@@ -28,17 +28,18 @@
 
 #import	<Foundation/NSObject.h>
 #import	<Foundation/NSRange.h>
+#import <Foundation/NSEnumerator.h>
+#import <GNUstepBase/GSBlocks.h>
 
 #if	defined(__cplusplus)
 extern "C" {
 #endif
 
-@class NSEnumerator;
 @class NSString;
 @class NSURL;
 @class NSIndexSet;
 
-@interface NSArray : NSObject <NSCoding, NSCopying, NSMutableCopying>
+@interface NSArray : NSObject <NSCoding, NSCopying, NSMutableCopying, NSFastEnumeration>
 
 + (id) array;
 + (id) arrayWithArray: (NSArray*)array;
@@ -48,14 +49,14 @@ extern "C" {
 #endif
 + (id) arrayWithObject: (id)anObject;
 + (id) arrayWithObjects: (id)firstObject, ...;
-+ (id) arrayWithObjects: (id*)objects count: (NSUInteger)count;
++ (id) arrayWithObjects: (const id[])objects count: (NSUInteger)count;
 
 - (NSArray*) arrayByAddingObject: (id)anObject;
 - (NSArray*) arrayByAddingObjectsFromArray: (NSArray*)anotherArray;
 - (BOOL) containsObject: anObject;
 - (NSUInteger) count;						// Primitive
-- (void) getObjects: (id*)aBuffer;
-- (void) getObjects: (id*)aBuffer range: (NSRange)aRange;
+- (void) getObjects: (__unsafe_unretained id[])aBuffer;
+- (void) getObjects: (__unsafe_unretained id[])aBuffer range: (NSRange)aRange;
 - (NSUInteger) indexOfObject: (id)anObject;
 - (NSUInteger) indexOfObject: (id)anObject inRange: (NSRange)aRange;
 - (NSUInteger) indexOfObjectIdenticalTo: (id)anObject;
@@ -70,7 +71,7 @@ extern "C" {
 - (id) initWithContentsOfURL: (NSURL*)aURL;
 #endif
 - (id) initWithObjects: firstObject, ...;
-- (id) initWithObjects: (id*)objects count: (NSUInteger)count;	// Primitive
+- (id) initWithObjects: (const id[])objects count: (NSUInteger)count;	// Primitive
 
 - (id) lastObject;
 - (id) objectAtIndex: (NSUInteger)index;			// Primitive
@@ -106,8 +107,8 @@ extern "C" {
 - (NSEnumerator*) reverseObjectEnumerator;
 
 - (NSString*) description;
-- (NSString*) descriptionWithLocale: (NSDictionary*)locale;
-- (NSString*) descriptionWithLocale: (NSDictionary*)locale
+- (NSString*) descriptionWithLocale: (id)locale;
+- (NSString*) descriptionWithLocale: (id)locale
 			     indent: (NSUInteger)level;
 
 - (BOOL) writeToFile: (NSString*)path atomically: (BOOL)useAuxiliaryFile;
@@ -115,6 +116,103 @@ extern "C" {
 - (BOOL) writeToURL: (NSURL*)url atomically: (BOOL)useAuxiliaryFile;
 - (id) valueForKey: (NSString*)key;
 #endif
+
+#if OS_API_VERSION(100600, GS_API_LATEST)
+
+DEFINE_BLOCK_TYPE(GSEnumeratorBlock, void, id, NSUInteger, BOOL*);
+DEFINE_BLOCK_TYPE(GSPredicateBlock, BOOL, id, NSUInteger, BOOL*);
+/**
+ * Enumerate over the collection using the given block.  The first argument is
+ * the object and the second is the index in the array.  The final argument is
+ * a pointer to a BOOL indicating whether the enumeration should stop.  Setting
+ * this to YES will interrupt the enumeration.
+ */
+- (void) enumerateObjectsUsingBlock: (GSEnumeratorBlock)aBlock;
+
+/**
+ * Enumerate over the collection using the given block.  The first argument is
+ * the object and the second is the index in the array.  The final argument is
+ * a pointer to a BOOL indicating whether the enumeration should stop.  Setting
+ * this to YES will interrupt the enumeration.
+ *
+ * The opts argument is a bitfield.  Setting the NSNSEnumerationConcurrent flag
+ * specifies that it is thread-safe.  The NSEnumerationReverse bit specifies
+ * that it should be enumerated in reverse order.
+ */
+- (void) enumerateObjectsWithOptions: (NSEnumerationOptions)opts 
+			  usingBlock: (GSEnumeratorBlock)aBlock;
+/**
+ * Enumerate over the specified indexes in the collection using the given
+ * block.  The first argument is the object and the second is the index in the
+ * array.  The final argument is a pointer to a BOOL indicating whether the
+ * enumeration should stop.  Setting this to YES will interrupt the
+ * enumeration.
+ *
+ * The opts argument is a bitfield.  Setting the NSNSEnumerationConcurrent flag
+ * specifies that it is thread-safe.  The NSEnumerationReverse bit specifies
+ * that it should be enumerated in reverse order.
+ */
+- (void) enumerateObjectsAtIndexes: (NSIndexSet*)indexSet
+			   options: (NSEnumerationOptions)opts
+			usingBlock: (GSEnumeratorBlock)block;
+/**
+ * Returns the indexes of the objects in a collection that match the condition
+ * specified by the block.
+ *
+ * The opts argument is a bitfield.  Setting the NSNSEnumerationConcurrent flag
+ * specifies that it is thread-safe.  The NSEnumerationReverse bit specifies
+ * that it should be enumerated in reverse order.
+ */
+- (NSIndexSet *) indexesOfObjectsWithOptions: (NSEnumerationOptions)opts 
+				 passingTest: (GSPredicateBlock)predicate;
+
+/**
+ * Returns the indexes of the objects in a collection that match the condition
+ * specified by the block.
+ */
+- (NSIndexSet*) indexesOfObjectsPassingTest: (GSPredicateBlock)predicate;
+
+/**
+ * Returns the indexes of the objects in a collection that match the condition
+ * specified by the block and are in the range specified by the index set.
+ *
+ * The opts argument is a bitfield.  Setting the NSNSEnumerationConcurrent flag
+ * specifies that it is thread-safe.  The NSEnumerationReverse bit specifies
+ * that it should be enumerated in reverse order.
+ */
+- (NSIndexSet*) indexesOfObjectsAtIndexes: (NSIndexSet*)indexSet
+				  options: (NSEnumerationOptions)opts
+			      passingTest: (GSPredicateBlock)predicate;
+
+/**
+ * Returns the index of the first object in the array that matches the
+ * condition specified by the block.
+ *
+ * The opts argument is a bitfield.  Setting the NSNSEnumerationConcurrent flag
+ * specifies that it is thread-safe.  The NSEnumerationReverse bit specifies
+ * that it should be enumerated in reverse order.
+ */
+- (NSUInteger) indexOfObjectWithOptions: (NSEnumerationOptions)opts 
+			    passingTest: (GSPredicateBlock)predicate;
+
+/**
+ * Returns the index of the first object in the array that matches the
+ * condition specified by the block.
+ */
+- (NSUInteger) indexOfObjectPassingTest: (GSPredicateBlock)predicate;
+
+/**
+ * Returns the index of the first object in the specified range in a collection
+ * that matches the condition specified by the block.
+ *
+ * The opts argument is a bitfield.  Setting the NSNSEnumerationConcurrent flag
+ * specifies that it is thread-safe.  The NSEnumerationReverse bit specifies
+ * that it should be enumerated in reverse order.
+ */
+- (NSUInteger) indexOfObjectAtIndexes: (NSIndexSet*)indexSet
+			      options: (NSEnumerationOptions)opts
+			  passingTest: (GSPredicateBlock)predicate;
+#endif
 @end
 
 
@@ -169,31 +267,12 @@ extern "C" {
 
 @end
 
-@interface	NSArray (GSCategories)
-/*
- *	Extension methods for working with sorted arrays - use a binary chop
- *	to determine the insertion location for an nobject.  If equal objects
- *	already exist in the array, they will be located immediately before
- *	the insertion position.
- * 
- *	The comparator function takes two items as arguments, the first is the
- *	item to be added, the second is the item already in the array.
- *      The function should return NSOrderedAscending if the item to be
- *      added is 'less than' the item in the array, NSOrderedDescending
- *      if it is greater, and NSOrderedSame if it is equal.
- *
- *	The selector version works the same - returning NSOrderedAscending if
- *	the receiver is 'less than' the item in the array.
- */
-- (NSUInteger) insertionPosition: (id)item
-		   usingFunction: (NSComparisonResult (*)(id, id, void *))sorter
-		         context: (void *)context;
-- (NSUInteger) insertionPosition: (id)item
-		   usingSelector: (SEL)comp;
-@end
-
 #if	defined(__cplusplus)
 }
 #endif
 
+#if	!NO_GNUSTEP && !defined(GNUSTEP_BASE_INTERNAL)
+#import	<GNUstepBase/NSArray+GNUstepBase.h>
+#endif
+
 #endif /* __NSArray_h_GNUSTEP_BASE_INCLUDE */
