@@ -107,6 +107,8 @@ GS_EXPORT NSString* const NSLoadedClasses;
 */
 @interface NSBundle : NSObject
 {
+#if	GS_EXPOSE(NSBundle)
+@public
   NSString		*_path;
   NSMutableArray	*_bundleClasses;
   Class			_principalClass;
@@ -116,6 +118,16 @@ GS_EXPORT NSString* const NSLoadedClasses;
   BOOL			_codeLoaded;
   unsigned		_version;
   NSString      	*_frameworkVersion;
+#endif
+#if     GS_NONFRAGILE
+#else
+  /* Pointer to private additional data used to avoid breaking ABI
+   * when we don't have the non-fragile ABI available.
+   * Use this mechanism rather than changing the instance variable
+   * layout (see Source/GSInternal.h for details).
+   */
+  @private id _internal GS_UNUSED_IVAR;
+#endif
 }
 
 /** Return an array enumerating all the bundles in the application.  This
@@ -172,15 +184,18 @@ GS_EXPORT NSString* const NSLoadedClasses;
  * not readable, return nil.  If you want the main bundle of an
  * application or a tool, it's better if you use +mainBundle.  */
 + (NSBundle*) bundleWithPath: (NSString*)path;
+#if OS_API_VERSION(100600,GS_API_LATEST) 
++ (NSBundle*) bundleWithURL: (NSURL*)url;
+#endif
 
 /**
   Returns an absolute path for a resource name with the extension
-  ext in the specified bundlePath.  See also
+  in the specified bundlePath.  See also
   -pathForResource:ofType:inDirectory: for more information on
   searching a bundle.  
  */
 + (NSString*) pathForResource: (NSString*)name
-		       ofType: (NSString*)ext
+		       ofType: (NSString*)extension
 		  inDirectory: (NSString*)bundlePath;
 
 /**
@@ -188,10 +203,17 @@ GS_EXPORT NSString* const NSLoadedClasses;
   so this method behaves exactly like +pathForResource:ofType:inDirectory:.
  */
 + (NSString*) pathForResource: (NSString*)name
-		       ofType: (NSString*)ext
+		       ofType: (NSString*)extension
 		  inDirectory: (NSString*)bundlePath
 		  withVersion: (int)version;
 
+#if OS_API_VERSION(100600,GS_API_LATEST) 
++ (NSURL*) URLForResource: (NSString*)name
+            withExtension: (NSString*)extension
+             subdirectory: (NSString*)subpath
+          inBundleWithURL: (NSURL*)bundleURL;
+#endif
+
 /** <init />
  * Init the bundle for reading resources from path.<br />
  * The MacOS-X documentation says that the path must be a full path to
@@ -214,6 +236,10 @@ GS_EXPORT NSString* const NSLoadedClasses;
  */
 - (id) initWithPath: (NSString*)path;
 
+#if OS_API_VERSION(100600,GS_API_LATEST) 
+- (id) initWithURL: (NSURL*)url;
+#endif
+
 /** Return the path to the bundle - an absolute path.  */
 - (NSString*) bundlePath;
 
@@ -256,7 +282,7 @@ GS_EXPORT NSString* const NSLoadedClasses;
 
 /**
   <p>
-   Returns an absolute path for a resource name with the extension ext
+   Returns an absolute path for a resource name with the extension
    in the specified bundlePath. Directories in the bundle are searched
    in the following order:
    </p>
@@ -271,21 +297,33 @@ GS_EXPORT NSString* const NSLoadedClasses;
    the bundle.
    </p>
    <p>
-   If ext is nil or empty, then the first file exactly matching name
+   If extension is nil or empty, then the first file exactly matching name
    (ie with no extension) is returned.
    </p>
 */
 - (NSString*) pathForResource: (NSString*)name
-		       ofType: (NSString*)ext
+		       ofType: (NSString*)extension
 		  inDirectory: (NSString*)subPath;
 
 /**
-   Returns an absolute path for a resource name with the extension ext
+   Returns an absolute path for a resource name with the extension
    in the receivers bundle path. 
    See -pathForResource:ofType:inDirectory:.
  */
 - (NSString*) pathForResource: (NSString*)name
-		       ofType: (NSString*)ext;
+		       ofType: (NSString*)extension;
+
+#if OS_API_VERSION(100600,GS_API_LATEST) 
+- (NSURL*) URLForResource: (NSString*)name
+            withExtension: (NSString*)extension;
+- (NSURL*) URLForResource: (NSString*)name
+            withExtension: (NSString*)extension
+             subdirectory: (NSString*)subpath;
+- (NSURL*) URLForResource: (NSString*)name 
+            withExtension: (NSString*)extension
+             subdirectory: (NSString*)subpath
+             localization: (NSString*)localizationName;
+#endif
 
 /**
    <p>Returns the value for the key found in the strings file tableName, or
@@ -296,9 +334,16 @@ GS_EXPORT NSString* const NSLoadedClasses;
    localized equivalent found.  This can be useful during development to check
    where a given string in the UI is "coming from".</p>
  */
+#ifdef __clang__
+- (NSString*) localizedStringForKey: (NSString*)key
+			      value: (NSString*)value
+			      table: (NSString*)tableName
+				  __attribute__((format_arg(1)));
+#else
 - (NSString*) localizedStringForKey: (NSString*)key
 			      value: (NSString*)value
 			      table: (NSString*)tableName;
+#endif
 
 /** Returns the absolute path to the resources directory of the bundle.  */
 - (NSString*) resourcePath;
@@ -355,7 +400,7 @@ GS_EXPORT NSString* const NSLoadedClasses;
  * resources.
  */
 - (NSString*) pathForResource: (NSString*)name
-		       ofType: (NSString*)ext
+		       ofType: (NSString*)extension
 		  inDirectory: (NSString*)subPath
 	      forLocalization: (NSString*)localizationName;
 
@@ -462,7 +507,7 @@ GS_EXPORT NSString* const NSLoadedClasses;
 
 /** Find a resource in the "Library" directory. */
 + (NSString*) pathForLibraryResource: (NSString*)name
-			      ofType: (NSString*)ext
+			      ofType: (NSString*)extension
 			 inDirectory: (NSString*)bundlePath;
 
 @end
@@ -569,4 +614,8 @@ GS_EXPORT NSString* const NSLoadedClasses;
 }
 #endif
 
+#if     !NO_GNUSTEP && !defined(GNUSTEP_BASE_INTERNAL)
+#import <GNUstepBase/NSBundle+GNUstepBase.h>
+#endif
+
 #endif	/* __NSBundle_h_GNUSTEP_BASE_INCLUDE */
