@@ -77,8 +77,14 @@ typedef enum _NSPredicateOperatorType
   NSUInteger			_options;
   NSPredicateOperatorType	_type;
 #endif
-#if	!GS_NONFRAGILE
-  void				*_unused;
+#if     GS_NONFRAGILE
+#else
+  /* Pointer to private additional data used to avoid breaking ABI
+   * when we don't have the non-fragile ABI available.
+   * Use this mechanism rather than changing the instance variable
+   * layout (see Source/GSInternal.h for details).
+   */
+  @private id _internal GS_UNUSED_IVAR;
 #endif
   @public
   NSExpression			*_left;
