@@ -31,6 +31,16 @@
 #ifndef __NSException_h_GNUSTEP_BASE_INCLUDE
 #define __NSException_h_GNUSTEP_BASE_INCLUDE
 #import	<GNUstepBase/GSVersionMacros.h>
+#import	<GNUstepBase/GSConfig.h>
+
+#if     defined(_NATIVE_OBJC_EXCEPTIONS)
+#define USER_NATIVE_OBJC_EXCEPTIONS       1
+#else
+#define USER_NATIVE_OBJC_EXCEPTIONS       0
+#endif
+#if     !BASE_NATIVE_OBJC_EXCEPTIONS && USER_NATIVE_OBJC_EXCEPTIONS
+#error  The current setting for native-objc-exceptions does not match that of gnustep-base ... please correct this.
+#endif
 
 #import	<Foundation/NSString.h>
 #include <setjmp.h>
@@ -83,10 +93,12 @@ extern "C" {
 */
 @interface NSException : NSObject <NSCoding, NSCopying>
 {    
+#if	GS_EXPOSE(NSException)
 @private
   NSString *_e_name;
   NSString *_e_reason;
   void *_reserved;
+#endif
 }
 
 /**
@@ -117,7 +129,7 @@ extern "C" {
 	format: (NSString*)format
      arguments: (va_list)argList;
 
-#if OS_API_VERSION(100500,GS_API_LATEST) && GS_API_VERSION(011501,GS_API_LATEST)
+#if OS_API_VERSION(100500,GS_API_LATEST) && GS_API_VERSION( 11501,GS_API_LATEST)
 /** Returns an array of the call stack return addresses at the point when
  * the exception was raised.  Re-raising the exception does not change
  * this value.
@@ -125,6 +137,16 @@ extern "C" {
 - (NSArray*) callStackReturnAddresses;
 #endif
 
+#if OS_API_VERSION(100600,GS_API_LATEST) && GS_API_VERSION( 11903,GS_API_LATEST)
+/**
+ * Returns an array of the symbolic names of the call stack return addresses.  
+ * Note that, on some platforms, symbols are only exported in
+ * position-independent code and so these may only return numeric addresses for
+ * code in static libraries or the main application.  
+ */
+- (NSArray*) callStackSymbols;
+#endif
+
 /**
  * <init/>Initializes a newly allocated NSException object with a
  * name, reason and a dictionary userInfo.
@@ -241,7 +263,7 @@ typedef struct _NSHandler
 {
     jmp_buf jumpState;			/* place to longjmp to */
     struct _NSHandler *next;		/* ptr to next handler */
-    NSException *exception;
+    __unsafe_unretained NSException *exception;
 } NSHandler;
 
 /**
@@ -306,7 +328,7 @@ NSSetUncaughtExceptionHandler(NSUncaughtExceptionHandler *handler);
    It is illegal to exit the first block of code by any other means than
    NS_VALRETURN, NS_VOIDRETURN, or just falling out the bottom.
  */
-#ifdef _NATIVE_OBJC_EXCEPTIONS
+#if     USER_NATIVE_OBJC_EXCEPTIONS
 
 # define NS_DURING       @try {
 # define NS_HANDLER      } @catch (NSException * localException) {
@@ -316,7 +338,7 @@ NSSetUncaughtExceptionHandler(NSUncaughtExceptionHandler *handler);
 # define NS_VALUERETURN(object, id)     return (object)
 # define NS_VOIDRETURN                  return
 
-#else // _NATIVE_OBJC_EXCEPTIONS
+#elif   !USER_NATIVE_OBJC_EXCEPTIONS && !BASE_NATIVE_OBJC_EXCEPTIONS
 
 /** Private support routine.  Do not call directly. */
 GS_EXPORT void _NSAddHandler( NSHandler *handler );
@@ -358,15 +380,16 @@ GS_EXPORT void _NSRemoveHandler( NSHandler *handler );
 - (void) handleFailureInFunction: (NSString*)functionName 
 			    file: (NSString*)fileName 
 		      lineNumber: (NSInteger)line 
-		     description: (NSString*)format,...;
+		     description: (NSString*)format,... GS_NORETURN_METHOD;
 
 - (void) handleFailureInMethod: (SEL)aSelector 
 			object: object 
 			  file: (NSString*)fileName 
 		    lineNumber: (NSInteger)line 
-		   description: (NSString*)format,...;
+		   description: (NSString*)format,... GS_NORETURN_METHOD;
 
 @end
+extern NSString *const NSAssertionHandlerKey;
 
 #ifdef	NS_BLOCK_ASSERTIONS
 #define _NSAssertArgs(condition, desc, args...)		
