@@ -157,8 +157,20 @@ GS_EXPORT NSString * const NSFileHandleOperationException;
 
 @interface NSPipe : NSObject
 {
-   NSFileHandle*	readHandle;
-   NSFileHandle*	writeHandle;
+#if	GS_EXPOSE(NSPipe)
+@private
+  NSFileHandle	*_readHandle;
+  NSFileHandle	*_writeHandle;
+#endif
+#if     GS_NONFRAGILE
+#else
+  /* Pointer to private additional data used to avoid breaking ABI
+   * when we don't have the non-fragile ABI available.
+   * Use this mechanism rather than changing the instance variable
+   * layout (see Source/GSInternal.h for details).
+   */
+  @private id _internal GS_UNUSED_IVAR;
+#endif
 }
 + (id) pipe;
 - (NSFileHandle*) fileHandleForReading;
@@ -202,22 +214,44 @@ GS_EXPORT NSString * const NSFileHandleOperationException;
 
 /**
  * Where OpenSSL is available, you can use the subclass returned by +sslClass
- * to handle SSL connections.
- *   The -sslAccept method is used to do SSL handshake and start an
- *   encrypted session on a channel where the connection was initiated
- *   from the far end.
- *   The -sslConnect method is used to do SSL handshake and start an
- *   encrypted session on a channel where the connection was initiated
- *   from the near end..
- *   The -sslDisconnect method is used to end the encrypted session.
- *   The -sslSetCertificate:privateKey:PEMpasswd: method is used to
- *   establish a client certificate before starting an encrypted session.
+ * to handle SSL connections.<br />
+ * The -sslAccept method is used to do SSL handshake and start an
+ * encrypted session on a channel where the connection was initiated
+ * from the far end.<br />
+ * The -sslConnect method is used to do SSL handshake and start an
+ * encrypted session on a channel where the connection was initiated
+ * from the near end.<br />
+ * The -sslDisconnect method is used to end the encrypted session.
+ * The -sslSetCertificate:privateKey:PEMpasswd: method is used to
+ * establish a client certificate before starting an encrypted session.<br />
+ * NB. Some of these methods may block while performing I/O on the network
+ * connection, (though they should run the current runloop while doing so)
+ * so you should structure your code to handle that.  In particular, if you
+ * are writing a server application, you should initiate a background accept
+ * to allow another incoming connection <em>before</em> you perform an
+ * -sslAccept on a connection you have just accepted.
  */
 @interface NSFileHandle (GNUstepOpenSSL)
 + (Class) sslClass;
 - (BOOL) sslAccept;
 - (BOOL) sslConnect;
 - (void) sslDisconnect;
+/** Make a non-blocking handshake attempt.  Calls to this method should be
+ * repeated until the method returns YES indicating that the handshake
+ * completed.  If the method returns YES indicating completion of the
+ * handshake, the result indicates whether the handshake succeeded in
+ * establishing a connection or not.
+ */
+- (BOOL) sslHandshakeEstablished: (BOOL*)result outgoing: (BOOL)isOutgoing;
+/** Sets certification data for the SSL connection.<br />
+ * The value of certFile is the path to a file containing a PEM encoded
+ * certificate for this host (optionally followed by other PEM encoded
+ * certificates in a chain leading to a root certificate authority).<br />
+ * The value of privatekey is the path of a file containing a PEM encoded key
+ * used to establish handshakes using the host certificate.<br />
+ * The value of PEMpasswd is a string used as the password to access the
+ * content of the key file.
+ */
 - (void) sslSetCertificate: (NSString*)certFile
                 privateKey: (NSString*)privateKey
                  PEMpasswd: (NSString*)PEMpasswd;
@@ -248,4 +282,8 @@ GS_EXPORT NSString * const GSFileHandleNotificationError;
 }
 #endif
 
+#if     !NO_GNUSTEP && !defined(GNUSTEP_BASE_INTERNAL)
+#import <GNUstepBase/NSFileHandle+GNUstepBase.h>
+#endif
+
 #endif /* __NSFileHandle_h_GNUSTEP_BASE_INCLUDE */
