@@ -42,8 +42,6 @@ extern "C" {
 @class NSLock;
 
 @interface NSNotification : NSObject <NSCopying, NSCoding>
-{
-}
 
 /* Creating a Notification Object */
 + (NSNotification*) notificationWithName: (NSString*)name
@@ -65,7 +63,10 @@ extern "C" {
 
 @interface NSNotificationCenter : NSObject
 {
-  void	*_table;
+#if	GS_EXPOSE(NSNotificationCenter)
+@private
+  GS_GC_STRONG void	*_table;
+#endif
 }
 
 + (NSNotificationCenter*) defaultCenter;
