@@ -22,7 +22,6 @@
    Boston, MA 02111 USA.
 
    AutogsdocSource: NSObject.m
-   AutogsdocSource: Additions/GSCategories.m
    */ 
 
 #ifndef __NSObject_h_GNUSTEP_BASE_INCLUDE
@@ -30,7 +29,6 @@
 
 #import	<Foundation/NSObjCRuntime.h>
 #import <objc/objc.h>
-#import <objc/typedstream.h>
 #import	<Foundation/NSZone.h>
 
 #ifndef	GS_WITH_GC
@@ -62,32 +60,156 @@ extern "C" {
  * in assuming that the receiver can handle it.
  */
 @protocol NSObject
-- (Class) class;			/** See [NSObject-class] */
-- (Class) superclass;			/** See [NSObject-superclass] */
-- (BOOL) isEqual: (id)anObject;		/** See [NSObject-isEqual:] */
-- (BOOL) isKindOfClass: (Class)aClass;	/** See [NSObject-isKindOfClass:] */
-- (BOOL) isMemberOfClass: (Class)aClass;/** See [NSObject-isMemberOfClass:] */
-- (BOOL) isProxy;			/** See [NSObject-isProxy] */
-- (NSUInteger) hash;			/** See [NSObject-hash] */
-- (id) self;				/** See [NSObject-self] */
-- (id) performSelector: (SEL)aSelector;	/** See [NSObject-performSelector:] */
-/** See [NSObject-performSelector:withObject:] */
+/**
+ * Returns the class of the receiver.  If the receiver is a proxy, then this
+ * may return the class of the proxy target.  Use -isProxy to determine whether
+ * the receiver is a proxy.  If you wish to find the real class of the
+ * receiver, ignoring proxies, then use object_getClass().  
+ */
+- (Class) class;
+/**
+ * Returns the superclass of receiver's class.  If the receiver is a proxy,
+ * then this may return the class of the proxy target.  Use -isProxy to
+ * determine whether the receiver is a proxy.  If you wish to find the real
+ * superclass of the receiver's class, ignoring proxies, then use
+ * class_getSuperclass(object_getClass()).
+ */
+- (Class) superclass;
+/**
+ * Returns whether the receiver is equal to the argument.  Defining equality is
+ * complex, so be careful when implementing this method.  Collections such as
+ * NSSet depend on the behaviour of this method.  In particular, this method
+ * must be commutative, so for any objects a and b:
+ *
+ * [a isEqual: b] == [b isEqual: a]
+ *
+ * This means that you must be very careful when returning YES if the argument
+ * is of another class.  For example, if you define a number class that returns
+ * YES if the argument is a string representation of the number, then this will
+ * break because the string will not recognise your object as being equal to
+ * itself.
+ *
+ * If two objects are equal, then they must have the same hash value, however
+ * equal hash values do not imply equality.
+ */
+- (BOOL) isEqual: (id)anObject;
+/**
+ * Returns YES if the receiver is an instance of the class, an instance of the
+ * subclass, or (in the case of proxies), an instance of something that can be
+ * treated as an instance of the class.
+ */
+- (BOOL) isKindOfClass: (Class)aClass;
+/**
+ * Returns YES if the receiver is an instance of the class or (in the case of
+ * proxies), an instance of something that can be treated as an instance of the
+ * class.
+ *
+ * Calling this method is rarely the correct thing to do.  In most cases, a
+ * subclass can be substituted for a superclass, so you should never need to
+ * check that an object is really an instance of a specific class and not a
+ * subclass.  
+ */
+- (BOOL) isMemberOfClass: (Class)aClass;
+/**
+ * Returns YES if the receiver is a proxy, NO otherwise.  The default
+ * implementation of this method in NSObject returns NO, while the
+ * implementation in NSProxy returns YES.
+ */
+- (BOOL) isProxy;
+/**
+ * Returns a hash value for the object.  All objects that are equal *MUST*
+ * return the same hash value.  For efficient storage in sets, or as keys in
+ * dictionaries, different objects should return hashes spread evenly over the
+ * range of an integer.
+ *
+ * An object may not return different values from this method after being
+ * stored in a collection.  This typically means that ether the hash value must
+ * be constant after the object's creation, or that the object may not be
+ * modified while stored in an unordered collection.
+ */
+- (NSUInteger) hash;
+/**
+ * Returns the receiver.  In a proxy, this may (but is not required to) return
+ * the proxied object.
+ */
+- (id) self;
+/**
+ * Performs the specified selector.  The selector must correspond to a method
+ * that takes no arguments.
+ */
+- (id) performSelector: (SEL)aSelector;
+/**
+ * Performs the specified selector, with the object as the argument.  This
+ * method does not perform any automatic unboxing, so the selector must
+ * correspond to a method that takes one object argument.
+ */
 - (id) performSelector: (SEL)aSelector
 	    withObject: (id)anObject;
-/** See [NSObject-performSelector:withObject:withObject:] */
+/**
+ * Performs the specified selector, with the objects as the arguments.  This
+ * method does not perform any automatic unboxing, so the selector must
+ * correspond to a method that takes two object arguments.
+ */
 - (id) performSelector: (SEL)aSelector
 	    withObject: (id)object1
 	    withObject: (id)object2;
-/** See [NSObject-respondsToSelector:] */
+/**
+ * Returns YES if the object can respond to messages with the specified
+ * selector.  The default implementation in NSObject returns YES if the
+ * receiver has a method corresponding to the method, but other classes may
+ * return YES if they can respond to a selector using one of the various
+ * forwarding mechanisms.
+ */
 - (BOOL) respondsToSelector: (SEL)aSelector;
-/** See [NSObject-conformsToProtocol:] */
+/**
+ * Returns YES if the receiver conforms to the specified protocol.
+ */
 - (BOOL) conformsToProtocol: (Protocol*)aProtocol;
-- (id) retain;				/** See [NSObject-retain] */
-- (id) autorelease			/** See [NSObject-autorelease] */;
-- (oneway void) release;		/** See [NSObject-release] */
-- (NSUInteger) retainCount;		/** See [NSObject-retainCount] */
-- (NSZone*) zone;			/** See [NSObject-zone] */
-- (NSString*) description;		/** See [NSObject-description] */
+/**
+ * Increments the reference count of the object and returns the receiver.  In
+ * garbage collected mode, this method does nothing.  In automated reference
+ * counting mode, you may neither implement this method nor call it directly.
+ */
+- (id) retain NS_AUTOMATED_REFCOUNT_UNAVAILABLE;
+/**
+ * Decrements the reference count of the object and destroys if it there are no
+ * remaining references.  In garbage collected mode, this method does nothing.
+ * In automated reference counting mode, you may neither implement this method
+ * nor call it directly.
+ */
+- (oneway void) release NS_AUTOMATED_REFCOUNT_UNAVAILABLE;
+/**
+ * Performs a deferred -release operation.  The object's reference count is
+ * decremented at the end of the scope of the current autorelease pool,
+ * identified either by a -drain message sent to the current NSAutoreleasePool
+ * instance, or in more recent versions of Objective-C by the end of an
+ * @autorelease_pool scope.
+ *
+ * In garbage collected mode, this method does nothing.  In automated reference
+ * counting mode, you may neither implement this method nor call it directly.
+ */
+- (id) autorelease NS_AUTOMATED_REFCOUNT_UNAVAILABLE;
+/**
+ * Returns the current retain count of an object.  This does not include the
+ * result of any pending autorelease operations.
+ *
+ * Code that relies on this method returning a sane value is broken.  For
+ * singletons, it may return NSUIntegerMax.  Even when it is tracking a retain
+ * count, it will not include on-stack pointers in manual retain/release mode,
+ * pointers marked as __unsafe_unretain or __weak in ARC mode, or pending
+ * autorelease operations.  Its value is therefore largely meaningless.  It can
+ * occasionally be useful for debugging.
+ */
+- (NSUInteger) retainCount NS_AUTOMATED_REFCOUNT_UNAVAILABLE;
+/**
+ * Returns the description of the object.  This is used by the %@ format
+ * specifier in strings.
+ */
+- (NSString*) description;
+/**
+ * Returns the zone of the object.
+ */
+- (NSZone*) zone NS_AUTOMATED_REFCOUNT_UNAVAILABLE;
 @end
 
 /**
@@ -187,7 +309,52 @@ extern "C" {
 + (id) allocWithZone: (NSZone*)z;
 + (id) alloc;
 + (Class) class;
-+ (NSString*) description;
+
+/**
+ * This method is automatically invoked on any class which implements it
+ * when the class is loaded into the runtime.<br />
+ * It is also invoked on any category where the method is implemented
+ * when that category is loaded into the runtime.<br />
+ * The +load method is called directly by the runtime and you should never
+ * send a +load message to a class yourself.<br />
+ * This method is called <em>before</em> the +initialize message is sent
+ * to the class, so you cannot depend on class initialisation having been
+ * performed, or upon other classes existing (apart from superclasses of
+ * the receiver, since +load is called on superclasses before it is called
+ * on their subclasses).<br />
+ * As a gross generalisation, it is safe to use C code, including
+ * most ObjectiveC runtime functions within +load, but attempting to send
+ * messages to ObjectiveC objects is likely to fail.<br />
+ * In GNUstep, this method is implemented for NSObject to perform some
+ * initialisation for the base library.<br />
+ * If you implement +load for a class, don't call [super load] in your
+ * implementation.
+ */
++ (void) load;
+
+/**
+ * This message is automatically sent to a class by the runtime.  It is
+ * sent once for each class, just before the class is used for the first
+ * time (excluding any automatic call to +load by the runtime).<br />
+ * The message is sent in a thread-safe manner ... other threads may not
+ * call methods of the class until +initialize has finished executing.<br />
+ * If the class has a superclass, its implementation of +initialize is
+ * called first.<br />
+ * If the class does not implement +initialize then the implementation
+ * in the closest superclass may be called.  This means that +initialize may
+ * be called more than once, and the recommended way to handle this by
+ * using the
+ * <code>
+ * if (self == [classname class])
+ * </code>
+ * conditional to check whether the method is being called for a subclass.<br />
+ * You should never call +initialize yourself ... let the runtime do it.<br />
+ * You can implement +initialize in your own class if you need to.
+ * NSObject's implementation handles essential root object and base
+ * library initialization.<br />
+ * It should be safe to call [super initialize] in your implementation
+ * of +initialize.
+ */
 + (void) initialize;
 + (IMP) instanceMethodForSelector: (SEL)aSelector;
 + (NSMethodSignature*) instanceMethodSignatureForSelector: (SEL)aSelector;
@@ -196,44 +363,77 @@ extern "C" {
 + (id) new;
 + (void) poseAsClass: (Class)aClassObject;
 + (id) setVersion: (NSInteger)aVersion;
-+ (Class) superclass;
 + (NSInteger) version;
 
-- (id) autorelease;
 - (id) awakeAfterUsingCoder: (NSCoder*)aDecoder;
-- (Class) class;
 - (Class) classForArchiver;
 - (Class) classForCoder;
-- (BOOL) conformsToProtocol: (Protocol*)aProtocol;
 - (id) copy;
 - (void) dealloc;
-- (NSString*) description;
 - (void) doesNotRecognizeSelector: (SEL)aSelector;
 - (void) forwardInvocation: (NSInvocation*)anInvocation;
-- (NSUInteger) hash;
 - (id) init;
-- (BOOL) isEqual: anObject;
-- (BOOL) isKindOfClass: (Class)aClass;
-- (BOOL) isMemberOfClass: (Class)aClass;
-- (BOOL) isProxy;
 - (IMP) methodForSelector: (SEL)aSelector;
 - (NSMethodSignature*) methodSignatureForSelector: (SEL)aSelector;
 - (id) mutableCopy;
-- (id) performSelector: (SEL)aSelector;
-- (id) performSelector: (SEL)aSelector
-	    withObject: (id)anObject;
-- (id) performSelector: (SEL)aSelector
-	    withObject: (id)object1
-	    withObject: (id)object2;
-- (void) release;
 - (id) replacementObjectForArchiver: (NSArchiver*)anArchiver;
 - (id) replacementObjectForCoder: (NSCoder*)anEncoder;
-- (BOOL) respondsToSelector: (SEL)aSelector;
-- (id) retain;
-- (NSUInteger) retainCount;
-- (id) self;
 - (Class) superclass;
-- (NSZone*) zone;
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
+/**
+ * This method will be called when attempting to send a message a class that
+ * does not understand it.  The class may install a new method for the given
+ * selector and return YES, otherwise it should return NO.
+ *
+ * Note: This method is only reliable when using the GNUstep runtime.  If you
+ * require compatibility with the GCC runtime, you must also implement
+ * -forwardInvocation: with equivalent semantics.  This will be considerably
+ *  slower, but more portable.
+ */
++ (BOOL) resolveClassMethod: (SEL)name;
+
+/**
+ * This method will be called when attempting to send a message an instance
+ * that does not understand it.  The class may install a new method for the
+ * given selector and return YES, otherwise it should return NO.
+ *
+ * Note: This method is only reliable when using the GNUstep runtime.  If you
+ * require compatibility with the GCC runtime, you must also implement
+ * -forwardInvocation: with equivalent semantics.  This will be considerably
+ *  slower, but more portable.
+ */
++ (BOOL) resolveInstanceMethod: (SEL)name;
+#endif
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
+/**
+ * Returns an auto-accessing proxy for the given object.  This proxy sends a
+ * -beginContentAccess message to the receiver when it is created and an
+ * -endContentAccess message when it is destroyed.  This prevents an object
+ * that implements the NSDiscardableContent protocol from having its contents
+ * discarded for as long as the proxy exists.  
+ *
+ * On systems using the GNUstep runtime, messages send to the proxy will be
+ * slightly slower than direct messages.  With the GCC runtime, they will be
+ * approximately two orders of magnitude slower.  The GNUstep runtime,
+ * therefore, is strongly recommended for code calling this method.
+ */
+- (id) autoContentAccessingProxy;
+
+/**
+ * If an object does not understand a message, it may delegate it to another
+ * object.  Returning nil indicates that forwarding should not take place.  The
+ * default implementation of this returns nil, but care should be taken when
+ * subclassing NSObject subclasses and overriding this method that
+ * the superclass implementation is called if returning nil.
+ *
+ * Note: This method is only reliable when using the GNUstep runtime and code
+ * compiled with clang.  If you require compatibility with GCC and the GCC
+ * runtime, you must also implement -forwardInvocation: with equivalent
+ * semantics.  This will be considerably slower, but more portable.
+ */
+- (id) forwardingTargetForSelector: (SEL)aSelector;
+
+#endif
 @end
 
 /**
@@ -243,7 +443,7 @@ extern "C" {
  * instance variables of the object.<br />
  * This function is used by the [NSObject+allocWithZone:] method.
  */
-GS_EXPORT NSObject *
+GS_EXPORT id
 NSAllocateObject(Class aClass, NSUInteger extraBytes, NSZone *zone);
 
 /**
@@ -251,7 +451,7 @@ NSAllocateObject(Class aClass, NSUInteger extraBytes, NSZone *zone);
  * This function is used by the [NSObject-dealloc] method.
  */
 GS_EXPORT void
-NSDeallocateObject(NSObject *anObject);
+NSDeallocateObject(id anObject);
 
 /**
  * Used to copy anObject.  This makes a bitwise copy of anObject to
@@ -282,19 +482,6 @@ NSExtraRefCount(id anObject);
 GS_EXPORT void
 NSIncrementExtraRefCount(id anObject);
 
-/**
- * Contains values <code>NSOrderedSame</code>, <code>NSOrderedAscending</code>
- * <code>NSOrderedDescending</code>, for left hand side equals, less than, or
- * greater than right hand side.
- */
-typedef enum _NSComparisonResult 
-{
-  NSOrderedAscending = -1, NSOrderedSame, NSOrderedDescending
-} 
-NSComparisonResult;
-
-enum {NSNotFound = 0x7fffffff};
-
 #if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 
 /** Global lock to be used by classes when operating on any global
@@ -310,61 +497,10 @@ GS_EXPORT NSRecursiveLock *gnustep_global_lock;
 
 #if GS_API_VERSION(GS_API_NONE, 011700)
 @interface NSObject (GNUstep)
-- (BOOL) isInstance;
-- (id) makeImmutableCopyOnFail: (BOOL)force;
-- (Class) transmuteClassTo: (Class)aClassObject;
-+ (Class) autoreleaseClass;
-+ (void) setAutoreleaseClass: (Class)aClass;
 + (void) enableDoubleReleaseCheck: (BOOL)enable;
-- (id) read: (TypedStream*)aStream;
-- (id) write: (TypedStream*)aStream;
 @end
 #endif
 
-/**
- * Provides a number of GNUstep-specific methods that are used to aid
- * implementation of the Base library.
- */
-@interface NSObject (GSCategories)
-
-/**
- * Message sent when an implementation wants to explicitly exclude a method
- * (but cannot due to compiler constraint), and wants to make sure it is not
- * called by mistake.  Default implementation raises an exception at runtime.
- */
-- (id) notImplemented: (SEL)aSel;
-
-/**
- * Message sent when an implementation wants to explicitly require a subclass
- * to implement a method (but cannot at compile time since there is no
- * <code>abstract</code> keyword in Objective-C).  Default implementation
- * raises an exception at runtime to alert developer that he/she forgot to
- * override a method.
- */
-- (id) subclassResponsibility: (SEL)aSel;
-
-/**
- * Message sent when an implementation wants to explicitly exclude a method
- * (but cannot due to compiler constraint) and forbid that subclasses
- * implement it.  Default implementation raises an exception at runtime.  If a
- * subclass <em>does</em> implement this method, however, the superclass's
- * implementation will not be called, so this is not a perfect mechanism.
- */
-- (id) shouldNotImplement: (SEL)aSel;
-
-/**
-  WARNING: The -compare: method for NSObject is deprecated
-           due to subclasses declaring the same selector with
-           conflicting signatures.
-           Comparison of arbitrary objects is not just meaningless
-           but also dangerous as most concrete implementations
-           expect comparable objects as arguments often accessing
-           instance variables directly.
-           This method will be removed in a future release.
-*/
-- (NSComparisonResult) compare: (id)anObject;
-@end
-
 #endif
 
 #import	<Foundation/NSDate.h>
@@ -407,6 +543,7 @@ GS_EXPORT NSRecursiveLock *gnustep_global_lock;
 	      afterDelay: (NSTimeInterval)seconds
 		 inModes: (NSArray*)modes;
 @end
+
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
 /**
  * The NSDiscardableContent protocol is used by objects which encapsulate data
@@ -414,33 +551,41 @@ GS_EXPORT NSRecursiveLock *gnustep_global_lock;
  * constraints are typically, but not always, related memory.  
  */
 @protocol NSDiscardableContent
+
 /**
  * This method is called before any access to the object.  It returns YES if
  * the object's content is still valid.  The caller must call -endContentAccess
  * once for every call to -beginContentAccess;
  */
-- (BOOL)beginContentAccess;
+- (BOOL) beginContentAccess;
+
+/**
+ * Discards the contents of the object if it is not currently being edited.
+ */
+- (void) discardContentIfPossible;
+
 /**
  * This method indicates that the caller has finished accessing the contents of
  * the object adopting this protocol.  Every call to -beginContentAccess must
  * be be paired with a call to this method after the caller has finished
  * accessing the contents.
  */
-- (void)endContentAccess;
-/**
- * Discards the contents of the object if it is not currently being edited.
- */
-- (void)discardContentIfPossible;
+- (void) endContentAccess;
+
 /**
  * Returns YES if the contents of the object have been discarded, either via a
  * call to -discardContentIfPossible while the object is not in use, or by some
- * implementation-dependent mechanism.  
+ * implementation dependent mechanism.  
  */
-- (BOOL)isContentDiscarded;
+- (BOOL) isContentDiscarded;
 @end
 #endif
 #if	defined(__cplusplus)
 }
 #endif
 
+#if     !NO_GNUSTEP && !defined(GNUSTEP_BASE_INTERNAL)
+#import <GNUstepBase/NSObject+GNUstepBase.h>
+#endif
+
 #endif /* __NSObject_h_GNUSTEP_BASE_INCLUDE */
