@@ -27,8 +27,7 @@
 #ifndef __NSPathUtilities_h_GNUSTEP_BASE_INCLUDE
 #define __NSPathUtilities_h_GNUSTEP_BASE_INCLUDE
 #import	<GNUstepBase/GSVersionMacros.h>
-
-#import	<GNUstepBase/GSObjCRuntime.h>
+#import	<Foundation/NSObjCRuntime.h>
 
 #if	defined(__cplusplus)
 extern "C" {
