@@ -39,6 +39,7 @@ extern "C" {
  */
 @interface NSScanner : NSObject <NSCopying>
 {
+#if	GS_EXPOSE(NSScanner)
 @private
   NSString		*_string;
   NSCharacterSet	*_charactersToBeSkipped;
@@ -48,6 +49,16 @@ extern "C" {
   unichar		_decimal;
   BOOL			_caseSensitive;
   BOOL			_isUnicode;
+#endif
+#if     GS_NONFRAGILE
+#else
+  /* Pointer to private additional data used to avoid breaking ABI
+   * when we don't have the non-fragile ABI available.
+   * Use this mechanism rather than changing the instance variable
+   * layout (see Source/GSInternal.h for details).
+   */
+  @private id _internal GS_UNUSED_IVAR;
+#endif
 }
 
 /*
