@@ -58,12 +58,20 @@ GS_EXPORT NSString *const NSGrammarUserDescription;
 #if	GS_EXPOSE(NSSpellServer)
 @private
   id _delegate;
-  BOOL _caseSensitive; 
-  unsigned char _dummy[3];
+  BOOL _caseSensitive GS_UNUSED_IVAR; 
+  unsigned char _dummy[3] GS_UNUSED_IVAR;
   NSMutableDictionary *_userDictionaries;
   NSString *_currentLanguage;
   NSArray *_ignoredWords;
-  void *_reserved;
+#endif
+#if     GS_NONFRAGILE
+#else
+  /* Pointer to private additional data used to avoid breaking ABI
+   * when we don't have the non-fragile ABI available.
+   * Use this mechanism rather than changing the instance variable
+   * layout (see Source/GSInternal.h for details).
+   */
+  @private id _internal GS_UNUSED_IVAR;
 #endif
 }
 
