@@ -110,6 +110,17 @@ enum
  NSWidthInsensitiveSearch = 256,
  NSForcedOrderingSearch = 512
 #endif
+#if OS_API_VERSION(100700,GS_API_LATEST) 
+ ,
+ /**
+  * Treats the search string as a regular expression.  This option may be
+  * combined with NSCaseInsensitiveSearch and NSAnchoredSearch, but no other
+  * search options.
+  *
+  * This option may only be used with the -rangeOfString: family of methods.
+  */
+ NSRegularExpressionSearch = 1024
+#endif
 };
 typedef NSUInteger NSStringCompareOptions;
 
@@ -339,7 +350,6 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 - (id) initWithData: (NSData*)data
 	   encoding: (NSStringEncoding)encoding;
 - (id) initWithContentsOfFile: (NSString*)path;
-- (id) init;
 
 // Getting a String's Length
 - (NSUInteger) length;
@@ -357,7 +367,6 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 // Dividing Strings into Substrings
 - (NSArray*) componentsSeparatedByString: (NSString*)separator;
 - (NSString*) substringFromIndex: (NSUInteger)index;
-- (NSString*) substringFromRange: (NSRange)aRange;
 - (NSString*) substringToIndex: (NSUInteger)index;
 
 // Finding Ranges of Characters and Substrings
@@ -758,9 +767,9 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 
 #if OS_API_VERSION(100300,GS_API_LATEST) 
 /** Not implemented */
-- (void) getParagraphStart: (NSUInteger *)startPtr
-                       end: (NSUInteger *)parEndPtr
-               contentsEnd: (NSUInteger *)contentsEndPtr
+- (void) getParagraphStart: (NSUInteger *)startIndex
+                       end: (NSUInteger *)parEndIndex
+               contentsEnd: (NSUInteger *)contentsEndIndex
                  forRange: (NSRange)range;
 /** Not implemented */
  - (NSRange) paragraphRangeForRange: (NSRange)range;
@@ -852,6 +861,7 @@ typedef NSUInteger NSStringEncodingConversionOptions;
  */
 @interface NXConstantString : NSString
 {
+@public
   const char * const nxcsptr;
   const unsigned int nxcslen;
 }
