@@ -72,8 +72,14 @@ extern "C" {
   id			_gcontext;
   void                  *_runLoopInfo;  // Per-thread runloop related info.
 #endif
-#if	!GS_NONFRAGILE
-  void                  *_unused;     // For future expansion
+#if     GS_NONFRAGILE
+#else
+  /* Pointer to private additional data used to avoid breaking ABI
+   * when we don't have the non-fragile ABI available.
+   * Use this mechanism rather than changing the instance variable
+   * layout (see Source/GSInternal.h for details).
+   */
+  @private id _internal GS_UNUSED_IVAR;
 #endif
 }
 
@@ -257,7 +263,7 @@ extern "C" {
 /**
  * Invokes -performSelectorOnMainThread:withObject:waitUntilDone:modes:
  * using the supplied arguments and an array containing common modes.<br />
- * These modes consist of NSRunLoopMode, NSConnectionreplyMode, and if
+ * These modes consist of NSRunLoopMode, NSConnectionReplyMode, and if
  * in an application, the NSApplication modes.
  */
 - (void) performSelectorOnMainThread: (SEL)aSelector
