@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,9 +26,6 @@
 #define __NSTimer_h_GNUSTEP_BASE_INCLUDE
 #import	<GNUstepBase/GSVersionMacros.h>
 
-/* This class is currently thrown together.  When it is cleaned up, it
-   may no longer be concrete. */
-
 #import	<Foundation/NSDate.h>
 
 #if	defined(__cplusplus)
@@ -39,9 +36,13 @@ extern "C" {
  *	NB. NSRunLoop is optimised using a hack that knows about the
  *	class layout for the fire date and invialidation flag in NSTimer.
  *	These MUST remain the first two items in the class.
+ *	Other classes must not attempt to use instance variables as
+ *	they are subject to change.
  */
 @interface NSTimer : NSObject
 {
+#if	GS_EXPOSE(NSTimer)
+@public
   NSDate 	*_date;		/* Must be first - for NSRunLoop optimisation */
   BOOL		_invalidated;	/* Must be 2nd - for NSRunLoop optimisation */
   BOOL		_repeats;
@@ -49,6 +50,16 @@ extern "C" {
   id		_target;
   SEL		_selector;
   id		_info;
+#endif
+#if     GS_NONFRAGILE
+#else
+  /* Pointer to private additional data used to avoid breaking ABI
+   * when we don't have the non-fragile ABI available.
+   * Use this mechanism rather than changing the instance variable
+   * layout (see Source/GSInternal.h for details).
+   */
+  @private id _internal GS_UNUSED_IVAR;
+#endif
 }
 
 /* Creating timer objects. */
@@ -76,17 +87,19 @@ extern "C" {
 - (void) invalidate;
 - (id) userInfo;
 
-
 #if	OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
+- (BOOL) isValid;
+- (NSTimeInterval) timeInterval;
+#endif
+
+#if	OS_API_VERSION(100200, GS_API_LATEST)
 - (id) initWithFireDate: (NSDate*)fd
 	       interval: (NSTimeInterval)ti
 		 target: (id)object
 	       selector: (SEL)selector
 	       userInfo: (id)info
 		repeats: (BOOL)f;
-- (BOOL) isValid;
 - (void) setFireDate: (NSDate*)fireDate;
-- (NSTimeInterval) timeInterval;
 #endif
 
 @end
