@@ -26,7 +26,7 @@
 #define __NSURLAuthenticationChallenge_h_GNUSTEP_BASE_INCLUDE
 #import	<GNUstepBase/GSVersionMacros.h>
 
-#if OS_API_VERSION(100200,GS_API_LATEST) && GS_API_VERSION(011300,GS_API_LATEST)
+#if OS_API_VERSION(100200,GS_API_LATEST) && GS_API_VERSION( 11300,GS_API_LATEST)
 
 #import	<Foundation/NSObject.h>
 
@@ -79,8 +79,9 @@ extern "C" {
  */
 @interface NSURLAuthenticationChallenge : NSObject
 {
-@private
+#if	GS_EXPOSE(NSURLAuthenticationChallenge)
   void	*_NSURLAuthenticationChallengeInternal;
+#endif
 }
 
 /**
