@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +26,7 @@
 #define __NSURLConnection_h_GNUSTEP_BASE_INCLUDE
 #import	<GNUstepBase/GSVersionMacros.h>
 
-#if OS_API_VERSION(100200,GS_API_LATEST) && GS_API_VERSION(011300,GS_API_LATEST)
+#if OS_API_VERSION(100200,GS_API_LATEST) && GS_API_VERSION( 11300,GS_API_LATEST)
 
 #import	<Foundation/NSObject.h>
 
@@ -45,8 +45,9 @@ extern "C" {
  */
 @interface NSURLConnection : NSObject
 {
-@private
+#if	GS_EXPOSE(NSURLConnection)
   void *_NSURLConnectionInternal;
+#endif
 }
 
 /**
@@ -72,7 +73,7 @@ extern "C" {
 
 /** <init />
  * Initialises the receiver with the specified request (performing
- * a deep copy so that ithe request does not change during loading)
+ * a deep copy so that the request does not change during loading)
  * and delegate.<br />
  * This automatically initiates an asynchronous load for the request.<br />
  * Processing of the request is done in the thread which calls this
