@@ -26,7 +26,7 @@
 #define __NSURLCredentialStorage_h_GNUSTEP_BASE_INCLUDE
 #import	<GNUstepBase/GSVersionMacros.h>
 
-#if OS_API_VERSION(100200,GS_API_LATEST) && GS_API_VERSION(011300,GS_API_LATEST)
+#if OS_API_VERSION(100200,GS_API_LATEST) && GS_API_VERSION( 11300,GS_API_LATEST)
 
 #import	<Foundation/NSObject.h>
 
@@ -49,8 +49,9 @@ extern NSString *const NSURLCredentialStorageChangedNotification;
  */
 @interface NSURLCredentialStorage : NSObject
 {
-@private
+#if	GS_EXPOSE(NSURLCredentialStorage)
   void *_NSURLCredentialStorageInternal;
+#endif
 }
 
 /**
