@@ -26,7 +26,7 @@
 #define __NSURLProtectionSpace_h_GNUSTEP_BASE_INCLUDE
 #import	<GNUstepBase/GSVersionMacros.h>
 
-#if OS_API_VERSION(100200,GS_API_LATEST) && GS_API_VERSION(011300,GS_API_LATEST)
+#if OS_API_VERSION(100200,GS_API_LATEST) && GS_API_VERSION( 11300,GS_API_LATEST)
 
 #import	<Foundation/NSObject.h>
 
@@ -60,8 +60,9 @@ extern NSString * const NSURLAuthenticationMethodHTTPDigest;
  */
 @interface NSURLProtectionSpace : NSObject <NSCopying>
 {
-@private
+#if	GS_EXPOSE(NSURLProtectionSpace)
   void *_NSURLProtectionSpaceInternal;
+#endif
 }
 
 /**
