@@ -308,7 +308,7 @@ extern "C" {
  *   <desc>%0.16g</desc>
  * </deflist>
  */
-- (NSString*) descriptionWithLocale: (NSDictionary*)locale;
+- (NSString*) descriptionWithLocale: (id)locale;
 
 /**
  * Compares receiver with otherNumber, using C type conversion if necessary,
@@ -349,20 +349,6 @@ extern "C" {
 
 #if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 
-/**
- * GNUstep specific (non-standard) additions to the NSNumber class.
- */
-@interface NSNumber(GSCategories)
-
-/**
- * Parses string as a <code>double</code>, <code>int</code>, or <code>unsigned
- * int</code> depending on what characters are present.  Uses
- * <code>atof</code> and <code>atoi</code> which don't report errors, so be
- * careful if the string might contain an invalid value.
- */
-+ (NSValue*) valueFromString: (NSString *)string;
-@end
-
 /** Note: Defines a method that is not in the OpenStep spec, but makes
     subclassing easier. */
 @interface NSValue (Subclassing)
@@ -371,29 +357,14 @@ extern "C" {
 + (Class) valueClassWithObjCType: (const char*)type;
 
 @end
-
-/**
- * Cache info for internal use by NSNumber concrete subclasses.
- * DO NOT USE.
- */
-typedef struct {
-  int		typeLevel;
-  void		(*getValue)(NSNumber*, SEL, void*);
-} GSNumberInfo;
-
-/** Internal method for caching. DO NOT USE. */
-GSNumberInfo	*GSNumberInfoFromObject(NSNumber *o);
-#define	GS_SMALL	16
-/**
- * Internal method: get cached values for integers in the range
- * - GS_SMALL to + GS_SMALL
- *   <br />DO NOT USE
- */
-unsigned	GSSmallHash(int n);
 #endif
 
 #if	defined(__cplusplus)
 }
 #endif
 
+#if     !NO_GNUSTEP && !defined(GNUSTEP_BASE_INTERNAL)
+#import <GNUstepBase/NSNumber+GNUstepBase.h>
+#endif
+
 #endif /* __NSValue_h_GNUSTEP_BASE_INCLUDE */
