@@ -32,24 +32,22 @@
 extern "C" {
 #endif
 
-
-@class NSDictionary, NSEnumerator, NSMutableArray;
+@class NSDictionary, NSEnumerator, NSMutableArray, NSMutableDictionary;
 
 /**
  * Represents an XML element.<br />
  */
 @interface NSXMLElement : NSXMLNode
 {
-#if	GS_EXPOSE(NSXMLElement)
-@protected
-  NSString              *_name;
-  NSMutableArray        *_attributes;
-  NSMutableArray        *_namespaces;
-  NSArray               *_children;
-  BOOL                   _childrenHaveMutated;
-  NSString              *_URI;	
-  NSInteger             _prefixIndex;
+#if     GS_NONFRAGILE
+#  if	defined(GS_NSXMLElement_IVARS)
+@public GS_NSXMLElement_IVARS
+#  endif
 #endif
+  /* The pointer to private additional data used to avoid breaking ABI
+   * when we don't have the non-fragile ABI available is inherited from
+   * NSXMLNode.  See Source/GSInternal.h for details.
+   */
 }
 
 /**
@@ -101,10 +99,18 @@ extern "C" {
 - (void) setAttributes: (NSArray*)attributes;
 
 /**
- * Sets attributes from the supplkied dictionary.
+ * Sets attributes from the supplied dictionary.<br />
+ * DEPRECATED ... use -setAttributesWithDictionary: instead.
  */
 - (void) setAttributesAsDictionary: (NSDictionary*)attributes;
 
+#if OS_API_VERSION(100700, GS_API_LATEST)
+/**
+ * Sets attributes from the supplied dictionary.
+ */
+- (void) setAttributesWithDictionary: (NSDictionary*)attributes;
+#endif
+
 /**
  * Returns the receiver's attributes.
  */
