@@ -187,9 +187,15 @@ GS_EXPORT NSString* const NSXMLParserErrorDomain;
 
 /**
  * Methods implemented by a delegate in order to make use of the parser.<br />
- * This is an informal protocol.
+ * This is now a formal protocol.
  */
+@protocol NSXMLParserDelegate <NSObject>
+#ifdef __clang__
+@optional
+#else
+@end
 @interface NSObject (NSXMLParserDelegateEventAdditions)
+#endif
 
 /** <override-dummy />
  */
