@@ -28,7 +28,7 @@
 
 #ifndef __NSZone_h_GNUSTEP_BASE_INCLUDE
 #define __NSZone_h_GNUSTEP_BASE_INCLUDE
-#import	<GNUstepBase/GSVersionMacros.h>
+#import	"GNUstepBase/GSVersionMacros.h"
 
 /**
  * Primary structure representing an <code>NSZone</code>.  Technically it
@@ -74,7 +74,7 @@ struct _NSZone
   struct NSZoneStats (*stats)(struct _NSZone *zone);
   
   size_t gran; // Zone granularity
-  NSString *name; // Name of zone (default is 'nil')
+  __unsafe_unretained NSString *name; // Name of zone (default is 'nil')
   NSZone *next;
 };
 
@@ -308,14 +308,14 @@ enum {
  * garbage collected itsself.<br />
  * In any case the memory returned is zero'ed.
  */
-GS_EXPORT void *
+GS_EXPORT GS_GC_STRONG void *
 NSAllocateCollectable(NSUInteger size, NSUInteger options);
 
 /** Reallocate memory to be of a different size and/or to have different
  * options settings.  The behavior of options is as for
  * the NSAllocateCollectable() function.
  */ 
-GS_EXPORT void *
+GS_EXPORT GS_GC_STRONG void *
 NSReallocateCollectable(void *ptr, NSUInteger size, NSUInteger options);
 
 #endif
