@@ -22,7 +22,7 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2011-06-07 19:36:12 +0100 (Tue, 07 Jun 2011) $ $Revision: 33258 $
+   $Date: 2011-07-24 07:09:22 -0600 (Sun, 24 Jul 2011) $ $Revision: 33621 $
 */
 
 #import "common.h"
@@ -93,7 +93,7 @@
   if ([NSThread isMultiThreaded] == YES)
     {
       DESTROY(self);
-      self = [NSLock new];
+      return (GSLazyLock*)[NSLock new];
     }
   else if (self != nil)
     {
@@ -246,7 +246,7 @@
   if ([NSThread isMultiThreaded] == YES)
     {
       DESTROY(self);
-      self = [NSRecursiveLock new];
+      return (GSLazyRecursiveLock*)[NSRecursiveLock new];
     }
   else
     {
