@@ -26,11 +26,10 @@
    Boston, MA 02111 USA.
 
    <title>GSObjCRuntime function and macro reference</title>
-   $Date: 2011-05-12 19:18:25 +0100 (Thu, 12 May 2011) $ $Revision: 33030 $
+   $Date: 2011-10-15 12:36:51 -0600 (Sat, 15 Oct 2011) $ $Revision: 34005 $
    */
 
 #import "common.h"
-#import "GNUstepBase/preface.h"
 #ifndef NeXT_Foundation_LIBRARY
 #import "Foundation/NSArray.h"
 #import "Foundation/NSAutoreleasePool.h"
@@ -71,7 +70,6 @@ extern struct objc_slot	*objc_get_slot(Class, SEL);
 #define BDBGPrintf(format, args...) \
   do { if (behavior_debug) { fprintf(stderr, (format) , ## args); } } while (0)
 
-
 Class
 GSObjCClass(id obj)
 {
@@ -138,10 +136,15 @@ GSSelectorFromName(const char *name)
   return sel_getUid(name);
 }
 
-// FIXME: Hack - need to provide these function declarations
-// for gcc 4.6 libobjc. They're called below, and they're declared 
-// in objc-api.h, but we're using runtime.h, so objc-api.h can't be imported.
-#if defined (__GNU_LIBOBJC__)
+#if defined (__GNU_LIBOBJC__) && (__GNU_LIBOBJC__ < 20110608)
+/* Don't use sel_registerTypedName() ... it's broken when first introduced
+ * into gcc (fails to correctly check for multiple registrations with same
+ * types but different layout info).
+ * Later versions of the runtime should be OK though.
+ * Hack - need to provide these function declarations
+ * for gcc 4.6 libobjc. They're called below, and they're declared 
+ * in objc-api.h, but we're using runtime.h, so objc-api.h can't be imported.
+ */
 SEL sel_get_any_typed_uid(const char *name);
 SEL sel_get_typed_uid(const char *name, const char*);
 SEL sel_register_name(const char *name);
@@ -153,16 +156,16 @@ GSSelectorFromNameAndTypes(const char *name, const char *types)
 {
 #if NeXT_RUNTIME
   return sel_getUid(name);
-/* Don't use sel_registerTypedName() ... it's broken when first introduced
- * into gcc (fails to correctly check for multple registrations with same
- * types but different layout info).
- *
-#elif defined (__GNU_LIBOBJC__)
+#elif defined (__GNU_LIBOBJC__) && (__GNU_LIBOBJC__ >= 20110608)
   return sel_registerTypedName(name, types);
- */
 #elif defined (__GNUSTEP_RUNTIME__)
   return sel_registerTypedName_np(name, types);
 #else
+extern SEL sel_get_any_typed_uid(const char*);
+extern SEL sel_get_typed_uid(const char*, const char*);
+extern SEL sel_register_name(const char*);
+extern SEL sel_register_typed_name(const char*, const char*);
+
   if (name == 0)
     {
       return 0;
@@ -1924,7 +1927,7 @@ NSArray *GSObjCDirectSubclassesOfClass(Class cls)
 void *
 GSAutoreleasedBuffer(unsigned size)
 {
-#if GS_WITH_GC
+#if GS_WITH_GC || __OBJC_GC__
   return NSAllocateCollectable(size, NSScannedOption);
 #else
 #ifdef ALIGN
@@ -2005,7 +2008,7 @@ GSPrintf (FILE *fptr, NSString* format, ...)
           ok = YES;
         }
     }
-  [arp release];
+  [arp drain];
   return ok;
 }
 
