@@ -28,6 +28,7 @@
    Boston, MA 02111 USA.
 */
 
+
 #import "common.h"
 #if	!defined(NeXT_Foundation_LIBRARY)
 #import "Foundation/NSArray.h"
@@ -70,7 +71,6 @@ typedef struct {unichar from; unsigned char to;} _ucc_;
 #else
 #include <iconv.h>
 #endif
-#include <errno.h>
 
 /*
  * The whole of the GNUstep code stores UNICODE in internal byte order,
@@ -304,7 +304,7 @@ static void GSSetupEncodingTable(void)
 		    }
 		}
 	    }
-	  encTable = NSZoneMalloc(NSDefaultMallocZone(),
+	  encTable = malloc(
 	    (encTableSize+1)*sizeof(struct _strenc_ *));
 	  memset(encTable, 0, (encTableSize+1)*sizeof(struct _strenc_ *));
 
@@ -331,13 +331,13 @@ static void GSSetupEncodingTable(void)
 		   * See if we can do a lossy conversion.
 		   */
 		  l = strlen(entry->iconv);
-		  lossy = NSZoneMalloc(NSDefaultMallocZone(), l + 11);
+		  lossy = malloc(l + 11);
 		  strncpy(lossy, entry->iconv, l);
 		  strncpy(lossy + l, "//TRANSLIT", 11);
-		  c = iconv_open(UNICODE_ENC, entry->iconv);
+		  c = iconv_open(entry->iconv, UNICODE_ENC);
 		  if (c == (iconv_t)-1)
 		    {
-		      NSZoneFree(NSDefaultMallocZone(),lossy);
+		      free(lossy);
 		    }
 		  else
 		    {
@@ -1351,7 +1351,7 @@ tables:
 	   */
 	  if (estr == 0)
 	    {
-	      NSLog(@"No iconv for encoding x%02x", enc);
+	      NSLog(@"GSToUnicode() No iconv for encoding x%02x", enc);
 	      result = NO;
 	      goto done;
 	    }
@@ -1362,7 +1362,7 @@ tables:
 	  cd = iconv_open(UNICODE_ENC, estr);
 	  if (cd == (iconv_t)-1)
 	    {
-	      NSLog(@"No iconv for encoding %@ tried to use %s",
+	      NSLog(@"GSToUnicode() No iconv for encoding %@ tried to use %s",
 		GSPrivateEncodingName(enc), estr);
 	      result = NO;
 	      goto done;
@@ -2314,7 +2314,7 @@ iconv_start:
 	   */
 	  if (estr == 0)
 	    {
-	      NSLog(@"No iconv for encoding x%02x", enc);
+	      NSLog(@"GSFromUnicode() No iconv for encoding x%02x", enc);
 	      result = NO;
 	      goto done;
 	    }
@@ -2325,7 +2325,7 @@ iconv_start:
 	  cd = iconv_open(estr, UNICODE_ENC);
 	  if (cd == (iconv_t)-1)
 	    {
-	      NSLog(@"No iconv for encoding %@ tried to use %s",
+	      NSLog(@"GSFromUnicode() No iconv for encoding %@ tried to use %s",
 		GSPrivateEncodingName(enc), estr);
 	      result = NO;
 	      goto done;
@@ -2520,7 +2520,7 @@ GSPrivateAvailableEncodings()
 	   * This is also the place where we determine the name we use
 	   * for iconv to support unicode.
 	   */
-	  encodings = NSZoneMalloc(NSDefaultMallocZone(),sizeof(NSStringEncoding) * (encTableSize+1));
+	  encodings = malloc(sizeof(NSStringEncoding) * (encTableSize+1));
 	  pos = 0;
 	  for (i = 0; i < encTableSize+1; i++)
 	    {
