@@ -22,7 +22,7 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2011-05-12 19:18:25 +0100 (Thu, 12 May 2011) $ $Revision: 33030 $
+   $Date: 2011-08-16 10:04:43 -0600 (Tue, 16 Aug 2011) $ $Revision: 33756 $
    */
 
 #import "common.h"
@@ -148,7 +148,7 @@ static Class	GSInlineArrayClass;
 }
 
 /* This is the designated initializer for NSArray. */
-- (id) initWithObjects: (id*)objects count: (NSUInteger)count
+- (id) initWithObjects: (const id[])objects count: (NSUInteger)count
 {
   if (count > 0)
     {
@@ -353,7 +353,7 @@ static Class	GSInlineArrayClass;
     }
 }
 
-- (void) getObjects: (id*)aBuffer
+- (void) getObjects: (__unsafe_unretained id[])aBuffer
 {
   NSUInteger i;
 
@@ -363,7 +363,7 @@ static Class	GSInlineArrayClass;
     }
 }
 
-- (void) getObjects: (id*)aBuffer range: (NSRange)aRange
+- (void) getObjects: (__unsafe_unretained id[])aBuffer range: (NSRange)aRange
 {
   NSUInteger i, j = 0, e = aRange.location + aRange.length;
 
@@ -376,7 +376,7 @@ static Class	GSInlineArrayClass;
 }
 
 - (NSUInteger) countByEnumeratingWithState: (NSFastEnumerationState*)state 	
-				   objects: (id*)stackbuf
+				   objects: (__unsafe_unretained id[])stackbuf
 				     count: (NSUInteger)len
 {
   /* For immutable arrays we can return the contents pointer directly. */
@@ -408,7 +408,7 @@ static Class	GSInlineArrayClass;
 {
   return [self initWithObjects: 0 count: 0];
 }
-- (id) initWithObjects: (id*)objects count: (NSUInteger)count
+- (id) initWithObjects: (const id[])objects count: (NSUInteger)count
 {
   _contents_array
     = (id*)(((void*)self) + class_getInstanceSize([self class]));
@@ -559,7 +559,7 @@ static Class	GSInlineArrayClass;
   return self;
 }
 
-- (id) initWithObjects: (id*)objects count: (NSUInteger)count
+- (id) initWithObjects: (const id[])objects count: (NSUInteger)count
 {
   self = [self initWithCapacity: count];
   if (self != nil && count > 0)
@@ -886,7 +886,7 @@ static Class	GSInlineArrayClass;
 }
 
 - (NSUInteger) countByEnumeratingWithState: (NSFastEnumerationState*)state 	
-				   objects: (id*)stackbuf
+				   objects: (__unsafe_unretained id[])stackbuf
 				     count: (NSUInteger)len
 {
   NSInteger count;
@@ -895,7 +895,7 @@ static Class	GSInlineArrayClass;
    * iteration.   If it changes during the iteration then
    * objc_enumerationMutation() will be called, throwing an exception.
    */
-  state->mutationsPtr = (unsigned long *)&_version;
+  state->mutationsPtr = &_version;
   count = MIN(len, _count - state->state);
   /* If a mutation has occurred then it's possible that we are being asked to
    * get objects from after the end of the array.  Don't pass negative values
@@ -1123,7 +1123,7 @@ static Class	GSInlineArrayClass;
                                               @"NS.objects"];
       if (array != nil)
 	{
-          return RETAIN(array);
+          return (GSPlaceholderArray*)RETAIN(array);
 	}
       else
         {
@@ -1155,11 +1155,11 @@ static Class	GSInlineArrayClass;
 				  at: a->_contents_array];
 	}
       a->_count = c;
-      return a;
+      return (GSPlaceholderArray*)a;
     }
 }
 
-- (id) initWithObjects: (id*)objects count: (NSUInteger)count
+- (id) initWithObjects: (const id[])objects count: (NSUInteger)count
 {
 #if	GS_WITH_GC
   self = (id)NSAllocateObject(GSArrayClass, 0, [self zone]);
@@ -1177,7 +1177,7 @@ static Class	GSInlineArrayClass;
   return 0;
 }
 
-- (void) release
+- (oneway void) release
 {
   return;		// placeholders never get released.
 }
