@@ -235,7 +235,8 @@ NSDataFromAvahiStringList(AvahiStringList* list)
   if (buffer == NULL)
     {
       // Should we raise an exception?
-      NSDebugLog(@"Couldn't allocate %lu bytes for txt record", len);
+      NSDebugLog(@"Couldn't allocate %"PRIuPTR" bytes for txt record",
+	(uintptr_t)len);
       return nil;
     }
 
@@ -726,8 +727,8 @@ didUpdateRecordData: (id)data
   [_info setObject: name   forKey: @"name"];
   [_info setObject: [NSNumber numberWithInteger: port]
             forKey: @"port"];
-  _browsers = NSCreateMapTable(NSIntMapKeyCallBacks, valueCallbacks, 10);
-  _browserTimeouts = NSCreateMapTable(NSIntMapKeyCallBacks,
+  _browsers = NSCreateMapTable(NSIntegerMapKeyCallBacks, valueCallbacks, 10);
+  _browserTimeouts = NSCreateMapTable(NSIntegerMapKeyCallBacks,
     NSObjectMapValueCallBacks, 10);
   if (port > 0)
     {
