@@ -30,6 +30,10 @@
 #import <callback.h>
 #import "callframe.h"
 
+#if !defined (__GNU_LIBOBJC__)
+#  include <objc/encoding.h>
+#endif
+
 #include <pthread.h>
 
 #import "GSInvocation.h"
@@ -808,7 +812,7 @@ gs_protocol_selector(const char *types)
 	{
 	  types++;
 	}
-      while(isdigit(*types))
+      while (isdigit(*types))
 	{
 	  types++;
 	}
