@@ -23,6 +23,11 @@
    */
 
 #import "common.h"
+
+#if !defined (__GNU_LIBOBJC__)
+#  include <objc/encoding.h>
+#endif
+
 #define	EXPOSE_NSInvocation_IVARS	1
 #import "Foundation/NSException.h"
 #import "Foundation/NSCoder.h"
@@ -143,6 +148,7 @@ static IMP gs_objc_msg_forward2 (id receiver, SEL sel)
   NSMutableData		*frame;
   cifframe_t            *cframe;
   ffi_closure           *cclosure;
+  void			*executable;
   NSMethodSignature     *sig = nil;
   GSCodeBuffer          *memory;
   const char            *types;
@@ -202,18 +208,28 @@ static IMP gs_objc_msg_forward2 (id receiver, SEL sel)
 
   memory = [GSCodeBuffer memoryWithSize: sizeof(ffi_closure)];
   cclosure = [memory buffer];
+  executable = [memory executable];
   if (cframe == NULL || cclosure == NULL)
     {
       [NSException raise: NSMallocException format: @"Allocating closure"];
     }
+#if	HAVE_FFI_PREP_CLOSURE_LOC
+  if (ffi_prep_closure_loc(cclosure, &(cframe->cif),
+    GSFFIInvocationCallback, frame, executable) != FFI_OK)
+    {
+      [NSException raise: NSGenericException format: @"Preping closure"];
+    }
+#else
+  executable = (void*)cclosure;
   if (ffi_prep_closure(cclosure, &(cframe->cif),
     GSFFIInvocationCallback, frame) != FFI_OK)
     {
       [NSException raise: NSGenericException format: @"Preping closure"];
     }
+#endif
   [memory protect];
 
-  return (IMP)cclosure;
+  return (IMP)executable;
 }
 
 static __attribute__ ((__unused__))
@@ -428,7 +444,14 @@ GSFFIInvokeWithTargetAndImp(NSInvocation *inv, id anObject, IMP imp)
       return;
     }
 
+  /* Make sure we have a typed selector for forwarding.
+   */
   NSAssert(_selector != 0, @"you must set the selector before invoking");
+  if (0 == GSTypesFromSelector(_selector))
+    {
+      _selector = GSSelectorFromNameAndTypes(sel_getName(_selector),
+	[_sig methodType]);
+    }
 
   /*
    *	Temporarily set new target and copy it (and the selector) into the
@@ -504,7 +527,7 @@ gs_protocol_selector(const char *types)
 	{
 	  types++;
 	}
-      while(isdigit(*types))
+      while (isdigit(*types))
 	{
 	  types++;
 	}
@@ -555,7 +578,8 @@ GSFFIInvocationCallback(ffi_cif *cif, void *retp, void **args, void *user)
   sig = nil;
   if (gs_protocol_selector(GSTypesFromSelector(selector)) == YES)
     {
-      sig = [NSMethodSignature signatureWithObjCTypes: GSTypesFromSelector(selector)];
+      sig = [NSMethodSignature signatureWithObjCTypes:
+	GSTypesFromSelector(selector)];
     }
   if (sig == nil)
     {
@@ -571,7 +595,7 @@ GSFFIInvocationCallback(ffi_cif *cif, void *retp, void **args, void *user)
       const char	*receiverTypes = [sig methodType];
       const char	*runtimeTypes = GSTypesFromSelector(selector);
 
-      if (runtimeTypes == 0 || strcmp(receiverTypes, runtimeTypes) != 0)
+      if (NO == GSSelectorTypesMatch(receiverTypes, runtimeTypes))
 	{
 	  const char	*runtimeName = sel_getName(selector);
 
