@@ -30,7 +30,6 @@
 #import "Foundation/NSData.h"
 #import "Foundation/NSArray.h"
 #import "Foundation/NSFileHandle.h"
-#import "GNUstepBase/GSFileHandle.h"
 #import "Foundation/NSException.h"
 #import "Foundation/NSRunLoop.h"
 #import "Foundation/NSNotification.h"
@@ -39,9 +38,10 @@
 #import "Foundation/NSByteOrder.h"
 #import "Foundation/NSProcessInfo.h"
 #import "Foundation/NSUserDefaults.h"
-#import "GSNetwork.h"
 #import "GSPrivate.h"
+#import "GSNetwork.h"
 #import "GNUstepBase/NSObject+GNUstepBase.h"
+#import "GSFileHandle.h"
 
 #import "../Tools/gdomap.h"
 
@@ -51,9 +51,19 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <sys/file.h>
+
+#if	defined(HAVE_SYS_FILE_H)
+#  include	<sys/file.h>
+#endif
+
 #include <sys/stat.h>
-#include <sys/fcntl.h>
+
+#if	defined(HAVE_SYS_FCNTL_H)
+#  include	<sys/fcntl.h>
+#elif	defined(HAVE_FCNTL_H)
+#  include	<fcntl.h>
+#endif
+
 #include <sys/ioctl.h>
 #ifdef	__svr4__
 #include <sys/filio.h>
@@ -61,10 +71,6 @@
 #include <netdb.h>
 
 #include <string.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#include <errno.h>
 
 /*
  *	Stuff for setting the sockets into non-blocking mode.
@@ -111,22 +117,28 @@ static NSString*	NotificationKey = @"NSFileHandleNotificationKey";
  */
 - (NSInteger) read: (void*)buf length: (NSUInteger)len
 {
+  int	result;
+
+  do
+    {
 #if	USE_ZLIB
-  if (gzDescriptor != 0)
-    {
-      len = gzread(gzDescriptor, buf, len);
-    }
-  else
+      if (gzDescriptor != 0)
+	{
+	  result = gzread(gzDescriptor, buf, len);
+	}
+      else
 #endif
-  if (isSocket)
-    {
-      len = recv(descriptor, buf, len, 0);
+      if (isSocket)
+	{
+	  result = recv(descriptor, buf, len, 0);
+	}
+      else
+	{
+	  result = read(descriptor, buf, len);
+	}
     }
-  else
-    {
-      len = read(descriptor, buf, len);
-    }
-  return len;
+  while (result < 0 && EINTR == errno);
+  return result;
 }
 
 /**
@@ -135,106 +147,28 @@ static NSString*	NotificationKey = @"NSFileHandleNotificationKey";
  */
 - (NSInteger) write: (const void*)buf length: (NSUInteger)len
 {
+  int	result;
+
+  do
+    {
 #if	USE_ZLIB
-  if (gzDescriptor != 0)
-    {
-      len = gzwrite(gzDescriptor, (char*)buf, len);
-    }
-  else
+    if (gzDescriptor != 0)
+      {
+	result = gzwrite(gzDescriptor, (char*)buf, len);
+      }
+    else
 #endif
-  if (isSocket)
-    {
-      len = send(descriptor, buf, len, 0);
-    }
-  else
-    {
-      len = write(descriptor, buf, len);
-    }
-  return len;
-}
-
-static BOOL
-getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
-{
-  const char		*proto = "tcp";
-  struct servent	*sp;
-
-  if (pcl)
-    {
-      proto = [pcl lossyCString];
-    }
-  memset(sin, '\0', sizeof(*sin));
-  sin->sin_family = AF_INET;
-
-  /*
-   *	If we were given a hostname, we use any address for that host.
-   *	Otherwise we expect the given name to be an address unless it is
-   *	a null (any address).
-   */
-  if (name)
-    {
-      NSHost*		host = [NSHost hostWithName: name];
-
-      if (host != nil)
+      if (isSocket)
 	{
-	  name = [host address];
-	}
-#ifndef	HAVE_INET_ATON
-      sin->sin_addr.s_addr = inet_addr([name lossyCString]);
-      if (sin->sin_addr.s_addr == INADDR_NONE)
-#else
-      if (inet_aton([name lossyCString], &sin->sin_addr) == 0)
-#endif
-	{
-	  return NO;
-	}
-    }
-  else
-    {
-      sin->sin_addr.s_addr = GSSwapHostI32ToBig(INADDR_ANY);
-    }
-  if (svc == nil)
-    {
-      sin->sin_port = 0;
-      return YES;
-    }
-  else if ((sp = getservbyname([svc lossyCString], proto)) == 0)
-    {
-      const char*     ptr = [svc lossyCString];
-      int             val = atoi(ptr);
-
-      while (isdigit(*ptr))
-	{
-	  ptr++;
-	}
-      if (*ptr == '\0' && val <= 0xffff)
-	{
-	  uint16_t	v = val;
-
-	  sin->sin_port = GSSwapHostI16ToBig(v);
-	  return YES;
-        }
-      else if (strcmp(ptr, "gdomap") == 0)
-	{
-	  uint16_t	v;
-#ifdef	GDOMAP_PORT_OVERRIDE
-	  v = GDOMAP_PORT_OVERRIDE;
-#else
-	  v = 538;	// IANA allocated port
-#endif
-	  sin->sin_port = GSSwapHostI16ToBig(v);
-	  return YES;
+	  result = send(descriptor, buf, len, 0);
 	}
       else
 	{
-	  return NO;
+	  result = write(descriptor, buf, len);
 	}
     }
-  else
-    {
-      sin->sin_port = sp->s_port;
-      return YES;
-    }
+  while (result < 0 && EINTR == errno);
+  return result;
 }
 
 + (id) allocWithZone: (NSZone*)z
@@ -664,8 +598,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
   static NSString	*dsocks = nil;
   static BOOL		beenHere = NO;
   int			net;
-  struct sockaddr_in	sin;
-  struct sockaddr_in	lsin;
+  struct sockaddr	sin;
+  struct sockaddr	lsin;
   NSString		*lhost = nil;
   NSString		*shost = nil;
   NSString		*sport = nil;
@@ -718,7 +652,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 	{
 	  p = nil;
 	}
-      if (getAddr(lhost, p, @"tcp", &lsin) == NO)
+      if (GSPrivateSockaddrSetup(lhost, 0, p, @"tcp", &lsin) == NO)
 	{
 	  NSLog(@"bad bind address specification");
 	  DESTROY(self);
@@ -765,7 +699,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
       p = @"tcp";
     }
 
-  if (getAddr(a, s, p, &sin) == NO)
+  if (GSPrivateSockaddrSetup(a, 0, s, p, &sin) == NO)
     {
       DESTROY(self);
       NSLog(@"bad address-service-protocol combination");
@@ -788,7 +722,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
     }
   if (shost != nil)
     {
-      if (getAddr(shost, sport, p, &sin) == NO)
+      if (GSPrivateSockaddrSetup(shost, 0, sport, p, &sin) == NO)
 	{
 	  NSLog(@"bad SOCKS host-port combination");
 	  DESTROY(self);
@@ -796,7 +730,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 	}
     }
 
-  if ((net = socket(AF_INET, SOCK_STREAM, PF_UNSPEC)) == -1)
+  if ((net = socket(sin.sa_family, SOCK_STREAM, PF_UNSPEC)) == -1)
     {
       NSLog(@"unable to create socket - %@", [NSError _last]);
       DESTROY(self);
@@ -810,10 +744,10 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
   if (lhost != nil)
     {
-      if (bind(net, (struct sockaddr *)&lsin, sizeof(lsin)) == -1)
+      if (bind(net, &lsin, GSPrivateSockaddrLength(&lsin)) == -1)
 	{
-	  NSLog(@"unable to bind to port %s:%d - %@", inet_ntoa(lsin.sin_addr),
-	    GSSwapBigI16ToHost(sin.sin_port), [NSError _last]);
+	  NSLog(@"unable to bind to port %@ - %@",
+	    GSPrivateSockaddrName(&lsin), [NSError _last]);
 	  (void) close(net);
 	  DESTROY(self);
 	  return nil;
@@ -827,13 +761,14 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
       isSocket = YES;
       [self setNonBlocking: YES];
-      if (connect(net, (struct sockaddr*)&sin, sizeof(sin)) == -1)
+      if (connect(net, &sin, GSPrivateSockaddrLength(&sin)) == -1)
 	{
-	  if (errno != EINPROGRESS)
+	  if (!GSWOULDBLOCK)
 	    {
-	      NSLog(@"unable to make connection to %s:%d - %@",
-		inet_ntoa(sin.sin_addr),
-		GSSwapBigI16ToHost(sin.sin_port), [NSError _last]);
+	      NSError	*e = [NSError _last];
+
+	      NSLog(@"unable to make socket connection to %@ - %@ (%d)",
+		GSPrivateSockaddrName(&sin), e, (int)[e code]);
 	      DESTROY(self);
 	      return nil;
 	    }
@@ -885,17 +820,17 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
   int			status = 1;
 #endif
   int			net;
-  struct sockaddr_in	sin;
+  struct sockaddr	sin;
   unsigned int		size = sizeof(sin);
 
-  if (getAddr(a, s, p, &sin) == NO)
+  if (GSPrivateSockaddrSetup(a, 0, s, p, &sin) == NO)
     {
       DESTROY(self);
       NSLog(@"bad address-service-protocol combination");
       return  nil;
     }
 
-  if ((net = socket(AF_INET, SOCK_STREAM, PF_UNSPEC)) == -1)
+  if ((net = socket(sin.sa_family, SOCK_STREAM, PF_UNSPEC)) == -1)
     {
       NSLog(@"unable to create socket - %@", [NSError _last]);
       DESTROY(self);
@@ -912,10 +847,12 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
   setsockopt(net, SOL_SOCKET, SO_REUSEADDR, (char *)&status, sizeof(status));
 #endif
 
-  if (bind(net, (struct sockaddr *)&sin, sizeof(sin)) == -1)
+  if (bind(net, &sin, GSPrivateSockaddrLength(&sin)) == -1)
     {
-      NSLog(@"unable to bind to port %s:%d - %@", inet_ntoa(sin.sin_addr),
-	GSSwapBigI16ToHost(sin.sin_port), [NSError _last]);
+      NSError	*e = [NSError _last];
+
+      NSLog(@"unable to bind to port %@ - %@",
+	GSPrivateSockaddrName(&sin), e);
       (void) close(net);
       DESTROY(self);
       return nil;
@@ -931,7 +868,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
       return nil;
     }
 
-  if (getsockname(net, (struct sockaddr*)&sin, &size) == -1)
+  if (getsockname(net, &sin, &size) == -1)
     {
       NSLog(@"unable to get socket name - %@", [NSError _last]);
       (void) close(net);
@@ -1917,11 +1854,11 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
   operation = [readInfo objectForKey: NotificationKey];
   if (operation == NSFileHandleConnectionAcceptedNotification)
     {
-      struct sockaddr_in	buf;
+      struct sockaddr	buf;
       int			desc;
       unsigned int		blen = sizeof(buf);
 
-      desc = accept(descriptor, (struct sockaddr*)&buf, &blen);
+      desc = accept(descriptor, &buf, &blen);
       if (desc == -1)
 	{
 	  NSString	*s;
@@ -1933,7 +1870,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
       else
 	{ // Accept attempt completed.
 	  GSFileHandle		*h;
-	  struct sockaddr_in	sin;
+	  struct sockaddr	sin;
 	  unsigned int		size = sizeof(sin);
 	  int			status;
 
@@ -1947,7 +1884,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 	  h = [[[self class] alloc] initWithFileDescriptor: desc
 						closeOnDealloc: YES];
 	  h->isSocket = YES;
-	  getpeername(desc, (struct sockaddr*)&sin, &size);
+	  getpeername(desc, &sin, &size);
 	  [h setAddr: &sin];
 	  [readInfo setObject: h
 		   forKey: NSFileHandleNotificationFileHandleItem];
@@ -2111,11 +2048,13 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
     }
 }
 
-- (void) setAddr: (struct sockaddr_in *)sin
+- (void) setAddr: (struct sockaddr *)sin
 {
-  address = [[NSString alloc] initWithCString: (char*)inet_ntoa(sin->sin_addr)];
-  service = [[NSString alloc] initWithFormat: @"%d",
-    (int)GSSwapBigI16ToHost(sin->sin_port)];
+  NSString	*s;
+
+  ASSIGN(address, GSPrivateSockaddrHost(sin));
+  s = [NSString stringWithFormat: @"%d", GSPrivateSockaddrPort(sin)];
+  ASSIGN(service, s);
   protocol = @"tcp";
 }
 
@@ -2173,16 +2112,16 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 - (NSString*) socketLocalAddress
 {
   NSString	*str = nil;
-  struct sockaddr_in sin = { 0 };
+  struct sockaddr sin;
   unsigned	size = sizeof(sin);
 
-  if (getsockname(descriptor, (struct sockaddr*)&sin, &size) == -1)
+  if (getsockname(descriptor, &sin, &size) == -1)
     {
       NSLog(@"unable to get socket name - %@", [NSError _last]);
     }
   else
     {
-      str = [NSString stringWithUTF8String: (char*)inet_ntoa(sin.sin_addr)];
+      str = GSPrivateSockaddrHost(&sin);
     }
   return str;
 }
@@ -2190,17 +2129,16 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 - (NSString*) socketLocalService
 {
   NSString	*str = nil;
-  struct sockaddr_in sin = { 0 };
+  struct sockaddr sin;
   unsigned	size = sizeof(sin);
 
-  if (getsockname(descriptor, (struct sockaddr*)&sin, &size) == -1)
+  if (getsockname(descriptor, &sin, &size) == -1)
     {
       NSLog(@"unable to get socket name - %@", [NSError _last]);
     }
   else
     {
-      str = [NSString stringWithFormat: @"%d",
-	(int)GSSwapBigI16ToHost(sin.sin_port)];
+      str = [NSString stringWithFormat: @"%d", GSPrivateSockaddrPort(&sin)];
     }
   return str;
 }
