@@ -83,11 +83,14 @@
 
 #include <string.h>		// for strstr()
 #include <sys/stat.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #include <sys/types.h>
-#include <fcntl.h>
+
+#if	defined(HAVE_SYS_FCNTL_H)
+#  include	<sys/fcntl.h>
+#elif	defined(HAVE_FCNTL_H)
+#  include	<fcntl.h>
+#endif
+
 #include <stdio.h>
 #ifdef HAVE_WCHAR_H
 #include <wchar.h>
