@@ -50,20 +50,21 @@
 #import "GSPrivate.h"
 
 #include <string.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#include <sys/file.h>
 
-#ifdef HAVE_SYS_FCNTL_H
-#include <sys/fcntl.h>		// For O_WRONLY, etc
+#ifdef	HAVE_SYS_FILE_H
+#  include <sys/file.h>
 #endif
+
+#if	defined(HAVE_SYS_FCNTL_H)
+#  include <sys/fcntl.h>
+#elif	defined(HAVE_FCNTL_H)
+#  include <fcntl.h>
+#endif
+
 #ifdef	HAVE_SYS_SOCKET_H
-#include <sys/socket.h>		// For MSG_PEEK, etc
+#  include <sys/socket.h>		// For MSG_PEEK, etc
 #endif
 
-#include <errno.h>
-
 /*
  * Implement map keys for strings with case insensitive comparisons,
  * so we can have case insensitive matching of http headers (correct
@@ -651,13 +652,40 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	  [nc removeObserver: self name: nil object: sock];
 
 	  ver = [[[document headerNamed: @"http"] value] floatValue];
-	  val = [[document headerNamed: @"connection"] value];
-	  if (ver < 1.1 || (val != nil && [val isEqual: @"close"] == YES))
+	  if (ver < 1.1)
 	    {
 	      [nc removeObserver: self name: nil object: sock];
 	      [sock closeFile];
 	      DESTROY(sock);
 	    }
+	  else if (nil != (val = [[document headerNamed: @"connection"] value]))
+	    {
+	      val = [val lowercaseString];
+	      if (YES == [val isEqualToString: @"close"])
+		{
+		  [nc removeObserver: self name: nil object: sock];
+		  [sock closeFile];
+		  DESTROY(sock);
+		}
+	      else if ([val length] > 5)
+		{
+		  NSEnumerator	*e;
+
+		  e = [[val componentsSeparatedByString: @","]
+		    objectEnumerator];
+		  while (nil != (val = [e nextObject]))
+		    {
+		      val = [val stringByTrimmingSpaces];
+		      if (YES == [val isEqualToString: @"close"])
+			{
+			  [nc removeObserver: self name: nil object: sock];
+			  [sock closeFile];
+			  DESTROY(sock);
+			  break;
+			}
+		    }
+		}
+	    }
 
 	  /*
 	   * Retrieve essential keys from document
