@@ -24,6 +24,7 @@
 */
 #import "common.h"
 #import "GNUstepBase/GSLocale.h"
+#import "GNUstepBase/NSObject+GNUstepBase.h"
 #import "Foundation/NSDictionary.h"
 #import "Foundation/NSArray.h"
 #import "Foundation/NSLock.h"
@@ -218,7 +219,7 @@ GSDomainFromDefaultLocale(void)
    */
   if (saved == nil)
     {
-      saved = [dict mutableCopy];
+      saved = [NSObject leak: dict];
     }
   [gnustep_global_lock unlock];
   return saved;
@@ -246,7 +247,8 @@ GSLanguageFromLocale(NSString *locale)
   NSString	*aliases = nil;
   NSBundle      *gbundle;
 
-  if (locale == nil || [locale isEqual: @"C"] || [locale isEqual: @"POSIX"])
+  if (locale == nil || [locale isEqual: @"C"] || [locale isEqual: @"POSIX"]
+      || [locale length] < 2)
     return @"English";
 
   gbundle = [NSBundle bundleForLibrary: @"gnustep-base"];
@@ -262,14 +264,49 @@ GSLanguageFromLocale(NSString *locale)
       if (language == nil && [locale pathExtension] != nil)
 	{
 	  locale = [locale stringByDeletingPathExtension];
+          if ([locale isEqual: @"C"] || [locale isEqual: @"POSIX"])
+            return @"English";
 	  language = [dict objectForKey: locale];
 	}
       if (language == nil)
 	{
-	  locale = [locale substringFromRange: NSMakeRange(0, 2)];
+	  locale = [locale substringWithRange: NSMakeRange(0, 2)];
 	  language = [dict objectForKey: locale];
 	}
     }
 
   return language;
 }
+
+NSArray *
+GSLocaleVariants(NSString *locale)
+{
+  NSRange under = [locale rangeOfString: @"_"];
+  if (under.location != NSNotFound)
+    {
+      return [NSArray arrayWithObjects:
+			locale,
+		      [locale substringToIndex: under.location],
+		      nil];
+    }
+  return [NSArray arrayWithObject: locale];
+}
+
+NSArray *
+GSLanguagesFromLocale(NSString *locale)
+{
+  NSArray *variants = GSLocaleVariants(locale);
+  NSMutableArray *result = [NSMutableArray arrayWithCapacity: [variants count]];
+
+  NSEnumerator *enumerator = [variants objectEnumerator];
+  NSString *variant;
+  while ((variant = [enumerator nextObject]) != nil)
+    {
+      NSString *language = GSLanguageFromLocale(variant);
+      if (language != nil)
+	{
+	  [result addObject: language];
+	}
+    }
+  return result;
+}
