@@ -3,29 +3,30 @@
 
    Written by:  Niels Grewe <niels.grewe@halbordnung.de>
    Date: March 2010
-   
+
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
-   
+
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
-   */ 
+   */
 
 #import "common.h"
-#define	EXPOSE_NSNetServices_IVARS	1
-#define	EXPOSE_NSNetServicesBrowser_IVARS	1
+#define	EXPOSE_NSNetService_IVARS	1
+#define	EXPOSE_NSNetServiceBrowser_IVARS	1
 #import	"Foundation/NSNetServices.h"
+#import "GNUstepBase/NSNetServices+GNUstepBase.h"
 
 // Subclasses using mDNSResponder:
 
@@ -113,6 +114,12 @@ NSString* GSNetServiceDotTerminatedNSStringFromString(const char* string);
                  name: (NSString*)name
          avahiIfIndex: (int)anIfIndex
         avahiProtocol: (int)aProtocol;
+
+#if GS_USE_AVAHI==1
+- (id<NSObject,GSNetServiceDelegate>)delegate;
+#else
+- (id<NSObject>)delegate;
+#endif
 @end
 
 /**
