@@ -28,15 +28,15 @@
    Boston, MA 02111 USA.
    */ 
 
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <fcntl.h>
+
+#if	defined(HAVE_SYS_FCNTL_H)
+#  include	<sys/fcntl.h>
+#elif	defined(HAVE_FCNTL_H)
+#  include	<fcntl.h>
+#endif
+
 
 #if     defined(__MINGW32__) || defined(__MINGW64__)
 
@@ -70,7 +70,10 @@
 #define INVALID_SOCKET  -1
 #define BADSOCKET(X)    ((X) < 0)
 #define GSNETERROR      errno
-#define GSWOULDBLOCK    (errno == EINPROGRESS)
+#define GSWOULDBLOCK    (EINPROGRESS == errno\
+ || EALREADY == errno\
+ || EINTR == errno\
+ || EAGAIN == errno)
 
 #endif  /* __MINGW__ */
 
@@ -95,4 +98,20 @@
 #  endif
 #endif
 
+NSString*
+GSPrivateSockaddrHost(struct sockaddr *addr) GS_ATTRIB_PRIVATE;
+
+unsigned
+GSPrivateSockaddrLength(struct sockaddr *addr) GS_ATTRIB_PRIVATE;
+
+NSString*
+GSPrivateSockaddrName(struct sockaddr *addr) GS_ATTRIB_PRIVATE;
+
+uint16_t
+GSPrivateSockaddrPort(struct sockaddr *addr) GS_ATTRIB_PRIVATE;
+
+BOOL
+GSPrivateSockaddrSetup(NSString *machine, uint16_t port, NSString *service,
+  NSString *protocol, struct sockaddr *sin) GS_ATTRIB_PRIVATE;
+
 #endif
