@@ -24,6 +24,8 @@
 #ifndef _GSPrivate_h_
 #define _GSPrivate_h_
 
+#include <errno.h>
+
 #import "Foundation/NSError.h"
 
 @class	_GSInsensitiveDictionary;
@@ -75,7 +77,7 @@ typedef struct objc_category* Category;
   unsigned	_count;
   unsigned	_capacity;
   int		_grow_factor;
-  int		_version;
+  unsigned long		_version;
 }
 @end
 
@@ -161,8 +163,8 @@ __attribute__((unused)) static void GSFreeTempBuffer(void **b)
  * Yet the optimization of the stored hash value is currently deemed
  * more important.
  */
-#define GS_REPLACE_CONSTANT_STRING(ID) \
-  ID = [[NSStringClass alloc] initWithCString: [ID cString]]
+#define GS_REPLACE_CONSTANT_STRING(ID) [(ID = [NSObject \
+  leak: [[NSString alloc] initWithUTF8String: [ID UTF8String]]]) release]
 /* Using cString here is OK here
    because NXConstantString returns a pointer
    to it's internal pointer.  */
@@ -524,9 +526,11 @@ GSPrivateUnloadModule(FILE *errorStream,
 {
   unsigned      size;
   void          *buffer;
+  void		*executable;
 }
 + (GSCodeBuffer*) memoryWithSize: (NSUInteger)_size;
 - (void*) buffer;
+- (void*) executable;
 - (id) initWithSize: (NSUInteger)_size;
 - (void) protect;
 @end
