@@ -72,6 +72,174 @@ GSPrivateSockaddrLength(struct sockaddr *addr)
   }
 }
 
+NSString *
+GSPrivateSockaddrHost(struct sockaddr *addr)
+{
+  char		buf[40];
+
+#if     defined(AF_INET6)
+  if (AF_INET6 == addr->sa_family)
+    {
+      struct sockaddr_in6	*addr6 = (struct sockaddr_in6*)addr;
+
+      inet_ntop(AF_INET, &addr6->sin6_addr, buf, sizeof(buf));
+      return [NSString stringWithUTF8String: buf];
+    } 
+#endif
+  inet_ntop(AF_INET, &((struct sockaddr_in*)addr)->sin_addr, buf, sizeof(buf));
+  return [NSString stringWithUTF8String: buf];
+}
+
+NSString *
+GSPrivateSockaddrName(struct sockaddr *addr)
+{
+  return [NSString stringWithFormat: @"%@:%d",
+    GSPrivateSockaddrHost(addr),
+    GSPrivateSockaddrPort(addr)];
+}
+
+uint16_t
+GSPrivateSockaddrPort(struct sockaddr *addr)
+{
+  uint16_t	port;
+
+#if     defined(AF_INET6)
+  if (AF_INET6 == addr->sa_family)
+    {
+      struct sockaddr_in6	*addr6 = (struct sockaddr_in6*)addr;
+
+      port = addr6->sin6_port;
+      port = GSSwapBigI16ToHost(port);
+      return port;
+    } 
+#endif
+  port = ((struct sockaddr_in*)addr)->sin_port;
+  port = GSSwapBigI16ToHost(port);
+  return port;
+}
+
+BOOL
+GSPrivateSockaddrSetup(NSString *machine, uint16_t port,
+  NSString *service, NSString *protocol, struct sockaddr *sin)
+{
+  memset(sin, '\0', sizeof(*sin));
+  sin->sa_family = AF_INET;
+
+  /* If we were given a hostname, we use any address for that host.
+   * Otherwise we expect the given name to be an address unless it is
+   * a null (any address).
+   */
+  if (0 != [machine length])
+    {
+      const char	*n;
+
+      n = [machine UTF8String];
+      if ((!isdigit(n[0]) || sscanf(n, "%*d.%*d.%*d.%*d") != 4)
+	&& 0 == strchr(n, ':'))
+	{
+	  machine = [[NSHost hostWithName: machine] address];
+	  n = [machine UTF8String];
+	}
+
+      if (0 == n)
+	{
+	  return NO;
+	}
+      if (0 == strchr(n, ':'))
+	{
+	  struct sockaddr_in	*addr = (struct sockaddr_in*)sin;
+
+	  if (inet_pton(AF_INET, n, &addr->sin_addr) <= 0)
+	    {
+	      return NO;
+	    }
+	}
+      else
+	{
+#if     defined(AF_INET6)
+	  struct sockaddr_in6	*addr6 = (struct sockaddr_in6*)sin;
+
+	  sin->sa_family = AF_INET6;
+	  if (inet_pton(AF_INET6, n, &addr6->sin6_addr) <= 0)
+	    {
+	      return NO;
+	    }
+#else
+	  return NO;
+#endif
+	}
+    }
+  else
+    {
+      ((struct sockaddr_in*)sin)->sin_addr.s_addr
+	= GSSwapHostI32ToBig(INADDR_ANY);
+    }
+
+  /* The optional service and protocol parameters may be used to
+   * look up the port
+   */
+  if (nil != service)
+    {
+      const char	*sname;
+      const char	*proto;
+      struct servent	*sp;
+
+      if (nil == protocol)
+	{
+	  proto = "tcp";
+	}
+      else
+	{
+	  proto = [protocol UTF8String];
+	}
+
+      sname = [service UTF8String];
+      if ((sp = getservbyname(sname, proto)) == 0)
+	{
+	  const char*     ptr = sname;
+	  int             val = atoi(ptr);
+
+	  while (isdigit(*ptr))
+	    {
+	      ptr++;
+	    }
+	  if (*ptr == '\0' && val <= 0xffff)
+	    {
+	      port = val;
+	    }
+	  else if (strcmp(ptr, "gdomap") == 0)
+	    {
+#ifdef	GDOMAP_PORT_OVERRIDE
+	      port = GDOMAP_PORT_OVERRIDE;
+#else
+	      port = 538;	// IANA allocated port
+#endif
+	    }
+	  else
+	    {
+	      return NO;
+	    }
+	}
+      else
+	{
+	  port = GSSwapBigI16ToHost(sp->s_port);
+	}
+    }
+
+#if     defined(AF_INET6)
+  if (AF_INET6 == sin->sa_family)
+    {
+      ((struct sockaddr_in6*)sin)->sin6_port = GSSwapHostI16ToBig(port);
+    }
+  else
+    {
+      ((struct sockaddr_in*)sin)->sin_port = GSSwapHostI16ToBig(port);
+    }
+#else
+  ((struct sockaddr_ind*)sin)->sin6_port = GSSwapHostI16ToBig(port);
+#endif
+  return YES;
+}
 
 /** The GSStreamHandler abstract class defines the methods used to
  * implement a handler object for a pair of streams.
@@ -492,6 +660,7 @@ static gnutls_anon_client_credentials_t anoncred;
 
   if ([proto isEqualToString: NSStreamSocketSecurityLevelTLSv1] == YES)
     {
+#if GNUTLS_VERSION_NUMBER < 0x020C00
       const int proto_prio[4] = {
 #if	defined(GNUTLS_TLS1_2)
         GNUTLS_TLS1_2,
@@ -500,13 +669,20 @@ static gnutls_anon_client_credentials_t anoncred;
         GNUTLS_TLS1_0,
         0 };
       gnutls_protocol_set_priority (session, proto_prio);
+#else
+      gnutls_priority_set_direct(session, "NORMAL:-VERS-SSL3.0:+VERS-TLS-ALL", NULL);
+#endif
     }
   if ([proto isEqualToString: NSStreamSocketSecurityLevelSSLv3] == YES)
     {
+#if GNUTLS_VERSION_NUMBER < 0x020C00
       const int proto_prio[2] = {
         GNUTLS_SSL3,
         0 };
       gnutls_protocol_set_priority (session, proto_prio);
+#else
+      gnutls_priority_set_direct(session, "NORMAL:-VERS-TLS-ALL:+VERS-SSL3.0", NULL);
+#endif
     }
 
 /*
@@ -529,7 +705,9 @@ static gnutls_anon_client_credentials_t anoncred;
   
   /* Set transport layer to use our low level stream code.
    */
+#if GNUTLS_VERSION_NUMBER < 0x020C00
   gnutls_transport_set_lowat (session, 0);
+#endif
   gnutls_transport_set_pull_function (session, GSTLSPull);
   gnutls_transport_set_push_function (session, GSTLSPush);
   gnutls_transport_set_ptr (session, (gnutls_transport_ptr_t)self);
@@ -1246,12 +1424,7 @@ socketError(int result)
 static inline BOOL
 socketWouldBlock()
 {
-#if	defined(__MINGW__)
-  int   e = WSAGetLastError();
-  return (e == WSAEWOULDBLOCK || e == WSAEINPROGRESS) ? YES : NO;
-#else
-  return (errno == EWOULDBLOCK || errno == EINPROGRESS) ? YES : NO;
-#endif
+  return GSWOULDBLOCK ? YES : NO;
 }
 
 
@@ -1321,103 +1494,40 @@ setNonBlocking(SOCKET fd)
 
   if (result == nil && _address.s.sa_family != AF_UNSPEC)
     {
-      SOCKET    s = [self _sock];
+      SOCKET    	s = [self _sock];
+      struct sockaddr	sin;
+      socklen_t	        size = sizeof(sin);
 
-      switch (_address.s.sa_family)
-        {
-          case AF_INET:
-            {
-              struct sockaddr_in sin = { 0 };
-              socklen_t	        size = sizeof(sin);
-
-              if ([key isEqualToString: GSStreamLocalAddressKey])
-                {
-                  if (getsockname(s, (struct sockaddr*)&sin, &size) != -1)
-                    {
-                      result = [NSString stringWithUTF8String:
-                        (char*)inet_ntoa(sin.sin_addr)];
-                    }
-                }
-              else if ([key isEqualToString: GSStreamLocalPortKey])
-                {
-                  if (getsockname(s, (struct sockaddr*)&sin, &size) != -1)
-                    {
-                      result = [NSString stringWithFormat: @"%d",
-                        (int)GSSwapBigI16ToHost(sin.sin_port)];
-                    }
-                }
-              else if ([key isEqualToString: GSStreamRemoteAddressKey])
-                {
-                  if (getpeername(s, (struct sockaddr*)&sin, &size) != -1)
-                    {
-                      result = [NSString stringWithUTF8String:
-                        (char*)inet_ntoa(sin.sin_addr)];
-                    }
-                }
-              else if ([key isEqualToString: GSStreamRemotePortKey])
-                {
-                  if (getpeername(s, (struct sockaddr*)&sin, &size) != -1)
-                    {
-                      result = [NSString stringWithFormat: @"%d",
-                        (int)GSSwapBigI16ToHost(sin.sin_port)];
-                    }
-                }
-            }
-            break;
-#if	defined(AF_INET6)
-          case AF_INET6:
-            {
-              struct sockaddr_in6 sin = { 0 };
-              socklen_t	        size = sizeof(sin);
-
-              if ([key isEqualToString: GSStreamLocalAddressKey])
-                {
-                  if (getsockname(s, (struct sockaddr*)&sin, &size) != -1)
-                    {
-                      char	buf[INET6_ADDRSTRLEN+1];
-
-                      if (inet_ntop(AF_INET6, &(sin.sin6_addr), buf,
-                        INET6_ADDRSTRLEN) == 0)
-                        {
-                          buf[INET6_ADDRSTRLEN] = '\0';
-                          result = [NSString stringWithUTF8String: buf];
-                        }
-                    }
-                }
-              else if ([key isEqualToString: GSStreamLocalPortKey])
-                {
-                  if (getsockname(s, (struct sockaddr*)&sin, &size) != -1)
-                    {
-                      result = [NSString stringWithFormat: @"%d",
-                        (int)GSSwapBigI16ToHost(sin.sin6_port)];
-                    }
-                }
-              else if ([key isEqualToString: GSStreamRemoteAddressKey])
-                {
-                  if (getpeername(s, (struct sockaddr*)&sin, &size) != -1)
-                    {
-                      char	buf[INET6_ADDRSTRLEN+1];
-
-                      if (inet_ntop(AF_INET6, &(sin.sin6_addr), buf,
-                        INET6_ADDRSTRLEN) == 0)
-                        {
-                          buf[INET6_ADDRSTRLEN] = '\0';
-                          result = [NSString stringWithUTF8String: buf];
-                        }
-                    }
-                }
-              else if ([key isEqualToString: GSStreamRemotePortKey])
-                {
-                  if (getpeername(s, (struct sockaddr*)&sin, &size) != -1)
-                    {
-                      result = [NSString stringWithFormat: @"%d",
-                        (int)GSSwapBigI16ToHost(sin.sin6_port)];
-                    }
-                }
-            }
-            break;
-#endif
-        }
+      if ([key isEqualToString: GSStreamLocalAddressKey])
+	{
+	  if (getsockname(s, (struct sockaddr*)&sin, &size) != -1)
+	    {
+	      result = GSPrivateSockaddrHost(&sin);
+	    }
+	}
+      else if ([key isEqualToString: GSStreamLocalPortKey])
+	{
+	  if (getsockname(s, (struct sockaddr*)&sin, &size) != -1)
+	    {
+	      result = [NSString stringWithFormat: @"%d",
+		(int)GSPrivateSockaddrPort(&sin)];
+	    }
+	}
+      else if ([key isEqualToString: GSStreamRemoteAddressKey])
+	{
+	  if (getpeername(s, (struct sockaddr*)&sin, &size) != -1)
+	    {
+	      result = GSPrivateSockaddrHost(&sin);
+	    }
+	}
+      else if ([key isEqualToString: GSStreamRemotePortKey])
+	{
+	  if (getpeername(s, (struct sockaddr*)&sin, &size) != -1)
+	    {
+	      result = [NSString stringWithFormat: @"%d",
+		(int)GSPrivateSockaddrPort(&sin)];
+	    }
+	}
     }
   return result;
 }
@@ -1470,7 +1580,7 @@ setNonBlocking(SOCKET fd)
           peer.sin_family = AF_INET;
           peer.sin_port = GSSwapHostI16ToBig(p);
           ptonReturn = inet_pton(AF_INET, addr_c, &peer.sin_addr);
-          if (ptonReturn == 0)   // error
+          if (ptonReturn <= 0)   // error
             {
               return NO;
             }
@@ -1493,7 +1603,7 @@ setNonBlocking(SOCKET fd)
           peer.sin6_family = AF_INET6;
           peer.sin6_port = GSSwapHostI16ToBig(p);
           ptonReturn = inet_pton(AF_INET6, addr_c, &peer.sin6_addr);
-          if (ptonReturn == 0)   // error
+          if (ptonReturn <= 0)   // error
             {
               return NO;
             }
