@@ -22,21 +22,22 @@
 
    */
 
-#include "config.h"
+#import "common.h"
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSByteOrder.h>
-#import <Foundation/NSData.h>
-#import <Foundation/NSDebug.h>
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSEnumerator.h>
-#import <Foundation/NSException.h>
-#import <Foundation/NSHost.h>
-#import <Foundation/NSRunLoop.h>
-#import <Foundation/NSValue.h>
+#import "Foundation/NSArray.h"
+#import "Foundation/NSByteOrder.h"
+#import "Foundation/NSData.h"
+#import "Foundation/NSDictionary.h"
+#import "Foundation/NSEnumerator.h"
+#import "Foundation/NSException.h"
+#import "Foundation/NSHost.h"
+#import "Foundation/NSRunLoop.h"
+#import "Foundation/NSValue.h"
 
 #import "GSStream.h"
 #import "GSPrivate.h"
+#import "GSSocketStream.h"
+#import "GNUstepBase/NSObject+GNUstepBase.h"
 
 NSString * const NSStreamDataWrittenToMemoryStreamKey
   = @"NSStreamDataWrittenToMemoryStreamKey";
@@ -76,15 +77,6 @@ NSString * const NSStreamSOCKSProxyVersion5
 NSString * const NSStreamSOCKSProxyVersionKey
   = @"NSStreamSOCKSProxyVersionKey";
 
-NSString * const GSStreamLocalAddressKey
-  = @"GSStreamLocalAddressKey";
-NSString * const GSStreamLocalPortKey
-  = @"GSStreamLocalPortKey";
-NSString * const GSStreamRemoteAddressKey
-  = @"GSStreamRemoteAddressKey";
-NSString * const GSStreamRemotePortKey
-  = @"GSStreamRemotePortKey";
-
 
 /*
  * Determine the type of event to use when adding a stream to the run loop.
@@ -97,7 +89,7 @@ NSString * const GSStreamRemotePortKey
  */
 static RunLoopEventType typeForStream(NSStream *aStream)
 {
-#if	defined(__MINGW32__)
+#if	defined(__MINGW__)
   if ([aStream _loopID] == (void*)aStream)
     {
       return ET_TRIGGER;
@@ -944,14 +936,8 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 
 @end
 
-
-@class  GSInetInputStream;
-@class  GSInet6InputStream;
-@class  GSInetOutputStream;
-@class  GSInet6OutputStream;
-@class  GSInetServerStream;
-@class  GSInet6ServerStream;
-@class  GSLocalServerStream;
+@interface	GSLocalServerStream : GSServerStream
+@end
 
 @implementation GSServerStream
 
@@ -978,7 +964,7 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 
 - (id) initToAddr: (NSString*)addr port: (NSInteger)port
 {
-  RELEASE(self);
+  DESTROY(self);
   // try inet first, then inet6
   self = [[GSInetServerStream alloc] initToAddr: addr port: port];
   if (!self)
@@ -988,7 +974,7 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 
 - (id) initToAddr: (NSString*)addr
 {
-  RELEASE(self);
+  DESTROY(self);
   return [[GSLocalServerStream alloc] initToAddr: addr];
 }
 
