@@ -38,6 +38,7 @@
 #import "Foundation/NSException.h"
 #import "Foundation/NSAffineTransform.h"
 #import "Foundation/NSCoder.h"
+#import "GNUstepBase/NSObject+GNUstepBase.h"
 
 /* Private definitions */
 #define A _matrix.m11
@@ -160,12 +161,15 @@ static NSAffineTransformStruct identityTransform = {
 
   if (_isIdentity)
     {
+      CGFloat newTX;
+
       A = aTransform->A;
       B = aTransform->B;
       C = aTransform->C;
       D = aTransform->D;
-      TX = TX * aTransform->A + TY * aTransform->C + aTransform->TX;
+      newTX = TX * aTransform->A + TY * aTransform->C + aTransform->TX;
       TY = TX * aTransform->B + TY * aTransform->D + aTransform->TY;
+      TX = newTX;
       _isIdentity = NO;  // because aTransform is not an identity transform.
       _isFlipY = aTransform->_isFlipY;
       check();
@@ -174,12 +178,15 @@ static NSAffineTransformStruct identityTransform = {
 
   if (_isFlipY)
     {
+      CGFloat newTX;
+
       A = aTransform->A;
       B = aTransform->B;
       C = -aTransform->C;
       D = -aTransform->D;
-      TX  = TX * aTransform->A + TY * aTransform->C + aTransform->TX;
+      newTX  = TX * aTransform->A + TY * aTransform->C + aTransform->TX;
       TY  = TX * aTransform->B + TY * aTransform->D + aTransform->TY;
+      TX = newTX;
       _isIdentity = NO;
       _isFlipY = NO;
       check();
@@ -280,16 +287,22 @@ static NSAffineTransformStruct identityTransform = {
 
   if (aTransform->_isIdentity)
     {
-      TX = aTransform->TX * A + aTransform->TY * C + TX;
+      CGFloat newTX;
+
+      newTX = aTransform->TX * A + aTransform->TY * C + TX;
       TY = aTransform->TX * B + aTransform->TY * D + TY;
+      TX = newTX;
       check();
       return;
     }
 
   if (aTransform->_isFlipY)
     {
-      TX  = aTransform->TX * A + aTransform->TY * C + TX;
-      TY  = aTransform->TX * B + aTransform->TY * D + TY;
+      CGFloat newTX;
+
+      newTX = aTransform->TX * A + aTransform->TY * C + TX;
+      TY = aTransform->TX * B + aTransform->TY * D + TY;
+      TX = newTX;
       C = -C;
       D = -D;
       if (_isIdentity)
@@ -579,9 +592,27 @@ static NSAffineTransformStruct identityTransform = {
 {
   NSAffineTransformStruct	replace;
     
-  [aCoder decodeArrayOfObjCType: @encode(CGFloat)
-			  count: 6
-			     at: (CGFloat*)&replace];
+  if ([aCoder allowsKeyedCoding])
+    {
+      // FIXME
+      return [self notImplemented: _cmd];
+      /*
+      NSData *data = [aCoder decodeObjectForKey: @"NSTransformStruct"];
+      unsigned int cursor = 0;
+      // We get lengths of 12 and 18
+      NSLog(@"length %d data %@", [data length], data);
+      [data deserializeDataAt: (CGFloat*)&replace
+	        ofObjCType: "[6f]"
+		  atCursor: &cursor
+		   context: nil];
+      */
+    }
+  else
+    {
+      [aCoder decodeArrayOfObjCType: @encode(CGFloat)
+                              count: 6
+                                 at: (CGFloat*)&replace];
+    }
   [self setTransformStruct: replace];
   return self;
 }
@@ -591,9 +622,16 @@ static NSAffineTransformStruct identityTransform = {
   NSAffineTransformStruct	replace;
     
   replace = [self transformStruct];
-  [aCoder encodeArrayOfObjCType: @encode(CGFloat)
-			  count: 6
-			     at: (CGFloat*)&replace];
+  if ([aCoder allowsKeyedCoding])
+    {
+      // FIXME
+    }
+  else
+    {
+      [aCoder encodeArrayOfObjCType: @encode(CGFloat)
+                              count: 6
+                                 at: (CGFloat*)&replace];
+    }
 }
 
 @end /* NSAffineTransform */
