@@ -22,10 +22,15 @@
    Boston, MA 02111 USA.
 
    <title>NSArchiver class reference</title>
-   $Date: 2011-03-29 10:40:14 +0100 (Tue, 29 Mar 2011) $ $Revision: 32733 $
+   $Date: 2011-05-19 02:19:24 -0600 (Thu, 19 May 2011) $ $Revision: 33069 $
    */
 
 #import "common.h"
+
+#if !defined (__GNU_LIBOBJC__)
+#  include <objc/encoding.h>
+#endif
+
 #define	EXPOSE_NSArchiver_IVARS	1
 #define	EXPOSE_NSUnarchiver_IVARS	1
 /*
