@@ -27,7 +27,7 @@
    Boston, MA 02111 USA.
 
    <title>NSArray class reference</title>
-   $Date: 2011-02-23 11:52:17 +0000 (Wed, 23 Feb 2011) $ $Revision: 32323 $
+   $Date: 2012-01-02 08:07:32 -0700 (Mon, 02 Jan 2012) $ $Revision: 34389 $
    */
 
 #import "common.h"
@@ -101,6 +101,13 @@ static SEL	oaiSel;
 static SEL	remSel;
 static SEL	rlSel;
 
++ (void) atExit
+{
+  DESTROY(defaultPlaceholderArray);
+  DESTROY(placeholderLock);
+  DESTROY(placeholderMap);
+}
+
 + (void) initialize
 {
   if (self == [NSArray class])
@@ -129,6 +136,7 @@ static SEL	rlSel;
       placeholderMap = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
 	NSNonRetainedObjectMapValueCallBacks, 0);
       placeholderLock = [NSLock new];
+      [self registerAtExit];
     }
 }
 
@@ -263,7 +271,7 @@ static SEL	rlSel;
  * Returns an autoreleased array containing the specified
  * objects, preserving order.
  */
-+ (id) arrayWithObjects: (id*)objects count: (NSUInteger)count
++ (id) arrayWithObjects: (const id[])objects count: (NSUInteger)count
 {
   return AUTORELEASE([[self allocWithZone: NSDefaultMallocZone()]
     initWithObjects: objects count: count]);
@@ -381,7 +389,7 @@ static SEL	rlSel;
 }
 
 - (NSUInteger) countByEnumeratingWithState: (NSFastEnumerationState*)state 	
-				   objects: (id*)stackbuf
+				   objects: (__unsafe_unretained id[])stackbuf
 				     count: (NSUInteger)len
 {
   NSUInteger size = [self count];
@@ -471,7 +479,7 @@ static SEL	rlSel;
  * Copies the objects from the receiver to aBuffer, which must be
  * an area of memory large enough to hold them.
  */
-- (void) getObjects: (id*)aBuffer
+- (void) getObjects: (__unsafe_unretained id[])aBuffer
 {
   unsigned i, c = [self count];
   IMP	get = [self methodForSelector: oaiSel];
@@ -484,7 +492,7 @@ static SEL	rlSel;
  * Copies the objects from the range aRange of the receiver to aBuffer,
  * which must be an area of memory large enough to hold them.
  */
-- (void) getObjects: (id*)aBuffer range: (NSRange)aRange
+- (void) getObjects: (__unsafe_unretained id[])aBuffer range: (NSRange)aRange
 {
   unsigned i, j = 0, c = [self count], e = aRange.location + aRange.length;
   IMP	get = [self methodForSelector: oaiSel];
@@ -871,7 +879,7 @@ static SEL	rlSel;
  * and needs to be re-implemented in subclasses in order to have all
  * other initialisers work.
  */
-- (id) initWithObjects: (id*)objects count: (NSUInteger)count
+- (id) initWithObjects: (const id[])objects count: (NSUInteger)count
 {
   self = [self init];
   return self;
@@ -917,7 +925,7 @@ static SEL	rlSel;
   //FIXME: probably slow!
   NSMutableArray *group = [NSMutableArray arrayWithCapacity: [indexes count]];
 
-  unsigned i = [indexes firstIndex];
+  NSUInteger i = [indexes firstIndex];
   while (i != NSNotFound)
     {
       [group addObject: [self objectAtIndex: i]];
@@ -1941,7 +1949,7 @@ compare(id elem1, id elem2, void* context)
 /**
  * Override our superclass's designated initializer to go our's
  */
-- (id) initWithObjects: (id*)objects count: (NSUInteger)count
+- (id) initWithObjects: (const id[])objects count: (NSUInteger)count
 {
   self = [self initWithCapacity: count];
   if (count > 0)
