@@ -22,10 +22,11 @@
    Boston, MA 02111 USA.
 
    <title>NSAssertionHandler class reference</title>
-   $Date: 2010-11-07 15:37:19 +0000 (Sun, 07 Nov 2010) $ $Revision: 31593 $
+   $Date: 2011-07-24 14:47:36 -0600 (Sun, 24 Jul 2011) $ $Revision: 33625 $
    */
 
 #import "common.h"
+#import "GNUstepBase/GSConfig.h"
 #import "Foundation/NSException.h"
 #import "Foundation/NSDictionary.h"
 #import "Foundation/NSThread.h"
@@ -103,6 +104,7 @@ NSString *const NSAssertionHandlerKey = @"NSAssertionHandler";
   [NSException raise: NSInternalInconsistencyException
 	      format: message arguments: ap];
   va_end(ap);
+  GS_UNREACHABLE();
   /* NOT REACHED */
 }
 
@@ -133,6 +135,7 @@ NSString *const NSAssertionHandlerKey = @"NSAssertionHandler";
 	      format: message arguments: ap];
   va_end(ap);
   /* NOT REACHED */
+  GS_UNREACHABLE();
 }
 
 @end
