@@ -31,7 +31,7 @@
    Boston, MA 02111 USA.
 
    <title>NSAttributedString class reference</title>
-   $Date: 2011-03-09 10:24:18 +0000 (Wed, 09 Mar 2011) $ $Revision: 32507 $
+   $Date: 2011-07-20 19:23:08 -0600 (Wed, 20 Jul 2011) $ $Revision: 33595 $
 */
 
 /* Warning -	[-initWithString:attributes:] is the designated initialiser,
@@ -366,7 +366,7 @@ appendUIntData(NSMutableData *d, NSUInteger i)
 //Creating an NSAttributedString
 - (id) init
 {
-  return [self initWithString: nil attributes: nil];
+  return [self initWithString: @"" attributes: nil];
 }
 
 /**
