@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSAutoreleasePool class reference</title>
-   $Date: 2011-05-24 16:42:54 +0100 (Tue, 24 May 2011) $ $Revision: 33105 $
+   $Date: 2011-07-11 08:31:36 -0600 (Mon, 11 Jul 2011) $ $Revision: 33521 $
    */
 
 #import "common.h"
@@ -34,12 +34,18 @@
 #import "Foundation/NSException.h"
 #import "Foundation/NSThread.h"
 
-#if	GS_WITH_GC
+
+#if GS_WITH_GC || __OBJC_GC__
+
+#if __OBJC_GC__
+@interface GSAutoreleasePool : NSAutoreleasePool @end
+#endif
 
 @implementation NSAutoreleasePool
 
 static NSAutoreleasePool	*pool = nil;
 
+#if GS_WITH_GC
 + (void) initialize
 {
   pool = NSAllocateObject(self, 0, NSDefaultMallocZone());
@@ -55,6 +61,42 @@ static NSAutoreleasePool	*pool = nil;
 {
   return pool;
 }
+#endif
+#ifdef __OBJC_GC__ 
+
+static Class PoolClass;
+
++ (void) initialize
+{
+  if ([NSGarbageCollector defaultCollector])
+    {
+      pool = NSAllocateObject(self, 0, NSDefaultMallocZone());
+    }
+  else
+    {
+      PoolClass = [GSAutoreleasePool class];
+    }
+  return;
+}
+
++ (id) allocWithZone: (NSZone*)zone
+{
+  if (nil == pool)
+    {
+      return NSAllocateObject(PoolClass, 0, 0);
+    }
+  return pool;
+}
+
++ (id) new
+{
+  if (nil == pool)
+    {
+      return [NSAllocateObject(PoolClass, 0, 0) init];
+    }
+  return pool;
+}
+#endif
 
 - (id) init
 {
@@ -120,6 +162,8 @@ static NSAutoreleasePool	*pool = nil;
 
 - (void) dealloc
 {
+  [NSException raise: NSGenericException
+    format: @"dealloc should not be called in garbage collected mode"];
   GSNOSUPERDEALLOC;
   return;
 }
@@ -163,7 +207,9 @@ static NSAutoreleasePool	*pool = nil;
 
 @end
 
-#else
+#endif
+
+#if !GS_WITH_GC
 
 /* When this is `NO', autoreleased objects are never actually recorded
    in an NSAutoreleasePool, and are not sent a `release' message.
@@ -247,7 +293,11 @@ pop_pool_from_cache (struct autorelease_thread_vars *tv)
 }
 
 
+#if __OBJC_GC__
+@implementation GSAutoreleasePool
+#else
 @implementation NSAutoreleasePool
+#endif
 
 + (void) initialize
 {
@@ -388,6 +438,7 @@ pop_pool_from_cache (struct autorelease_thread_vars *tv)
 {
   NSThread		*t = GSCurrentThread();
   NSAutoreleasePool	*pool;
+  NSAssert(nil != t, @"Creating autorelease pool on nonexistent thread!");
 
   pool = t->_autorelease_vars.current_pool;
   if (pool == nil && t->_active == NO)
@@ -413,7 +464,7 @@ pop_pool_from_cache (struct autorelease_thread_vars *tv)
 	{
 	  NSLog(@"autorelease called without pool for nil object.");
 	}
-      [arp release];
+      [arp drain];
     }
 }
 
@@ -463,7 +514,9 @@ pop_pool_from_cache (struct autorelease_thread_vars *tv)
 
 - (void) drain
 {
-  [self release];
+  // Don't call -release, make both -release and -drain have the same cost in
+  // non-GC mode.
+  [self dealloc];
 }
 
 - (id) retain
