@@ -28,17 +28,18 @@
 
 
    <title>NSBundle class reference</title>
-   $Date: 2011-05-23 11:53:12 +0100 (Mon, 23 May 2011) $ $Revision: 33089 $
+   $Date: 2012-01-16 01:15:07 -0700 (Mon, 16 Jan 2012) $ $Revision: 34555 $
 */
 
-#import "common.h"
 #define	EXPOSE_NSBundle_IVARS	1
+#import "common.h"
 #include "objc-load.h"
 #import "Foundation/NSBundle.h"
 #import "Foundation/NSException.h"
 #import "Foundation/NSArray.h"
 #import "Foundation/NSDictionary.h"
 #import "Foundation/NSEnumerator.h"
+#import "Foundation/NSNull.h"
 #import "Foundation/NSProcessInfo.h"
 #import "Foundation/NSUserDefaults.h"
 #import "Foundation/NSNotification.h"
@@ -48,6 +49,7 @@
 #import "Foundation/NSFileManager.h"
 #import "Foundation/NSPathUtilities.h"
 #import "Foundation/NSData.h"
+#import "Foundation/NSURL.h"
 #import "Foundation/NSValue.h"
 #import "GNUstepBase/NSObject+GNUstepBase.h"
 #import "GNUstepBase/NSString+GNUstepBase.h"
@@ -59,9 +61,6 @@ NSString * const NSBundleDidLoadNotification = @"NSBundleDidLoadNotification";
 NSString * const NSShowNonLocalizedStrings = @"NSShowNonLocalizedStrings";
 NSString * const NSLoadedClasses = @"NSLoadedClasses";
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #include <string.h>
 
 static NSFileManager	*
@@ -76,6 +75,8 @@ manager()
   return mgr;
 }
 
+static NSLock *pathCacheLock = nil;
+static NSMutableDictionary *pathCache = nil;
 
 @interface NSObject (PrivateFrameworks)
 + (NSString*) frameworkEnv;
@@ -210,10 +211,10 @@ AbsolutePathOfExecutable(NSString *path, BOOL atLaunch)
 #if defined(__WIN32__)
 	  else
 	    {
-	      NSString	*ext = [path pathExtension];
+	      NSString	*extension = [path pathExtension];
 
 	      /* Also add common executable extensions on windows */
-	      if (ext == nil || [ext length] == 0)
+	      if (extension == nil || [extension length] == 0)
 		{
 		  NSString *wpath;
 
@@ -301,17 +302,32 @@ GSPrivateExecutablePath()
   return executablePath;
 }
 
-static BOOL
+static NSArray *
 bundle_directory_readable(NSString *path)
 {
-  NSFileManager	*mgr = manager();
-  BOOL		directory;
+  id	found;
 
-  if ([mgr fileExistsAtPath: path isDirectory: &directory] == NO
-    || !directory)
-    return NO;
+  [pathCacheLock lock];
+  found = [[[pathCache objectForKey: path] retain] autorelease];
+  [pathCacheLock unlock];
+  if (nil == found)
+    {
+      NSFileManager	*mgr = manager();
 
-  return [mgr isReadableFileAtPath: path];
+      found = [mgr directoryContentsAtPath: path];
+      if (nil == found)
+	{
+	  found = [NSNull null];
+	}
+      [pathCacheLock lock];
+      [pathCache setObject: found forKey: path];
+      [pathCacheLock unlock];
+    }
+  if ((id)[NSNull null] == found)
+    {
+      found = nil;
+    }
+  return (NSArray*)found;
 }
 
 /* Get the object file that should be located in the bundle of the same name */
@@ -373,29 +389,59 @@ bundle_object_name(NSString *path, NSString* executable)
 }
 
 /* Construct a path from components */
-static NSString *
-_bundle_resource_path(NSString *primary, NSString* bundlePath, NSString *lang)
+static void
+addBundlePath(NSMutableArray *list, NSArray *contents,
+  NSString *path, NSString *subdir, NSString *lang)
 {
-  if (bundlePath)
-    primary = [primary stringByAppendingPathComponent: bundlePath];
-  if (lang)
-    primary = [primary stringByAppendingPathComponent:
-      [NSString stringWithFormat: @"%@.lproj", lang]];
-  return primary;
+  if (nil == contents)
+    {
+      return;
+    }
+  if (nil != subdir)
+    {
+      NSEnumerator *e = [[subdir pathComponents] objectEnumerator];
+      NSString *subdirComponent;
+      while ((subdirComponent = [e nextObject]) != nil)
+	{
+	  if (NO == [contents containsObject: subdirComponent])
+	    {
+	      return;
+	    }
+	  path = [path stringByAppendingPathComponent: subdirComponent];
+	  if (nil == (contents = bundle_directory_readable(path)))
+	    {
+	      return;
+	    }	  
+	}
+    }
+  if (nil != lang)
+    {
+      lang = [lang stringByAppendingPathExtension: @"lproj"];
+      if (NO == [contents containsObject: lang])
+	{
+	  return;
+	}
+      path = [path stringByAppendingPathComponent: lang];
+      if (nil == (contents = bundle_directory_readable(path)))
+	{
+	  return;
+	}
+    }
+  [list addObject: path];
 }
 
 /* Try to locate name framework in standard places
    which are like /Library/Frameworks/(name).framework */
 static inline NSString *
 _find_framework(NSString *name)
-{                
+{
   NSArray	*paths;
   NSFileManager *file_mgr = manager();
   NSString	*file_name = [name stringByAppendingPathExtension:@"framework"];
   NSString	*file_path;
   NSString	*path;
   NSEnumerator	*enumerator;
-           
+
   NSCParameterAssert(name != nil);
 
   paths = NSSearchPathForDirectoriesInDomains(GSFrameworksDirectory,
@@ -412,7 +458,7 @@ _find_framework(NSString *name)
         }
     }
   return nil;
-}        
+}
 
 
 /* Try to locate resources for tool name (which is this tool) in
@@ -446,7 +492,7 @@ _find_main_bundle_for_tool(NSString *toolName)
     }
 
   tail = [@"Tools" stringByAppendingPathComponent:
-	     [@"Resources" stringByAppendingPathComponent: 
+	     [@"Resources" stringByAppendingPathComponent:
 		 toolName]];
 
   paths = NSSearchPathForDirectoriesInDomains (NSLibraryDirectory,
@@ -594,14 +640,14 @@ _find_main_bundle_for_tool(NSString *toolName)
 	   */
 	  if ([bundlePath isAbsolutePath] == NO)
 	    {
-	      bundlePath = [_launchDirectory 
+	      bundlePath = [_launchDirectory
 			     stringByAppendingPathComponent: bundlePath];
 
 	    }
-	  
+
 	  /* Dereference symlinks, and standardize path.  This will
 	   * only work properly if the original bundlePath is
-	   * absolute.  
+	   * absolute.
 	   */
 	  bundlePath = [bundlePath stringByStandardizingPath];
 
@@ -659,7 +705,7 @@ _find_main_bundle_for_tool(NSString *toolName)
 	      bundlePath = [bundlePath stringByDeletingLastPathComponent];
 #endif
 	      pathComponent = [bundlePath lastPathComponent];
-	
+
 	      if ([pathComponent isEqualToString:
 				   [NSString stringWithFormat: @"%@%@",
 					     name, @".framework"]])
@@ -729,7 +775,7 @@ _find_main_bundle_for_tool(NSString *toolName)
 
 	  value = [NSValue valueWithPointer: (void*)class];
 	  [bundle->_bundleClasses addObject: value];
-	
+
 	  fmClasses++;
 	}
 
@@ -863,6 +909,9 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
        */
       manager();
 
+      pathCacheLock = [NSLock new];
+      pathCache = [NSMutableDictionary new];
+
       /* Need to make this recursive since both mainBundle and
        * initWithPath: want to lock the thread.
        */
@@ -880,22 +929,22 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
 	gnustep_target_cpu = RETAIN(str);
       else if ((str = [env objectForKey: @"GNUSTEP_HOST_CPU"]) != nil)
 	gnustep_target_cpu = RETAIN(str);
-      
+
       if ((str = [env objectForKey: @"GNUSTEP_TARGET_OS"]) != nil)
 	gnustep_target_os = RETAIN(str);
       else if ((str = [env objectForKey: @"GNUSTEP_HOST_OS"]) != nil)
 	gnustep_target_os = RETAIN(str);
-      
+
       if ((str = [env objectForKey: @"GNUSTEP_TARGET_DIR"]) != nil)
 	gnustep_target_dir = RETAIN(str);
       else if ((str = [env objectForKey: @"GNUSTEP_HOST_DIR"]) != nil)
 	gnustep_target_dir = RETAIN(str);
-      
+
       if ((str = [env objectForKey: @"LIBRARY_COMBO"]) != nil)
 	library_combo = RETAIN(str);
-      
+
       _launchDirectory = RETAIN([manager() currentDirectoryPath]);
-      
+
       _gnustep_bundle = RETAIN([self bundleForLibrary: @"gnustep-base"
 					      version: _base_version]);
 #if 0
@@ -923,12 +972,12 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
       {
 	void	*state = NULL;
 	Class	class;
-	
+
 	while ((class = objc_next_class(&state)))
 	  {
 	    const char *className = class_getName(class);
 	    unsigned int len = strlen (className);
-	    
+
 	    if (len > sizeof("NSFramework_")
 		&& !strncmp("NSFramework_", className, 12))
 	      {
@@ -940,7 +989,7 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
 #if 0
       //  _bundle_load_callback(class, NULL);
       //  bundle = (NSBundle *)NSMapGet(_bundles, bundlePath);
-      
+
       objc_close_main_module(handle);
       _loadingBundle = nil;
 #endif
@@ -1098,7 +1147,7 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
       if (isApplication == YES)
 	{
 	  s = [path lastPathComponent];
-    
+
 	  if ([s hasSuffix: @".app"] == NO
 	    && [s hasSuffix: @".debug"] == NO
 	    && [s hasSuffix: @".profile"] == NO
@@ -1169,7 +1218,7 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
 		   */
 		  maybePath = _find_main_bundle_for_tool (toolName);
 		}
-	      
+
 	      /* If that didn't work, maybe the tool was created with
 	       * gnustep-make version 1.  So we try {tool location on
 	       * disk after walking up the non-flattened
@@ -1218,7 +1267,10 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
    */
   if (aClass == [NSObject class])
     {
-      return _gnustep_bundle;
+      if (nil != _gnustep_bundle)
+	{
+	  return _gnustep_bundle;
+	}
     }
 
   [load_lock lock];
@@ -1296,6 +1348,11 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
   return AUTORELEASE([[self alloc] initWithPath: path]);
 }
 
++ (NSBundle*) bundleWithURL: (NSURL*)url
+{
+  return AUTORELEASE([[self alloc] initWithURL: url]);
+}
+
 + (NSBundle*) bundleWithIdentifier: (NSString*)identifier
 {
   NSBundle	*bundle = nil;
@@ -1305,13 +1362,21 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
     {
       bundle = (NSBundle *)NSMapGet(_byIdentifier, identifier);
 IF_NO_GC(
-      if (bundle != nil)
-	{
-	  [bundle retain]; /* retain - look as if we were alloc'ed */
-	}
+	[bundle retain]; /* retain - look as if we were alloc'ed */
 )
     }
   [load_lock unlock];
+  // Some OS X apps  try to get the foundation bundle by looking it up by
+  // identifier.  This is expected to be faster than looking it up by class, so
+  // we lazily insert it into the table if it's requested.
+  if (nil == bundle && [@"com.apple.Foundation" isEqualToString: identifier])
+    {
+      NSBundle *foundation = [self bundleForClass: self];
+      [load_lock lock];
+      NSMapInsert(_byIdentifier, @"com.apple.Foundation", foundation);
+      [load_lock unlock];
+      return foundation;
+    }
   return AUTORELEASE(bundle);
 }
 
@@ -1399,7 +1464,7 @@ IF_NO_GC(
     }
   [load_lock unlock];
 
-  if (bundle_directory_readable(path) == NO)
+  if (bundle_directory_readable(path) == nil)
     {
       NSDebugMLLog(@"NSBundle", @"Could not access path %@ for bundle", path);
       // if this is not the main bundle ... deallocate and return.
@@ -1456,6 +1521,12 @@ IF_NO_GC(
   return self;
 }
 
+- (id) initWithURL: (NSURL*)url
+{
+  // FIXME
+  return [self initWithPath: [url path]];
+}
+
 - (void) dealloc
 {
   if ([self isLoaded] == YES && self != _mainBundle
@@ -1472,7 +1543,10 @@ IF_NO_GC(
     }
   if (_path != nil)
     {
-      NSString	*identifier = [self bundleIdentifier];
+      NSString		*identifier = [self bundleIdentifier];
+      NSUInteger	plen = [_path length];
+      NSEnumerator	*enumerator;
+      NSString		*path;
 
       [load_lock lock];
       NSMapRemove(_bundles, _path);
@@ -1481,6 +1555,41 @@ IF_NO_GC(
 	  NSMapRemove(_byIdentifier, identifier);
         }
       [load_lock unlock];
+
+      /* Clean up path cache for this bundle.
+       */
+      [pathCacheLock lock];
+      enumerator = [pathCache keyEnumerator];
+      while (nil != (path = [enumerator nextObject]))
+	{
+	  if (YES == [path hasPrefix: _path])
+	    {
+	      if ([path length] == plen)
+		{
+		  /* Remove the bundle directory path from the cache.
+		   */
+		  [pathCache removeObjectForKey: path];
+		}
+	      else
+		{
+		  unichar	c = [path characterAtIndex: plen];
+
+		  /* if the directory is inside the bundle, remove from cache.
+		   */
+		  if ('/' == c)
+		    {
+		      [pathCache removeObjectForKey: path];
+		    }
+#if defined(__MINGW__)
+		  else if ('\\' == c)
+		    {
+		      [pathCache removeObjectForKey: path];
+		    }
+#endif
+		}
+	    }
+	}
+      [pathCacheLock unlock];
       RELEASE(_path);
     }
   TEST_RELEASE(_frameworkVersion);
@@ -1680,7 +1789,7 @@ IF_NO_GC(
   return YES;
 }
 
-- (void) release
+- (oneway void) release
 {
   /* We lock during release so that other threads can't grab the
    * object between us checking the reference count and deallocating.
@@ -1710,6 +1819,7 @@ IF_NO_GC(
   NSString		*primary;
   NSString		*language;
   NSArray		*languages;
+  NSArray		*contents;
   NSMutableArray	*array;
   NSEnumerator		*enumerate;
 
@@ -1718,72 +1828,78 @@ IF_NO_GC(
     stringArrayForKey: @"NSLanguages"];
 
   primary = [rootPath stringByAppendingPathComponent: @"Resources"];
-  [array addObject: _bundle_resource_path(primary, subPath, nil)];
+  contents = bundle_directory_readable(primary);
+  addBundlePath(array, contents, primary, subPath, nil);
   /* If we have been asked for a specific localization, we add it.
    */
   if (localization != nil)
     {
-      [array addObject: _bundle_resource_path(primary, subPath, localization)];
+      addBundlePath(array, contents, primary, subPath, localization);
     }
   else
     {
       /* This matches OS X behavior, which only searches languages that
-       * are in the user's preference. Don't use -preferredLocalizations - 
+       * are in the user's preference. Don't use -preferredLocalizations -
        * that would cause a recursive loop.
        */
       enumerate = [languages objectEnumerator];
       while ((language = [enumerate nextObject]))
 	{
-	  [array addObject: _bundle_resource_path(primary, subPath, language)];
+	  addBundlePath(array, contents, primary, subPath, language);
 	}
     }
   primary = rootPath;
-  [array addObject: _bundle_resource_path(primary, subPath, nil)];
+  contents = bundle_directory_readable(primary);
+  addBundlePath(array, contents, primary, subPath, nil);
   if (localization != nil)
     {
-      [array addObject: _bundle_resource_path(primary, subPath, localization)];
+      addBundlePath(array, contents, primary, subPath, localization);
     }
   else
     {
       enumerate = [languages objectEnumerator];
       while ((language = [enumerate nextObject]))
 	{
-	  [array addObject: _bundle_resource_path(primary, subPath, language)];
+	  addBundlePath(array, contents, primary, subPath, language);
 	}
     }
   return array;
 }
 
 + (NSString *) _pathForResource: (NSString *)name
-			 ofType: (NSString *)ext	
+			 ofType: (NSString *)extension
 		     inRootPath: (NSString *)rootPath
 		    inDirectory: (NSString *)subPath
 {
   NSFileManager	*mgr = manager();
   NSString	*path;
+  NSString	*file;
   NSEnumerator	*pathlist;
 
   if (name == nil)
     {
       name = @"";
     }
-  if ([ext length] == 0)
+  if ([extension length] == 0)
     {
-      ext = nil;
+      extension = nil;
+      file = name;
+    }
+  else
+    {
+      file = [name stringByAppendingPathExtension: extension];
     }
 
   pathlist = [[self _bundleResourcePathsWithRootPath: rootPath
     subPath: subPath localization: nil] objectEnumerator];
   while ((path = [pathlist nextObject]) != nil)
     {
-      if (bundle_directory_readable(path))
+      NSArray	*paths = bundle_directory_readable(path);
+
+      if (YES == [paths containsObject: file])
 	{
-	  path = [path stringByAppendingPathComponent: name];
-	  if (ext != nil)
-	    {
-	      path = [path stringByAppendingPathExtension: ext];
-	    }
-	  if ([mgr isReadableFileAtPath: path])
+	  path = [path stringByAppendingPathComponent: file];
+	  if (YES == [mgr isReadableFileAtPath: path])
 	    {
 	      return path;
 	    }
@@ -1795,36 +1911,48 @@ IF_NO_GC(
 
 
 + (NSString *) pathForResource: (NSString *)name
-			ofType: (NSString *)ext	
+			ofType: (NSString *)extension
 		   inDirectory: (NSString *)bundlePath
 		   withVersion: (int)version
 {
   return [self _pathForResource: name
-			 ofType: ext
+			 ofType: extension
 		     inRootPath: bundlePath
 		    inDirectory: nil];
 }
 
 + (NSString *) pathForResource: (NSString *)name
-			ofType: (NSString *)ext	
+			ofType: (NSString *)extension
 		   inDirectory: (NSString *)bundlePath
 {
   return [self _pathForResource: name
-			 ofType: ext
+			 ofType: extension
 		     inRootPath: bundlePath
 		    inDirectory: nil];
 }
 
++ (NSURL*) URLForResource: (NSString*)name
+            withExtension: (NSString*)extension
+             subdirectory: (NSString*)subpath
+          inBundleWithURL: (NSURL*)bundleURL
+{
+  NSBundle *root = [self bundleWithURL: bundleURL];
+
+  return [root URLForResource: name
+                withExtension: extension
+                 subdirectory: subpath];
+}
+
 - (NSString *) pathForResource: (NSString *)name
-			ofType: (NSString *)ext
+			ofType: (NSString *)extension
 {
   return [self pathForResource: name
-			ofType: ext
+			ofType: extension
 		   inDirectory: nil];
 }
 
 - (NSString *) pathForResource: (NSString *)name
-			ofType: (NSString *)ext
+			ofType: (NSString *)extension
 		   inDirectory: (NSString *)subPath
 {
   NSString *rootPath;
@@ -1838,11 +1966,48 @@ IF_NO_GC(
     rootPath = [self bundlePath];
 
   return [NSBundle _pathForResource: name
-			     ofType: ext
+			     ofType: extension
 			 inRootPath: rootPath
 		        inDirectory: subPath];
 }
 
+- (NSURL *) URLForResource: (NSString *)name
+	     withExtension: (NSString *)extension
+{
+  return [self URLForResource: name
+                withExtension: extension
+                 subdirectory: nil
+                 localization: nil];
+}
+
+- (NSURL *) URLForResource: (NSString *)name
+	     withExtension: (NSString *)extension
+              subdirectory: (NSString *)subpath
+{
+  return [self URLForResource: name
+                withExtension: extension
+                 subdirectory: subpath
+                 localization: nil];
+}
+
+- (NSURL *) URLForResource: (NSString *)name
+	     withExtension: (NSString *)extension
+	      subdirectory: (NSString *)subpath
+	      localization: (NSString *)localizationName
+{
+  NSString	*path;
+
+  path = [self pathForResource: name
+			ofType: extension
+		   inDirectory: subpath
+	       forLocalization: localizationName];
+  if (nil == path)
+    {
+      return nil;
+    }
+  return [NSURL fileURLWithPath: path];
+}
+
 + (NSArray*) _pathsForResourcesOfType: (NSString*)extension
 		      inRootDirectory: (NSString*)bundlePath
 		       inSubDirectory: (NSString*)subPath
@@ -1852,7 +2017,6 @@ IF_NO_GC(
   NSString *path;
   NSMutableArray *resources;
   NSEnumerator *pathlist;
-  NSFileManager	*mgr = manager();
 
   pathlist = [[NSBundle _bundleResourcePathsWithRootPath: bundlePath
     subPath: subPath localization: localization] objectEnumerator];
@@ -1862,9 +2026,9 @@ IF_NO_GC(
   while ((path = [pathlist nextObject]))
     {
       NSEnumerator *filelist;
-      NSString *match;
+      NSString	*match;
 
-      filelist = [[mgr directoryContentsAtPath: path] objectEnumerator];
+      filelist = [bundle_directory_readable(path) objectEnumerator];
       while ((match = [filelist nextObject]))
 	{
 	  if (allfiles || [extension isEqual: [match pathExtension]])
@@ -1921,16 +2085,16 @@ IF_NO_GC(
 	}
       else if ([localizationName length] > 0
 	&& [[theDir lastPathComponent] hasPrefix: localizationName])
-	{ 
+	{
 	  [result insertObject: path atIndex: 0];
 	}
     }
-  
+
   return result;
 }
 
 - (NSString*) pathForResource: (NSString*)name
-		       ofType: (NSString*)ext
+		       ofType: (NSString*)extension
 		  inDirectory: (NSString*)subPath
 	      forLocalization: (NSString*)localizationName
 {
@@ -1938,15 +2102,15 @@ IF_NO_GC(
   NSString		*result = nil;
   NSArray		*array;
 
-  if ([ext length] == 0)
+  if ([extension length] == 0)
     {
-      ext = [name pathExtension];
-      if (ext != nil)
+      extension = [name pathExtension];
+      if (extension != nil)
 	{
 	  name = [name stringByDeletingPathExtension];
 	}
     }
-  array = [self pathsForResourcesOfType: ext
+  array = [self pathsForResourcesOfType: extension
                             inDirectory: subPath
                         forLocalization: localizationName];
 
@@ -1955,7 +2119,7 @@ IF_NO_GC(
       NSEnumerator	*enumerator = [array objectEnumerator];
       NSString		*path;
 
-      name = [name stringByAppendingPathExtension: ext];
+      name = [name stringByAppendingPathExtension: extension];
       while ((path = [enumerator nextObject]) != nil)
 	{
 	  NSString	*found = [path lastPathComponent];
@@ -1968,7 +2132,7 @@ IF_NO_GC(
 	}
     }
   [result retain];
-  [arp release];
+  [arp drain];
   return [result autorelease];
 }
 
@@ -2037,7 +2201,7 @@ IF_NO_GC(
 
 - (id) objectForInfoDictionaryKey: (NSString *)key
 {
-  return nil;
+  return [[self infoDictionary] objectForKey: key];
 }
 
 - (NSString*) developmentLocalization
@@ -2068,7 +2232,7 @@ IF_NO_GC(
   return [NSBundle preferredLocalizationsFromArray: [self localizations]];
 }
 
-- (NSString *) localizedStringForKey: (NSString *)key	
+- (NSString *) localizedStringForKey: (NSString *)key
                                value: (NSString *)value
                                table: (NSString *)tableName
 {
@@ -2109,7 +2273,7 @@ IF_NO_GC(
           NSData		*tableData;
           const unsigned char	*bytes;
           unsigned		length;
-          
+
           tableData = [[NSData alloc] initWithContentsOfFile: tablePath];
           bytes = [tableData bytes];
           length = [tableData length];
@@ -2248,10 +2412,10 @@ IF_NO_GC(
   if (object == nil || [object length] == 0)
     {
       object = [[self infoDictionary] objectForKey: @"CFBundleExecutable"];
-      if(object == nil || [object length] == 0)
-	  {
-	    return nil;
-	  }
+      if (object == nil || [object length] == 0)
+	{
+	  return nil;
+	}
     }
   if (_bundleType == NSBUNDLE_FRAMEWORK)
     {
@@ -2259,11 +2423,11 @@ IF_NO_GC(
        * which really is a class name.
        */
       NSString *mangledName = object;
-      mangledName = [mangledName stringByReplacingString: @"_"  
+      mangledName = [mangledName stringByReplacingString: @"_"
 				 withString: @"__"];
-      mangledName = [mangledName stringByReplacingString: @"-" 
+      mangledName = [mangledName stringByReplacingString: @"-"
 				 withString: @"_0"];
-      mangledName = [mangledName stringByReplacingString: @"+" 
+      mangledName = [mangledName stringByReplacingString: @"+"
 				 withString: @"_1"];
 
 #if !defined(__MINGW__)
@@ -2510,14 +2674,14 @@ IF_NO_GC(
    *
    * if no <interfaceVersion> is specified, and if can't find any versioned
    * resources in those directories, we'll also accept the old unversioned
-   * subdirectory: 
+   * subdirectory:
    *
    * Libraries/Resources/<libraryName>/
    *
    */
   paths = NSSearchPathForDirectoriesInDomains (NSLibraryDirectory,
 					       NSAllDomainsMask, YES);
-  
+
   enumerator = [paths objectEnumerator];
   while ((path = [enumerator nextObject]) != nil)
     {
@@ -2559,7 +2723,7 @@ IF_NO_GC(
 	      if ([fm fileExistsAtPath: path  isDirectory: &isDir]  &&  isDir)
 		{
 		  b = [self bundleWithPath: path];
-		  
+
 		  if (b != nil && b->_bundleType == NSBUNDLE_BUNDLE)
 		    {
 		      b->_bundleType = NSBUNDLE_LIBRARY;
@@ -2582,7 +2746,7 @@ IF_NO_GC(
 		  /* TODO: Ignore subdirectories.  */
 		  NSEnumerator *fileEnumerator;
 		  NSString *potentialPath;
-		  
+
 		  fileEnumerator = [fm enumeratorAtPath: versionsPath];
 		  while ((potentialPath = [fileEnumerator nextObject]) != nil)
 		    {
@@ -2594,7 +2758,7 @@ IF_NO_GC(
 				   isDirectory: &isDir]  &&  isDir)
 			{
 			  b = [self bundleWithPath: potentialPath];
-			  
+
 			  if (b != nil && b->_bundleType == NSBUNDLE_BUNDLE)
 			    {
 			      b->_bundleType = NSBUNDLE_LIBRARY;
@@ -2638,7 +2802,7 @@ IF_NO_GC(
 }
 
 + (NSString *) pathForLibraryResource: (NSString *)name
-			       ofType: (NSString *)ext	
+			       ofType: (NSString *)extension
 			  inDirectory: (NSString *)bundlePath
 {
   NSString	*path = nil;
@@ -2656,7 +2820,7 @@ IF_NO_GC(
     {
       bundle = [self bundleWithPath: bundle_path];
       path = [bundle pathForResource: name
-                              ofType: ext
+                              ofType: extension
                          inDirectory: bundlePath];
     }
 
