@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSCharacterSet class reference</title>
-   $Date: 2010-09-24 12:05:11 +0100 (Fri, 24 Sep 2010) $ $Revision: 31404 $
+   $Date: 2011-07-24 07:09:22 -0600 (Sun, 24 Jul 2011) $ $Revision: 33621 $
 */
 
 #import "common.h"
@@ -76,7 +76,7 @@
 }
 @end
 
-@interface	NSDataStatic : NSObject	// Help the compiler
+@interface	NSDataStatic : NSData	// Help the compiler
 @end
 
 /* Private class from NSIndexSet.m
@@ -620,7 +620,7 @@ static Class concreteMutableClass = nil;
 
 - (id) initWithBitmap: (NSData*)bitmap number: (int)number
 {
-  if ((self = [(NSBitmapCharSet*)self initWithBitmap: bitmap]) != nil)
+  if ((self = (_GSStaticCharSet*)[(NSBitmapCharSet*)self initWithBitmap: bitmap]) != nil)
     {
       _index = number;
     }
