@@ -23,10 +23,15 @@
    Boston, MA 02111 USA.
 
    <title>NSCoder class reference</title>
-   $Date: 2010-02-19 08:12:46 +0000 (Fri, 19 Feb 2010) $ $Revision: 29669 $
+   $Date: 2011-05-19 02:19:24 -0600 (Thu, 19 May 2011) $ $Revision: 33069 $
    */
 
 #import "common.h"
+
+#if !defined (__GNU_LIBOBJC__)
+#  include <objc/encoding.h>
+#endif
+
 #define	EXPOSE_NSCoder_IVARS	1
 #import "Foundation/NSData.h"
 #import "Foundation/NSCoder.h"
