@@ -97,9 +97,28 @@ typedef GSIMapNode_t *GSIMapNode;
 #define GSI_MAP_RETAIN_VAL(M, X)\
  (M->legacy ? M->cb.old.v.retain(M, X.ptr) \
  : pointerFunctionsAcquire(&M->cb.pf.v, &X.ptr, X.ptr))
+
+#define GSI_MAP_WRITE_KEY(M, addr, x) \
+	if (M->legacy) \
+		*(addr) = x;\
+	else\
+	 pointerFunctionsAssign(&M->cb.pf.k, (void**)addr, (x).obj);
+#define GSI_MAP_WRITE_VAL(M, addr, x) \
+	if (M->legacy) \
+		*(addr) = x;\
+	else\
+	 pointerFunctionsAssign(&M->cb.pf.v, (void**)addr, (x).obj);
+#define GSI_MAP_READ_KEY(M,addr) \
+	(M->legacy ? *(addr) :\
+	 (typeof(*addr))pointerFunctionsRead(&M->cb.pf.k, (void**)addr))
+#define GSI_MAP_READ_VALUE(M,addr) \
+	(M->legacy ? *(addr) :\
+	 (typeof(*addr))pointerFunctionsRead(&M->cb.pf.v, (void**)addr))
 #define GSI_MAP_ZEROED(M)\
  (M->legacy ? 0 \
- : ((M->cb.pf.k.options & NSPointerFunctionsZeroingWeakMemory) ? YES : NO))
+ : (((M->cb.pf.k.options | M->cb.pf.v.options) & NSPointerFunctionsZeroingWeakMemory) ?\
+      YES : NO))
+
 
 #define	GSI_MAP_ENUMERATOR	NSMapEnumerator
 
@@ -500,7 +519,7 @@ NSEndMapTableEnumeration(NSMapEnumerator *enumerator)
        * 'bucket' field.
        */
       [(id)enumerator->node release];
-      memset(enumerator, '\0', sizeof(GSIMapEnumerator));
+      memset(enumerator, '\0', sizeof(NSMapEnumerator));
     }
 }
 
@@ -1183,6 +1202,7 @@ const NSMapTableValueCallBacks NSOwnedPointerMapValueCallBacks =
 
 - (NSUInteger) count
 {
+  GSIMapRemoveWeak(self);
   return (NSUInteger)nodeCount;
 }
 
@@ -1190,37 +1210,9 @@ const NSMapTableValueCallBacks NSOwnedPointerMapValueCallBacks =
 				   objects: (id*)stackbuf
 				     count: (NSUInteger)len
 {
-  NSInteger 		count;
-  NSMapEnumerator	enumerator;
-
-  state->mutationsPtr = (unsigned long *)version;
-  if (state->state == 0 && state->extra[0] == 0)
-    {
-      enumerator = GSIMapEnumeratorForMap(self);
-    }
-  else
-    {
-      enumerator.map = self;
-      enumerator.node = (GSIMapNode)(uintptr_t)state->state;
-      enumerator.bucket = state->extra[0];
-    }
-  for (count = 0; count < len; count++)
-    {
-      GSIMapNode	node = GSIMapEnumeratorNextNode(&enumerator);
-
-      if (node == 0)
-	{
-	  break;
-	}
-      else
-	{
-	  stackbuf[count] = node->key.obj;
-	}
-    }
-  state->state = (unsigned long)(uintptr_t)enumerator.node;
-  state->extra[0] = enumerator.bucket;
-  state->itemsPtr = stackbuf;
-  return count;
+  state->mutationsPtr = (unsigned long *)&version;
+  return GSIMapCountByEnumeratingWithStateObjectsCount
+    (self, state, stackbuf, len);
 }
 
 - (void) dealloc
