@@ -22,15 +22,16 @@
    Boston, MA 02111 USA.
 
    <title>NSCopyObject class reference</title>
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2011-02-20 05:28:12 -0700 (Sun, 20 Feb 2011) $ $Revision: 32243 $
    */
 
 #import "common.h"
 
 NSObject *NSCopyObject(NSObject *anObject, NSUInteger extraBytes, NSZone *zone)
 {
-  id copy = NSAllocateObject(((id)anObject)->class_pointer, extraBytes, zone);
-  memcpy(copy, anObject,
-    class_getInstanceSize(object_getClass(anObject)) + extraBytes);
+  Class	c = object_getClass(anObject);
+  id copy = NSAllocateObject(c, extraBytes, zone);
+
+  memcpy(copy, anObject, class_getInstanceSize(c) + extraBytes);
   return copy;
 }
