@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSCountedSet class reference</title>
-   $Date: 2010-02-25 18:49:31 +0000 (Thu, 25 Feb 2010) $ $Revision: 29753 $
+   $Date: 2011-07-24 07:09:22 -0600 (Sun, 24 Jul 2011) $ $Revision: 33621 $
    */
 
 #import "common.h"
@@ -43,7 +43,7 @@
 /*
  *	Class variables for uniquing objects;
  */
-static NSRecursiveLock	*uniqueLock = nil;
+static GSLazyRecursiveLock	*uniqueLock = nil;
 static NSCountedSet	*uniqueSet = nil;
 static IMP		uniqueImp = 0;
 static IMP		lockImp = 0;
@@ -73,7 +73,7 @@ static Class NSCountedSet_concrete_class;
     {
       NSCountedSet_abstract_class = self;
       NSCountedSet_concrete_class = [GSCountedSet class];
-      uniqueLock = [GSLazyLock new];
+      uniqueLock = [GSLazyRecursiveLock new];
       lockImp = [uniqueLock methodForSelector: @selector(lock)];
       unlockImp = [uniqueLock methodForSelector: @selector(unlock)];
     }
