@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSData class reference</title>
-   $Date: 2011-04-05 13:12:34 +0100 (Tue, 05 Apr 2011) $ $Revision: 32780 $
+   $Date: 2012-01-09 01:28:27 -0700 (Mon, 09 Jan 2012) $ $Revision: 34468 $
    */
 
 /* NOTES	-	Richard Frith-Macdonald 1997
@@ -68,6 +68,11 @@
  */
 
 #import "common.h"
+
+#if !defined (__GNU_LIBOBJC__)
+#  include <objc/encoding.h>
+#endif
+
 #import "GNUstepBase/GSObjCRuntime.h"
 #import "Foundation/NSByteOrder.h"
 #import "Foundation/NSCoder.h"
@@ -81,14 +86,16 @@
 #import "GSPrivate.h"
 #import "GNUstepBase/NSObject+GNUstepBase.h"
 #include <stdio.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>             /* SEEK_* on SunOS 4 */
-#endif
 
 #ifdef	HAVE_MMAP
-#include	<unistd.h>
-#include	<sys/mman.h>
-#include	<fcntl.h>
+#include <sys/mman.h>
+
+#if	defined(HAVE_SYS_FCNTL_H)
+#  include	<sys/fcntl.h>
+#elif	defined(HAVE_FCNTL_H)
+#  include	<fcntl.h>
+#endif
+
 #ifndef	MAP_FAILED
 #define	MAP_FAILED	((void*)-1)	/* Failure address.	*/
 #endif
@@ -311,7 +318,7 @@ failure:
 @interface	NSDataStatic : NSData
 {
   NSUInteger	length;
-  void		*bytes;
+  __strong void	*bytes;
 }
 @end
 
@@ -324,7 +331,7 @@ failure:
 @interface	NSMutableDataMalloc : NSMutableData
 {
   NSUInteger	length;
-  void		*bytes;
+  __strong void	*bytes;
 #if	GS_WITH_GC
   BOOL		owned;
 #else
@@ -570,11 +577,7 @@ failure:
 	    format: @"[%@-initWithBytes:length:] called with "
 	    @"length but null bytes", NSStringFromClass([self class])];
 	}
-#if	GS_WITH_GC
       ptr = NSAllocateCollectable(bufferSize, 0);
-#else
-      ptr = NSZoneMalloc(NSDefaultMallocZone(), bufferSize);
-#endif
       if (ptr == 0)
 	{
 	  DESTROY(self);
@@ -1267,7 +1270,7 @@ failure:
 {
   if ([coder allowsKeyedCoding])
     {
-      [coder encodeObject: self];
+      [coder encodeObject: self forKey: @"NS.bytes"];
     }
   else
     {
@@ -1277,12 +1280,11 @@ failure:
 
 - (id) initWithCoder: (NSCoder*)coder
 {
-  id	obj = nil;
+  id obj = nil;
 
   if ([coder allowsKeyedCoding])
     {
-      // FIXME
-      obj = [coder decodeObject];
+      obj = [coder decodeObjectForKey: @"NS.bytes"];
     }
   else
     {
@@ -2388,7 +2390,7 @@ failure:
 + (id) dataWithShmID: (int)anID length: (NSUInteger)length
 {
 #ifdef	HAVE_SHMCTL
-  NSDataShared	*d;
+  NSMutableDataShared	*d;
 
   d = [NSMutableDataShared allocWithZone: NSDefaultMallocZone()];
   d = [d initWithShmID: anID length: length];
@@ -2501,7 +2503,7 @@ failure:
 
 + (id) allocWithZone: (NSZone*)z
 {
-  return (NSData*)NSAllocateObject(self, 0, z);
+  return NSAllocateObject(self, 0, z);
 }
 
 /*	Creation and Destruction of objects.	*/
@@ -2978,7 +2980,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 @implementation	NSDataMappedFile
 + (id) allocWithZone: (NSZone*)z
 {
-  return (NSData*)NSAllocateObject([NSDataMappedFile class], 0, z);
+  return NSAllocateObject([NSDataMappedFile class], 0, z);
 }
 
 - (void) dealloc
@@ -3008,7 +3010,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
   int		fd;
 	
 #if defined(__MINGW__)
-  const unichar	*thePath = (const unichar*)[path filesystemRepresentation];
+  const unichar	*thePath = (const unichar*)[path fileSystemRepresentation];
 #else
   const char	*thePath = [path fileSystemRepresentation];
 #endif
@@ -3069,7 +3071,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 @implementation	NSDataShared
 + (id) allocWithZone: (NSZone*)z
 {
-  return (NSData*)NSAllocateObject([NSDataShared class], 0, z);
+  return NSAllocateObject([NSDataShared class], 0, z);
 }
 
 - (void) dealloc
@@ -3109,8 +3111,8 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
       shmid = shmget(IPC_PRIVATE, bufferSize, IPC_CREAT|VM_RDONLY);
       if (shmid == -1)			/* Created memory? */
 	{
-	  NSLog(@"[-initWithBytes:length:] shared mem get failed for %u - %@",
-	    bufferSize, [NSError _last]);
+	  NSLog(@"[-initWithBytes:length:] shared mem get failed for %"
+	    PRIuPTR" - %@", bufferSize, [NSError _last]);
 	  DESTROY(self);
 	  self = [dataMalloc allocWithZone: NSDefaultMallocZone()];
 	  return [self initWithBytes: aBuffer length: bufferSize];
@@ -3119,8 +3121,8 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
     bytes = shmat(shmid, 0, 0);
     if (bytes == (void*)-1)
       {
-	NSLog(@"[-initWithBytes:length:] shared mem attach failed for %u - %@",
-	  bufferSize, [NSError _last]);
+	NSLog(@"[-initWithBytes:length:] shared mem attach failed for %"
+	  PRIuPTR" - %@", bufferSize, [NSError _last]);
 	bytes = 0;
 	DESTROY(self);
 	self = [dataMalloc allocWithZone: NSDefaultMallocZone()];
@@ -3183,7 +3185,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 
 + (id) allocWithZone: (NSZone*)z
 {
-  return (NSData*)NSAllocateObject(mutableDataMalloc, 0, z);
+  return NSAllocateObject(mutableDataMalloc, 0, z);
 }
 
 - (Class) classForCoder
@@ -3299,7 +3301,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
       if (bytes == 0)
 	{
 	  NSLog(@"[NSMutableDataMalloc -initWithCapacity:] out of memory "
-	    @"for %u bytes - %@", size, [NSError _last]);
+	    @"for %"PRIuPTR" bytes - %@", size, [NSError _last]);
 	  DESTROY(self);
 	  return nil;
 	}
@@ -3796,7 +3798,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 @implementation	NSMutableDataShared
 + (id) allocWithZone: (NSZone*)z
 {
-  return (NSData*)NSAllocateObject([NSMutableDataShared class], 0, z);
+  return NSAllocateObject([NSMutableDataShared class], 0, z);
 }
 
 - (void) dealloc
@@ -3843,7 +3845,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
   if (shmid == -1)			/* Created memory? */
     {
       NSLog(@"[NSMutableDataShared -initWithCapacity:] shared memory "
-	@"get failed for %u - %@", bufferSize, [NSError _last]);
+	@"get failed for %"PRIuPTR" - %@", bufferSize, [NSError _last]);
       DESTROY(self);
       self = [mutableDataMalloc allocWithZone: NSDefaultMallocZone()];
       return [self initWithCapacity: bufferSize];
@@ -3853,7 +3855,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
   if (bytes == (void*)-1)
     {
       NSLog(@"[NSMutableDataShared -initWithCapacity:] shared memory "
-	@"attach failed for %u - %@", bufferSize, [NSError _last]);
+	@"attach failed for %"PRIuPTR" - %@", bufferSize, [NSError _last]);
       bytes = 0;
       DESTROY(self);
       self = [mutableDataMalloc allocWithZone: NSDefaultMallocZone()];
