@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDate class reference</title>
-   $Date: 2011-03-09 10:24:18 +0000 (Wed, 09 Mar 2011) $ $Revision: 32507 $
+   $Date: 2011-11-04 04:38:16 -0600 (Fri, 04 Nov 2011) $ $Revision: 34117 $
    */
 
 #import "common.h"
@@ -78,8 +78,8 @@ static Class	calendarClass = nil;
 @interface	GSDateFuture : GSDateSingle
 @end
 
-static NSDate	*_distantPast = nil;
-static NSDate	*_distantFuture = nil;
+static id _distantPast = nil;
+static id _distantFuture = nil;
 
 
 static NSString*
@@ -1226,15 +1226,20 @@ otherTime(NSDate* other)
 /**
  * Returns the time interval between the receivers value and that of the
  * otherDate argument.  If otherDate is earlier than the receiver, the
- * returned value will be positive, if it is later it will be negative.
+ * returned value will be positive, if it is later it will be negative.<br />
+ * For current (2011) OSX compatibility, this method returns NaN if otherDate
+ * is nil ... do not write code depending on that behavior.
  */
 - (NSTimeInterval) timeIntervalSinceDate: (NSDate*)otherDate
 {
   if (otherDate == nil)
+    return nan("");
+/*
     {
       [NSException raise: NSInvalidArgumentException
 		  format: @"nil argument for timeIntervalSinceDate:"];
     }
+*/
   return otherTime(self) - otherTime(otherDate);
 }
 
@@ -1518,7 +1523,7 @@ otherTime(NSDate* other)
   return self;
 }
 
-- (void) release
+- (oneway void) release
 {
 }
 
