@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDebug utilities reference</title>
-   $Date: 2011-03-07 15:34:06 +0000 (Mon, 07 Mar 2011) $ $Revision: 32485 $
+   $Date: 2011-12-15 02:42:39 -0700 (Thu, 15 Dec 2011) $ $Revision: 34290 $
    */
 
 #import "common.h"
@@ -66,7 +66,7 @@ static table_entry*	the_table = 0;
 
 static BOOL	debug_allocation = NO;
 
-static NSLock	*uniqueLock = nil;
+static GSLazyRecursiveLock	*uniqueLock = nil;
 
 static const char*	_GSDebugAllocationList(BOOL difference);
 static const char*	_GSDebugAllocationListAll(void);
@@ -856,7 +856,12 @@ __builtin_extract_return_address(__builtin_return_address(a + 1)) : 0); break;
  * Of course this will fail horribly if an exception occurs in one of the
  * few methods we use to manage the per-thread jump buffer.
  */
-#include <signal.h>
+#if	defined(HAVE_SYS_SIGNAL_H)
+#  include	<sys/signal.h>
+#elif	defined(HAVE_SIGNAL_H)
+#  include	<signal.h>
+#endif
+
 #include <setjmp.h>
 
 #if	defined(__MINGW__)
@@ -1188,7 +1193,7 @@ GSPrivateStackAddresses(void)
       signal(SIGBUS, env->bus);
     }
 #endif
-  [pool release];
+  [pool drain];
   return stack;
 }
 
