@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDictionary class reference</title>
-   $Date: 2011-02-23 11:52:17 +0000 (Wed, 23 Feb 2011) $ $Revision: 32323 $
+   $Date: 2011-10-24 08:33:30 -0600 (Mon, 24 Oct 2011) $ $Revision: 34049 $
    */
 
 #import "common.h"
@@ -41,6 +41,7 @@
 #import "Foundation/NSKeyedArchiver.h"
 #import "GNUstepBase/NSObject+GNUstepBase.h"
 #import "GSPrivate.h"
+#import "GSFastEnumeration.h"
 
 static BOOL GSMacOSXCompatiblePropertyLists(void)
 {
@@ -165,8 +166,8 @@ static SEL	appSel;
  * and needs to be re-implemented in subclasses in order to have all
  * other initialisers work.
  */
-- (id) initWithObjects: (id*)objects
-	       forKeys: (id*)keys
+- (id) initWithObjects: (const id[])objects
+	       forKeys: (const id[])keys
 		 count: (NSUInteger)count
 {
   self = [self init];
@@ -314,13 +315,22 @@ static SEL	appSel;
 {
   if ([aCoder allowsKeyedCoding])
     {
-      id	keys;
-      id	objects;
+      id keys = nil;
+      id objects = nil;
+
+      if ([aCoder containsValueForKey: @"NS.keys"])
+        {
+          keys = [(NSKeyedUnarchiver*)aCoder _decodeArrayOfObjectsForKey:
+                                        @"NS.keys"];
+          objects = [(NSKeyedUnarchiver*)aCoder _decodeArrayOfObjectsForKey:
+                                           @"NS.objects"];
+        }
+      else if ([aCoder containsValueForKey: @"dict.sortedKeys"])
+	{
+          keys = [aCoder decodeObjectForKey: @"dict.sortedKeys"];
+          objects = [aCoder decodeObjectForKey: @"dict.values"];
+        }
 
-      keys = [(NSKeyedUnarchiver*)aCoder _decodeArrayOfObjectsForKey:
-					       @"NS.keys"];
-      objects = [(NSKeyedUnarchiver*)aCoder _decodeArrayOfObjectsForKey:
-						  @"NS.objects"];
       if (keys == nil)
 	{
 	  unsigned	i = 0;
@@ -396,8 +406,8 @@ static SEL	appSel;
  * The n th element of the objects array is associated with the n th
  * element of the keys array.
  */
-+ (id) dictionaryWithObjects: (id*)objects
-		     forKeys: (id*)keys
++ (id) dictionaryWithObjects: (const id[])objects
+		     forKeys: (const id[])keys
 		       count: (NSUInteger)count
 {
   return AUTORELEASE([[self allocWithZone: NSDefaultMallocZone()]
@@ -814,6 +824,15 @@ static SEL	appSel;
       return AUTORELEASE(result);
     }
 }
+- (void)getObjects: (__unsafe_unretained id[])objects
+           andKeys: (__unsafe_unretained id[])keys
+{
+  int i=0;
+  FOR_IN(id, key, self)
+    keys[i] = key;
+    objects[i] = [self objectForKey: key];
+  END_FOR_IN(self)
+}
 
 /**
  * Returns an array containing all the dictionary's keys that are
@@ -1103,7 +1122,7 @@ compareIt(id o1, id o2, void* context)
   return o;
 }
 - (NSUInteger) countByEnumeratingWithState: (NSFastEnumerationState*)state 	
-                                   objects: (id*)stackbuf
+                                   objects: (__unsafe_unretained id[])stackbuf
                                      count: (NSUInteger)len
 {
     [self subclassResponsibility: _cmd];
@@ -1231,8 +1250,8 @@ compareIt(id o1, id o2, void* context)
  * The n th element of the objects array is associated with the n th
  * element of the keys array.
  */
-- (id) initWithObjects: (id*)objects
-	       forKeys: (id*)keys
+- (id) initWithObjects: (const id[])objects
+	       forKeys: (const id[])keys
 		 count: (NSUInteger)count
 {
   self = [self initWithCapacity: count];
