@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDistributedLock class reference</title>
-   $Date: 2010-02-25 18:49:31 +0000 (Thu, 25 Feb 2010) $ $Revision: 29753 $
+   $Date: 2011-12-15 02:42:39 -0700 (Thu, 15 Dec 2011) $ $Revision: 34290 $
    */
 
 #import "common.h"
@@ -34,7 +34,13 @@
 #import "Foundation/NSValue.h"
 #import "GSPrivate.h"
 
-#include <fcntl.h>
+
+#if	defined(HAVE_SYS_FCNTL_H)
+#  include	<sys/fcntl.h>
+#elif	defined(HAVE_FCNTL_H)
+#  include	<fcntl.h>
+#endif
+
 
 static NSFileManager	*mgr = nil;
 
