@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDistributedNotificationCenter class reference</title>
-   $Date: 2011-02-28 19:49:57 +0000 (Mon, 28 Feb 2011) $ $Revision: 32417 $
+   $Date: 2011-06-19 03:26:03 -0600 (Sun, 19 Jun 2011) $ $Revision: 33344 $
    */
 
 #import	"common.h"
@@ -38,6 +38,7 @@
 #import	"Foundation/NSRunLoop.h"
 #import	"Foundation/NSTask.h"
 #import	"GNUstepBase/NSTask+GNUstepBase.h"
+#import	"GNUstepBase/NSObject+GNUstepBase.h"
 #import	"Foundation/NSDistributedNotificationCenter.h"
 #import	"Foundation/NSUserDefaults.h"
 #import	"Foundation/NSHost.h"
@@ -151,7 +152,8 @@ static NSDistributedNotificationCenter	*netCenter = nil;
 		      NSAllocateObject(self, 0, NSDefaultMallocZone());
 		    tmp->_centerLock = [NSRecursiveLock new];
 		    tmp->_type = RETAIN(NSLocalNotificationCenterType);
-		    locCenter = tmp;
+		    locCenter = [NSObject leak: tmp];
+		    [tmp release];
 		  }
 		NS_HANDLER
 		  {
@@ -179,7 +181,8 @@ static NSDistributedNotificationCenter	*netCenter = nil;
 		      NSAllocateObject(self, 0, NSDefaultMallocZone());
 		    tmp->_centerLock = [NSRecursiveLock new];
 		    tmp->_type = RETAIN(GSPublicNotificationCenterType);
-		    pubCenter = tmp;
+		    pubCenter = [NSObject leak: tmp];
+		    [tmp release];
 		  }
 		NS_HANDLER
 		  {
@@ -207,7 +210,8 @@ static NSDistributedNotificationCenter	*netCenter = nil;
 		      NSAllocateObject(self, 0, NSDefaultMallocZone());
 		    tmp->_centerLock = [NSRecursiveLock new];
 		    tmp->_type = RETAIN(GSNetworkNotificationCenterType);
-		    netCenter = tmp;
+		    netCenter = [NSObject leak: tmp];
+		    [tmp release];
 		  }
 		NS_HANDLER
 		  {
@@ -743,7 +747,7 @@ static NSDistributedNotificationCenter	*netCenter = nil;
 		rootProxyForConnectionWithRegisteredName: service
 		host: host usingNameServer: ns];
               [_remote retain];
-              [pool release];
+              [pool drain];
 	    }
 	  if (_remote == nil)
 	    {
