@@ -21,7 +21,7 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2011-02-28 19:49:57 +0000 (Mon, 28 Feb 2011) $ $Revision: 32417 $
+   $Date: 2012-01-03 02:34:10 -0700 (Tue, 03 Jan 2012) $ $Revision: 34398 $
 */
 
 #import "common.h"
@@ -771,7 +771,7 @@ _NSFoundationUncaughtExceptionHandler (NSException *exception)
 	[[[exception _callStack] description] lossyCString]);
     }
   fflush(stderr);	/* NEEDED UNDER MINGW */
-  [pool release];
+  [pool drain];
   _terminate();
 }
 
@@ -947,7 +947,7 @@ callUncaughtHandler(id value)
         [super description], _e_name, _e_reason];
     }
   [result retain];
-  [pool release];
+  [pool drain];
   return [result autorelease];
 }
 
@@ -957,7 +957,11 @@ callUncaughtHandler(id value)
     {
       _reserved = NSZoneCalloc([self zone], 2, sizeof(id));
     }
-  _e_stack = [GSStackTrace new];
+  if (nil == _e_stack)
+    {
+      // Only set the stack when first raised
+      _e_stack = [GSStackTrace new];
+    }
 
 #if     defined(_NATIVE_OBJC_EXCEPTIONS)
   @throw self;
