@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSFileHandle class reference</title>
-   $Date: 2011-02-15 06:25:54 +0000 (Tue, 15 Feb 2011) $ $Revision: 32173 $
+   $Date: 2011-10-19 09:25:38 -0600 (Wed, 19 Oct 2011) $ $Revision: 34029 $
    */
 
 #import "common.h"
@@ -30,8 +30,10 @@
 #import "Foundation/NSData.h"
 #import "Foundation/NSFileHandle.h"
 #import "Foundation/NSPathUtilities.h"
-#import "GNUstepBase/GSFileHandle.h"
 #import "GNUstepBase/NSObject+GNUstepBase.h"
+#import "GSPrivate.h"
+#import "GSNetwork.h"
+#import "GSFileHandle.h"
 
 // GNUstep Notification names
 
@@ -728,7 +730,7 @@ NSString * const NSFileHandleOperationException
 /** <override-dummy />
  * Establishes an SSL connection from the system that the handle
  * is talking to.<br />
- * This is implented by an SSL handling subclass.<br />
+ * This is implemented by an SSL handling subclass.<br />
  * The default implementation just returns NO.
  */
 - (BOOL) sslAccept
@@ -739,7 +741,7 @@ NSString * const NSFileHandleOperationException
 /** <override-dummy />
  * Establishes an SSL connection to the system that the handle
  * is talking to.<br />
- * This is implented by an SSL handling subclass.<br />
+ * This is implemented by an SSL handling subclass.<br />
  * The default implementation just returns NO.
  */
 - (BOOL) sslConnect
@@ -755,7 +757,14 @@ NSString * const NSFileHandleOperationException
 }
 
 /** <override-dummy />
- * Sets the certificate to be used to identify this process to the server
+ */
+- (BOOL) sslHandshakeEstablished: (BOOL*)result outgoing: (BOOL)isOutgoing
+{
+  return NO;
+}
+
+/** <override-dummy />
+ * Sets the certificate chain to be used to identify this process to the server
  * at the opposite end of the network connection.
  */
 - (void) sslSetCertificate: (NSString*)certFile
@@ -763,5 +772,6 @@ NSString * const NSFileHandleOperationException
                  PEMpasswd: (NSString*)PEMpasswd
 {
 }
+
 @end
 
