@@ -34,7 +34,7 @@
    Boston, MA 02111 USA.
 
    <title>NSFileManager class reference</title>
-   $Date: 2011-04-07 06:09:08 +0100 (Thu, 07 Apr 2011) $ $Revision: 32794 $
+   $Date: 2012-01-26 12:59:58 -0700 (Thu, 26 Jan 2012) $ $Revision: 34637 $
 */
 
 #define _FILE_OFFSET_BITS 64
@@ -77,9 +77,6 @@
 # include <ndir.h>
 #endif
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #ifdef HAVE_WINDOWS_H
 #  include <windows.h>
 #endif
@@ -139,21 +136,24 @@
 # include <sys/statfs.h>
 #endif
 
-#ifdef HAVE_SYS_FILE_H
-#include <sys/file.h>
+#if	defined(HAVE_SYS_FILE_H)
+# include <sys/file.h>
 #endif
 
 #ifdef HAVE_SYS_MOUNT_H
 #include <sys/mount.h>
 #endif
 
-#include <errno.h>
-
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
 
-#include <fcntl.h>
+#if	defined(HAVE_SYS_FCNTL_H)
+#  include	<sys/fcntl.h>
+#elif	defined(HAVE_FCNTL_H)
+#  include	<fcntl.h>
+#endif
+
 #ifdef HAVE_PWD_H
 #include <pwd.h>     /* For struct passwd */
 #endif
@@ -681,7 +681,7 @@ static NSStringEncoding	defaultEncoding;
 	    {
 	      ok = [self contentsEqualAtPath: p1 andPath: p2];
 	    }
-	  RELEASE(pool);
+	  [pool drain];
 	}
       return ok;
     }
@@ -691,6 +691,11 @@ static NSStringEncoding	defaultEncoding;
     }
 }
 
+- (NSArray*) contentsOfDirectoryAtPath: (NSString*)path error: (NSError**)error
+{
+  return [self directoryContentsAtPath: path];
+}
+
 /**
  * Creates a new directory and all intermediate directories
  * if flag is YES, creates only the last directory in the path
@@ -1440,7 +1445,7 @@ static NSStringEncoding	defaultEncoding;
 	  item = [contents objectAtIndex: i];
 	  next = [path stringByAppendingPathComponent: item];
 	  result = [self removeFileAtPath: next handler: handler];
-	  [arp release];
+	  [arp drain];
 	  if (result == NO)
 	    {
 	      return NO;
@@ -1462,6 +1467,15 @@ static NSStringEncoding	defaultEncoding;
     }
 }
 
+- (BOOL) removeItemAtPath: (NSString*)path
+		    error: (NSError**)error
+{
+  BOOL  result;
+
+  result = [self removeFileAtPath: path handler: nil];
+  return result;
+}
+
 /**
  * Returns YES if a file (or directory etc) exists at the specified path.
  */
@@ -2641,11 +2655,11 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
 
 #else
   NSDictionary	*attributes;
-  int		i;
+  unsigned long long	fileSize;
+  unsigned long long	i;
   int		bufsize = 8096;
   int		sourceFd;
   int		destFd;
-  int		fileSize;
   int		fileMode;
   int		rbytes;
   int		wbytes;
@@ -2820,7 +2834,7 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
 	}
       [self changeFileAttributes: attributes atPath: destinationFile];
     }
-  [pool release];
+  [pool drain];
 
   return YES;
 }
@@ -2910,7 +2924,7 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
 	}
       [self changeFileAttributes: attributes atPath: destinationFile];
     }
-  [pool release];
+  [pool drain];
   return YES;
 #else
   return NO;
