@@ -23,11 +23,12 @@
 
    */
 
-#include        "Foundation/NSCoder.h"
-#include        "Foundation/NSData.h"
-#include	"Foundation/NSIndexSet.h"
-#include	"Foundation/NSException.h"
-#include	"Foundation/NSZone.h"
+#import "common.h"
+#define	EXPOSE_NSIndexSet_IVARS	1
+#import "Foundation/NSCoder.h"
+#import "Foundation/NSData.h"
+#import	"Foundation/NSIndexSet.h"
+#import	"Foundation/NSException.h"
 
 #define	GSI_ARRAY_TYPE	NSRange
 
@@ -52,7 +53,14 @@ static void sanity(GSIArray array)
 	{
 	  NSRange	r = GSIArrayItemAtIndex(array, i).ext;
 
-	  NSCAssert(r.location >= last, @"Overlap ranges");
+	  if (i > 0)
+	    {
+	      NSCAssert(r.location > last, @"Overlap or touching ranges");
+	    }
+	  else
+	    {
+	      NSCAssert(r.location >= last, @"Overlap ranges");
+	    }
 	  NSCAssert(NSMaxRange(r) > r.location, @"Bad range length");
 	  last = NSMaxRange(r);
 	}
@@ -1228,6 +1236,22 @@ static NSUInteger posForIndex(GSIArray array, NSUInteger index)
 		  GSIArraySetItemAtIndex(_array, (GSIArrayItem)r, c);
 		}
 	    }
+	  if (pos > 0)
+	    {
+	      c = GSIArrayCount(_array);
+	      if (pos < c)
+		{
+		  NSRange	r0 = GSIArrayItemAtIndex(_array, pos - 1).ext;
+		  NSRange	r1 = GSIArrayItemAtIndex(_array, pos).ext;
+
+	          if (NSMaxRange(r0) == r1.location)
+		    {
+		      r0.length += r1.length;
+		      GSIArraySetItemAtIndex(_array, (GSIArrayItem)r0, pos - 1);
+		      GSIArrayRemoveItemAtIndex(_array, pos);
+		    }
+		}
+	    }
 	}
     }
   SANITY();
@@ -1292,7 +1316,8 @@ static NSUInteger posForIndex(GSIArray array, NSUInteger index)
 
 - (id) _initWithBytes: (const void*)bytes length: (NSUInteger)length
 {
-  NSAssert(length % sizeof(NSRange) == 0, NSInvalidArgumentException);
+  NSAssert(length % sizeof(GSIArrayItem) == 0, NSInvalidArgumentException);
+  NSAssert(length % __alignof__(GSIArrayItem) == 0, NSInvalidArgumentException);
   length /= sizeof(NSRange);
   _data = NSZoneMalloc([self zone], sizeof(GSIArray_t));
   _array->ptr = (GSIArrayItem*)bytes;
