@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSInvocation class reference</title>
-   $Date: 2011-03-09 10:24:18 +0000 (Wed, 09 Mar 2011) $ $Revision: 32507 $
+   $Date: 2012-01-29 09:57:06 -0700 (Sun, 29 Jan 2012) $ $Revision: 34660 $
    */
 
 #import "common.h"
@@ -47,10 +47,6 @@
 #include <sys/mman.h>
 #endif
 
-#if     defined(HAVE_UNISTD_H)
-#import <unistd.h>
-#endif
-
 #if     defined(HAVE_MMAP)
 #  if   !defined(MAP_ANONYMOUS)
 #    if defined(MAP_ANON)
@@ -77,6 +73,9 @@
 {
   if (size > 0)
     {
+#if	defined(HAVE_FFI_PREP_CLOSURE_LOC)
+      ffi_closure_free(buffer);
+#else
 #if     defined(HAVE_MMAP)
       munmap(buffer, size);
 #else
@@ -88,14 +87,34 @@
 #endif
       NSDeallocateMemoryPages(buffer, NSPageSize());
 #endif
+#endif
+      buffer = 0;
+      executable = 0;
+      size = 0;
     }
   [super dealloc];
 }
 
+- (void*) executable
+{
+  return executable;
+}
+
 - (id) initWithSize: (NSUInteger)_size
 {
   NSAssert(_size > 0, @"Tried to allocate zero length buffer.");
   NSAssert(_size <= NSPageSize(), @"Tried to allocate more than one page.");
+#if	defined(HAVE_FFI_PREP_CLOSURE_LOC)
+  buffer = ffi_closure_alloc(_size, &executable);
+  if (0 == buffer)
+    {
+      executable = 0;
+    }  
+  else
+    {
+      size = _size;
+    }
+#else
 #if     defined(HAVE_MMAP)
 #if     defined(HAVE_MPROTECT)
   /* We have mprotect, so we create memory as writable and change it to
@@ -118,15 +137,19 @@
 
   if (buffer == (void*)0)
     {
-      NSLog(@"Failed to map %u bytes for execute: %@", _size, [NSError _last]);
+      NSLog(@"Failed to map %"PRIuPTR
+	" bytes for execute: %@", _size, [NSError _last]);
       buffer = 0;
+      executable = 0;
       [self dealloc];
       self = nil;
     }
   else
     {
+      executable = buffer;
       size = _size;
     }
+#endif	/* USE_LIBFFI */
   return self;
 }
 
@@ -135,7 +158,8 @@
  */
 - (void) protect
 {
-#if   defined(__MINGW__)
+#if	!defined(HAVE_FFI_PREP_CLOSURE_LOC)
+#if	defined(__MINGW__)
   DWORD old;
   if (VirtualProtect(buffer, size, PAGE_EXECUTE, &old) == 0)
     {
@@ -147,26 +171,13 @@
       NSLog(@"Failed to protect memory as executable: %@", [NSError _last]);
     }
 #endif
+#endif
 }
 @end
 
 static Class   NSInvocation_abstract_class;
 static Class   NSInvocation_concrete_class;
 
-@interface GSInvocationProxy
-{
-@public
-  Class		isa;
-  id		target;
-  NSInvocation	*invocation;
-}
-+ (id) _newWithTarget: (id)t;
-- (NSInvocation*) _invocation;
-- (void) forwardInvocation: (NSInvocation*)anInvocation;
-- (NSMethodSignature*) methodSignatureForSelector: (SEL)aSelector;
-@end
-@interface GSMessageProxy : GSInvocationProxy
-@end
 
 
 
@@ -788,26 +799,6 @@ _arg_addr(NSInvocation *inv, int index)
   return nil;
 }
 
-/**
- * Internal use.<br />
- * Provides a return frame that the ObjectiveC runtime can use to
- * return the result of an invocation to a calling function.
- */
-
-+ (id) _newProxyForInvocation: (id)target
-{
-  return [GSInvocationProxy _newWithTarget: target];
-}
-+ (id) _newProxyForMessage: (id)target
-{
-  return [GSMessageProxy _newWithTarget: target];
-}
-+ (NSInvocation*) _returnInvocationAndDestroyProxy: (id)proxy
-{
-  NSInvocation	*inv = [proxy _invocation];
-  NSDeallocateObject(proxy);
-  return inv;
-}
 @end
 
 @implementation NSInvocation (BackwardCompatibility)
@@ -849,34 +840,3 @@ _arg_addr(NSInvocation *inv, int index)
 @end
 #endif
 
-
-@implementation	GSInvocationProxy
-+ (id) _newWithTarget: (id)t
-{
-  GSInvocationProxy	*o;
-  o = (GSInvocationProxy*) NSAllocateObject(self, 0, NSDefaultMallocZone());
-  o->target = RETAIN(t);
-  return o;
-}
-- (NSInvocation*) _invocation
-{
-  return invocation;
-}
-- (void) forwardInvocation: (NSInvocation*)anInvocation
-{
-  invocation = anInvocation;
-}
-- (NSMethodSignature*) methodSignatureForSelector: (SEL)aSelector
-{
-  return [target methodSignatureForSelector: aSelector];
-}
-@end
-
-@implementation	GSMessageProxy
-- (NSInvocation*) _invocation
-{
-  [invocation setTarget: target];
-  return invocation;
-}
-@end
-
