@@ -21,7 +21,7 @@
    Boston, MA 02111 USA.
 
    <title>NSKeyValueCoding informal protocol reference</title>
-   $Date: 2011-03-09 10:32:48 +0000 (Wed, 09 Mar 2011) $ $Revision: 32510 $
+   $Date: 2012-01-30 04:31:40 -0700 (Mon, 30 Jan 2012) $ $Revision: 34665 $
    */
 
 #import "common.h"
@@ -604,7 +604,7 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
     {
       SEL		sel = 0;
       const char	*type = NULL;
-      int		off;
+      int		off = 0;
       const char	*name;
       char		key[size + 1];
       char		buf[size + 5];
@@ -711,6 +711,7 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
 
       name = buf;	// _setKey:
       type = NULL;
+      off = 0;
       sel = sel_getUid(name);
       if (sel == 0 || [self respondsToSelector: sel] == NO)
 	{
@@ -802,7 +803,7 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
 {
   SEL		sel = 0;
   const char	*type = 0;
-  int		off;
+  int		off = 0;
   unsigned	size = [aKey length] * 8;
   char		key[size + 1];
 
