@@ -21,7 +21,7 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2011-03-09 10:42:16 +0000 (Wed, 09 Mar 2011) $ $Revision: 32512 $
+   $Date: 2011-07-24 07:09:22 -0600 (Sun, 24 Jul 2011) $ $Revision: 33621 $
 */
 
 #import "common.h"
@@ -174,7 +174,7 @@ setup()
 @interface	GSKVOInfo : NSObject
 {
   NSObject	        *instance;	// Not retained.
-  NSLock	        *iLock;
+  GSLazyRecursiveLock	        *iLock;
   NSMapTable	        *paths;
 }
 - (GSKVOPathInfo *) lockReturningPathInfoForKey: (NSString *)key;
