@@ -259,11 +259,12 @@ static NSMapTable	*globalClassMap = 0;
       GSIArraySetItemAtIndex(_objMap, (GSIArrayItem)obj, index);
     }
 
-  if (obj == nil)
+  if ((obj == nil) || [@"$null" isEqual: obj])
     {
       // Record NilMarker for decoded object.
       o = GSIArrayItemAtIndex(_objMap, 0).obj;
       GSIArraySetItemAtIndex(_objMap, (GSIArrayItem)o, index);
+      obj = nil;
     }
 
   return obj;
@@ -380,7 +381,8 @@ static NSMapTable	*globalClassMap = 0;
 			 count: (NSUInteger)expected
 			    at: (void*)buf
 {
-  id	 o = [self decodeObject];
+  id	 	o = [self decodeObject];
+  NSUInteger	size;
 
   if ([o isKindOfClass: [_NSKeyedCoderOldStyleArray class]] == NO)
     {
@@ -400,7 +402,8 @@ static NSMapTable	*globalClassMap = 0;
 		  format: @"[%@ +%@]: count missmatch",
 	NSStringFromClass([self class]), NSStringFromSelector(_cmd), o];
     }
-  memcpy(buf, [o bytes], expected * objc_sizeof_type(type));
+  NSGetSizeAndAlignment(type, 0, &size);
+  memcpy(buf, [o bytes], expected * size);
 }
 
 - (BOOL) decodeBoolForKey: (NSString*)aKey
@@ -498,8 +501,8 @@ static NSMapTable	*globalClassMap = 0;
   if (i > INT_MAX || i < INT_MIN)
     {
       [NSException raise: NSRangeException
-	          format: @"[%@ +%@]: value for key(%@) is out of range",
-	NSStringFromClass([self class]), NSStringFromSelector(_cmd), aKey];
+	format: @"[%@ +%@]: value %"PRIu64" for key(%@) is out of range",
+        NSStringFromClass([self class]), NSStringFromSelector(_cmd), i, aKey];
     }
 #endif
   return (int)i;
@@ -531,8 +534,8 @@ static NSMapTable	*globalClassMap = 0;
   if (i > INT32_MAX || i < INT32_MIN)
     {
       [NSException raise: NSRangeException
-	          format: @"[%@ +%@]: value for key(%@) is out of range",
-	NSStringFromClass([self class]), NSStringFromSelector(_cmd), aKey];
+	format: @"[%@ +%@]: value %"PRIu64" for key(%@) is out of range",
+	NSStringFromClass([self class]), NSStringFromSelector(_cmd), i, aKey];
     }
   return (int32_t)i;
 }
