@@ -25,10 +25,10 @@
 #import "common.h"
 #include <pthread.h>
 #import "GNUstepBase/GSConfig.h"
+#import "GSPrivate.h"
 #define	gs_cond_t	pthread_cond_t
 #define	gs_mutex_t	pthread_mutex_t
 #include <math.h>
-#include <errno.h>
 
 #define	EXPOSE_NSLock_IVARS	1
 #define	EXPOSE_NSRecursiveLock_IVARS	1
@@ -106,7 +106,7 @@
 	  return YES;\
 	}\
       sched_yield();\
-    } while([limit timeIntervalSinceNow] < 0);\
+    } while ([limit timeIntervalSinceNow] > 0);\
   return NO;\
 }
 #define	MTRYLOCK \
@@ -219,7 +219,7 @@ MLOCK
 	  _NSLockError(self, _cmd, NO);
 	}
       sched_yield();
-    } while([limit timeIntervalSinceNow] < 0);
+    } while ([limit timeIntervalSinceNow] > 0);
   return NO;
 }
 
