@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSLog reference</title>
-   $Date: 2011-02-28 19:49:57 +0000 (Mon, 28 Feb 2011) $ $Revision: 32417 $
+   $Date: 2011-10-14 09:46:13 -0600 (Fri, 14 Oct 2011) $ $Revision: 33993 $
    */
 
 #import "common.h"
@@ -62,11 +62,6 @@
 # endif
 #endif	// HAVE_SYSLOG
 
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #import "GSPrivate.h"
 
 extern NSThread	*GSCurrentThread();
@@ -357,6 +352,6 @@ NSLogv (NSString* format, va_list args)
 
   [myLock unlock];
 
-  [arp release];
+  [arp drain];
 }
 
