@@ -44,30 +44,36 @@
 #import "Foundation/NSFileManager.h"
 #import "Foundation/NSProcessInfo.h"
 
-#import "GSNetwork.h"
 #import "GSPrivate.h"
+#import "GSNetwork.h"
 #import "GSPortPrivate.h"
 
 #include <stdio.h>
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #include <sys/param.h>		/* for MAXHOSTNAMELEN */
 #include <sys/types.h>
 #include <sys/un.h>
 #include <arpa/inet.h>		/* for inet_ntoa() */
-#include <errno.h>
 #include <string.h>		/* for strchr() */
 #include <ctype.h>		/* for strchr() */
-#include <fcntl.h>
+
+#if	defined(HAVE_SYS_FCNTL_H)
+#  include	<sys/fcntl.h>
+#elif	defined(HAVE_FCNTL_H)
+#  include	<fcntl.h>
+#endif
+
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <netdb.h>
 #include <sys/socket.h>
-#include <sys/file.h>
+
+#if	defined(HAVE_SYS_FILE_H)
+#  include	<sys/file.h>
+#endif
+
 #include <sys/stat.h>
+
 /*
  *	Stuff for setting the sockets into non-blocking mode.
  */
@@ -381,7 +387,7 @@ static Class	runLoopClass;
 
   if (connect(desc, (struct sockaddr*)&sockAddr, SUN_LEN(&sockAddr)) < 0)
     {
-      if (errno != EINPROGRESS)
+      if (!GSWOULDBLOCK)
 	{
 	  NSLog(@"unable to make connection to %s - %@",
 	    sockAddr.sun_path, [NSError _last]);
@@ -1100,7 +1106,7 @@ static void clean_up_sockets(void)
 	unlink([name bytes]);
     }
   NSEndMapTableEnumeration(&mEnum);
-  [arp release];
+  [arp drain];
   if (unknownThread == YES)
     {
       GSUnregisterCurrentThread();
@@ -1236,7 +1242,7 @@ typedef	struct {
       ((internal*)(port->_internal))->_name = theName;
       ((internal*)(port->_internal))->_listener = -1;
       ((internal*)(port->_internal))->_handles
-	= NSCreateMapTable(NSIntMapKeyCallBacks,
+	= NSCreateMapTable(NSIntegerMapKeyCallBacks,
 	NSObjectMapValueCallBacks, 0);
       ((internal*)(port->_internal))->_myLock = [GSLazyRecursiveLock new];
       port->_is_valid = YES;
@@ -1696,7 +1702,7 @@ typedef	struct {
     }
 }
 
-- (void) release
+- (oneway void) release
 {
   M_LOCK(messagePortLock);
   if (NSDecrementExtraRefCountWasZero(self))
@@ -1786,7 +1792,7 @@ typedef	struct {
   rl = [self reservedSpaceLength];
   if (length != 0 && length != rl)
     {
-      NSLog(@"bad reserved length - %u", length);
+      NSLog(@"bad reserved length - %"PRIuPTR, length);
       return NO;
     }
   if ([receivingPort isKindOfClass: messagePortClass] == NO)
