@@ -20,7 +20,7 @@
    Boston, MA 02111 USA.
 
    <title>NSMessagePortNameServer class reference</title>
-   $Date: 2011-03-07 15:34:06 +0000 (Mon, 07 Mar 2011) $ $Revision: 32485 $
+   $Date: 2011-12-15 02:42:39 -0700 (Thu, 15 Dec 2011) $ $Revision: 34290 $
    */
 
 #import "common.h"
@@ -40,9 +40,20 @@
 #import "GSPortPrivate.h"
 
 #include <sys/stat.h>
-#include <unistd.h>
-#include <signal.h>
-#include <fcntl.h>
+
+#if	defined(HAVE_SYS_SIGNAL_H)
+#  include	<sys/signal.h>
+#elif	defined(HAVE_SIGNAL_H)
+#  include	<signal.h>
+#endif
+
+
+#if	defined(HAVE_SYS_FCNTL_H)
+#  include	<sys/fcntl.h>
+#elif	defined(HAVE_FCNTL_H)
+#  include	<fcntl.h>
+#endif
+
 #include <string.h>
 #include <sys/un.h>
 
@@ -99,7 +110,7 @@ static void clean_up_names(void)
       [defaultServer removePort: port];
     }
   NSEndMapTableEnumeration(&mEnum);
-  [arp release];
+  [arp drain];
   if (unknownThread == YES)
     {
       GSUnregisterCurrentThread();
