@@ -24,13 +24,15 @@
    Boston, MA 02111 USA.
 
    <title>NSMethodSignature class reference</title>
-   $Date: 2011-03-07 15:34:06 +0000 (Mon, 07 Mar 2011) $ $Revision: 32485 $
+   $Date: 2011-05-25 05:15:08 -0600 (Wed, 25 May 2011) $ $Revision: 33109 $
    */
 
 #import "common.h"
-#ifndef __GNU_LIBOBJC__
-#include <objc/encoding.h>
+
+#if !defined (__GNU_LIBOBJC__)
+#  include <objc/encoding.h>
 #endif
+
 #define	EXPOSE_NSMethodSignature_IVARS	1
 
 #import "Foundation/NSMethodSignature.h"
@@ -439,9 +441,9 @@ next_arg(const char *typePtr, NSArgumentInfo *info, char *outTypes)
 	  p = objc_skip_type_qualifiers (p);
 	}
       alen = ptr - args;
-      sprintf(ret + rlen, "%d", (int)_argFrameLength);
+      rlen += sprintf(ret + rlen, "%d", (int)_argFrameLength);
 
-      _methodTypes = NSZoneMalloc(NSDefaultMallocZone(), alen + rlen + 1);
+      _methodTypes = NSAllocateCollectable(alen + rlen + 1, 0);
       strncpy((char*)_methodTypes, ret, rlen);
       strncpy(((char*)_methodTypes) + rlen, args, alen);
       ((char*)_methodTypes)[alen + rlen] = '\0';
