@@ -26,7 +26,7 @@
    Boston, MA 02111 USA.
 
    <title>NSNotificationCenter class reference</title>
-   $Date: 2011-06-20 13:52:55 +0100 (Mon, 20 Jun 2011) $ $Revision: 33361 $
+   $Date: 2012-01-30 04:31:40 -0700 (Mon, 30 Jan 2012) $ $Revision: 34665 $
 */
 
 #import "common.h"
@@ -36,6 +36,7 @@
 #import "Foundation/NSLock.h"
 #import "Foundation/NSThread.h"
 #import "GNUstepBase/GSLock.h"
+#import "GNUstepBase/NSObject+GNUstepBase.h"
 
 static NSZone	*_zone = 0;
 
@@ -139,8 +140,31 @@ struct	NCTbl;		/* Notification Center Table structure	*/
  * If 'next' is 0 then the observation is unused (ie it has been
  * removed from, or not yet added to  any list).  The end of a
  * list is marked by 'next' being set to 'ENDOBS'.
+ *
+ * This is normally a structure which handles memory management using a fast
+ * reference count mechanism, but when built with clang for GC, a structure
+ * can't hold a zeroing weak pointer to an observer so it's implemented as a
+ * trivial class instead ... and gets managed by the garbage collector.
  */
 
+#ifdef __OBJC_GC__
+
+@interface	GSObservation : NSObject
+{
+  @public
+  __weak id	observer;	/* Object to receive message.	*/
+  SEL		selector;	/* Method selector.		*/
+  IMP		method;		/* Method implementation.	*/
+  struct Obs	*next;		/* Next item in linked list.	*/
+  struct NCTbl	*link;		/* Pointer back to chunk table	*/
+}
+@end
+@implementation	GSObservation
+@end
+#define	Observation	GSObservation
+
+#else
+
 typedef	struct	Obs {
   id		observer;	/* Object to receive message.	*/
   SEL		selector;	/* Method selector.		*/
@@ -150,6 +174,8 @@ typedef	struct	Obs {
   struct NCTbl	*link;		/* Pointer back to chunk table	*/
 } Observation;
 
+#endif
+
 #define	ENDOBS	((Observation*)-1)
 
 static inline unsigned doHash(NSString* key)
@@ -188,14 +214,35 @@ static inline BOOL doEqual(NSString* key1, NSString* key2)
  * Setup for inline operation on arrays of Observers.
  */
 static void listFree(Observation *list);
+
+#ifdef __OBJC_GC__
+
+/* Observations are managed by the GC system because they need to be
+ * instances of a class in order to implement weak pointer to observer.
+ */
+#define	obsRetain(X)
+#define	obsFree(X)
+
+#else
+
+/* Observations have retain/release counts managed explicitly by fast
+ * function calls.
+ */
 static void obsRetain(Observation *o);
 static void obsFree(Observation *o);
 
+#endif
+
 
 #define GSI_ARRAY_TYPES	0
 #define GSI_ARRAY_TYPE	Observation*
+#ifdef __OBJC_GC__
+#define GSI_ARRAY_NO_RELEASE	1
+#define GSI_ARRAY_NO_RETAIN	1
+#else
 #define GSI_ARRAY_RELEASE(A, X)   obsFree(X.ext)
 #define GSI_ARRAY_RETAIN(A, X)    obsRetain(X.ext)
+#endif
 
 #include "GNUstepBase/GSIArray.h"
 
@@ -221,8 +268,6 @@ static GC_descr	nodeDesc;	// Type descriptor for map node.
 
 #include "GNUstepBase/GSIMap.h"
 
-@class	GSLazyRecursiveLock;
-
 /*
  * An NC table is used to keep track of memory allocated to store
  * Observation structures. When an Observation is removed from the
@@ -251,7 +296,7 @@ typedef struct NCTbl {
   GSIMapTable		nameless;	/* Get messages for any name.	*/
   GSIMapTable		named;		/* Getting named messages only.	*/
   unsigned		lockCount;	/* Count recursive operations.	*/
-  GSLazyRecursiveLock	*_lock;		/* Lock out other threads.	*/
+  NSRecursiveLock	*_lock;		/* Lock out other threads.	*/
   Observation		*freeList;
   Observation		**chunks;
   unsigned		numChunks;
@@ -266,10 +311,35 @@ typedef struct NCTbl {
 #define	NAMED		(TABLE->named)
 #define	LOCKCOUNT	(TABLE->lockCount)
 
-static Observation *obsNew(NCTable* t)
+static Observation *
+obsNew(NCTable *t, SEL s, IMP m, id o)
 {
   Observation	*obs;
 
+#if __OBJC_GC__
+
+  /* With clang GC, observations are garbage collected and we don't
+   * use a cache.  However, because the reference to the observer must be
+   * weak, the observation has to be an instance of a class ...
+   */
+  static Class observationClass;
+
+  if (0 == observationClass)
+    {
+      observationClass = [GSObservation class];
+    }
+  obs = NSAllocateObject(observationClass, 0, _zone);
+
+#else
+
+  /* Generally, observations are cached and we create a 'new' observation
+   * by retrieving from the cache or by allocating a block of observations
+   * in one go.  This works nicely to both hide observations from the
+   * garbage collector (when using gcc for GC) and to provide high
+   * performance for situations where apps add/remove lots of observers
+   * very frequently (poor design, but something which happens in the
+   * real world unfortunately).
+   */
   if (t->freeList == 0)
     {
       Observation	*block;
@@ -281,22 +351,12 @@ static Observation *obsNew(NCTable* t)
 	  t->numChunks++;
 
 	  size = t->numChunks * sizeof(Observation*);
-#if	GS_WITH_GC
 	  t->chunks = (Observation**)NSReallocateCollectable(
 	    t->chunks, size, NSScannedOption);
-#else
-	  t->chunks = (Observation**)NSZoneRealloc(NSDefaultMallocZone(),
-	    t->chunks, size);
-#endif
 
 	  size = CHUNKSIZE * sizeof(Observation);
-#if	GS_WITH_GC
 	  t->chunks[t->numChunks - 1]
 	    = (Observation*)NSAllocateCollectable(size, 0);
-#else
-	  t->chunks[t->numChunks - 1]
-	    = (Observation*)NSZoneMalloc(NSDefaultMallocZone(), size);
-#endif
 	  t->chunkIndex = 0;
 	}
       block = t->chunks[t->numChunks - 1];
@@ -307,22 +367,32 @@ static Observation *obsNew(NCTable* t)
   obs = t->freeList;
   t->freeList = (Observation*)obs->link;
   obs->link = (void*)t;
+  obs->retained = 0;
+  obs->next = 0;
+#endif
+
+  obs->selector = s;
+  obs->method = m;
+#if	GS_WITH_GC
+  GSAssignZeroingWeakPointer((void**)&obs->observer, (void*)o);
+#else
+  obs->observer = o;
+#endif
+
   return obs;
 }
 
 static GSIMapTable	mapNew(NCTable *t)
 {
   if (t->cacheIndex > 0)
-    return t->cache[--t->cacheIndex];
+    {
+      return t->cache[--t->cacheIndex];
+    }
   else
     {
       GSIMapTable	m;
 
-#if	GS_WITH_GC
       m = NSAllocateCollectable(sizeof(GSIMapTable_t), NSScannedOption);
-#else
-      m = NSZoneMalloc(_zone, sizeof(GSIMapTable_t));
-#endif
       GSIMapInitWithZoneAndCapacity(m, _zone, 2);
       return m;
     }
@@ -348,6 +418,8 @@ static void endNCTable(NCTable *t)
   GSIMapNode		n0;
   Observation		*l;
 
+  TEST_RELEASE(t->_lock);
+
   /*
    * Free observations without notification names or numbers.
    */
@@ -403,34 +475,21 @@ static void endNCTable(NCTable *t)
     }
   NSZoneFree(NSDefaultMallocZone(), t->chunks);
   NSZoneFree(NSDefaultMallocZone(), t);
-
-  TEST_RELEASE(t->_lock);
 }
 
 static NCTable *newNCTable(void)
 {
   NCTable	*t;
 
-#if	GS_WITH_GC
   t = (NCTable*)NSAllocateCollectable(sizeof(NCTable), NSScannedOption);
-#else
-  t = (NCTable*)NSZoneMalloc(_zone, sizeof(NCTable));
-  memset((void*)t, '\0', sizeof(NCTable));
-#endif
   t->chunkIndex = CHUNKSIZE;
   t->wildcard = ENDOBS;
 
-#if	GS_WITH_GC
   t->nameless = NSAllocateCollectable(sizeof(GSIMapTable_t), NSScannedOption);
   t->named = NSAllocateCollectable(sizeof(GSIMapTable_t), NSScannedOption);
-#else
-  t->nameless = NSZoneMalloc(_zone, sizeof(GSIMapTable_t));
-  t->named = NSZoneMalloc(_zone, sizeof(GSIMapTable_t));
-#endif
   GSIMapInitWithZoneAndCapacity(t->nameless, _zone, 16);
   GSIMapInitWithZoneAndCapacity(t->named, _zone, 128);
 
-  // t->_lock = [GSLazyRecursiveLock new];
   t->_lock = [NSRecursiveLock new];
   return t;
 }
@@ -447,6 +506,7 @@ static inline void unlockNCTable(NCTable* t)
   [t->_lock unlock];
 }
 
+#ifndef __OBJC_GC__
 static void obsFree(Observation *o)
 {
   NSCAssert(o->retained >= 0, NSInternalInconsistencyException);
@@ -462,6 +522,12 @@ static void obsFree(Observation *o)
     }
 }
 
+static void obsRetain(Observation *o)
+{
+  o->retained++;
+}
+#endif
+
 static void listFree(Observation *list)
 {
   while (list != ENDOBS)
@@ -515,11 +581,6 @@ static Observation *listPurge(Observation *list, id observer)
   return list;
 }
 
-static void obsRetain(Observation *o)
-{
-  o->retained++;
-}
-
 /*
  * Utility function to remove all the observations from a particular
  * map table node that match the specified observer.  If the observer
@@ -624,6 +685,14 @@ purgeCollectedFromMapNode(GSIMapTable map, GSIMapNode node)
 
 static NSNotificationCenter *default_center = nil;
 
++ (void) atExit
+{
+  id	tmp = default_center;
+
+  default_center = nil;
+  [tmp release];
+}
+
 + (void) initialize
 {
   if (self == [NSNotificationCenter class])
@@ -648,6 +717,7 @@ static NSNotificationCenter *default_center = nil;
        */
       default_center = [self alloc];
       [default_center init];
+      [self registerAtExit];
     }
 }
 
@@ -751,16 +821,7 @@ static NSNotificationCenter *default_center = nil;
 
   lockNCTable(TABLE);
 
-  o = obsNew(TABLE);
-  o->selector = selector;
-  o->method = method;
-#if	GS_WITH_GC
-  GSAssignZeroingWeakPointer((void**)&o->observer, (void*)observer);
-#else
-  o->observer = observer;
-#endif
-  o->retained = 0;
-  o->next = 0;
+  o = obsNew(TABLE, selector, method, observer);
 
   if (object != nil)
     {
@@ -1215,9 +1276,9 @@ static NSNotificationCenter *default_center = nil;
   GSNotification	*notification;
 
   notification = (id)NSAllocateObject(concrete, 0, NSDefaultMallocZone());
-  name = notification->_name = [name copyWithZone: [self zone]];
-  object = notification->_object = TEST_RETAIN(object);
-  notification->_info = TEST_RETAIN(info);
+  notification->_name = [name copyWithZone: [self zone]];
+  notification->_object = [object retain];
+  notification->_info = [info retain];
   [self _postAndRelease: notification];
 }
 
