@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSNull class reference</title>
-   $Date: 2010-02-19 08:12:46 +0000 (Fri, 19 Feb 2010) $ $Revision: 29669 $
+   $Date: 2011-07-31 09:31:39 -0600 (Sun, 31 Jul 2011) $ $Revision: 33660 $
    */
 
 #import "common.h"
@@ -104,7 +104,7 @@ static NSNull	*null = 0;
     return NO;
 }
 
-- (void) release
+- (oneway void) release
 {
 }
 
