@@ -47,6 +47,9 @@
 #import "Foundation/NSException.h"
 #import "Foundation/NSValue.h"
 #import "GNUstepBase/NSObject+GNUstepBase.h"
+#if __has_include(<objc/runtime.h>)
+#  include <objc/runtime.h>
+#endif
 
 /*
  * NSNumber implementation.  This matches the behaviour of Apple's
@@ -345,6 +348,230 @@ return NSOrderedSame;
 #include "NSNumberMethods.h"
 @end
 
+#ifdef OBJC_SMALL_OBJECT_SHIFT
+static BOOL useSmallInt;
+static BOOL useSmallExtendedDouble;
+static BOOL useSmallRepeatingDouble;
+#define SMALL_INT_MASK 1
+#define SMALL_EXTENDED_DOUBLE_MASK 2
+#define SMALL_REPEATING_DOUBLE_MASK 3
+
+
+@interface NSSmallInt : NSSignedIntegerNumber
+@end
+
+@implementation NSSmallInt
+#undef VALUE
+#define VALUE (((intptr_t)self) >> OBJC_SMALL_OBJECT_SHIFT)
+#if OBJC_SMALL_OBJECT_SHIFT == 1
+#define FORMAT @"%d"
+#else
+#define FORMAT @"%lld"
+#endif
+#include "NSNumberMethods.h"
+
++ (void) load
+{
+  useSmallInt = objc_registerSmallObjectClass_np(self, SMALL_INT_MASK);
+}
+
++ (id) alloc
+{
+  return (id)1;
+}
+
++ (id) allocWithZone: (NSZone*)aZone
+{
+  return (id)1;
+}
+
+- (id) copy
+{
+  return self;
+}
+
+- (id) copyWithZone: (NSZone*)aZone
+{
+  return self;
+}
+
+- (id) retain
+{
+  return self;
+}
+
+- (id) autorelease
+{
+  return self;
+}
+
+- (oneway void) release
+{
+  return;
+}
+@end
+
+#if OBJC_SMALL_OBJECT_SHIFT == 3
+
+union BoxedDouble
+{
+  id obj;
+  uintptr_t bits;
+  double d;
+};
+
+@interface NSSmallExtendedDouble : NSFloatingPointNumber
+@end
+
+@implementation NSSmallExtendedDouble
+static double
+unboxSmallExtendedDouble(uintptr_t boxed)
+{
+  // The low bit of the mantissa
+  uintptr_t mask = boxed & 8;
+  union BoxedDouble ret;
+  // Clear the class pointer
+  boxed &= ~7;
+  ret.bits = boxed | (mask >> 1) | (mask >> 2) | (mask >> 3);
+  return ret.d;
+}
+
+static BOOL
+isSmallExtendedDouble(double d)
+{
+  union BoxedDouble b = {.d=d};
+  return unboxSmallExtendedDouble(b.bits) == d;
+}
+
+static double
+unboxSmallRepeatingDouble(uintptr_t boxed)
+{
+  // The low bit of the mantissa
+  uintptr_t mask = boxed & 56;
+  union BoxedDouble ret;
+  // Clear the class pointer
+  boxed &= ~7;
+  ret.bits = boxed | (mask >> 3);
+  return ret.d;
+}
+
+static BOOL
+isSmallRepeatingDouble(double d)
+{
+  union BoxedDouble b = {.d=d};
+  return unboxSmallRepeatingDouble(b.bits) == d;
+}
+
+static id
+boxDouble(double d, uintptr_t mask)
+{
+  union BoxedDouble b = {.d=d};
+  b.bits &= ~OBJC_SMALL_OBJECT_MASK;
+  b.bits |= mask;
+  return b.obj;
+}
+
+#undef VALUE
+#define VALUE (unboxSmallExtendedDouble((uintptr_t)self))
+#define FORMAT @"%0.16g"
+#include "NSNumberMethods.h"
+
++ (void)
+load
+{
+  useSmallExtendedDouble = objc_registerSmallObjectClass_np
+    (self, SMALL_EXTENDED_DOUBLE_MASK);
+}
+
++ (id) alloc
+{
+  return (id)SMALL_EXTENDED_DOUBLE_MASK;
+}
+
++ (id) allocWithZone: (NSZone*)aZone
+{
+  return (id)SMALL_EXTENDED_DOUBLE_MASK;
+}
+
+- (id) copy
+{
+  return self;
+}
+
+- (id) copyWithZone: (NSZone*)aZone
+{
+  return self;
+}
+
+- (id) retain
+{
+  return self;
+}
+
+- (id) autorelease
+{
+  return self;
+}
+
+- (oneway void) release
+{
+  return;
+}
+@end
+
+@interface NSSmallRepeatingDouble : NSFloatingPointNumber
+@end
+
+@implementation NSSmallRepeatingDouble
+#undef VALUE
+#define VALUE (unboxSmallRepeatingDouble((uintptr_t)self))
+#define FORMAT @"%0.16g"
+#include "NSNumberMethods.h"
+
++ (void) load
+{
+  useSmallRepeatingDouble = objc_registerSmallObjectClass_np
+    (self, SMALL_REPEATING_DOUBLE_MASK);
+}
+
++ (id) alloc
+{
+  return (id)SMALL_REPEATING_DOUBLE_MASK;
+}
+
++ (id) allocWithZone: (NSZone*)aZone
+{
+  return (id)SMALL_REPEATING_DOUBLE_MASK;
+}
+
+- (id) copy
+{
+  return self;
+}
+
+- (id) copyWithZone: (NSZone*)aZone
+{
+  return self;
+}
+
+- (id) retain
+{
+  return self;
+}
+
+- (id) autorelease
+{
+  return self;
+}
+
+- (oneway void) release
+{
+  return;
+}
+@end
+#endif
+#endif
+
 @implementation NSNumber
 
 static Class NSNumberClass;
@@ -537,6 +764,14 @@ if (aValue >= -1 && aValue <= 12)\
     }
 
   CHECK_SINGLETON (aValue);
+#ifdef OBJC_SMALL_OBJECT_SHIFT
+  if (useSmallInt &&
+      (aValue < (INT_MAX>>OBJC_SMALL_OBJECT_SHIFT)) &&
+      (aValue > -(INT_MAX>>OBJC_SMALL_OBJECT_SHIFT)))
+    {
+       return (id)((aValue << OBJC_SMALL_OBJECT_SHIFT) | SMALL_INT_MASK);
+    }
+#endif
   n = NSAllocateObject (NSIntNumberClass, 0, 0);
   n->value = aValue;
   return AUTORELEASE(n);
@@ -624,6 +859,12 @@ if (aValue >= -1 && aValue <= 12)\
       return [[[self alloc] initWithBytes: (const void *)&aValue
         objCType: @encode(float)] autorelease];
     }
+#if OBJC_SMALL_OBJECT_SHIFT == 3
+  if (useSmallRepeatingDouble)
+    {
+      return boxDouble(aValue, SMALL_REPEATING_DOUBLE_MASK);
+    }
+#endif
   n = NSAllocateObject (NSFloatNumberClass, 0, 0);
   n->value = aValue;
   return AUTORELEASE(n);
@@ -638,6 +879,16 @@ if (aValue >= -1 && aValue <= 12)\
       return [[[self alloc] initWithBytes: (const void *)&aValue
         objCType: @encode(double)] autorelease];
     }
+#if OBJC_SMALL_OBJECT_SHIFT == 3
+  if (useSmallRepeatingDouble && isSmallRepeatingDouble(aValue))
+    {
+      return boxDouble(aValue, SMALL_REPEATING_DOUBLE_MASK);
+    }
+  if (useSmallExtendedDouble && isSmallExtendedDouble(aValue))
+    {
+      return boxDouble(aValue, SMALL_EXTENDED_DOUBLE_MASK);
+    }
+#endif
   n = NSAllocateObject (NSDoubleNumberClass, 0, 0);
   n->value = aValue;
   return AUTORELEASE(n);
@@ -724,7 +975,7 @@ if (aValue >= -1 && aValue <= 12)\
 - (void) encodeWithCoder: (NSCoder *) coder
 {
   const char *type = [self objCType];
-  char buffer[16];
+  char buffer[16] __attribute__ ((aligned (16)));
 
   [coder encodeValueOfObjCType: @encode (char) at: type];
   /* The most we currently store in an NSNumber is 8 bytes (double or long
@@ -745,7 +996,7 @@ if (aValue >= -1 && aValue <= 12)\
 - (id) initWithCoder: (NSCoder *) coder
 {
   char type[2] = { 0 };
-  char buffer[16];
+  char buffer[16] __attribute__ ((aligned (16)));
 
   [coder decodeValueOfObjCType: @encode (char) at: type];
   [coder decodeValueOfObjCType: type at: buffer];
@@ -757,12 +1008,16 @@ if (aValue >= -1 && aValue <= 12)\
   return [self stringValue];
 }
 
-/* Return nil for an NSNumber that is allocated and initalized without
+/* Return nil for an NSNumber that is allocated and initialized without
  * providing a real value.  Yes, this seems weird, but it is actually what
  * happens on OS X.
  */
 - (id) init
 {
+  if (object_getClass(self) != NSNumberClass)
+    {
+      return [super init];
+    }
   DESTROY(self);
   return nil;
 }
