@@ -24,9 +24,27 @@
    Boston, MA 02111 USA.
 
    <title>NSNumberFormatter class reference</title>
-   $Date: 2011-05-12 19:18:25 +0100 (Thu, 12 May 2011) $ $Revision: 33030 $
+   $Date: 2012-01-25 18:40:12 -0700 (Wed, 25 Jan 2012) $ $Revision: 34634 $
    */
 
+/* Unfortunately, libicu does not define the maximum values allowed for all
+   of these attributes.  We define them here to, though.
+   These are based off libicu version 4.6.
+ */
+#define MAX_SYMBOLS 27
+#define MAX_TEXTATTRIBUTES 8
+#define MAX_ATTRIBUTES 20
+
+#define GS_NSNumberFormatter_IVARS \
+  NSUInteger	_behavior; \
+  BOOL		_genDecimal; \
+  NSUInteger	_style; \
+  NSLocale	*_locale; \
+  void		*_formatter; \
+  id		_symbols[MAX_SYMBOLS]; \
+  id		_textAttributes[MAX_TEXTATTRIBUTES]; \
+  int		_attributes[MAX_ATTRIBUTES]
+
 #import "common.h"
 #define	EXPOSE_NSNumberFormatter_IVARS	1
 #import "Foundation/NSAttributedString.h"
@@ -51,14 +69,6 @@
 
 #define BUFFER_SIZE 1024
 
-/* Unfortunately, libicu does not define the maximum values allowed for all
-   of these attributes.  We define them here to, though.
-   These are based off libicu version 4.6.
- */
-#define MAX_SYMBOLS 27
-#define MAX_TEXTATTRIBUTES 8
-#define MAX_ATTRIBUTES 20
-
 #if GS_USE_ICU == 1
 static inline UNumberFormatStyle
 NSToICUFormatStyle (NSNumberFormatterStyle style)
@@ -199,6 +209,8 @@ ICUToNSRoundingMode (UNumberFormatRoundingMode mode)
       case UNUM_ROUND_HALFUP:
         result = NSNumberFormatterRoundHalfUp;
         break;
+      default:
+        result = NSNumberFormatterRoundHalfUp;
     }
   
   return result;
@@ -252,22 +264,16 @@ ICUToNSRoundingMode (UNumberFormatRoundingMode mode)
 #define	UNUM_ZERO_DIGIT_SYMBOL	0
 
 #endif
-
-#define GS_NSNumberFormatter_IVARS \
-  NSUInteger	_behavior; \
-  BOOL		_genDecimal; \
-  NSUInteger	_style; \
-  NSLocale	*_locale; \
-  void		*_formatter; \
-  id		_symbols[MAX_SYMBOLS]; \
-  id		_textAttributes[MAX_TEXTATTRIBUTES]; \
-  int		_attributes[MAX_ATTRIBUTES]
  
 #define GSInternal              NSNumberFormatterInternal
 #include        "GSInternal.h"
 GS_PRIVATE_INTERNAL(NSNumberFormatter)
 
+#if GS_NONFRAGILE
+@interface NSNumberFormatter (Internal)
+#else
 @interface	NSNumberFormatterInternal (Methods)
+#endif
 - (int32_t) attributeForKey: (int)key;
 - (NSString*) symbolForKey: (int)key;
 - (NSString*) textAttributeForKey: (int)key;
@@ -276,7 +282,11 @@ GS_PRIVATE_INTERNAL(NSNumberFormatter)
 - (void) setTextAttribute: (NSString*)value forKey: (int)key;
 @end
 
+#if GS_NONFRAGILE
+@implementation NSNumberFormatter (Internal)
+#else
 @implementation	NSNumberFormatterInternal (Methods)
+#endif
 
 - (int32_t) attributeForKey: (int)key
 {
@@ -437,6 +447,7 @@ static NSUInteger _defaultBehavior = NSNumberFormatterBehavior10_4;
 - (NSAttributedString*) attributedStringForObjectValue: (id)anObject
 				 withDefaultAttributes: (NSDictionary*)attr
 {
+  NSString *stringForObjectValue;
   NSDecimalNumber *zeroNumber = [NSDecimalNumber zero];
   NSDecimalNumber *nanNumber = [NSDecimalNumber notANumber];
 
@@ -468,8 +479,15 @@ static NSUInteger _defaultBehavior = NSNumberFormatterBehavior10_4;
       attr = _attributesForNegativeValues;
     }
 
+  stringForObjectValue = [self stringForObjectValue: anObject];
+  
+  if (stringForObjectValue == nil)
+    {
+      stringForObjectValue = @"";
+    }
+
   return AUTORELEASE([[NSAttributedString alloc] initWithString:
-    [self stringForObjectValue: anObject] attributes: attr]);
+     stringForObjectValue attributes: attr]);
 }
 
 - (NSAttributedString*) attributedStringForNil
@@ -1037,20 +1055,15 @@ static NSUInteger _defaultBehavior = NSNumberFormatterBehavior10_4;
 
 #define STRING_FROM_NUMBER(function, number) do \
   { \
-    UChar *outStr = buffer; \
     UErrorCode err = U_ZERO_ERROR; \
     int32_t len; \
     NSString *result; \
     \
-    len = function (internal->_formatter, number, outStr, \
+    len = function (internal->_formatter, number, buffer, \
       BUFFER_SIZE, NULL, &err); \
     if (len > BUFFER_SIZE) \
-      outStr = NSZoneMalloc ([self zone], len * sizeof(UChar));\
-    err = U_ZERO_ERROR; \
-    function (internal->_formatter, number, outStr, BUFFER_SIZE, NULL, &err); \
-    result = [NSString stringWithCharacters: outStr length: len]; \
-    if (len > BUFFER_SIZE) \
-      NSZoneFree ([self zone], outStr); \
+      len = BUFFER_SIZE; \
+    result = [NSString stringWithCharacters: buffer length: len]; \
     return result; \
   } while (0)
 
@@ -1087,9 +1100,11 @@ static NSUInteger _defaultBehavior = NSNumberFormatterBehavior10_4;
            * don't think it matters, because we don't bother with anything
            * smaller than int for NSNumbers
 	   */
+#if	defined(_C_BOOL)
           case _C_BOOL:
             STRING_FROM_NUMBER(unum_format, (int)[anObject boolValue]);
             break;
+#endif
           /* If it's not a type encoding that we recognise, let the receiver
            * cast it to a double, which probably has enough precision for what
            * we need.  This needs testing with NSDecimalNumber though, because
@@ -1260,7 +1275,7 @@ static NSUInteger _defaultBehavior = NSNumberFormatterBehavior10_4;
         {
           while (([placeHolders characterIsMember:
             [useFormat characterAtIndex: NSMaxRange(intPartRange)]]
-            || [[useFormat substringFromRange:
+            || [[useFormat substringWithRange:
               NSMakeRange(NSMaxRange(intPartRange), 1)] isEqual:
           defaultThousandsSeparator])
             && NSMaxRange(intPartRange) < [useFormat length] - 1)
