@@ -22,10 +22,15 @@
    Boston, MA 02111 USA.
 
    <title>NSObjCRuntime class reference</title>
-   $Date: 2011-02-07 22:53:35 +0000 (Mon, 07 Feb 2011) $ $Revision: 32012 $
+   $Date: 2011-05-19 02:19:24 -0600 (Thu, 19 May 2011) $ $Revision: 33069 $
    */
 
 #import "common.h"
+
+#if !defined (__GNU_LIBOBJC__)
+#  include <objc/encoding.h>
+#endif
+
 #import "Foundation/NSException.h"
 #include <string.h>
 
