@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSObject class reference</title>
-   $Date: 2011-06-18 13:57:44 +0100 (Sat, 18 Jun 2011) $ $Revision: 33339 $
+   $Date: 2011-12-15 02:42:39 -0700 (Thu, 15 Dec 2011) $ $Revision: 34290 $
    */
 
 /* On some versions of mingw we need to work around bad function declarations
@@ -47,15 +47,15 @@
 #import "Foundation/NSNotification.h"
 #import "Foundation/NSMapTable.h"
 #import "GNUstepBase/GSLocale.h"
+#import "GNUstepBase/NSObject+GNUstepBase.h"
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
 
-#ifdef	HAVE_SIGNAL_H
-#include	<signal.h>
-#endif
-#ifdef	HAVE_SYS_SIGNAL_H
-#include	<sys/signal.h>
+#if	defined(HAVE_SYS_SIGNAL_H)
+#  include	<sys/signal.h>
+#elif	defined(HAVE_SIGNAL_H)
+#  include	<signal.h>
 #endif
 
 #if __GNUC__ >= 4
@@ -64,10 +64,22 @@
 #endif
 #endif // __GNUC__
 
+#ifdef __OBJC_GC__
+#include <objc/objc-auto.h>
+#endif
+
 #define	IN_NSOBJECT_M	1
 #import "GSPrivate.h"
 
 
+#ifdef __GNUSTEP_RUNTIME__
+#include <objc/capabilities.h>
+#include <objc/hooks.h>
+#ifdef OBJC_CAP_ARC
+#include <objc/objc-arc.h>
+#endif
+#endif
+
 /* When this is `YES', every call to release/autorelease, checks to
    make sure isn't being set up to release itself too many times.
    This does not need mutex protection. */
@@ -105,7 +117,7 @@ static Class	NSConstantStringClass;
 - (NSMethodSignature*) methodSignatureForSelector: (SEL)aSelector;
 @end
 
-@interface GSContentAccessingProxy : NSProxy 
+@interface GSContentAccessingProxy : NSProxy
 {
   NSObject<NSDiscardableContent> *object;
 }
@@ -113,7 +125,7 @@ static Class	NSConstantStringClass;
 @end
 
 /*
- * allocationLock is needed when running multi-threaded for 
+ * allocationLock is needed when running multi-threaded for
  * protecting the map table of zombie information.
  */
 static NSLock *allocationLock;
@@ -132,7 +144,7 @@ static void GSMakeZombie(NSObject *o)
 
   c = object_getClass(o);
   object_setClass(o, zombieClass);
-  if (NSDeallocateZombies == NO)
+  if (0 != zombieMap)
     {
       [allocationLock lock];
       NSMapInsert(zombieMap, (void*)o, (void*)c);
@@ -145,7 +157,7 @@ static void GSLogZombie(id o, SEL sel)
 {
   Class	c = 0;
 
-  if (NSDeallocateZombies == NO)
+  if (0 != zombieMap)
     {
       [allocationLock lock];
       c = NSMapGet(zombieMap, (void*)o);
@@ -190,6 +202,7 @@ static void GSLogZombie(id o, SEL sel)
 #undef	GSATOMICREAD
 #endif
 
+
 #if	defined(__MINGW__)
 #ifndef _WIN64
 #undef InterlockedIncrement
@@ -209,13 +222,13 @@ typedef int32_t volatile *gsatomic_t;
 #define	GSAtomicDecrement(X)	InterlockedDecrement((LONG volatile*)X)
 
 
-#elif defined(__llvm__) || (defined(USE_ATOMIC_BUILDINS) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 1)))
+#elif defined(__llvm__) || (defined(USE_ATOMIC_BUILTINS) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 1)))
 /* Use the GCC atomic operations with recent GCC versions */
 
 typedef int32_t volatile *gsatomic_t;
 #define GSATOMICREAD(X) (*(X))
-#define GSAtomicIncrement(X)    __sync_fetch_and_add(X, 1)
-#define GSAtomicDecrement(X)    __sync_fetch_and_sub(X, 1)
+#define GSAtomicIncrement(X)    __sync_add_and_fetch(X, 1)
+#define GSAtomicDecrement(X)    __sync_sub_and_fetch(X, 1)
 
 
 #elif	defined(__linux__) && (defined(__i386__) || defined(__x86_64__))
@@ -384,11 +397,14 @@ static inline NSLock *GSAllocationLockForObject(id p)
 
 #endif
 
-
 #ifdef ALIGN
 #undef ALIGN
 #endif
-#define	ALIGN __alignof__(double)
+#if defined(__GNUC__) && __GNUC__ < 4
+#define __builtin_offsetof(s, f) (uintptr_t)(&(((s*)0)->f))
+#endif
+#define alignof(type) __builtin_offsetof(struct { const char c; type member; }, member)
+#define	ALIGN alignof(double)
 
 /*
  *	Define a structure to hold information that is held locally
@@ -396,7 +412,6 @@ static inline NSLock *GSAllocationLockForObject(id p)
  */
 typedef struct obj_layout_unpadded {
     NSUInteger	retained;
-    NSZone	*zone;
 } unp;
 #define	UNP sizeof(unp)
 
@@ -406,13 +421,32 @@ typedef struct obj_layout_unpadded {
  *	structure correct.
  */
 struct obj_layout {
-    NSUInteger	retained;
-    NSZone	*zone;
     char	padding[ALIGN - ((UNP % ALIGN) ? (UNP % ALIGN) : ALIGN)];
+    NSUInteger	retained;
 };
 typedef	struct obj_layout *obj;
 
+#ifdef __OBJC_GC__
 
+/**
+ * If -base is compiled in GC mode, then we want to still support manual
+ * reference counting if we are linked with non-GC code.  
+ */
+static BOOL GSDecrementExtraRefCountWasZero(id anObject);
+
+BOOL
+NSDecrementExtraRefCountWasZero(id anObject)
+{
+  if (!objc_collecting_enabled())
+    {
+      return GSDecrementExtraRefCountWasZero(anObject);
+    }
+  return NO;
+}
+
+
+static BOOL GSDecrementExtraRefCountWasZero(id anObject)
+#else
 /**
  * Examines the extra reference count for the object and, if non-zero
  * decrements it, otherwise leaves it unchanged.<br />
@@ -422,6 +456,7 @@ typedef	struct obj_layout *obj;
  */
 BOOL
 NSDecrementExtraRefCountWasZero(id anObject)
+#endif
 {
 #if	!GS_WITH_GC
   if (double_release_check_enabled)
@@ -497,6 +532,12 @@ NSDecrementExtraRefCountWasZero(id anObject)
 inline NSUInteger
 NSExtraRefCount(id anObject)
 {
+#ifdef __OBJC_GC__
+  if (objc_collecting_enabled())
+    {
+      return UINT_MAX-1;
+    }
+#endif
 #if	GS_WITH_GC
   return UINT_MAX - 1;
 #else	/* GS_WITH_GC */
@@ -504,6 +545,25 @@ NSExtraRefCount(id anObject)
 #endif /* GS_WITH_GC */
 }
 
+#ifdef __OBJC_GC__
+
+/**
+ * If -base is compiled in GC mode, then we want to still support manual
+ * reference counting if we are linked with non-GC code.  
+ */
+static void GSIncrementExtraRefCount(id anObject);
+
+inline void NSIncrementExtraRefCount(id anObject)
+{
+  if (!objc_collecting_enabled())
+    {
+      GSIncrementExtraRefCount(anObject);
+    }
+}
+
+
+static void GSIncrementExtraRefCount(id anObject)
+#else
 /**
  * Increments the extra reference count for anObject.<br />
  * The GNUstep version raises an exception if the reference count
@@ -512,8 +572,9 @@ NSExtraRefCount(id anObject)
  */
 inline void
 NSIncrementExtraRefCount(id anObject)
+#endif
 {
-#if	GS_WITH_GC
+#if	GS_WITH_GC || __OBJC_GC__
   return;
 #else	/* GS_WITH_GC */
   if (allocationLock != 0)
@@ -559,8 +620,8 @@ NSIncrementExtraRefCount(id anObject)
 #define	AADD(c, o) GSDebugAllocationAdd(c, o)
 #define	AREM(c, o) GSDebugAllocationRemove(c, o)
 #else
-#define	AADD(c, o) 
-#define	AREM(c, o) 
+#define	AADD(c, o)
+#define	AREM(c, o)
 #endif
 
 static SEL cxx_construct, cxx_destruct;
@@ -572,7 +633,7 @@ static SEL cxx_construct, cxx_destruct;
  * -.cxx_destruct.  The -.cxx_construct methods must be called in order from
  *  the root class to all subclasses, to ensure that subclass ivars are
  *  initialised after superclass ones.  This must be done in reverse for
- *  destruction.  
+ *  destruction.
  *
  *  This function first calls itself recursively on the superclass, to get the
  *  IMP for the constructor function in the superclass.  It then compares the
@@ -702,17 +763,47 @@ NSDeallocateObject(id anObject)
 
 #else	/* GS_WITH_GC */
 
+#if !__OBJC_GC__
 inline NSZone *
 GSObjCZone(NSObject *object)
 {
   GSOnceFLog(@"GSObjCZone() is deprecated ... use -zone instead");
   if (object_getClass(object) == NSConstantStringClass)
     return NSDefaultMallocZone();
-  return ((obj)object)[-1].zone;
+  return NSZoneFromPointer(object);
 }
+#endif
+
+#if __OBJC_GC__
+inline NSZone *
+GSObjCZone(NSObject *object)
+{
+  return NSDefaultMallocZone();
+}
+static inline id
+GSAllocateObject (Class aClass, NSUInteger extraBytes, NSZone *zone);
 
+inline id
+NSAllocateObject(Class aClass, NSUInteger extraBytes, NSZone *zone)
+{
+  if (!objc_collecting_enabled())
+    {
+      GSAllocateObject(aClass, extraBytes, zone);
+    }
+  id	new = class_createInstance(aClass, extraBytes);
+  if (0 == cxx_construct)
+    {
+      cxx_construct = sel_registerName(".cxx_construct");
+      cxx_destruct = sel_registerName(".cxx_destruct");
+    }
+  return new;
+}
+inline id
+GSAllocateObject (Class aClass, NSUInteger extraBytes, NSZone *zone)
+#else
 inline id
 NSAllocateObject (Class aClass, NSUInteger extraBytes, NSZone *zone)
+#endif
 {
   id	new;
   int	size;
@@ -727,7 +818,6 @@ NSAllocateObject (Class aClass, NSUInteger extraBytes, NSZone *zone)
   if (new != nil)
     {
       memset (new, 0, size);
-      ((obj)new)->zone = zone;
       new = (id)&((obj)new)[1];
       object_setClass(new, aClass);
       AADD(aClass, new);
@@ -748,16 +838,29 @@ NSAllocateObject (Class aClass, NSUInteger extraBytes, NSZone *zone)
 
   return new;
 }
+#if __OBJC_GC__
+static void GSDeallocateObject(id anObject);
 
+inline void NSDeallocateObject(id anObject)
+{
+  if (!objc_collecting_enabled())
+    {
+      GSDeallocateObject(anObject);
+    }
+}
+
+static void GSDeallocateObject(id anObject)
+#else
 inline void
 NSDeallocateObject(id anObject)
+#endif
 {
   Class aClass = object_getClass(anObject);
 
   if ((anObject != nil) && !class_isMetaClass(aClass))
     {
       obj	o = &((obj)anObject)[-1];
-      NSZone	*z = o->zone;
+      NSZone	*z = NSZoneFromPointer(o);
 
       /* Call the default finalizer to handle C++ destructors.
        */
@@ -786,7 +889,10 @@ NSDeallocateObject(id anObject)
 BOOL
 NSShouldRetainWithZone (NSObject *anObject, NSZone *requestedZone)
 {
-#if	GS_WITH_GC
+#if	GS_WITH_GC || __OBJC_GC__
+  // If we're running in hybrid mode, we disable all of the clever zone stuff
+  // for non-GC code, so this is always true if we're compiled for GC, even if
+  // we're compiled for GC but not using GC.
   return YES;
 #else
   return (!requestedZone || requestedZone == NSDefaultMallocZone()
@@ -856,6 +962,7 @@ NSShouldRetainWithZone (NSObject *anObject, NSZone *requestedZone)
  * </p>
  */
 @implementation NSObject
+- (void)_ARCCompliantRetainRelease {}
 
 + (void) _becomeMultiThreaded: (NSNotification *)aNotification
 {
@@ -891,11 +998,21 @@ GSGarbageCollectorLog(char *msg, GC_word arg)
  * Semi-private function in libobjc2 that initialises the classes used for
  * blocks.
  */
-extern BOOL 
+extern BOOL
 objc_create_block_classes_as_subclasses_of(Class super);
 
+#ifdef OBJC_CAP_ARC
+static id gs_weak_load(id obj)
+{
+	return [obj retainCount] > 0 ? obj : nil;
+}
+#endif
+
 + (void) load
 {
+#ifdef OBJC_CAP_ARC
+  _objc_weak_load = gs_weak_load;
+#endif
   objc_create_block_classes_as_subclasses_of(self);
 }
 
@@ -914,8 +1031,8 @@ objc_create_block_classes_as_subclasses_of(Class super);
 #ifdef __MINGW__
       {
         // See libgnustep-base-entry.m
-        extern void gnustep_base_socket_init(void);	
-        gnustep_base_socket_init();	
+        extern void gnustep_base_socket_init(void);
+        gnustep_base_socket_init();
       }
 #else /* __MINGW__ */
 
@@ -1002,6 +1119,14 @@ objc_create_block_classes_as_subclasses_of(Class super);
       GSObjCBehaviorDebug(GSPrivateEnvironmentFlag("GNUSTEP_BEHAVIOR_DEBUG",
 	GSObjCBehaviorDebug(-1)));
 
+      /* See if we should cleanup at process exit.
+       */
+      if (YES == GSPrivateEnvironmentFlag("GNUSTEP_SHOULD_CLEAN_UP", NO))
+	{
+	  [self setShouldCleanUp: YES];
+	  [self registerAtExit: @selector(_atExit)];
+	}
+
       /* Set up the autorelease system ... we must do this before using any
        * other class whose +initialize might autorelease something.
        */
@@ -1013,6 +1138,7 @@ objc_create_block_classes_as_subclasses_of(Class super);
        * string constants etc.
        */
       NSConstantStringClass = [NSString constantStringClass];
+
       GSPrivateBuildStrings();
 
       /* Determine zombie management flags and set up a map to store
@@ -1042,6 +1168,11 @@ objc_create_block_classes_as_subclasses_of(Class super);
   return;
 }
 
++ (void) _atExit
+{
+  DESTROY(zombieMap);
+}
+
 /**
  * Allocates a new instance of the receiver from the default
  * zone, by invoking +allocWithZone: with
@@ -1290,8 +1421,12 @@ objc_create_block_classes_as_subclasses_of(Class super);
 
 - (void) finalize
 {
-  Class	destructorClass = object_getClass(self);
+  Class	destructorClass = Nil;
   IMP	destructor = 0;
+#ifdef OBJC_SMALL_OBJECT_MASK
+  if (((NSUInteger)self & OBJC_SMALL_OBJECT_MASK) == 0)
+#endif
+  destructorClass = isa;
 
   /* C++ destructors must be called in the opposite order to their
    * creators, so start at the leaf class and then go up the tree until we
@@ -1307,12 +1442,13 @@ objc_create_block_classes_as_subclasses_of(Class super);
 
       newDestructor
 	= class_getMethodImplementation(destructorClass, cxx_destruct);
+      destructorClass = class_getSuperclass(destructorClass);
+
       if (newDestructor != destructor)
-	{
+	  {
 	  newDestructor(self, cxx_destruct);
 	  destructor = newDestructor;
 	}
-      destructorClass = class_getSuperclass(destructorClass);
     }
   return;
 }
@@ -1701,7 +1837,7 @@ objc_create_block_classes_as_subclasses_of(Class super);
  */
 - (id) autorelease
 {
-#if	GS_WITH_GC == 0
+#if	!GS_WITH_GC
   if (double_release_check_enabled)
     {
       NSUInteger release_count;
@@ -1913,9 +2049,12 @@ objc_create_block_classes_as_subclasses_of(Class super);
  */
 - (oneway void) release
 {
-#if	GS_WITH_GC == 0
+#if	(GS_WITH_GC == 0)
   if (NSDecrementExtraRefCountWasZero(self))
     {
+#  ifdef OBJC_CAP_ARC
+      objc_delete_weak_refs(self);
+#  endif
       [self dealloc];
     }
 #endif
@@ -1963,7 +2102,7 @@ objc_create_block_classes_as_subclasses_of(Class super);
  */
 - (id) retain
 {
-#if	GS_WITH_GC == 0
+#if	(GS_WITH_GC == 0)
   NSIncrementExtraRefCount(self);
 #endif
   return self;
@@ -2020,15 +2159,22 @@ objc_create_block_classes_as_subclasses_of(Class super);
  */
 - (NSZone*) zone
 {
-#if	GS_WITH_GC
+#if	GS_WITH_GC || __OBJC_GC__
   /* MacOS-X 10.5 seems to return the default malloc zone if GC is enabled.
    */
   return NSDefaultMallocZone();
 #else
-  return (((obj)self)[-1]).zone;
+  return NSZoneFromPointer(self);
 #endif
 }
 
+#if	!GS_WITH_GC && !__OBJC_GC__
++ (NSZone *) zone
+{
+  return NSDefaultMallocZone();
+}
+#endif
+
 /**
  * Called to encode the instance variables of the receiver to aCoder.<br />
  * Subclasses should call the superclass method at the start of their
@@ -2164,21 +2310,6 @@ objc_create_block_classes_as_subclasses_of(Class super);
   return [self instanceMethodForSelector:aSel];
 }
 
-+ (NSMethodSignature*) instanceMethodSignatureForSelector: (SEL)aSelector
-{
-  struct objc_method* mth;
-
-  if (aSelector == 0)
-    [NSException raise: NSInvalidArgumentException
-		format: @"%@ null selector given", NSStringFromSelector(_cmd)];
-
-  mth = GSGetMethod(self, aSelector, YES, YES);
-  if (0 == mth)
-    return nil;
-  return [NSMethodSignature
-    signatureWithObjCTypes: method_getTypeEncoding(mth)];
-}
-
 - (IMP) methodFor: (SEL)aSel
 {
   return [self methodForSelector: aSel];
@@ -2353,7 +2484,7 @@ objc_create_block_classes_as_subclasses_of(Class super);
 }
 - (Class) originalClass
 {
-  return NSMapGet(zombieMap, (void*)self);
+  return zombieMap ? NSMapGet(zombieMap, (void*)self) : Nil;
 }
 - (void) forwardInvocation: (NSInvocation*)anInvocation
 {
@@ -2374,13 +2505,13 @@ objc_create_block_classes_as_subclasses_of(Class super);
       return nil;
     }
   [allocationLock lock];
-  c = NSMapGet(zombieMap, (void*)self);
+  c = zombieMap ? NSMapGet(zombieMap, (void*)self) : Nil;
   [allocationLock unlock];
   return [c instanceMethodSignatureForSelector: aSelector];
 }
 @end
 
-@implementation GSContentAccessingProxy 
+@implementation GSContentAccessingProxy
 - (void) dealloc
 {
   [object endContentAccess];
