@@ -39,6 +39,7 @@
   BOOL concurrent; \
   BOOL executing; \
   BOOL finished; \
+  BOOL blocked; \
   BOOL ready; \
   NSMutableArray *dependencies;
 
@@ -47,10 +48,11 @@
   NSConditionLock	*cond; \
   NSMutableArray	*operations; \
   NSMutableArray	*waiting; \
+  NSMutableArray	*starting; \
   NSString		*name; \
   BOOL			suspended; \
-  NSInteger		threads; \
-  NSInteger		idle; \
+  NSInteger		executing; \
+  NSInteger		threadCount; \
   NSInteger		count;
 
 #import "Foundation/NSOperation.h"
@@ -67,8 +69,16 @@
 #include	"GSInternal.h"
 GS_PRIVATE_INTERNAL(NSOperation)
 
+/* The pool of threads for 'non-concurrent' operations in a queue.
+ */
+#define	POOL	8
+
 static NSArray	*empty = nil;
 
+@interface	NSOperation (Private)
+- (void) _finish;
+@end
+
 @implementation NSOperation
 
 + (BOOL) automaticallyNotifiesObserversForKey: (NSString*)theKey
@@ -85,7 +95,7 @@ static NSArray	*empty = nil;
 
 - (void) addDependency: (NSOperation *)op
 {
-  if (NO == [op isKindOfClass: [self class]])
+  if (NO == [op isKindOfClass: [NSOperation class]])
     {
       [NSException raise: NSInvalidArgumentException
 		  format: @"[%@-%@] dependency is not an NSOperation",
@@ -222,7 +232,6 @@ static NSArray	*empty = nil;
       internal->threadPriority = 0.5;
       internal->ready = YES;
       internal->lock = [NSRecursiveLock new];
-      internal->cond = [[NSConditionLock alloc] initWithCondition: 0];
     }
   return self;
 }
@@ -262,21 +271,37 @@ static NSArray	*empty = nil;
                          change: (NSDictionary *)change
                         context: (void *)context
 {
-  /* Some dependency has finished (or been removed) ...
-   * so we need to check to see if we are now ready unless we know we are.
-   * This is protected by locks so that an update due to an observed
-   * change in one thread won't interrupt anything in another thread.
-   */
   [internal->lock lock];
-  if (NO == internal->ready)
+
+  /* We only observe isFinished changes, and we can remove self as an
+   * observer once we know the operation has finished since it can never
+   * become unfinished.
+   */
+  [object removeObserver: self
+	      forKeyPath: @"isFinished"];
+
+  if (object == self)
+    {
+      /* We have finished and need to unlock the condition lock so that
+       * any waiting thread can continue.
+       */
+      [internal->cond lock];
+      [internal->cond unlockWithCondition: 1];
+    }
+  else if (NO == internal->ready)
     {
       NSEnumerator	*en;
       NSOperation	*op;
 
+      /* Some dependency has finished (or been removed) ...
+       * so we need to check to see if we are now ready unless we know we are.
+       * This is protected by locks so that an update due to an observed
+       * change in one thread won't interrupt anything in another thread.
+       */
       en = [internal->dependencies objectEnumerator];
       while ((op = [en nextObject]) != nil)
         {
-          if (NO == [op isReady])
+          if (NO == [op isFinished])
 	    break;
         }
       if (op == nil)
@@ -373,8 +398,8 @@ static NSArray	*empty = nil;
 
 - (void) start
 {
-  CREATE_AUTORELEASE_POOL(pool);
-  double	prio = [NSThread  threadPriority];
+  NSAutoreleasePool	*pool = [NSAutoreleasePool new];
+  double		prio = [NSThread  threadPriority];
 
   [internal->lock lock];
   NS_DURING
@@ -403,10 +428,12 @@ static NSArray	*empty = nil;
 		      format: @"[%@-%@] called on operation which is not ready",
 	    NSStringFromClass([self class]), NSStringFromSelector(_cmd)];
 	}
-
-      [self willChangeValueForKey: @"isExecuting"];
-      internal->executing = YES;
-      [self didChangeValueForKey: @"isExecuting"];
+      if (NO == internal->executing)
+	{
+	  [self willChangeValueForKey: @"isExecuting"];
+	  internal->executing = YES;
+	  [self didChangeValueForKey: @"isExecuting"];
+	}
     }
   NS_HANDLER
     {
@@ -431,28 +458,8 @@ static NSArray	*empty = nil;
     }
   NS_ENDHANDLER;
 
-  [internal->lock lock];
-  NS_DURING
-    {
-      /* Notify KVO system of changes to isExecuting and isFinished
-       */
-      [self willChangeValueForKey: @"isExecuting"];
-      [self willChangeValueForKey: @"isFinished"];
-      internal->executing = NO;
-      internal->finished = YES;
-      [self didChangeValueForKey: @"isFinished"];
-      [self didChangeValueForKey: @"isExecuting"];
-      [internal->cond lock];
-      [internal->cond unlockWithCondition: 1];
-    }
-  NS_HANDLER
-    {
-      [internal->lock unlock];
-      [localException raise];
-    }
-  NS_ENDHANDLER
-  [internal->lock unlock];
-  RELEASE(pool);
+  [self _finish];
+  [pool release];
 }
 
 - (double) threadPriority
@@ -462,11 +469,67 @@ static NSArray	*empty = nil;
 
 - (void) waitUntilFinished
 {
-  [internal->cond lockWhenCondition: 1];	// Wait for finish
-  [internal->cond unlockWithCondition: 1];	// Signal any other watchers
+  if (NO == [self isFinished])
+    {
+      [internal->lock lock];
+      if (nil == internal->cond)
+	{
+	  /* Set up condition to wait on and observer to unblock.
+	   */
+	  internal->cond = [[NSConditionLock alloc] initWithCondition: 0];
+	  [self addObserver: self
+		 forKeyPath: @"isFinished"
+		    options: NSKeyValueObservingOptionNew
+		    context: NULL];
+	  /* Some other thread could have marked us as finished while we
+	   * were setting up ... so we can fake the observation if needed.
+	   */
+          if (YES == [self isFinished])
+	    {
+	      [self observeValueForKeyPath: @"isFinished"
+				  ofObject: self
+				    change: nil
+				   context: nil];
+	    }
+	}
+      [internal->lock unlock];
+      [internal->cond lockWhenCondition: 1];	// Wait for finish
+      [internal->cond unlockWithCondition: 1];	// Signal any other watchers
+    }
 }
 @end
 
+@implementation	NSOperation (Private)
+- (void) _finish
+{
+  /* retain while finishing so that we don't get deallocated when our
+   * queue removes and releases us.
+   */
+  [self retain];
+  [internal->lock lock];
+  if (NO == internal->finished)
+    {
+      if (NO == internal->executing)
+        {
+	  [self willChangeValueForKey: @"isExecuting"];
+	  [self willChangeValueForKey: @"isFinished"];
+	  internal->executing = NO;
+	  internal->finished = YES;
+	  [self didChangeValueForKey: @"isFinished"];
+	  [self didChangeValueForKey: @"isExecuting"];
+	}
+      else
+	{
+	  [self willChangeValueForKey: @"isFinished"];
+	  internal->finished = YES;
+	  [self didChangeValueForKey: @"isFinished"];
+	}
+    }
+  [internal->lock unlock];
+  [self release];
+}
+
+@end
 
 #undef	GSInternal
 #define	GSInternal	NSOperationQueueInternal
@@ -476,15 +539,15 @@ GS_PRIVATE_INTERNAL(NSOperationQueue)
 
 @interface	NSOperationQueue (Private)
 + (void) _mainQueue;
+- (void) _execute;
+- (void) _thread;
 - (void) observeValueForKeyPath: (NSString *)keyPath
 		       ofObject: (id)object
                          change: (NSDictionary *)change
                         context: (void *)context;
-- (void) _thread;
-- (void) _updateThreads;
 @end
 
-static NSInteger	maxThreads = 200;	// FIXME ... how many really?
+static NSInteger	maxConcurrent = 200;	// Thread pool size
 
 static NSComparisonResult
 sortFunc(id o1, id o2, void *ctxt)
@@ -570,8 +633,8 @@ static NSOperationQueue *mainQueue = nil;
   if (total > 0)
     {
       BOOL		invalidArg = NO;
-      GS_BEGINITEMBUF(buf, total, id)
       NSUInteger	toAdd = total;
+      GS_BEGINITEMBUF(buf, total, id)
 
       [ops getObjects: buf];
       for (index = 0; index < total; index++)
@@ -653,39 +716,13 @@ static NSOperationQueue *mainQueue = nil;
 
 - (void) cancelAllOperations
 {
-  NSUInteger	index;
-  NSOperation	*o;
-
-  [internal->cond lock];
-  while ((o = [internal->waiting lastObject]) != nil)
-    {
-      [o removeObserver: self
-	     forKeyPath: @"isReady"];
-      [o cancel];
-      [internal->waiting removeLastObject];
-    }
-  [internal->cond unlockWithCondition: 0];	// Nothing waiting to execute
-
-  [internal->lock lock];
-  index = [internal->operations count];
-  while (index-- > 0)
-    {
-      NSOperation	*o;
-
-      o = [internal->operations objectAtIndex: index];
-      if (NO == [o isCancelled])
-	{
-          [o removeObserver: self
-	         forKeyPath: @"isReady"];
-          [o cancel];
-	}
-    }
-  [internal->lock unlock];
+  [[self operations] makeObjectsPerformSelector: @selector(cancel)];
 }
 
 - (void) dealloc
 {
   [internal->operations release];
+  [internal->starting release];
   [internal->waiting release];
   [internal->name release];
   [internal->cond release];
@@ -702,9 +739,10 @@ static NSOperationQueue *mainQueue = nil;
       internal->suspended = NO;
       internal->count = NSOperationQueueDefaultMaxConcurrentOperationCount;
       internal->operations = [NSMutableArray new];
+      internal->starting = [NSMutableArray new];
       internal->waiting = [NSMutableArray new];
-      internal->cond = [[NSConditionLock alloc] initWithCondition: 0];
       internal->lock = [NSRecursiveLock new];
+      internal->cond = [[NSConditionLock alloc] initWithCondition: 0];
     }
   return self;
 }
@@ -756,8 +794,6 @@ static NSOperationQueue *mainQueue = nil;
 
 - (void) setMaxConcurrentOperationCount: (NSInteger)cnt
 {
-  BOOL	unSuspend = NO;
-
   if (cnt < 0
     && cnt != NSOperationQueueDefaultMaxConcurrentOperationCount)
     {
@@ -768,28 +804,12 @@ static NSOperationQueue *mainQueue = nil;
   [internal->lock lock];
   if (cnt != internal->count)
     {
-      if (cnt > internal->count)
-	{
-	  unSuspend = YES;		// May need to add more threads.
-	}
       [self willChangeValueForKey: @"maxConcurrentOperationCount"];
       internal->count = cnt;
       [self didChangeValueForKey: @"maxConcurrentOperationCount"];
     }
   [internal->lock unlock];
-  if (YES == unSuspend)
-    {
-      [internal->cond lock];
-      if ([internal->waiting count] > 0)
-	{
-	  [internal->cond unlockWithCondition: 1];
-	  [self _updateThreads];
-	}
-      else
-	{
-	  [internal->cond unlockWithCondition: 0];
-	}
-    }
+  [self _execute];
 }
 
 - (void) setName: (NSString*)s
@@ -808,33 +828,15 @@ static NSOperationQueue *mainQueue = nil;
 
 - (void) setSuspended: (BOOL)flag
 {
-  BOOL	unSuspend = NO;
-
   [internal->lock lock];
   if (flag != internal->suspended)
     {
-      if (YES == flag)
-	{
-	  unSuspend = YES;
-	}
       [self willChangeValueForKey: @"suspended"];
       internal->suspended = flag;
       [self didChangeValueForKey: @"suspended"];
     }
   [internal->lock unlock];
-  if (YES == unSuspend)
-    {
-      [internal->cond lock];
-      if ([internal->waiting count] > 0)
-	{
-	  [internal->cond unlockWithCondition: 1];
-	  [self _updateThreads];
-	}
-      else
-	{
-	  [internal->cond unlockWithCondition: 0];
-	}
-    }
+  [self _execute];
 }
 
 - (void) waitUntilAllOperationsAreFinished
@@ -871,186 +873,159 @@ static NSOperationQueue *mainQueue = nil;
                          change: (NSDictionary *)change
                         context: (void *)context
 {
-  [internal->cond lock];
-  if (YES == [object isReady])
+  [internal->lock lock];
+  if (YES == [object isFinished])
     {
-      if ([internal->waiting indexOfObjectIdenticalTo: object] == NSNotFound)
-	{
-          [internal->waiting addObject: object];
- 	}
+      internal->executing--;
+      [object removeObserver: self
+		  forKeyPath: @"isFinished"];
+      [self willChangeValueForKey: @"operations"];
+      [self willChangeValueForKey: @"operationCount"];
+      [internal->operations removeObjectIdenticalTo: object];
+      [self didChangeValueForKey: @"operationCount"];
+      [self didChangeValueForKey: @"operations"];
     }
-  else
+  else if (YES == [object isReady])
     {
-      NSUInteger	index;
-
-      index = [internal->waiting indexOfObjectIdenticalTo: object];
-      if (index != NSNotFound)
-	{
-          [internal->waiting removeObjectAtIndex: index];
-	}
-    }
-  if ([internal->waiting count] > 0)
-    {
-      [internal->cond unlockWithCondition: 1];
-      [self _updateThreads];
-    }
-  else
-    {
-      [internal->cond unlockWithCondition: 0];
+      [object removeObserver: self
+		  forKeyPath: @"isReady"];
+      [internal->waiting addObject: object];
     }
+  [internal->lock unlock];
+  [self _execute];
 }
 
 - (void) _thread
 {
-  BOOL	terminate = NO;
+  NSAutoreleasePool	*pool = [NSAutoreleasePool new];
 
-  /* Record which operation queue the current thread is running.
-   */
-  [[[NSThread currentThread] threadDictionary] setObject: self
-						  forKey: threadKey];
-
-  [internal->lock lock];
-  while (NO == terminate)
+  for (;;)
     {
       NSOperation	*op;
-      NSUInteger	index;
-      NSDate		*timeout;
+      NSDate		*when;
+      BOOL		found;
 
-      /* Unlock the queue while we are waiting for another operation
-       * to perform.
-       */
-      [internal->lock unlock];
-
-      /* Wait for an operation to become available.
-       * If there's nothing to do within 5 seconds, we might terminate,
-       * but only if there are no operations still queued.
-       * The figure 5 was chosen because it seems to be what OSX 10.6 uses.
-       */
-      timeout = [[NSDate alloc] initWithTimeIntervalSinceNow: 5.0];
-      if (NO == [internal->cond lockWhenCondition: 1 beforeDate: timeout])
+      when = [[NSDate alloc] initWithTimeIntervalSinceNow: 5.0];
+      found = [internal->cond lockWhenCondition: 1 beforeDate: when];
+      [when release];
+      if (NO == found)
 	{
-	  terminate = YES;
+	  break;	// Idle for 5 seconds ... exit thread.
 	}
-      [timeout release];
-      if (NO == terminate)
+
+      if ([internal->starting count] > 0)
 	{
-	  op = nil;
-
-	  if ((unsigned)internal->threads > (unsigned)internal->count)
-	    {
-	      /* We know that the 'unlimited' thread count is -1 so an
-	       * unsigned comparison is safe to tell is we need to end
-	       * a thread.
-	       */
-	      terminate = YES;	// This thread is not needed
-	    }
-	  else if (internal->count != 0 && NO == internal->suspended
-	    && [internal->waiting count] > 0)
-	    {
-	      /* If concurrent operations are permitted an the queue
-	       * is not suspended, and there are operations waiting,
-	       * then we can get one.
-	       */
-	      [internal->waiting sortUsingFunction: sortFunc context: 0];
-	      op = [[internal->waiting objectAtIndex: 0] retain];
-	      [internal->waiting removeObjectAtIndex: 0];
-	    }
-	  if ([internal->waiting count] == 0)
-	    {
-	      [internal->cond unlockWithCondition: 0];
-	    }
-	  else
-	    {
-	      [internal->cond unlockWithCondition: 1];
-	    }
-
-	  /* Restore the queue lock 
-	   */
-	  [internal->lock lock];
-	  if (YES == [op isReady])
-	    {
-	      if (NO == [op isCancelled])
-		{
-		  internal->idle--;
-
-		  /* Unlock the queue while the operation is executing.
-		   */
-		  [internal->lock unlock];
-		  [op start];
-		  [op waitUntilFinished];
-
-		  /* Lock the queue again to perform cleanup etc.
-		   */
-		  [internal->lock lock];
-		  internal->idle++;
-		}
-	      [self willChangeValueForKey: @"operations"];
-	      [self willChangeValueForKey: @"operationCount"];
-	      [internal->operations removeObjectIdenticalTo: op];
-	      [self didChangeValueForKey: @"operationCount"];
-	      [self didChangeValueForKey: @"operations"];
-	    }
-	  [op release];
+          op = [internal->starting objectAtIndex: 0];
+	  [internal->starting removeObjectAtIndex: 0];
 	}
       else
 	{
-	  /* Restore the queue lock so we can clean up operations in queue.
-	   */
-	  [internal->lock lock];
+	  op = nil;
 	}
 
-      /* And now make sure we clean up any finished operations.
-       */
-      index = [internal->operations count];
-      while (index-- > 0)
+      if ([internal->starting count] > 0)
 	{
-	  op = [internal->operations objectAtIndex: index];
-	  if (YES == [op isFinished])
-	    {
-	      [self willChangeValueForKey: @"operations"];
-	      [self willChangeValueForKey: @"operationCount"];
-	      [internal->operations removeObjectAtIndex: index];
-	      [self didChangeValueForKey: @"operationCount"];
-	      [self didChangeValueForKey: @"operations"];
-	    }
+	  // Signal any other idle threads,
+          [internal->cond unlockWithCondition: 1];
 	}
-      if ([internal->operations count] > 0)
+      else
 	{
-	  terminate = NO;	// Still stuff to do
+	  // There are no more operations starting.
+          [internal->cond unlockWithCondition: 0];
+	}
+
+      if (nil != op)
+	{
+          NS_DURING
+	    {
+	      NSAutoreleasePool	*opPool = [NSAutoreleasePool new];
+
+	      if (NO == [op isCancelled])
+		{
+		  [NSThread setThreadPriority: [op threadPriority]];
+		  [op main];
+		}
+	      [opPool release];
+	    }
+          NS_HANDLER
+	    {
+	      NSLog(@"Problem running operation %@ ... %@",
+		op, localException);
+	    }
+          NS_ENDHANDLER
+	  [op _finish];
 	}
     }
-  internal->idle--;
-  internal->threads--;
+
+  [internal->lock lock];
+  internal->threadCount--;
   [internal->lock unlock];
+  [pool release];
+  [NSThread exit];
 }
 
-/* Just check to see if a new thread needs to be started.
+/* Check for operations which can be executed and start them.
  */
-- (void) _updateThreads
+- (void) _execute
 {
+  NSInteger	max;
+
   [internal->lock lock];
-  if (0 == internal->idle
-    && NO == [self isSuspended]
-    && [self maxConcurrentOperationCount] != 0
+
+  max = [self maxConcurrentOperationCount];
+  if (NSOperationQueueDefaultMaxConcurrentOperationCount == max)
+    {
+      max = maxConcurrent;
+    }
+
+  while (NO == [self isSuspended]
+    && max > internal->executing
     && [internal->waiting count] > 0)
     {
-      NSInteger	total = internal->count;
+      NSOperation	*op;
 
-      if (total == NSOperationQueueDefaultMaxConcurrentOperationCount)
+      /* Make sure we have a sorted queue of operations waiting to execute.
+       */
+      [internal->waiting sortUsingFunction: sortFunc context: 0];
+
+      /* Take the first operation from the queue and start it executing.
+       * We set ourselves up as an observer for the operating finishing
+       * and we keep track of the count of operations we have started,
+       * but the actual startup is left to the NSOperation -start method.
+       */
+      op = [internal->waiting objectAtIndex: 0];
+      [internal->waiting removeObjectAtIndex: 0];
+      [op addObserver: self
+	   forKeyPath: @"isFinished"
+	      options: NSKeyValueObservingOptionNew
+	      context: NULL];
+      internal->executing++;
+      if (YES == [op isConcurrent])
 	{
-	  total = maxThreads;	// Limit number of allowed threads
+          [op start];
 	}
-      if (internal->threads < total)
+      else
 	{
-	  /* All threads are in use, but we don't have the maximum
-	   * number of threads, so we can create a new one for the
-	   * waiting operation.
+	  NSUInteger	pending;
+
+	  [internal->cond lock];
+	  pending = [internal->starting count];
+	  [internal->starting addObject: op];
+
+	  /* Create a new thread if all existing threads are busy and
+	   * we haven't reached the pool limit.
 	   */
-	  internal->threads++;
-	  internal->idle++;
-	  [NSThread detachNewThreadSelector: @selector(_thread)
-				   toTarget: self
-				 withObject: nil];
+	  if (0 == internal->threadCount
+	    || (pending > 0 && internal->threadCount < POOL))
+	    {
+	      internal->threadCount++;
+	      [NSThread detachNewThreadSelector: @selector(_thread)
+				       toTarget: self
+				     withObject: nil];
+	    }
+	  /* Tell the thread pool that there is an operation to start.
+	   */
+	  [internal->cond unlockWithCondition: 1];
 	}
     }
   [internal->lock unlock];
