@@ -22,14 +22,11 @@
    Boston, MA 02111 USA.
 
    <title>NSPage class reference</title>
-   $Date: 2010-06-17 20:08:08 +0100 (Thu, 17 Jun 2010) $ $Revision: 30772 $
+   $Date: 2011-10-14 09:46:13 -0600 (Fri, 14 Oct 2011) $ $Revision: 33993 $
    */
 
 #import "common.h"
 #include <string.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #include <stdio.h>
 #if __mach__
 #include <mach.h>
