@@ -26,7 +26,7 @@
    Boston, MA 02111 USA.
 
    <title>NSPathUtilities function reference</title>
-   $Date: 2011-04-13 15:34:39 +0100 (Wed, 13 Apr 2011) $ $Revision: 32841 $
+   $Date: 2011-11-14 03:01:24 -0700 (Mon, 14 Nov 2011) $ $Revision: 34169 $
    */
 
 /**
@@ -73,9 +73,6 @@
 
 #import "GSPrivate.h"
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>		// for getuid()
-#endif
 #ifdef	HAVE_PWD_H
 #include <pwd.h>		// for getpwnam_r() and getpwuid_r()
 #endif
@@ -242,8 +239,8 @@ static void ShutdownPathUtilities(void);
 })
 
 /* Grab a path from the config file, making it relative to the config
- * file location if it begins with './' or '../', and checking that the
- * result is an absolute path.
+ * file location if it is '.' or begins with './' or '../', and checking
+ * that the result is an absolute path.
  */
 #define ASSIGN_PATH(var, dictionary, key) ({\
   id val = getPathConfig(dictionary, key);\
@@ -418,77 +415,138 @@ getPathConfig(NSDictionary *dict, NSString *key)
 static void ExtractValuesFromConfig(NSDictionary *config)
 {
   NSMutableDictionary	*c = [config mutableCopy];
+  BOOL			createLibraryPath;
   id		        extra;
 
   /*
    * Move values out of the dictionary and into variables for rapid reference.
    */
-  ASSIGN_IF_SET(gnustepUserDefaultsDir, c, @"GNUSTEP_USER_DEFAULTS_DIR");
+  ASSIGN_IF_SET(gnustepUserDefaultsDir, c,
+    @"GNUSTEP_USER_DEFAULTS_DIR");
 
-  ASSIGN_PATH(gnustepMakefiles, c, @"GNUSTEP_MAKEFILES");
+  ASSIGN_PATH(gnustepMakefiles, c,
+    @"GNUSTEP_MAKEFILES");
 
-  ASSIGN_PATH(gnustepSystemUsersDir, c, @"GNUSTEP_SYSTEM_USERS_DIR");
-  ASSIGN_PATH(gnustepNetworkUsersDir, c, @"GNUSTEP_NETWORK_USERS_DIR");
-  ASSIGN_PATH(gnustepLocalUsersDir, c, @"GNUSTEP_LOCAL_USERS_DIR");
+  ASSIGN_PATH(gnustepSystemUsersDir, c,
+    @"GNUSTEP_SYSTEM_USERS_DIR");
+  ASSIGN_PATH(gnustepNetworkUsersDir, c,
+    @"GNUSTEP_NETWORK_USERS_DIR");
+  ASSIGN_PATH(gnustepLocalUsersDir, c,
+    @"GNUSTEP_LOCAL_USERS_DIR");
 
-  ASSIGN_PATH(gnustepSystemApps, c, @"GNUSTEP_SYSTEM_APPS");
-  ASSIGN_PATH(gnustepSystemAdminApps, c, @"GNUSTEP_SYSTEM_ADMIN_APPS");
-  ASSIGN_PATH(gnustepSystemWebApps, c, @"GNUSTEP_SYSTEM_WEB_APPS");
-  ASSIGN_PATH(gnustepSystemTools, c, @"GNUSTEP_SYSTEM_TOOLS");
-  ASSIGN_PATH(gnustepSystemAdminTools, c, @"GNUSTEP_SYSTEM_ADMIN_TOOLS");
-  ASSIGN_PATH(gnustepSystemLibrary, c, @"GNUSTEP_SYSTEM_LIBRARY");
-  ASSIGN_PATH(gnustepSystemLibraries, c, @"GNUSTEP_SYSTEM_LIBRARIES");
-  ASSIGN_PATH(gnustepSystemHeaders, c, @"GNUSTEP_SYSTEM_HEADERS");
-  ASSIGN_PATH(gnustepSystemDocumentation, c, @"GNUSTEP_SYSTEM_DOC");
-  ASSIGN_PATH(gnustepSystemDocumentationMan, c, @"GNUSTEP_SYSTEM_DOC_MAN");
-  ASSIGN_PATH(gnustepSystemDocumentationInfo, c, @"GNUSTEP_SYSTEM_DOC_INFO");
+  ASSIGN_PATH(gnustepSystemApps, c,
+    @"GNUSTEP_SYSTEM_APPS");
+  ASSIGN_PATH(gnustepSystemAdminApps, c,
+    @"GNUSTEP_SYSTEM_ADMIN_APPS");
+  ASSIGN_PATH(gnustepSystemWebApps, c,
+    @"GNUSTEP_SYSTEM_WEB_APPS");
+  ASSIGN_PATH(gnustepSystemTools, c,
+    @"GNUSTEP_SYSTEM_TOOLS");
+  ASSIGN_PATH(gnustepSystemAdminTools, c,
+    @"GNUSTEP_SYSTEM_ADMIN_TOOLS");
+  ASSIGN_PATH(gnustepSystemLibrary, c,
+    @"GNUSTEP_SYSTEM_LIBRARY");
+  ASSIGN_PATH(gnustepSystemLibraries, c,
+    @"GNUSTEP_SYSTEM_LIBRARIES");
+  ASSIGN_PATH(gnustepSystemHeaders, c,
+    @"GNUSTEP_SYSTEM_HEADERS");
+  ASSIGN_PATH(gnustepSystemDocumentation, c,
+    @"GNUSTEP_SYSTEM_DOC");
+  ASSIGN_PATH(gnustepSystemDocumentationMan, c,
+    @"GNUSTEP_SYSTEM_DOC_MAN");
+  ASSIGN_PATH(gnustepSystemDocumentationInfo, c,
+    @"GNUSTEP_SYSTEM_DOC_INFO");
 
-  ASSIGN_PATH(gnustepNetworkApps, c, @"GNUSTEP_NETWORK_APPS");
-  ASSIGN_PATH(gnustepNetworkAdminApps, c, @"GNUSTEP_NETWORK_ADMIN_APPS");
-  ASSIGN_PATH(gnustepNetworkWebApps, c, @"GNUSTEP_NETWORK_WEB_APPS");
-  ASSIGN_PATH(gnustepNetworkTools, c, @"GNUSTEP_NETWORK_TOOLS");
-  ASSIGN_PATH(gnustepNetworkAdminTools, c, @"GNUSTEP_NETWORK_ADMIN_TOOLS");
-  ASSIGN_PATH(gnustepNetworkLibrary, c, @"GNUSTEP_NETWORK_LIBRARY");
-  ASSIGN_PATH(gnustepNetworkLibraries, c, @"GNUSTEP_NETWORK_LIBRARIES");
-  ASSIGN_PATH(gnustepNetworkHeaders, c, @"GNUSTEP_NETWORK_HEADERS");
-  ASSIGN_PATH(gnustepNetworkDocumentation, c, @"GNUSTEP_NETWORK_DOC");
-  ASSIGN_PATH(gnustepNetworkDocumentationMan, c, @"GNUSTEP_NETWORK_DOC_MAN");
-  ASSIGN_PATH(gnustepNetworkDocumentationInfo, c, @"GNUSTEP_NETWORK_DOC_INFO");
+  ASSIGN_PATH(gnustepNetworkApps, c,
+    @"GNUSTEP_NETWORK_APPS");
+  ASSIGN_PATH(gnustepNetworkAdminApps, c,
+    @"GNUSTEP_NETWORK_ADMIN_APPS");
+  ASSIGN_PATH(gnustepNetworkWebApps, c,
+    @"GNUSTEP_NETWORK_WEB_APPS");
+  ASSIGN_PATH(gnustepNetworkTools, c,
+    @"GNUSTEP_NETWORK_TOOLS");
+  ASSIGN_PATH(gnustepNetworkAdminTools, c,
+    @"GNUSTEP_NETWORK_ADMIN_TOOLS");
+  ASSIGN_PATH(gnustepNetworkLibrary, c,
+    @"GNUSTEP_NETWORK_LIBRARY");
+  ASSIGN_PATH(gnustepNetworkLibraries, c,
+    @"GNUSTEP_NETWORK_LIBRARIES");
+  ASSIGN_PATH(gnustepNetworkHeaders, c,
+    @"GNUSTEP_NETWORK_HEADERS");
+  ASSIGN_PATH(gnustepNetworkDocumentation, c,
+    @"GNUSTEP_NETWORK_DOC");
+  ASSIGN_PATH(gnustepNetworkDocumentationMan, c,
+    @"GNUSTEP_NETWORK_DOC_MAN");
+  ASSIGN_PATH(gnustepNetworkDocumentationInfo, c,
+    @"GNUSTEP_NETWORK_DOC_INFO");
 
-  ASSIGN_PATH(gnustepLocalApps, c, @"GNUSTEP_LOCAL_APPS");
-  ASSIGN_PATH(gnustepLocalAdminApps, c, @"GNUSTEP_LOCAL_ADMIN_APPS");
-  ASSIGN_PATH(gnustepLocalWebApps, c, @"GNUSTEP_LOCAL_WEB_APPS");
-  ASSIGN_PATH(gnustepLocalTools, c, @"GNUSTEP_LOCAL_TOOLS");
-  ASSIGN_PATH(gnustepLocalAdminTools, c, @"GNUSTEP_LOCAL_ADMIN_TOOLS");
-  ASSIGN_PATH(gnustepLocalLibrary, c, @"GNUSTEP_LOCAL_LIBRARY");
-  ASSIGN_PATH(gnustepLocalLibraries, c, @"GNUSTEP_LOCAL_LIBRARIES");
-  ASSIGN_PATH(gnustepLocalHeaders, c, @"GNUSTEP_LOCAL_HEADERS");
-  ASSIGN_PATH(gnustepLocalDocumentation, c, @"GNUSTEP_LOCAL_DOC");
-  ASSIGN_PATH(gnustepLocalDocumentationMan, c, @"GNUSTEP_LOCAL_DOC_MAN");
-  ASSIGN_PATH(gnustepLocalDocumentationInfo, c, @"GNUSTEP_LOCAL_DOC_INFO");
+  ASSIGN_PATH(gnustepLocalApps, c,
+    @"GNUSTEP_LOCAL_APPS");
+  ASSIGN_PATH(gnustepLocalAdminApps, c,
+    @"GNUSTEP_LOCAL_ADMIN_APPS");
+  ASSIGN_PATH(gnustepLocalWebApps, c,
+    @"GNUSTEP_LOCAL_WEB_APPS");
+  ASSIGN_PATH(gnustepLocalTools, c,
+    @"GNUSTEP_LOCAL_TOOLS");
+  ASSIGN_PATH(gnustepLocalAdminTools, c,
+    @"GNUSTEP_LOCAL_ADMIN_TOOLS");
+  ASSIGN_PATH(gnustepLocalLibrary, c,
+    @"GNUSTEP_LOCAL_LIBRARY");
+  ASSIGN_PATH(gnustepLocalLibraries, c,
+    @"GNUSTEP_LOCAL_LIBRARIES");
+  ASSIGN_PATH(gnustepLocalHeaders, c,
+    @"GNUSTEP_LOCAL_HEADERS");
+  ASSIGN_PATH(gnustepLocalDocumentation, c,
+    @"GNUSTEP_LOCAL_DOC");
+  ASSIGN_PATH(gnustepLocalDocumentationMan, c,
+    @"GNUSTEP_LOCAL_DOC_MAN");
+  ASSIGN_PATH(gnustepLocalDocumentationInfo, c,
+    @"GNUSTEP_LOCAL_DOC_INFO");
 
-  ASSIGN_USER(gnustepUserDirApps, c, @"GNUSTEP_USER_DIR_APPS");
-  TEST_ASSIGN(gnustepUserDirApps, @GNUSTEP_TARGET_USER_DIR_APPS);
-  ASSIGN_USER(gnustepUserDirAdminApps, c, @"GNUSTEP_USER_DIR_ADMIN_APPS");
-  TEST_ASSIGN(gnustepUserDirAdminApps, @GNUSTEP_TARGET_USER_DIR_ADMIN_APPS);
-  ASSIGN_USER(gnustepUserDirWebApps, c, @"GNUSTEP_USER_DIR_WEB_APPS");
-  TEST_ASSIGN(gnustepUserDirWebApps, @GNUSTEP_TARGET_USER_DIR_WEB_APPS);
-  ASSIGN_USER(gnustepUserDirTools, c, @"GNUSTEP_USER_DIR_TOOLS");
-  TEST_ASSIGN(gnustepUserDirTools, @GNUSTEP_TARGET_USER_DIR_TOOLS);
-  ASSIGN_USER(gnustepUserDirAdminTools, c, @"GNUSTEP_USER_DIR_ADMIN_TOOLS");
-  TEST_ASSIGN(gnustepUserDirAdminTools, @GNUSTEP_TARGET_USER_DIR_ADMIN_TOOLS);
-  ASSIGN_USER(gnustepUserDirLibrary, c, @"GNUSTEP_USER_DIR_LIBRARY");
-  TEST_ASSIGN(gnustepUserDirLibrary, @GNUSTEP_TARGET_USER_DIR_LIBRARY);
-  ASSIGN_USER(gnustepUserDirLibraries, c, @"GNUSTEP_USER_DIR_LIBRARIES");
-  TEST_ASSIGN(gnustepUserDirLibraries, @GNUSTEP_TARGET_USER_DIR_LIBRARIES);
-  ASSIGN_USER(gnustepUserDirHeaders, c, @"GNUSTEP_USER_DIR_HEADERS");
-  TEST_ASSIGN(gnustepUserDirHeaders, @GNUSTEP_TARGET_USER_DIR_HEADERS);
-  ASSIGN_USER(gnustepUserDirDocumentation, c, @"GNUSTEP_USER_DIR_DOC");
-  TEST_ASSIGN(gnustepUserDirDocumentation, @GNUSTEP_TARGET_USER_DIR_DOC);
-  ASSIGN_USER(gnustepUserDirDocumentationMan, c, @"GNUSTEP_USER_DIR_DOC_MAN");
-  TEST_ASSIGN(gnustepUserDirDocumentationMan, @GNUSTEP_TARGET_USER_DIR_DOC_MAN);
-  ASSIGN_USER(gnustepUserDirDocumentationInfo, c, @"GNUSTEP_USER_DIR_DOC_INFO");
-  TEST_ASSIGN(gnustepUserDirDocumentationInfo, @GNUSTEP_TARGET_USER_DIR_DOC_INFO);
+  ASSIGN_USER(gnustepUserDirApps, c,
+    @"GNUSTEP_USER_DIR_APPS");
+  TEST_ASSIGN(gnustepUserDirApps,
+    @GNUSTEP_TARGET_USER_DIR_APPS);
+  ASSIGN_USER(gnustepUserDirAdminApps, c,
+    @"GNUSTEP_USER_DIR_ADMIN_APPS");
+  TEST_ASSIGN(gnustepUserDirAdminApps,
+    @GNUSTEP_TARGET_USER_DIR_ADMIN_APPS);
+  ASSIGN_USER(gnustepUserDirWebApps, c,
+    @"GNUSTEP_USER_DIR_WEB_APPS");
+  TEST_ASSIGN(gnustepUserDirWebApps,
+    @GNUSTEP_TARGET_USER_DIR_WEB_APPS);
+  ASSIGN_USER(gnustepUserDirTools, c,
+    @"GNUSTEP_USER_DIR_TOOLS");
+  TEST_ASSIGN(gnustepUserDirTools,
+    @GNUSTEP_TARGET_USER_DIR_TOOLS);
+  ASSIGN_USER(gnustepUserDirAdminTools, c,
+    @"GNUSTEP_USER_DIR_ADMIN_TOOLS");
+  TEST_ASSIGN(gnustepUserDirAdminTools,
+    @GNUSTEP_TARGET_USER_DIR_ADMIN_TOOLS);
+  ASSIGN_USER(gnustepUserDirLibrary, c,
+    @"GNUSTEP_USER_DIR_LIBRARY");
+  TEST_ASSIGN(gnustepUserDirLibrary,
+    @GNUSTEP_TARGET_USER_DIR_LIBRARY);
+  ASSIGN_USER(gnustepUserDirLibraries, c,
+    @"GNUSTEP_USER_DIR_LIBRARIES");
+  TEST_ASSIGN(gnustepUserDirLibraries,
+    @GNUSTEP_TARGET_USER_DIR_LIBRARIES);
+  ASSIGN_USER(gnustepUserDirHeaders, c,
+    @"GNUSTEP_USER_DIR_HEADERS");
+  TEST_ASSIGN(gnustepUserDirHeaders,
+    @GNUSTEP_TARGET_USER_DIR_HEADERS);
+  ASSIGN_USER(gnustepUserDirDocumentation, c,
+    @"GNUSTEP_USER_DIR_DOC");
+  TEST_ASSIGN(gnustepUserDirDocumentation,
+    @GNUSTEP_TARGET_USER_DIR_DOC);
+  ASSIGN_USER(gnustepUserDirDocumentationMan, c,
+    @"GNUSTEP_USER_DIR_DOC_MAN");
+  TEST_ASSIGN(gnustepUserDirDocumentationMan,
+    @GNUSTEP_TARGET_USER_DIR_DOC_MAN);
+  ASSIGN_USER(gnustepUserDirDocumentationInfo, c,
+    @"GNUSTEP_USER_DIR_DOC_INFO");
+  TEST_ASSIGN(gnustepUserDirDocumentationInfo,
+    @GNUSTEP_TARGET_USER_DIR_DOC_INFO);
 
   /*
    * The GNUSTEP_EXTRA field may contain a list of extra keys which
@@ -514,6 +572,14 @@ static void ExtractValuesFromConfig(NSDictionary *config)
     }
   [c removeObjectForKey: @"GNUSTEP_SYSTEM_DEFAULTS_FILE"];
 
+  /* If GNUSTEP_CREATE_DIRECTORIES is YES then we should ensure that the
+   * per-user directory and the Library subdirectory exist so resources
+   * can safely be stored in them.
+   */
+  createLibraryPath
+    = [[c objectForKey: @"GNUSTEP_CREATE_LIBRARY_PATH"] boolValue];
+  [c removeObjectForKey: @"GNUSTEP_CREATE_LIBRARY_PATH"];
+
   /*
    * Remove any other dictionary entries we have used.
    */
@@ -596,10 +662,9 @@ static void ExtractValuesFromConfig(NSDictionary *config)
   ASSIGN_USER_PATH(gnustepUserDocumentationInfo, gnustepUserDocumentationInfo);
 
   /*
-   * Try to ensure that essential user directories exist.
-   * FIXME  ... Check/creation should perhaps be configurable.
+   * Try to ensure that essential user directories exist if we need to.
    */
-  if (1)
+  if (YES == createLibraryPath)
     {
       NSFileManager	*manager;
       NSString		*path;
@@ -622,47 +687,84 @@ static void ExtractValuesFromConfig(NSDictionary *config)
   /*
    * Finally set default locations for the essential paths if required.
    */
-  ASSIGN_DEFAULT_PATH(gnustepSystemApps, @GNUSTEP_TARGET_SYSTEM_APPS);
-  ASSIGN_DEFAULT_PATH(gnustepSystemAdminApps, @GNUSTEP_TARGET_SYSTEM_ADMIN_APPS);
-  ASSIGN_DEFAULT_PATH(gnustepSystemWebApps, @GNUSTEP_TARGET_SYSTEM_WEB_APPS);
-  ASSIGN_DEFAULT_PATH(gnustepSystemTools, @GNUSTEP_TARGET_SYSTEM_TOOLS);
-  ASSIGN_DEFAULT_PATH(gnustepSystemAdminTools, @GNUSTEP_TARGET_SYSTEM_ADMIN_TOOLS);
-  ASSIGN_DEFAULT_PATH(gnustepSystemLibrary, @GNUSTEP_TARGET_SYSTEM_LIBRARY);
-  ASSIGN_DEFAULT_PATH(gnustepSystemLibraries, @GNUSTEP_TARGET_SYSTEM_LIBRARIES);
-  ASSIGN_DEFAULT_PATH(gnustepSystemHeaders, @GNUSTEP_TARGET_SYSTEM_HEADERS);
-  ASSIGN_DEFAULT_PATH(gnustepSystemDocumentation, @GNUSTEP_TARGET_SYSTEM_DOC);
-  ASSIGN_DEFAULT_PATH(gnustepSystemDocumentationMan, @GNUSTEP_TARGET_SYSTEM_DOC_MAN);
-  ASSIGN_DEFAULT_PATH(gnustepSystemDocumentationInfo, @GNUSTEP_TARGET_SYSTEM_DOC_INFO);
+  ASSIGN_DEFAULT_PATH(gnustepSystemApps,
+    @GNUSTEP_TARGET_SYSTEM_APPS);
+  ASSIGN_DEFAULT_PATH(gnustepSystemAdminApps,
+    @GNUSTEP_TARGET_SYSTEM_ADMIN_APPS);
+  ASSIGN_DEFAULT_PATH(gnustepSystemWebApps,
+    @GNUSTEP_TARGET_SYSTEM_WEB_APPS);
+  ASSIGN_DEFAULT_PATH(gnustepSystemTools,
+    @GNUSTEP_TARGET_SYSTEM_TOOLS);
+  ASSIGN_DEFAULT_PATH(gnustepSystemAdminTools,
+    @GNUSTEP_TARGET_SYSTEM_ADMIN_TOOLS);
+  ASSIGN_DEFAULT_PATH(gnustepSystemLibrary,
+    @GNUSTEP_TARGET_SYSTEM_LIBRARY);
+  ASSIGN_DEFAULT_PATH(gnustepSystemLibraries,
+    @GNUSTEP_TARGET_SYSTEM_LIBRARIES);
+  ASSIGN_DEFAULT_PATH(gnustepSystemHeaders,
+    @GNUSTEP_TARGET_SYSTEM_HEADERS);
+  ASSIGN_DEFAULT_PATH(gnustepSystemDocumentation,
+    @GNUSTEP_TARGET_SYSTEM_DOC);
+  ASSIGN_DEFAULT_PATH(gnustepSystemDocumentationMan,
+    @GNUSTEP_TARGET_SYSTEM_DOC_MAN);
+  ASSIGN_DEFAULT_PATH(gnustepSystemDocumentationInfo,
+    @GNUSTEP_TARGET_SYSTEM_DOC_INFO);
 
-  ASSIGN_DEFAULT_PATH(gnustepNetworkApps, @GNUSTEP_TARGET_NETWORK_APPS);
-  ASSIGN_DEFAULT_PATH(gnustepNetworkAdminApps, @GNUSTEP_TARGET_NETWORK_ADMIN_APPS);
-  ASSIGN_DEFAULT_PATH(gnustepNetworkWebApps, @GNUSTEP_TARGET_NETWORK_WEB_APPS);
-  ASSIGN_DEFAULT_PATH(gnustepNetworkTools, @GNUSTEP_TARGET_NETWORK_TOOLS);
-  ASSIGN_DEFAULT_PATH(gnustepNetworkAdminTools, @GNUSTEP_TARGET_NETWORK_ADMIN_TOOLS);
-  ASSIGN_DEFAULT_PATH(gnustepNetworkLibrary, @GNUSTEP_TARGET_NETWORK_LIBRARY);
-  ASSIGN_DEFAULT_PATH(gnustepNetworkLibraries, @GNUSTEP_TARGET_NETWORK_LIBRARIES);
-  ASSIGN_DEFAULT_PATH(gnustepNetworkHeaders, @GNUSTEP_TARGET_NETWORK_HEADERS);
-  ASSIGN_DEFAULT_PATH(gnustepNetworkDocumentation, @GNUSTEP_TARGET_NETWORK_DOC);
-  ASSIGN_DEFAULT_PATH(gnustepNetworkDocumentationMan, @GNUSTEP_TARGET_NETWORK_DOC_MAN);
-  ASSIGN_DEFAULT_PATH(gnustepNetworkDocumentationInfo, @GNUSTEP_TARGET_NETWORK_DOC_INFO);
+  ASSIGN_DEFAULT_PATH(gnustepNetworkApps,
+    @GNUSTEP_TARGET_NETWORK_APPS);
+  ASSIGN_DEFAULT_PATH(gnustepNetworkAdminApps,
+    @GNUSTEP_TARGET_NETWORK_ADMIN_APPS);
+  ASSIGN_DEFAULT_PATH(gnustepNetworkWebApps,
+    @GNUSTEP_TARGET_NETWORK_WEB_APPS);
+  ASSIGN_DEFAULT_PATH(gnustepNetworkTools,
+    @GNUSTEP_TARGET_NETWORK_TOOLS);
+  ASSIGN_DEFAULT_PATH(gnustepNetworkAdminTools,
+    @GNUSTEP_TARGET_NETWORK_ADMIN_TOOLS);
+  ASSIGN_DEFAULT_PATH(gnustepNetworkLibrary,
+    @GNUSTEP_TARGET_NETWORK_LIBRARY);
+  ASSIGN_DEFAULT_PATH(gnustepNetworkLibraries,
+    @GNUSTEP_TARGET_NETWORK_LIBRARIES);
+  ASSIGN_DEFAULT_PATH(gnustepNetworkHeaders,
+    @GNUSTEP_TARGET_NETWORK_HEADERS);
+  ASSIGN_DEFAULT_PATH(gnustepNetworkDocumentation,
+    @GNUSTEP_TARGET_NETWORK_DOC);
+  ASSIGN_DEFAULT_PATH(gnustepNetworkDocumentationMan,
+    @GNUSTEP_TARGET_NETWORK_DOC_MAN);
+  ASSIGN_DEFAULT_PATH(gnustepNetworkDocumentationInfo,
+    @GNUSTEP_TARGET_NETWORK_DOC_INFO);
 
-  ASSIGN_DEFAULT_PATH(gnustepLocalApps, @GNUSTEP_TARGET_LOCAL_APPS);
-  ASSIGN_DEFAULT_PATH(gnustepLocalAdminApps, @GNUSTEP_TARGET_LOCAL_ADMIN_APPS);
-  ASSIGN_DEFAULT_PATH(gnustepLocalWebApps, @GNUSTEP_TARGET_LOCAL_WEB_APPS);
-  ASSIGN_DEFAULT_PATH(gnustepLocalTools, @GNUSTEP_TARGET_LOCAL_TOOLS);
-  ASSIGN_DEFAULT_PATH(gnustepLocalAdminTools, @GNUSTEP_TARGET_LOCAL_ADMIN_TOOLS);
-  ASSIGN_DEFAULT_PATH(gnustepLocalLibrary, @GNUSTEP_TARGET_LOCAL_LIBRARY);
-  ASSIGN_DEFAULT_PATH(gnustepLocalLibraries, @GNUSTEP_TARGET_LOCAL_LIBRARIES);
-  ASSIGN_DEFAULT_PATH(gnustepLocalHeaders, @GNUSTEP_TARGET_LOCAL_HEADERS);
-  ASSIGN_DEFAULT_PATH(gnustepLocalDocumentation, @GNUSTEP_TARGET_LOCAL_DOC);
-  ASSIGN_DEFAULT_PATH(gnustepLocalDocumentationMan, @GNUSTEP_TARGET_LOCAL_DOC_MAN);
-  ASSIGN_DEFAULT_PATH(gnustepLocalDocumentationInfo, @GNUSTEP_TARGET_LOCAL_DOC_INFO);
+  ASSIGN_DEFAULT_PATH(gnustepLocalApps,
+    @GNUSTEP_TARGET_LOCAL_APPS);
+  ASSIGN_DEFAULT_PATH(gnustepLocalAdminApps,
+    @GNUSTEP_TARGET_LOCAL_ADMIN_APPS);
+  ASSIGN_DEFAULT_PATH(gnustepLocalWebApps,
+    @GNUSTEP_TARGET_LOCAL_WEB_APPS);
+  ASSIGN_DEFAULT_PATH(gnustepLocalTools,
+    @GNUSTEP_TARGET_LOCAL_TOOLS);
+  ASSIGN_DEFAULT_PATH(gnustepLocalAdminTools,
+    @GNUSTEP_TARGET_LOCAL_ADMIN_TOOLS);
+  ASSIGN_DEFAULT_PATH(gnustepLocalLibrary,
+    @GNUSTEP_TARGET_LOCAL_LIBRARY);
+  ASSIGN_DEFAULT_PATH(gnustepLocalLibraries,
+    @GNUSTEP_TARGET_LOCAL_LIBRARIES);
+  ASSIGN_DEFAULT_PATH(gnustepLocalHeaders,
+    @GNUSTEP_TARGET_LOCAL_HEADERS);
+  ASSIGN_DEFAULT_PATH(gnustepLocalDocumentation,
+    @GNUSTEP_TARGET_LOCAL_DOC);
+  ASSIGN_DEFAULT_PATH(gnustepLocalDocumentationMan,
+    @GNUSTEP_TARGET_LOCAL_DOC_MAN);
+  ASSIGN_DEFAULT_PATH(gnustepLocalDocumentationInfo,
+    @GNUSTEP_TARGET_LOCAL_DOC_INFO);
 
-  ASSIGN_DEFAULT_PATH(gnustepMakefiles, @GNUSTEP_TARGET_MAKEFILES);
+  ASSIGN_DEFAULT_PATH(gnustepMakefiles,
+    @GNUSTEP_TARGET_MAKEFILES);
 
-  ASSIGN_DEFAULT_PATH(gnustepSystemUsersDir, @GNUSTEP_TARGET_SYSTEM_USERS_DIR);
-  ASSIGN_DEFAULT_PATH(gnustepNetworkUsersDir, @GNUSTEP_TARGET_NETWORK_USERS_DIR);
-  ASSIGN_DEFAULT_PATH(gnustepLocalUsersDir, @GNUSTEP_TARGET_LOCAL_USERS_DIR);
+  ASSIGN_DEFAULT_PATH(gnustepSystemUsersDir,
+    @GNUSTEP_TARGET_SYSTEM_USERS_DIR);
+  ASSIGN_DEFAULT_PATH(gnustepNetworkUsersDir,
+    @GNUSTEP_TARGET_NETWORK_USERS_DIR);
+  ASSIGN_DEFAULT_PATH(gnustepLocalUsersDir,
+    @GNUSTEP_TARGET_LOCAL_USERS_DIR);
 }
 
 static void
@@ -680,13 +782,13 @@ addDefaults(NSString *defs, NSMutableDictionary *conf)
 	{
 #if defined(__MINGW__)
 	  fprintf(stderr,
-	    "The file '%S' is writable by someone other than"
+	    "\nThe file '%S' is writable by someone other than"
 	    " its owner (permissions 0%lo).\nIgnoring it.\n",
 	    [defs fileSystemRepresentation],
 	    (long)[attributes filePosixPermissions]);
 #else
 	  fprintf(stderr,
-	    "The file '%s' is writable by someone other than"
+	    "\nThe file '%s' is writable by someone other than"
 	    " its owner (permissions 0%lo).\nIgnoring it.\n",
 	    [defs fileSystemRepresentation],
 	    (long)[attributes filePosixPermissions]);
@@ -695,7 +797,50 @@ addDefaults(NSString *defs, NSMutableDictionary *conf)
 	}
       else
 	{
-	  d = [NSDictionary dictionaryWithContentsOfFile: defs];
+	  NSString	*s;
+
+	  /* Here we load the string and convert to a property list
+	   * rather than using higher-level methods likely to raise
+	   * an exception or otherwise use the NSUserDefaults
+           * system and cause recursion on error.
+	   */
+	  s = [[NSString allocWithZone: NSDefaultMallocZone()]
+	    initWithContentsOfFile: defs];
+	  if (nil == s)
+	    {
+	      d = nil;
+	    }
+	  else
+	    {
+	      NS_DURING
+		{
+		  d = [s propertyList];
+		  if (NO == [d isKindOfClass: [NSDictionary class]])
+		    {
+		      d = nil;
+		    }
+		}
+	      NS_HANDLER
+		{
+		  d = nil;
+		}
+	      NS_ENDHANDLER
+	      RELEASE(s);
+	    }
+	  if (nil == d)
+	    {
+#if defined(__MINGW__)
+	  fprintf(stderr,
+	    "\nThe file '%S' is not parseable as a property list"
+	    " containing a dictionary.\nIgnoring it.\n",
+	    [defs fileSystemRepresentation]);
+#else
+	  fprintf(stderr,
+	    "\nThe file '%s' is not parseable as a property list"
+	    " containing a dictionary.\nIgnoring it.\n",
+	    [defs fileSystemRepresentation]);
+#endif
+	    }
 	}
 
       if (d != nil)
@@ -754,9 +899,12 @@ GNUstepConfig(NSDictionary *newConfig)
 	{
 	  if (newConfig == nil)
 	    {
-	      NSString	*file = nil;
-	      BOOL	fromEnvironment = YES;
-	      BOOL	bareDirectory = NO;
+	      NSString		*file = nil;
+	      NSString		*path;
+	      NSEnumerator	*e;
+	      NSString      	*defs;
+	      BOOL		fromEnvironment = YES;
+	      BOOL		bareDirectory = NO;
 
 	      conf = [[NSMutableDictionary alloc] initWithCapacity: 32];
 
@@ -789,8 +937,8 @@ GNUstepConfig(NSDictionary *newConfig)
 	        || [file hasPrefix: @"../"] == YES)
 		{
 		  Class		c = [NSProcessInfo class];
-		  NSString	*path = GSPrivateSymbolPath (c, 0);
 
+		  path = GSPrivateSymbolPath (c, 0);
 		  // Remove library name from path
 		  path = [path stringByDeletingLastPathComponent];
                   if ([file hasPrefix: @"./"] == YES)
@@ -834,46 +982,44 @@ GNUstepConfig(NSDictionary *newConfig)
 		    }
 #endif
 		}
+
 	      if (bareDirectory == YES)
 		{
+		  /* Set the directory name, but don't try to read file.
+		   */
 		  gnustepConfigPath = RETAIN(file);
 		}
 	      else
 		{
-		  NSEnumerator	*e;
-                  NSString      *defs;
-                  NSString      *path;
-
 		  gnustepConfigPath
 		    = RETAIN([file stringByDeletingLastPathComponent]);
 		  ParseConfigurationFile(file, conf, nil);
-
 		  if (nil != [conf objectForKey: @"GNUSTEP_EXTRA"])
 		    {
 		      NSLog(@"Warning: use of GNUSTEP_EXTRA in your GNUstep.conf file is deprecated.  Please use a GlobalDefaults.plist instead.\n");
 		    }
-		  /* Merge in any values from property lists in the 
-		   * GlobalDefaults directory.
-		   */
-                  path = [gnustepConfigPath stringByAppendingPathComponent:
-                    @"GlobalDefaults"];
-		  e = [[MGR() directoryContentsAtPath: path] objectEnumerator];
-		  while ((defs = [e nextObject]) != nil)
-		    {
-		      if ([[defs pathExtension] isEqualToString: @"plist"])
-			{
-			  defs = [path stringByAppendingPathComponent: defs];
-			  addDefaults(defs, conf);
-			}
-		    }
-
-		  /* And merge in value from GloablDefaults.plist
-		   */
-                  defs = [gnustepConfigPath stringByAppendingPathComponent:
-                    @"GlobalDefaults.plist"];
-
-		  addDefaults(defs, conf);
 		}
+
+	      /* Merge in any values from property lists in the 
+	       * GlobalDefaults directory.
+	       */
+	      path = [gnustepConfigPath stringByAppendingPathComponent:
+		@"GlobalDefaults"];
+	      e = [[MGR() directoryContentsAtPath: path] objectEnumerator];
+	      while ((defs = [e nextObject]) != nil)
+		{
+		  if ([[defs pathExtension] isEqualToString: @"plist"])
+		    {
+		      defs = [path stringByAppendingPathComponent: defs];
+		      addDefaults(defs, conf);
+		    }
+		}
+
+	      /* And merge in value from GlobalDefaults.plist
+	       */
+	      defs = [gnustepConfigPath stringByAppendingPathComponent:
+		@"GlobalDefaults.plist"];
+	      addDefaults(defs, conf);
 	    }
 	  else
 	    {
@@ -1908,7 +2054,7 @@ NSArray *
 NSSearchPathForDirectoriesInDomains(NSSearchPathDirectory directoryKey,
   NSSearchPathDomainMask domainMask, BOOL expandTilde)
 {
-  NSMutableArray  *paths = [NSMutableArray new];
+  NSMutableArray  *paths;
   NSString        *path;
   unsigned        i;
   unsigned        count;
@@ -1916,6 +2062,7 @@ NSSearchPathForDirectoriesInDomains(NSSearchPathDirectory directoryKey,
   InitialisePathUtilities();
 
   NSCAssert(gnustepMakefiles!=nil,@"Path utilities without initialisation!");
+  paths = [NSMutableArray array];
 
   /*
    * The order in which we return paths is important - user must come
@@ -2161,12 +2308,16 @@ if (domainMask & mask) \
 	    }
 
 	  ADD_PLATFORM_PATH(NSUserDomainMask, gnustepUserLibraries);
-	  if (full) ADD_PATH(NSUserDomainMask, gnustepUserLibraries, full);
-	  if (part) ADD_PATH(NSUserDomainMask, gnustepUserLibraries, part);
+	  if (full)
+	    ADD_PATH(NSUserDomainMask, gnustepUserLibraries, full);
+	  if (part)
+	    ADD_PATH(NSUserDomainMask, gnustepUserLibraries, part);
 
 	  ADD_PLATFORM_PATH(NSLocalDomainMask, gnustepLocalLibraries);
-	  if (full) ADD_PATH(NSLocalDomainMask, gnustepLocalLibraries, full);
-	  if (part) ADD_PATH(NSLocalDomainMask, gnustepLocalLibraries, part);
+	  if (full)
+	    ADD_PATH(NSLocalDomainMask, gnustepLocalLibraries, full);
+	  if (part)
+	    ADD_PATH(NSLocalDomainMask, gnustepLocalLibraries, part);
 
 	  ADD_PLATFORM_PATH(NSNetworkDomainMask, gnustepNetworkLibraries);
 	  if (full)
@@ -2175,8 +2326,10 @@ if (domainMask & mask) \
 	    ADD_PATH(NSNetworkDomainMask, gnustepNetworkLibraries, part);
 
 	  ADD_PLATFORM_PATH(NSSystemDomainMask, gnustepSystemLibraries);
-	  if (full) ADD_PATH(NSSystemDomainMask, gnustepSystemLibraries, full);
-	  if (part) ADD_PATH(NSSystemDomainMask, gnustepSystemLibraries, part);
+	  if (full)
+	    ADD_PATH(NSSystemDomainMask, gnustepSystemLibraries, full);
+	  if (part)
+	    ADD_PATH(NSSystemDomainMask, gnustepSystemLibraries, part);
 	}
 	break;
 
@@ -2197,20 +2350,28 @@ if (domainMask & mask) \
 	    }
 
 	  ADD_PLATFORM_PATH(NSUserDomainMask, gnustepUserTools);
-	  if (full) ADD_PATH(NSUserDomainMask, gnustepUserTools, full);
-	  if (part) ADD_PATH(NSUserDomainMask, gnustepUserTools, part);
+	  if (full)
+	    ADD_PATH(NSUserDomainMask, gnustepUserTools, full);
+	  if (part)
+	    ADD_PATH(NSUserDomainMask, gnustepUserTools, part);
 
 	  ADD_PLATFORM_PATH(NSLocalDomainMask, gnustepLocalTools);
-	  if (full) ADD_PATH(NSLocalDomainMask, gnustepLocalTools, full);
-	  if (part) ADD_PATH(NSLocalDomainMask, gnustepLocalTools, part);
+	  if (full)
+	    ADD_PATH(NSLocalDomainMask, gnustepLocalTools, full);
+	  if (part)
+	    ADD_PATH(NSLocalDomainMask, gnustepLocalTools, part);
 
 	  ADD_PLATFORM_PATH(NSNetworkDomainMask, gnustepNetworkTools);
-	  if (full) ADD_PATH(NSNetworkDomainMask, gnustepNetworkTools, full);
-	  if (part) ADD_PATH(NSNetworkDomainMask, gnustepNetworkTools, part);
+	  if (full)
+	    ADD_PATH(NSNetworkDomainMask, gnustepNetworkTools, full);
+	  if (part)
+	    ADD_PATH(NSNetworkDomainMask, gnustepNetworkTools, part);
 
 	  ADD_PLATFORM_PATH(NSSystemDomainMask, gnustepSystemTools);
-	  if (full) ADD_PATH(NSSystemDomainMask, gnustepSystemTools, full);
-	  if (part) ADD_PATH(NSSystemDomainMask, gnustepSystemTools, part);
+	  if (full)
+	    ADD_PATH(NSSystemDomainMask, gnustepSystemTools, full);
+	  if (part)
+	    ADD_PATH(NSSystemDomainMask, gnustepSystemTools, part);
 	}
 	break;
 
@@ -2231,20 +2392,28 @@ if (domainMask & mask) \
 	    }
 
 	  ADD_PLATFORM_PATH(NSUserDomainMask, gnustepUserAdminTools);
-	  if (full) ADD_PATH(NSUserDomainMask, gnustepUserAdminTools, full);
-	  if (part) ADD_PATH(NSUserDomainMask, gnustepUserAdminTools, part);
+	  if (full)
+	    ADD_PATH(NSUserDomainMask, gnustepUserAdminTools, full);
+	  if (part)
+	    ADD_PATH(NSUserDomainMask, gnustepUserAdminTools, part);
 
 	  ADD_PLATFORM_PATH(NSLocalDomainMask, gnustepLocalAdminTools);
-	  if (full) ADD_PATH(NSLocalDomainMask, gnustepLocalAdminTools, full);
-	  if (part) ADD_PATH(NSLocalDomainMask, gnustepLocalAdminTools, part);
+	  if (full)
+	    ADD_PATH(NSLocalDomainMask, gnustepLocalAdminTools, full);
+	  if (part)
+	    ADD_PATH(NSLocalDomainMask, gnustepLocalAdminTools, part);
 
 	  ADD_PLATFORM_PATH(NSNetworkDomainMask, gnustepNetworkAdminTools);
-	  if (full) ADD_PATH(NSNetworkDomainMask, gnustepNetworkAdminTools, full);
-	  if (part) ADD_PATH(NSNetworkDomainMask, gnustepNetworkAdminTools, part);
+	  if (full)
+	    ADD_PATH(NSNetworkDomainMask, gnustepNetworkAdminTools, full);
+	  if (part)
+	    ADD_PATH(NSNetworkDomainMask, gnustepNetworkAdminTools, part);
 
 	  ADD_PLATFORM_PATH(NSSystemDomainMask, gnustepSystemAdminTools);
-	  if (full) ADD_PATH(NSSystemDomainMask, gnustepSystemAdminTools, full);
-	  if (part) ADD_PATH(NSSystemDomainMask, gnustepSystemAdminTools, part);
+	  if (full)
+	    ADD_PATH(NSSystemDomainMask, gnustepSystemAdminTools, full);
+	  if (part)
+	    ADD_PATH(NSSystemDomainMask, gnustepSystemAdminTools, part);
 	}
 	break;
 
@@ -2278,6 +2447,5 @@ if (domainMask & mask) \
         }
     }
 
-  IF_NO_GC(AUTORELEASE (paths);)
   return paths;
 }
