@@ -21,7 +21,7 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2010-03-19 12:10:11 +0000 (Fri, 19 Mar 2010) $ $Revision: 30001 $
+   $Date: 2011-10-14 09:46:13 -0600 (Fri, 14 Oct 2011) $ $Revision: 33993 $
    */
 
 #import "common.h"
@@ -30,9 +30,6 @@
 
 #import "Foundation/NSFileHandle.h"
 #import "GSPrivate.h"
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 
 /**
  * <p>The NSPipe provides an encapsulation of the UNIX concept of pipe.<br />
