@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSPort class reference</title>
-   $Date: 2011-02-28 19:49:57 +0000 (Mon, 28 Feb 2011) $ $Revision: 32417 $
+   $Date: 2011-07-31 09:31:39 -0600 (Sun, 31 Jul 2011) $ $Revision: 33660 $
    */
 
 #import "common.h"
@@ -160,7 +160,7 @@ static Class	NSPort_concrete_class;
   [[NSNotificationCenter defaultCenter]
     postNotificationName: NSPortDidBecomeInvalidNotification
 		  object: self];
-  [arp release];
+  [arp drain];
 }
 
 - (BOOL) isValid
@@ -184,7 +184,7 @@ static Class	NSPort_concrete_class;
   return [super autorelease];
 }
 
-- (void) release
+- (oneway void) release
 {
   if (_is_valid && [self retainCount] == 1)
     {
