@@ -30,10 +30,15 @@
    Boston, MA 02111 USA.
 
    <title>NSPortCoder class reference</title>
-   $Date: 2011-03-29 10:40:14 +0100 (Tue, 29 Mar 2011) $ $Revision: 32733 $
+   $Date: 2011-12-17 07:05:25 -0700 (Sat, 17 Dec 2011) $ $Revision: 34308 $
    */
 
 #import "common.h"
+
+#if !defined (__GNU_LIBOBJC__)
+#  include <objc/encoding.h>
+#endif
+
 #define	EXPOSE_NSPortCoder_IVARS	1
 #import "Foundation/NSException.h"
 #import "Foundation/NSByteOrder.h"
@@ -1920,7 +1925,7 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 - (NSInteger) versionForClassName: (NSString*)className
 {
   GSClassInfo	*info = nil;
-  unsigned	version = NSNotFound;
+  NSInteger	version = NSNotFound;
   unsigned	count = GSIArrayCount(_clsAry);
 
   /*
