@@ -52,6 +52,14 @@
 // For pow()
 #include <math.h>
 
+#if     defined(HAVE_UNICODE_UREGEX_H)
+#include <unicode/uregex.h>
+#endif
+
+/* Object to represent the expression beign evaluated.
+ */
+static NSExpression	*evaluatedObjectExpression = nil;
+
 @interface GSPredicateScanner : NSScanner
 {
   NSEnumerator	*_args;		// Not retained.
@@ -804,109 +812,194 @@
      }
 }
 
-- (BOOL) _evaluateLeftValue: (id)leftResult rightValue: (id)rightResult
+#if	GS_USE_ICU == 1
+static BOOL
+GSICUStringMatchesRegex(NSString *string, NSString *regex, NSStringCompareOptions opts)
 {
-   unsigned compareOptions = 0;
-   BOOL leftIsNil;
-   BOOL rightIsNil;
-	
-   leftIsNil = (leftResult == nil || [leftResult isEqual: [NSNull null]]);
-   rightIsNil = (rightResult == nil || [rightResult isEqual: [NSNull null]]);
-   if (leftIsNil || rightIsNil)
-     {
-       /* One of the values is nil. The result is YES,
-        * if both are nil and equlality is requested.
-        */
-       return ((leftIsNil == rightIsNil)
-         && ((_type == NSEqualToPredicateOperatorType)
-         || (_type == NSLessThanOrEqualToPredicateOperatorType)
-         || (_type == NSGreaterThanOrEqualToPredicateOperatorType)));
-     }
+  BOOL result = NO;
+  UErrorCode error = 0;
+  uint32_t flags = 0;
+  NSUInteger stringLength = [string length];
+  NSUInteger regexLength = [regex length];
+  unichar *stringBuffer;
+  unichar *regexBuffer;
+  URegularExpression *icuregex = NULL;  
 
-   // Change predicate options into string options.
-   if (!(_options & NSDiacriticInsensitivePredicateOption))
-     {
-       compareOptions |= NSLiteralSearch;
-     }
-   if (_options & NSCaseInsensitivePredicateOption)
-     {
-       compareOptions |= NSCaseInsensitiveSearch;
-     }
+  stringBuffer = malloc(stringLength * sizeof(unichar));
+  if (NULL == stringBuffer) { return NO; }
+  regexBuffer = malloc(regexLength * sizeof(unichar));
+  if (NULL == regexBuffer) { free(stringBuffer); return NO; }
 
-   /* This is a very optimistic implementation,
-    * hoping that the values are of the right type.
-    */
-   switch (_type)
-     {
-       case NSLessThanPredicateOperatorType:
-         return ([leftResult compare: rightResult] == NSOrderedAscending);
-       case NSLessThanOrEqualToPredicateOperatorType:
-         return ([leftResult compare: rightResult] != NSOrderedDescending);
-       case NSGreaterThanPredicateOperatorType:
-         return ([leftResult compare: rightResult] == NSOrderedDescending);
-       case NSGreaterThanOrEqualToPredicateOperatorType:
-         return ([leftResult compare: rightResult] != NSOrderedAscending);
-       case NSEqualToPredicateOperatorType:
-         return [leftResult isEqual: rightResult];
-       case NSNotEqualToPredicateOperatorType:
-         return ![leftResult isEqual: rightResult];
-       case NSMatchesPredicateOperatorType:
-         // FIXME: Missing implementation of matches.
-         return [leftResult compare: rightResult options: compareOptions] == NSOrderedSame;  
-       case NSLikePredicateOperatorType:
-         // FIXME: Missing implementation of like.
-         return [leftResult compare: rightResult options: compareOptions] == NSOrderedSame;  
-       case NSBeginsWithPredicateOperatorType:
-         {
-           NSRange range = NSMakeRange(0, [rightResult length]);
-           return ([leftResult compare: rightResult options: compareOptions range: range] == NSOrderedSame);
-         }
-       case NSEndsWithPredicateOperatorType:
-         {
-           NSRange range = NSMakeRange([leftResult length] - [rightResult length], [rightResult length]);
-           return ([leftResult compare: rightResult options: compareOptions range: range] == NSOrderedSame);
-         }
-       case NSInPredicateOperatorType:
-         // Handle special case where rightResult is a collection and leftResult an element of it.
-         if (![rightResult isKindOfClass: [NSString class]])
-           {
-             NSEnumerator *e;
-             id value;
+  [string getCharacters: stringBuffer range: NSMakeRange(0, stringLength)];
+  [regex getCharacters: regexBuffer range: NSMakeRange(0, regexLength)];
 
-             if (![rightResult respondsToSelector: @selector(objectEnumerator)])
-               {
-                 [NSException raise: NSInvalidArgumentException 
-                              format: @"The right hand side for an IN operator must be a collection"];
-               }
+  flags |= UREGEX_DOTALL; // . is supposed to recognize newlines
+  if ((opts & NSCaseInsensitiveSearch) != 0) { flags |= UREGEX_CASE_INSENSITIVE; }
 
-             e = [rightResult objectEnumerator];
-             while ((value = [e nextObject]))
-               {
-                 if ([value isEqual: leftResult]) 
-                   return YES;		
-               }
+  icuregex = uregex_open(regexBuffer, regexLength, flags, NULL, &error);
+  if (icuregex != NULL && U_SUCCESS(error))
+    {
+      uregex_setText(icuregex, stringBuffer, stringLength, &error);
+      result = uregex_matches(icuregex, 0, &error);
+    }
+  uregex_close(icuregex);
 
-             return NO;
-           }
-         return ([rightResult rangeOfString: leftResult options: compareOptions].location != NSNotFound);
-       case NSCustomSelectorPredicateOperatorType:
-         {
-           BOOL (*function)(id,SEL,id) = (BOOL (*)(id,SEL,id))[leftResult methodForSelector: _selector];
-           return function(leftResult, _selector, rightResult);
-         }
-       default:
-         return NO;
-     }
+  free(stringBuffer);
+  free(regexBuffer);
+
+  return result;
+}
+#endif
+
+- (BOOL) _evaluateLeftValue: (id)leftResult
+		 rightValue: (id)rightResult
+		     object: (id)object
+{
+  unsigned compareOptions = 0;
+  BOOL leftIsNil;
+  BOOL rightIsNil;
+
+  if (leftResult == evaluatedObjectExpression)
+    {
+      leftResult = object;
+    }
+  if (rightResult == evaluatedObjectExpression)
+    {
+      rightResult = object;
+    }
+
+  leftIsNil = (leftResult == nil || [leftResult isEqual: [NSNull null]]);
+  rightIsNil = (rightResult == nil || [rightResult isEqual: [NSNull null]]);
+  if (leftIsNil || rightIsNil)
+    {
+      /* One of the values is nil. The result is YES,
+       * if both are nil and equality is requested.
+       */
+      return ((leftIsNil == rightIsNil)
+	&& ((_type == NSEqualToPredicateOperatorType)
+	|| (_type == NSLessThanOrEqualToPredicateOperatorType)
+	|| (_type == NSGreaterThanOrEqualToPredicateOperatorType)));
+    }
+
+  // Change predicate options into string options.
+  if (!(_options & NSDiacriticInsensitivePredicateOption))
+    {
+      compareOptions |= NSLiteralSearch;
+    }
+  if (_options & NSCaseInsensitivePredicateOption)
+    {
+      compareOptions |= NSCaseInsensitiveSearch;
+    }
+
+  /* This is a very optimistic implementation,
+   * hoping that the values are of the right type.
+   */
+  switch (_type)
+    {
+      case NSLessThanPredicateOperatorType:
+	return ([leftResult compare: rightResult] == NSOrderedAscending);
+      case NSLessThanOrEqualToPredicateOperatorType:
+	return ([leftResult compare: rightResult] != NSOrderedDescending);
+      case NSGreaterThanPredicateOperatorType:
+	return ([leftResult compare: rightResult] == NSOrderedDescending);
+      case NSGreaterThanOrEqualToPredicateOperatorType:
+	return ([leftResult compare: rightResult] != NSOrderedAscending);
+      case NSEqualToPredicateOperatorType:
+	return [leftResult isEqual: rightResult];
+      case NSNotEqualToPredicateOperatorType:
+	return ![leftResult isEqual: rightResult];
+      case NSMatchesPredicateOperatorType:
+#if	GS_USE_ICU == 1
+	return GSICUStringMatchesRegex(leftResult, rightResult, compareOptions);
+#else
+	return [leftResult compare: rightResult options: compareOptions]
+	  == NSOrderedSame;  
+#endif
+      case NSLikePredicateOperatorType:
+#if	GS_USE_ICU == 1
+	{
+	  NSString *regex;
+
+	  /* The right hand is a pattern with '?' meaning match one character,
+	   * and '*' meaning match zero or more characters, so translate that
+	   * into a regex.
+	   */
+	  regex = [rightResult stringByReplacingOccurrencesOfString: @"*"
+							 withString: @".*"];
+	  regex = [regex stringByReplacingOccurrencesOfString: @"?"
+						   withString: @".?"];
+	  regex = [NSString stringWithFormat: @"^%@$", regex];
+	  return GSICUStringMatchesRegex(leftResult, regex, compareOptions);
+	}
+#else
+	return [leftResult compare: rightResult options: compareOptions]
+	  == NSOrderedSame;
+#endif
+      case NSBeginsWithPredicateOperatorType:
+	{
+	  NSRange range = NSMakeRange(0, [rightResult length]);
+	  return ([leftResult compare: rightResult
+			      options: compareOptions
+				range: range] == NSOrderedSame);
+	}
+      case NSEndsWithPredicateOperatorType:
+	{
+	  NSRange range;
+
+	  range = NSMakeRange([leftResult length] - [rightResult length],
+	    [rightResult length]);
+	  return ([leftResult compare: rightResult
+			      options: compareOptions
+				range: range] == NSOrderedSame);
+	}
+      case NSInPredicateOperatorType:
+	/* Handle special case where rightResult is a collection
+	 * and leftResult an element of it.
+	 */
+	if (![rightResult isKindOfClass: [NSString class]])
+	  {
+	    NSEnumerator *e;
+	    id value;
+
+	    if (![rightResult respondsToSelector: @selector(objectEnumerator)])
+	      {
+		[NSException raise: NSInvalidArgumentException 
+			    format: @"The right hand side for an IN operator "
+		  @"must be a collection"];
+	      }
+
+	    e = [rightResult objectEnumerator];
+	    while ((value = [e nextObject]))
+	      {
+		if ([value isEqual: leftResult]) 
+		  return YES;		
+	      }
+
+	    return NO;
+	  }
+	return ([rightResult rangeOfString: leftResult
+				   options: compareOptions].location
+	  != NSNotFound);
+      case NSCustomSelectorPredicateOperatorType:
+	{
+	  BOOL (*function)(id,SEL,id) = (BOOL (*)(id,SEL,id))[leftResult methodForSelector: _selector];
+	  return function(leftResult, _selector, rightResult);
+	}
+      default:
+	return NO;
+    }
 }
 
 - (BOOL) evaluateWithObject: (id)object
 {
   id leftValue = [_left expressionValueWithObject: object context: nil];
   id rightValue = [_right expressionValueWithObject: object context: nil];
-	
+
   if (_modifier == NSDirectPredicateModifier)
     {
-      return [self _evaluateLeftValue: leftValue rightValue: rightValue];
+      return [self _evaluateLeftValue: leftValue
+			   rightValue: rightValue
+			       object: object];
     }
   else
     {		
@@ -914,6 +1007,11 @@
       NSEnumerator *e;
       id value;
 
+      if (leftValue == evaluatedObjectExpression)
+	{
+	  leftValue = object;
+	}
+
       if (![leftValue respondsToSelector: @selector(objectEnumerator)])
         {
           [NSException raise: NSInvalidArgumentException 
@@ -923,7 +1021,9 @@
       e = [leftValue objectEnumerator];
       while ((value = [e nextObject]))
         {
-          BOOL eval = [self _evaluateLeftValue: value rightValue: rightValue];
+          BOOL eval = [self _evaluateLeftValue: value
+				    rightValue: rightValue
+					object: object];
           if (eval != result) 
             return eval;		
         }
@@ -966,6 +1066,14 @@
 
 @implementation NSExpression
 
++ (void) initialize
+{
+  if (self == [NSExpression class] && nil == evaluatedObjectExpression)
+    {
+      evaluatedObjectExpression = [GSEvaluatedObjectExpression new];
+    }
+}
+
 + (NSExpression *) expressionForConstantValue: (id)obj
 {
   GSConstantValueExpression *e;
@@ -978,11 +1086,7 @@
 
 + (NSExpression *) expressionForEvaluatedObject
 {
-  GSEvaluatedObjectExpression *e;
-
-  e = [[GSEvaluatedObjectExpression alloc] 
-          initWithExpressionType: NSEvaluatedObjectExpressionType];
-  return AUTORELEASE(e);
+  return evaluatedObjectExpression;
 }
 
 + (NSExpression *) expressionForFunction: (NSString *)name
