@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSProcessInfo class reference</title>
-   $Date: 2011-03-17 22:33:31 +0000 (Thu, 17 Mar 2011) $ $Revision: 32638 $
+   $Date: 2012-01-03 03:31:41 -0700 (Tue, 03 Jan 2012) $ $Revision: 34399 $
 */
 
 /*************************************************************************
@@ -55,13 +55,6 @@
  *************************************************************************/
 
 #import "common.h"
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#ifdef HAVE_STRERROR
-#include <errno.h>
-#endif /* HAVE_STRERROR */
 
 #include <stdio.h>
 #include <string.h>
@@ -70,24 +63,39 @@
 #  include <windows.h>
 #endif
 
-#ifdef  HAVE_SYS_SIGNAL_H
-#include <sys/signal.h>
-#endif
-#ifdef  HAVE_SIGNAL_H
-#include <signal.h>
+#ifdef HAVE_STRERROR
+#  include <errno.h>
 #endif
 
-#include <sys/file.h>
-#ifdef HAVE_SYS_FCNTL_H
-#include <sys/fcntl.h>
+#if	defined(HAVE_SYS_SIGNAL_H)
+#  include	<sys/signal.h>
+#elif	defined(HAVE_SIGNAL_H)
+#  include	<signal.h>
 #endif
+
+#if	defined(HAVE_SYS_FILE_H)
+#  include <sys/file.h>
+#endif
+
+#if	defined(HAVE_SYS_FCNTL_H)
+#  include <sys/fcntl.h>
+#elif	defined(HAVE_FCNTL_H)
+#  include <fcntl.h>
+#endif
+
 #ifdef HAVE_SYS_UTSNAME_H
 #include <sys/utsname.h>
 #endif
 
 #ifdef HAVE_KVM_ENV
 #include <kvm.h>
-#include <fcntl.h>
+
+#if	defined(HAVE_SYS_FCNTL_H)
+#  include	<sys/fcntl.h>
+#elif	defined(HAVE_FCNTL_H)
+#  include	<fcntl.h>
+#endif
+
 #include <sys/param.h>
 #endif /* HAVE_KVM_ENV */
 
@@ -108,9 +116,11 @@
 #import "GNUstepBase/GSConfig.h"
 #import "Foundation/NSArray.h"
 #import "Foundation/NSSet.h"
+#import "Foundation/NSCharacterSet.h"
 #import "Foundation/NSDictionary.h"
 #import "Foundation/NSDate.h"
 #import "Foundation/NSException.h"
+#import "Foundation/NSFileManager.h"
 #import "Foundation/NSProcessInfo.h"
 #import "Foundation/NSAutoreleasePool.h"
 #import "Foundation/NSHost.h"
@@ -211,7 +221,7 @@ static NSString		*_gnu_processName = nil;
 static NSArray		*_gnu_arguments = nil;
 
 // Dictionary of environment vars and their values
-static NSMutableDictionary	*_gnu_environment = nil;
+static NSDictionary	*_gnu_environment = nil;
 
 // The operating system we are using.
 static unsigned int	_operatingSystem = 0;
@@ -449,7 +459,7 @@ _gnu_process_args(int argc, char *argv[], char *env[])
     IF_NO_GC(RELEASE(keys));
     IF_NO_GC(RELEASE(values));
   }
-  IF_NO_GC(RELEASE(arp));
+  [arp drain];
 }
 
 #if !GS_FAKE_MAIN && ((defined(HAVE_PROCFS)  || defined(HAVE_KVM_ENV) || defined(HAVE_PROCFS_PSINFO) || defined(__APPLE__)) && (defined(HAVE_LOAD_METHOD)))
@@ -890,7 +900,7 @@ _gnu_noobjc_free_vars(void)
   if (self == [NSProcessInfo class]
     && !_gnu_processName && !_gnu_arguments && !_gnu_environment)
     {
-      if(_gnu_noobjc_argv == 0 || _gnu_noobjc_env == 0)
+      if (_gnu_noobjc_argv == 0 || _gnu_noobjc_env == 0)
 	{
           _NSLog_printf_handler(_GNU_MISSING_MAIN_FUNCTION_CALL);
           exit(1);
@@ -1458,7 +1468,8 @@ GSInitializeProcess(int argc, char **argv, char **envp)
   int		desc;
 
 #if	defined(__MINGW__)
-  desc = _wopen([path fileSystemRepresentation], O_RDWR|O_CREAT|O_APPEND, 0644);
+  desc = _wopen((wchar_t*)[path fileSystemRepresentation],
+    O_RDWR|O_CREAT|O_APPEND, 0644);
 #else
   desc = open([path fileSystemRepresentation], O_RDWR|O_CREAT|O_APPEND, 0644);
 #endif
