@@ -86,7 +86,7 @@ extern BOOL GSScanDouble(unichar*, unsigned, double*);
   NSXMLParser				*theParser;
   NSMutableString			*value;
   NSMutableArray			*stack;
-  NSString				*key;
+  id					key;
   BOOL					inArray;
   BOOL					inDictionary;
   BOOL					inString;
@@ -2370,7 +2370,7 @@ static BOOL	classInitialized = NO;
 + (NSData *) dataWithPropertyList: (id)aPropertyList
                            format: (NSPropertyListFormat)aFormat
                           options: (NSPropertyListWriteOptions)anOption
-                            error: (NSError**)error
+                            error: (out NSError**)error
 {
   NSMutableData	*dest;
   NSDictionary	*loc;
@@ -2520,7 +2520,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
 + (id) propertyListWithData: (NSData*)data
                     options: (NSPropertyListReadOptions)anOption
                      format: (NSPropertyListFormat*)aFormat
-                      error: (NSError**)error
+                      error: (out NSError**)error
 {
   NSPropertyListFormat	format = 0;
   NSString           *errorStr = nil;
@@ -2672,7 +2672,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
 + (id) propertyListWithStream: (NSInputStream*)stream
                       options: (NSPropertyListReadOptions)anOption
                        format: (NSPropertyListFormat*)aFormat
-                        error: (NSError**)error
+                        error: (out NSError**)error
 {
   // FIXME
   return nil;
@@ -2682,7 +2682,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
                        toStream: (NSOutputStream*)stream
                          format: (NSPropertyListFormat)aFormat
                         options: (NSPropertyListWriteOptions)anOption
-                          error: (NSError**)error
+                          error: (out NSError**)error
 {
   // FIXME: The NSData operations should be implemented on top of this method, 
   // not the other way round,
@@ -2998,28 +2998,18 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
       unsigned		i;
       unsigned char	buffer[16];
 
+      if (len > sizeof(unsigned long long))
+        {
+          [NSException raise: NSInvalidArgumentException
+                      format: @"Stored number too long (%d bytes) in property list", len];
+       }
+
       [data getBytes: buffer range: NSMakeRange(counter, len)];
       for (i = 0; i < len; i++)
         {
 	  num = (num << 8) + buffer[i];
 	}
-
-      if (next == 0x10)
-        {
-	  result = [NSNumber numberWithUnsignedChar: (unsigned char)num];
-	}
-      else if (next == 0x11)
-        {
-	  result = [NSNumber numberWithUnsignedShort: (unsigned short)num];
-	}
-      else if ((next == 0x12) || (next == 13))
-        {
-	  result = [NSNumber numberWithUnsignedInt: (unsigned int)num];
-	}
-      else
-        {
-	  result = [NSNumber numberWithUnsignedLongLong: num];
-	}
+      result = [NSNumber numberWithLongLong: (long long)num];
     }
   else if (next == 0x22)
     {
