@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSProxy class reference</title>
-   $Date: 2011-02-20 16:21:43 +0000 (Sun, 20 Feb 2011) $ $Revision: 32247 $
+   $Date: 2011-07-26 04:47:16 -0600 (Tue, 26 Jul 2011) $ $Revision: 33634 $
    */
 
 #import "common.h"
@@ -34,11 +34,15 @@
 #import "Foundation/NSDistantObject.h"
 #import "Foundation/NSPortCoder.h"
 
-@class	NSDistantObject;
-
-#ifndef NeXT_RUNTIME
-extern BOOL __objc_responds_to(id, SEL);
+// Get objc_delete_weak_refs(), if it is present in the runtime.
+#ifdef __GNUSTEP_RUNTIME__
+#include <objc/capabilities.h>
+#ifdef OBJC_CAP_ARC
+#include <objc/objc-arc.h>
 #endif
+#endif
+
+@class	NSDistantObject;
 
 /**
  * <p>The NSProxy class provides a basic implementation of a class whose
@@ -168,7 +172,7 @@ extern BOOL __objc_responds_to(id, SEL);
 /**
  * A dummy method to ensure that the class can safely be held in containers.
  */
-+ (void) release
++ (oneway void) release
 {
   /* Do nothing	*/
 }
@@ -178,7 +182,7 @@ extern BOOL __objc_responds_to(id, SEL);
  */
 + (BOOL) respondsToSelector: (SEL)aSelector
 {
-  if (__objc_responds_to(self, aSelector))
+  if (class_respondsToSelector(object_getClass(self), aSelector))
     return YES;
   else
     return NO;
@@ -449,17 +453,16 @@ extern BOOL __objc_responds_to(id, SEL);
  * Decrement the retain count for the receiver ... deallocate if it would
  * become negative.
  */
-- (void) release
+- (oneway void) release
 {
-#if	GS_WITH_GC == 0
-  if (_retain_count == 0)
+#if	(GS_WITH_GC == 0)
+  if (NSDecrementExtraRefCountWasZero(self))
     {
+#  ifdef OBJC_CAP_ARC
+      objc_delete_weak_refs(self);
+#  endif
       [self dealloc];
     }
-  else
-    {
-      _retain_count--;
-    }
 #endif
 }
 
@@ -497,7 +500,7 @@ extern BOOL __objc_responds_to(id, SEL);
     {
       return NO;
     }
-  if (__objc_responds_to(self, aSelector))
+  if (class_respondsToSelector(object_getClass(self), aSelector))
     {
       return YES;
     }
@@ -522,9 +525,7 @@ extern BOOL __objc_responds_to(id, SEL);
  */
 - (id) retain
 {
-#if	GS_WITH_GC == 0
-  _retain_count++;
-#endif
+  NSIncrementExtraRefCount(self);
   return self;
 }
 
@@ -533,7 +534,7 @@ extern BOOL __objc_responds_to(id, SEL);
  */
 - (NSUInteger) retainCount
 {
-  return _retain_count + 1;
+  return NSExtraRefCount(self) + 1;
 }
 
 /**
