@@ -26,7 +26,7 @@
    Boston, MA 02111 USA.
 
    <title>NSRunLoop class reference</title>
-   $Date: 2011-03-09 12:40:50 +0000 (Wed, 09 Mar 2011) $ $Revision: 32517 $
+   $Date: 2011-10-14 09:46:13 -0600 (Fri, 14 Oct 2011) $ $Revision: 33993 $
 */
 
 #import "common.h"
@@ -59,9 +59,6 @@
 #ifdef HAVE_POLL_F
 #include <poll.h>
 #endif
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #include <math.h>
 #include <time.h>
 #include <string.h>		/* for memset() */
@@ -486,7 +483,7 @@ static inline BOOL timerInvalidated(NSTimer *t)
 	    }
 	}
     }
-  [arp release];
+  [arp drain];
 }
 
 /**
@@ -757,7 +754,7 @@ static inline BOOL timerInvalidated(NSTimer *t)
                                            userInfo: nil
                                             repeats: YES];
           context->housekeeper = timer;
-          [arp release];
+          [arp drain];
         }
     }
   return current;
@@ -1163,7 +1160,7 @@ updateTimer(NSTimer *t, NSDate *d, NSTimeInterval now)
 	    }
 	  GSPrivateNotifyASAP(_currentMode);
 	  _currentMode = savedMode;
-	  [arp release];
+	  [arp drain];
 	  NS_VOIDRETURN;
 	}
 
@@ -1221,7 +1218,7 @@ updateTimer(NSTimer *t, NSDate *d, NSTimeInterval now)
       [localException raise];
     }
   NS_ENDHANDLER
-  [arp release];
+  [arp drain];
 }
 
 /**
@@ -1247,7 +1244,7 @@ updateTimer(NSTimer *t, NSDate *d, NSTimeInterval now)
   d = [self limitDateForMode: mode];
   if (d == nil)
     {
-      [arp release];
+      [arp drain];
       return NO;
     }
 
@@ -1266,7 +1263,7 @@ updateTimer(NSTimer *t, NSDate *d, NSTimeInterval now)
   [self acceptInputForMode: mode beforeDate: d];
 
   [d release];
-  [arp release];
+  [arp drain];
   return YES;
 }
 
