@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSScanner class reference</title>
-   $Date: 2010-09-09 22:41:40 +0100 (Thu, 09 Sep 2010) $ $Revision: 31278 $
+   $Date: 2011-10-14 04:57:40 -0600 (Fri, 14 Oct 2011) $ $Revision: 33982 $
 */
 
 #import "common.h"
@@ -72,7 +72,6 @@ static Class		GSCStringClass;
 static Class		GSUnicodeStringClass;
 static Class		GSMutableStringClass;
 static Class		GSPlaceholderStringClass;
-static Class		NSConstantStringClass;
 static id		_holder;
 static NSCharacterSet	*defaultSkipSet;
 static SEL		memSel;
@@ -139,7 +138,6 @@ typedef GSString	*ivars;
       GSUnicodeStringClass = [GSUnicodeString class];
       GSMutableStringClass = [GSMutableString class];
       GSPlaceholderStringClass = [GSPlaceholderString class];
-      NSConstantStringClass = [NSString constantStringClass];
       _holder = (id)NSAllocateObject(GSPlaceholderStringClass, 0, 0);
       externalEncoding = [NSString defaultCStringEncoding];
       if (GSPrivateIsByteEncoding(externalEncoding) == YES)
@@ -203,40 +201,20 @@ typedef GSString	*ivars;
     }
 
   c = object_getClass(aString);
+  if (GSObjCIsKindOf(c, GSMutableStringClass) == YES)
+    {
+      _string = [_holder initWithString: aString];
+    }
   if (GSObjCIsKindOf(c, GSUnicodeStringClass) == YES)
     {
-      _isUnicode = YES;
       _string = RETAIN(aString);
     }
   else if (GSObjCIsKindOf(c, GSCStringClass) == YES)
     {
-      _isUnicode = NO;
-      _string = RETAIN(aString);
-    }
-  else if (GSObjCIsKindOf(c, GSMutableStringClass) == YES)
-    {
-      if (((ivars)aString)->_flags.wide == 1)
-	{
-	  _isUnicode = YES;
-	  _string = [_holder initWithCharacters: ((ivars)aString)->_contents.u
-					 length: ((ivars)aString)->_count];
-	}
-      else
-	{
-	  _isUnicode = NO;
-	  _string = [_holder initWithBytes: ((ivars)aString)->_contents.c
-				    length: ((ivars)aString)->_count
-				  encoding: internalEncoding];
-	}
-    }
-  else if (c == NSConstantStringClass)
-    {
-      _isUnicode = NO;
       _string = RETAIN(aString);
     }
   else if ([aString isKindOfClass: NSStringClass])
     {
-      _isUnicode = YES;
       _string = [_holder initWithString: aString];
     }
   else
@@ -245,6 +223,11 @@ typedef GSString	*ivars;
       NSLog(@"Scanner initialised with something not a string");
       return nil;
     }
+  c = object_getClass(_string);
+  if (GSObjCIsKindOf(c, GSUnicodeStringClass) == YES)
+    {
+      _isUnicode = YES;
+    }
   [self setCharactersToBeSkipped: defaultSkipSet];
   _decimal = '.';
   return self;
