@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSSet class reference</title>
-   $Date: 2011-03-05 15:01:35 +0000 (Sat, 05 Mar 2011) $ $Revision: 32459 $
+   $Date: 2012-01-01 00:38:53 -0700 (Sun, 01 Jan 2012) $ $Revision: 34374 $
    */
 
 #import "common.h"
@@ -110,7 +110,7 @@ static Class NSMutableSet_concrete_class;
 /**
  *  New set containing (unique elements of) objects.
  */
-+ (id) setWithObjects: (id*)objects
++ (id) setWithObjects: (const id[])objects
 	        count: (NSUInteger)count
 {
   return AUTORELEASE([[self allocWithZone: NSDefaultMallocZone()]
@@ -314,7 +314,7 @@ static Class NSMutableSet_concrete_class;
  * and needs to be re-implemented in subclasses in order to have all
  * other initialisers work.
  */
-- (id) initWithObjects: (id*)objects
+- (id) initWithObjects: (const id[])objects
 		 count: (NSUInteger)count
 {
   self = [self init];
@@ -872,10 +872,10 @@ static Class NSMutableSet_concrete_class;
   FOR_IN (id, obj, enumerator)
   {
     CALL_BLOCK(aBlock, obj, &shouldStop);
-    if(shouldStop)
-    {
-      return;
-    }
+    if (shouldStop)
+      {
+	return;
+      }
   }
   END_FOR_IN(enumerator)
 }
@@ -1005,7 +1005,7 @@ static Class NSMutableSet_concrete_class;
   [self subclassResponsibility: _cmd];
 }
 
-- (id) initWithObjects: (id*)objects
+- (id) initWithObjects: (const id[])objects
 		 count: (NSUInteger)count
 {
   self = [self initWithCapacity: count];
