@@ -43,14 +43,11 @@
 #import "Foundation/NSThread.h"
 #import "Foundation/NSConnection.h"
 
-#import "GSNetwork.h"
 #import "GSPortPrivate.h"
 #import "GSPrivate.h"
+#import "GSNetwork.h"
 
 #include <stdio.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 
 #ifdef __MINGW__
 #define close closesocket
@@ -60,10 +57,15 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>		/* for inet_ntoa() */
 #endif /* !__MINGW__ */
-#include <errno.h>
 #include <string.h>		/* for strchr() */
 #include <ctype.h>		/* for strchr() */
-#include <fcntl.h>
+
+#if	defined(HAVE_SYS_FCNTL_H)
+#  include	<sys/fcntl.h>
+#elif	defined(HAVE_FCNTL_H)
+#  include	<fcntl.h>
+#endif
+
 #ifdef __MINGW__
 #include <winsock2.h>
 #include <wininet.h>
@@ -74,7 +76,11 @@
 #include <sys/resource.h>
 #include <netdb.h>
 #include <sys/socket.h>
-#include <sys/file.h>
+
+#if	defined(HAVE_SYS_FILE_H)
+#  include	<sys/file.h>
+#endif
+
 /*
  *	Stuff for setting the sockets into non-blocking mode.
  */
@@ -216,7 +222,7 @@ typedef enum {
   BOOL			valid;
   NSSocketPort		*recvPort;
   NSSocketPort		*sendPort;
-  struct sockaddr_in	sockAddr;	/* Far end of connection.	*/
+  struct sockaddr	sockAddr;	/* Far end of connection.	*/
   NSString		*defaultAddress;
 }
 
@@ -502,7 +508,7 @@ static Class	runLoopClass;
     }
   while (gotAddr == NO)
     {
-      const char	*addr;
+      NSString	*addr;
 
       if (addrNum >= [addrs count])
 	{
@@ -511,40 +517,28 @@ static Class	runLoopClass;
 	  M_UNLOCK(myLock);
 	  return NO;
 	}
-      addr = [[addrs objectAtIndex: addrNum++] cString];
+      addr = [addrs objectAtIndex: addrNum++];
 
-      memset(&sockAddr, '\0', sizeof(sockAddr));
-      sockAddr.sin_family = AF_INET;
-#ifndef HAVE_INET_ATON
-      sockAddr.sin_addr.s_addr = inet_addr(addr);
-      if (sockAddr.sin_addr.s_addr == INADDR_NONE)
-#else
-      if (inet_aton(addr, &sockAddr.sin_addr) == 0)
-#endif
+      if (NO == GSPrivateSockaddrSetup(addr,
+	[aPort portNumber], nil, nil, &sockAddr))
 	{
-	  NSLog(@"bad ip address - '%s'", addr);
+	  NSLog(@"bad address - '%@'", addr);
 	}
       else
 	{
 	  gotAddr = YES;
-	  NSDebugMLLog(@"GSTcpHandle", @"Connecting to %s:%d using desc %d",
+	  NSDebugMLLog(@"GSTcpHandle", @"Connecting to %@:%d using desc %d",
 	    addr, [aPort portNumber], desc);
 	}
     }
-  sockAddr.sin_port = GSSwapHostI16ToBig([aPort portNumber]);
 
-  if (connect(desc, (struct sockaddr*)&sockAddr, sizeof(sockAddr))
-    == SOCKET_ERROR)
+  if (connect(desc, (struct sockaddr*)&sockAddr,
+    GSPrivateSockaddrLength(&sockAddr)) == SOCKET_ERROR)
     {
-#ifdef __MINGW__
-      if (WSAGetLastError() != WSAEWOULDBLOCK)
-#else
-      if (errno != EINPROGRESS)
-#endif
+      if (!GSWOULDBLOCK)
 	{
-	  NSLog(@"unable to make connection to %s:%d - %@",
-	    inet_ntoa(sockAddr.sin_addr),
-	    GSSwapBigI16ToHost(sockAddr.sin_port), [NSError _last]);
+	  NSLog(@"unable to make connection to %@ - %@",
+	    GSPrivateSockaddrName(&sockAddr), [NSError _last]);
 	  if (addrNum < [addrs count])
 	    {
 	      BOOL	result;
@@ -684,7 +678,7 @@ static Class	runLoopClass;
 - (NSString*) description
 {
   return [NSString stringWithFormat: @"Handle (%d) to %s:%d",
-    desc, inet_ntoa(sockAddr.sin_addr), ntohs(sockAddr.sin_port)];
+    desc, GSPrivateSockaddrName(&sockAddr)];
 }
 
 - (int) descriptor
@@ -1118,9 +1112,7 @@ static Class	runLoopClass;
           len = send(desc, [d bytes], [d length], 0);
           if (len == (int)[d length])
             {
-	      RELEASE(defaultAddress);
-	      defaultAddress = RETAIN([NSString stringWithUTF8String:
-	        inet_ntoa(sockAddr.sin_addr)]);
+	      ASSIGN(defaultAddress, GSPrivateSockaddrHost(&sockAddr));
 	      NSDebugMLLog(@"GSTcpHandle",
 	        @"wrote %d bytes on 0x%x", len, self);
 	      state = GS_H_CONNECTED;
@@ -1482,7 +1474,7 @@ static Class		tcpPortClass;
   if (self == [NSSocketPort class])
     {
       tcpPortClass = self;
-      tcpPortMap = NSCreateMapTable(NSIntMapKeyCallBacks,
+      tcpPortMap = NSCreateMapTable(NSIntegerMapKeyCallBacks,
 	NSNonOwnedPointerMapValueCallBacks, 0);
 
       tcpPortLock = [GSLazyRecursiveLock new];
@@ -1579,12 +1571,12 @@ static Class		tcpPortClass;
       port->listener = -1;
       port->host = RETAIN(aHost);
       port->address = [addr copy];
-      port->handles = NSCreateMapTable(NSIntMapKeyCallBacks,
+      port->handles = NSCreateMapTable(NSIntegerMapKeyCallBacks,
 	NSObjectMapValueCallBacks, 0);
 #if	defined(__MINGW__)
       port->eventListener = WSA_INVALID_EVENT;
-      port->events = NSCreateMapTable(NSIntMapKeyCallBacks,
-        NSIntMapValueCallBacks, 0);
+      port->events = NSCreateMapTable(NSIntegerMapKeyCallBacks,
+        NSIntegerMapValueCallBacks, 0);
 #endif
       port->myLock = [GSLazyRecursiveLock new];
       port->_is_valid = YES;
@@ -1596,34 +1588,16 @@ static Class		tcpPortClass;
 #endif
 	  SOCKET desc;
 	  BOOL	addrOk = YES;
-	  struct sockaddr_in	sockaddr;
+	  struct sockaddr	sockaddr;
 
 	  /*
 	   * Creating a new port on the local host - so we must create a
 	   * listener socket to accept incoming connections.
 	   */
-	  memset(&sockaddr, '\0', sizeof(sockaddr));
-	  sockaddr.sin_family = AF_INET;
-	  if (addr == nil)
-	    {
-	      sockaddr.sin_addr.s_addr = GSSwapHostI32ToBig(INADDR_ANY);
-	    }
-	  else
-	    {
-#ifndef HAVE_INET_ATON
-	      sockaddr.sin_addr.s_addr = inet_addr([addr cString]);
-	      if (sockaddr.sin_addr.s_addr == INADDR_NONE)
-#else
-	      if (inet_aton([addr cString], &sockaddr.sin_addr) == 0)
-#endif
-		{
-		  addrOk = NO;
-		}
-	    }
-	  sockaddr.sin_port = GSSwapHostI16ToBig(number);
+	  addrOk = GSPrivateSockaddrSetup(addr, number, nil, nil, &sockaddr);
 
 	  /*
-           * Need size of buffer for getsockbyname() later.
+	   * Need size of buffer for getsockbyname() later.
 	   */
 	  slen = sizeof(sockaddr);
 
@@ -1632,7 +1606,7 @@ static Class		tcpPortClass;
 	      NSLog(@"Bad address (%@) specified for listening port", addr);
 	      DESTROY(port);
 	    }
-	  else if ((desc = socket(AF_INET, SOCK_STREAM, PF_UNSPEC))
+	  else if ((desc = socket(sockaddr.sa_family, SOCK_STREAM, PF_UNSPEC))
 	    == INVALID_SOCKET)
 	    {
 	      NSLog(@"unable to create socket - %@", [NSError _last]);
@@ -1654,11 +1628,11 @@ static Class		tcpPortClass;
               DESTROY(port);
 	    }
 #endif
-	  else if (bind(desc, (struct sockaddr *)&sockaddr,
-	    sizeof(sockaddr)) == SOCKET_ERROR)
+	  else if (bind(desc, &sockaddr,
+	    GSPrivateSockaddrLength(&sockaddr)) == SOCKET_ERROR)
 	    {
-	      NSLog(@"unable to bind to port %s:%d - %@",
-		inet_ntoa(sockaddr.sin_addr), number, [NSError _last]);
+	      NSLog(@"unable to bind to port %@ - %@",
+		GSPrivateSockaddrName(&sockaddr), [NSError _last]);
 	      (void) close(desc);
               DESTROY(port);
 	    }
@@ -1686,7 +1660,7 @@ static Class		tcpPortClass;
 	       * we did the 'bind' call.
 	       */
 	      port->listener = desc;
-	      port->portNum = GSSwapBigI16ToHost(sockaddr.sin_port); 
+	      port->portNum = GSPrivateSockaddrPort(&sockaddr); 
 #if	defined(__MINGW__)
               port->eventListener = (WSAEVENT)CreateEvent(NULL,NO,NO,NULL);
               if (port->eventListener == WSA_INVALID_EVENT)
@@ -1735,7 +1709,7 @@ static Class		tcpPortClass;
 	       * No known ports within this port number -
 	       * create the map table to add the new port to.
 	       */
-	      thePorts = NSCreateMapTable(NSIntMapKeyCallBacks,
+	      thePorts = NSCreateMapTable(NSIntegerMapKeyCallBacks,
 			      NSNonOwnedPointerMapValueCallBacks, 0);
 	      NSMapInsert(tcpPortMap,
 		(void*)(uintptr_t)number, (void*)thePorts);
@@ -2147,7 +2121,7 @@ static Class		tcpPortClass;
   if (desc == listener)
 #endif
     {
-      struct sockaddr_in	sockAddr;
+      struct sockaddr	sockAddr;
       socklen_t size = sizeof(sockAddr);
 
       desc = accept(listener, (struct sockaddr*)&sockAddr, &size);
@@ -2172,8 +2146,7 @@ static Class		tcpPortClass;
 	   */
 	  handle = [GSTcpHandle handleWithDescriptor: desc];
 	  memcpy(&handle->sockAddr, &sockAddr, sizeof(sockAddr));
-	  handle->defaultAddress = RETAIN([NSString stringWithUTF8String:
-	    inet_ntoa(sockAddr.sin_addr)]);
+	  ASSIGN(handle->defaultAddress, GSPrivateSockaddrHost(&sockAddr));
 
 	  [handle setState: GS_H_ACCEPT];
 	  [self addHandle: handle forSend: NO];
@@ -2214,7 +2187,7 @@ static Class		tcpPortClass;
     }
 }
 
-- (void) release
+- (oneway void) release
 {
   M_LOCK(tcpPortLock);
   if (NSDecrementExtraRefCountWasZero(self))
@@ -2316,7 +2289,7 @@ static Class		tcpPortClass;
   rl = [self reservedSpaceLength];
   if (length != 0 && length != rl)
     {
-      NSLog(@"bad reserved length - %u", length);
+      NSLog(@"bad reserved length - %"PRIuPTR, length);
       return NO;
     }
   if ([receivingPort isKindOfClass: tcpPortClass] == NO)
