@@ -1,5 +1,5 @@
 /* Implementation for NSSortDescriptor for GNUStep
-   Copyright (C) 2005 Free Software Foundation, Inc.
+   Copyright (C) 2005-2011 Free Software Foundation, Inc.
 
    Written by:  Saso Kiselkov <diablos@manga.sk>
    Date: 2005
@@ -27,7 +27,6 @@
 #define	EXPOSE_NSSortDescriptor_IVARS	1
 #import "Foundation/NSSortDescriptor.h"
 
-#import "Foundation/NSBundle.h"
 #import "Foundation/NSCoder.h"
 #import "Foundation/NSException.h"
 #import "Foundation/NSKeyValueCoding.h"
@@ -82,7 +81,7 @@
   [super dealloc];
 }
 
-- (unsigned) hash
+- (NSUInteger) hash
 {
   const char	*sel = sel_getName(_selector);
 
@@ -164,10 +163,10 @@
 {
   if ([coder allowsKeyedCoding])
     {
-      [coder encodeObject: _key forKey: @"Key"];
-      [coder encodeBool: _ascending forKey: @"Ascending"];
+      [coder encodeObject: _key forKey: @"NSKey"];
+      [coder encodeBool: _ascending forKey: @"NSAscending"];
       [coder encodeObject: NSStringFromSelector(_selector)
-                   forKey: @"Selector"];
+                   forKey: @"NSSelector"];
     }
   else
     {
@@ -183,10 +182,10 @@
     {
       if ([decoder allowsKeyedCoding])
         {
-          ASSIGN(_key, [decoder decodeObjectForKey: @"Key"]);
-          _ascending = [decoder decodeBoolForKey: @"Ascending"];
+          ASSIGN(_key, [decoder decodeObjectForKey: @"NSKey"]);
+          _ascending = [decoder decodeBoolForKey: @"NSAscending"];
           _selector = NSSelectorFromString([decoder
-            decodeObjectForKey: @"Selector"]);
+            decodeObjectForKey: @"NSSelector"]);
         }
       else
         {
