@@ -44,6 +44,8 @@ NSString *const NSGrammarRange = @"NSGrammarRange";
 NSString *const NSGrammarUserDescription = @"NSGrammarUserDescription";
 NSString *const NSGrammarCorrections = @"NSGrammarCorrections";
 
+static NSConnection *spellServerConnection = nil;
+
 /* User dictionary location */
 static NSString *GNU_UserDictionariesDir = @"Dictionaries";
 
@@ -79,9 +81,12 @@ GSSpellServerName(NSString *vendor, NSString *language)
 // Non-private Instance methods
 - (id) init
 {
-  NSArray *userLanguages = [NSUserDefaults userLanguages];  
-  NSString *currentLanguage = [userLanguages objectAtIndex: 0];
+  NSArray *userPreference;
+  NSString *currentLanguage;
 
+  userPreference = [[NSUserDefaults standardUserDefaults]
+    stringArrayForKey: @"NSLanguages"];  
+  currentLanguage = [userPreference objectAtIndex: 0];
   if ((self = [super init]) != nil)
     {
       _delegate = nil;
@@ -111,7 +116,6 @@ GSSpellServerName(NSString *vendor, NSString *language)
 		 byVendor: (NSString *)vendor
 {
   NSString *serverName = GSSpellServerName(vendor, language);
-  NSConnection *connection = nil;
   BOOL result = NO;
 
   if (serverName == nil)
@@ -119,12 +123,11 @@ GSSpellServerName(NSString *vendor, NSString *language)
       return NO;
     }
 
-  connection = [[NSConnection alloc] init];
-  if (connection)
+  spellServerConnection = [[NSConnection alloc] init];
+  if (spellServerConnection)
     {
-      IF_NO_GC(RETAIN(connection);)
-      [connection setRootObject: self];
-      result = [connection registerName: serverName];
+      [spellServerConnection setRootObject: self];
+      result = [spellServerConnection registerName: serverName];
     }
 
   return result;
