@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSTask class reference</title>
-   $Date: 2011-04-03 10:54:57 +0100 (Sun, 03 Apr 2011) $ $Revision: 32766 $
+   $Date: 2011-12-15 02:42:39 -0700 (Thu, 15 Dec 2011) $ $Revision: 34290 $
    */
 
 #import "common.h"
@@ -49,9 +49,6 @@
 #import "GSPrivate.h"
 
 #include <string.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #include <sys/types.h>
 #if defined(__FreeBSD__) || defined(__OpenBSD__)
 #include <fcntl.h>
@@ -62,18 +59,22 @@
 #  include <windows.h>
 #endif
 
-#ifdef	HAVE_SYS_SIGNAL_H
-#include <sys/signal.h>
+#if	defined(HAVE_SYS_SIGNAL_H)
+#  include <sys/signal.h>
+#elif	defined(HAVE_SIGNAL_H)
+#  include <signal.h>
 #endif
-#ifdef	HAVE_SIGNAL_H
-#include <signal.h>
+
+#if	defined(HAVE_SYS_FILE_H)
+#  include	<sys/file.h>
 #endif
-#ifdef	HAVE_SYS_FILE_H
-#include <sys/file.h>
-#endif
-#ifdef	HAVE_SYS_FCNTL_H
-#include <sys/fcntl.h>
+
+#if	defined(HAVE_SYS_FCNTL_H)
+#  include <sys/fcntl.h>
+#elif	defined(HAVE_FCNTL_H)
+#  include <fcntl.h>
 #endif
+
 #ifdef	HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
@@ -281,7 +282,7 @@ pty_slave(const char* name)
 	   * t2: complete notification ... attempt to release task object
 	   * but it's already deallocated.
 	   */
-          activeTasks = NSCreateMapTable(NSIntMapKeyCallBacks,
+          activeTasks = NSCreateMapTable(NSIntegerMapKeyCallBacks,
                 NSObjectMapValueCallBacks, 0);
         }
       [gnustep_global_lock unlock];
@@ -1182,7 +1183,7 @@ quotedFromString(NSString *aString)
 	  [data appendBytes: &terminator length: 2];	// end of setting
 	}
       [data appendBytes: &terminator length: 2];	// end of environment
-      RELEASE(pool);
+      [pool drain];
       envp = [data mutableBytes];
     }
 
