@@ -25,7 +25,7 @@
    Boston, MA 02111 USA.
 
    <title>NSTimeZone class reference</title>
-   $Date: 2011-06-16 08:24:22 +0100 (Thu, 16 Jun 2011) $ $Revision: 33313 $
+   $Date: 2011-07-31 09:31:39 -0600 (Sun, 31 Jul 2011) $ $Revision: 33660 $
  */
 
 /* Use the system time zones if available. In other cases, use an
@@ -408,7 +408,7 @@ static NSString *_time_zone_path(NSString *subpath, NSString *type)
     {
       zone = RETAIN(localTimeZone);
       DESTROY(self);
-      return zone;
+      return (GSPlaceholderTimeZone*)zone;
     }
 
   /*
@@ -557,10 +557,10 @@ static NSString *_time_zone_path(NSString *subpath, NSString *type)
 	}
     }
   DESTROY(self);
-  return zone;
+  return (GSPlaceholderTimeZone*)zone;
 }
 
-- (void) release
+- (oneway void) release
 {
   return;		// placeholders never get released.
 }
@@ -620,7 +620,7 @@ static NSString *_time_zone_path(NSString *subpath, NSString *type)
   return [[NSTimeZoneClass defaultTimeZone] name];
 }
 
-- (void) release
+- (oneway void) release
 {
 }
 
@@ -668,6 +668,7 @@ static NSMapTable	*absolutes = 0;
     {
       absolutes = NSCreateMapTable(NSIntegerMapKeyCallBacks,
 	NSNonOwnedPointerMapValueCallBacks, 0);
+      [[NSObject leakAt: (id*)&absolutes] release];
     }
 }
 
@@ -1340,18 +1341,23 @@ static NSMapTable	*absolutes = 0;
       NSTimeZoneClass = self;
       GSPlaceholderTimeZoneClass = [GSPlaceholderTimeZone class];
       zoneDictionary = [[NSMutableDictionary alloc] init];
+      [[NSObject leakAt: &zoneDictionary] release];
 
       /*
        * Set up infrastructure for placeholder timezones.
        */
       defaultPlaceholderTimeZone = (GSPlaceholderTimeZone*)
 	NSAllocateObject(GSPlaceholderTimeZoneClass, 0, NSDefaultMallocZone());
+      [[NSObject leakAt: &defaultPlaceholderTimeZone] release];
       placeholderMap = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
 	NSNonRetainedObjectMapValueCallBacks, 0);
+      [[NSObject leakAt: (id*)&placeholderMap] release];
 
       localTimeZone = [[NSLocalTimeZone alloc] init];
+      [[NSObject leakAt: (id*)&localTimeZone] release];
 
       zone_mutex = [GSLazyRecursiveLock new];
+      [[NSObject leakAt: (id*)&zone_mutex] release];
 
       [[NSNotificationCenter defaultCenter] addObserver: self
         selector: @selector(_notified:)
@@ -1521,9 +1527,14 @@ static NSMapTable	*absolutes = 0;
 	      while (nil != (s = [e nextObject]))
 		{
 		  s = [s stringByTrimmingSpaces];
-		  if ([s hasPrefix: @"ZONE"])
+                  // OpenSuse uses the non-standard key TIMEZONE
+		  if ([s hasPrefix: @"ZONE"] || [s hasPrefix: @"TIMEZONE"])
 		    {
-		      s = [s substringFromIndex: 4];
+                      if ([s hasPrefix: @"ZONE"])
+                        s = [s substringFromIndex: 4];
+                      else
+                        s = [s substringFromIndex: 8];
+
 		      s = [s stringByTrimmingSpaces];
 		      if ([s hasPrefix: @"="])
 			{
