@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSTimer class reference</title>
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2010-09-25 15:28:07 -0600 (Sat, 25 Sep 2010) $ $Revision: 31420 $
    */
 
 #import "common.h"
@@ -224,36 +224,51 @@ static Class	NSDate_class;
  */
 - (void) fire
 {
-  if (_selector == 0)
+  id	target;
+
+  /* We retain the target so it won't be deallocated while we are using it
+   * (if this timer gets invalidated while we are firing).
+   */
+  target = [_target retain];
+
+  /* We check that we have not been invalidated before we fire.
+   */
+  if (NO == _invalidated)
     {
-      NS_DURING
+      if (_selector == 0)
 	{
-	  [(NSInvocation*)_target invoke];
+	  NS_DURING
+	    {
+	      [(NSInvocation*)target invoke];
+	    }
+	  NS_HANDLER
+	    {
+	      NSLog(@"*** NSTimer ignoring exception '%@' (reason '%@') "
+	        @"raised during posting of timer with target %p "
+		@"and selector '%@'",
+		[localException name], [localException reason], target,
+		NSStringFromSelector([target selector]));
+	    }
+	  NS_ENDHANDLER
 	}
-      NS_HANDLER
+      else
 	{
-	  NSLog(@"*** NSTimer ignoring exception '%@' (reason '%@') "
-	   @"raised during posting of timer with target %p and selector '%@'",
-	    [localException name], [localException reason], _target,
-	    NSStringFromSelector([_target selector]));
+	  NS_DURING
+	    {
+	      [target performSelector: _selector withObject: self];
+	    }
+	  NS_HANDLER
+	    {
+	      NSLog(@"*** NSTimer ignoring exception '%@' (reason '%@') "
+		@"raised during posting of timer with target %p and "
+		@"selector '%@'",
+		[localException name], [localException reason], target,
+		NSStringFromSelector(_selector));
+	    }
+	  NS_ENDHANDLER
 	}
-      NS_ENDHANDLER
-    }
-  else
-    {
-      NS_DURING
-	{
-	  [_target performSelector: _selector withObject: self];
-	}
-      NS_HANDLER
-	{
-	  NSLog(@"*** NSTimer ignoring exception '%@' (reason '%@') "
-	    @"raised during posting of timer with target %p and selector '%@'",
-	    [localException name], [localException reason], _target,
-	    NSStringFromSelector(_selector));
-	}
-      NS_ENDHANDLER
     }
+  [target release];
 
   if (_repeats == NO)
     {
