@@ -25,7 +25,7 @@
    Boston, MA 02111 USA.
 
    <title>NSURL class reference</title>
-   $Date: 2011-03-07 11:34:17 +0000 (Mon, 07 Mar 2011) $ $Revision: 32483 $
+   $Date: 2012-02-02 10:12:42 -0700 (Thu, 02 Feb 2012) $ $Revision: 34699 $
 */
 
 /*
@@ -42,6 +42,7 @@ function may be incorrect
 #import "Foundation/NSCoder.h"
 #import "Foundation/NSData.h"
 #import "Foundation/NSDictionary.h"
+#import "Foundation/NSError.h"
 #import "Foundation/NSException.h"
 #import "Foundation/NSFileManager.h"
 #import "Foundation/NSLock.h"
@@ -52,6 +53,9 @@ function may be incorrect
 #import "Foundation/NSURLHandle.h"
 #import "Foundation/NSValue.h"
 
+#import "GNUstepBase/NSURL+GNUstepBase.h"
+
+
 NSString * const NSURLErrorDomain = @"NSURLErrorDomain";
 NSString * const NSErrorFailingURLStringKey = @"NSErrorFailingURLStringKey";
 
@@ -60,8 +64,9 @@ NSString * const NSErrorFailingURLStringKey = @"NSErrorFailingURLStringKey";
 @end
 
 @implementation	NSString (NSURLPrivate)
+
 /* Like the normal percent escape method, but with additional characters
- * escaped.
+ * escaped (for use by file scheme URLs).
  */
 - (NSString*) _stringByAddingPercentEscapes
 {
@@ -120,6 +125,38 @@ NSString * const NSErrorFailingURLStringKey = @"NSErrorFailingURLStringKey";
     }
   return s;
 }
+
+@end
+
+@interface	NSURL (GSPrivate)
+- (NSURL*) _URLBySettingPath: (NSString*)newPath; 
+@end
+
+@implementation	NSURL (GSPrivate)
+
+- (NSURL*) _URLBySettingPath: (NSString*)newPath 
+{
+  if ([self isFileURL]) 
+    {
+      return [NSURL fileURLWithPath: newPath];
+    }
+  else
+    {
+      NSURL	*u;
+
+      u = [[NSURL alloc] initWithScheme: [self scheme]
+				   user: [self user]
+			       password: [self password]
+				   host: [self host]
+				   port: [self port]
+			       fullPath: newPath
+			parameterString: [self parameterString]
+				  query: [self query]
+			       fragment: [self fragment]];
+      return [u autorelease];
+    }
+}
+
 @end
 
 /*
@@ -563,56 +600,34 @@ static char *unescape(const char *from, char * to)
 
 
 
-/**
- * This class permits manipulation of URLs and the resources to which they
- * refer.  They can be used to represent absolute URLs or relative URLs
- * which are based upon an absolute URL.  The relevant RFCs describing
- * how a URL is formatted, and what is legal in a URL are -
- * 1808, 1738, and 2396.<br />
- * Handling of the underlying resources is carried out by NSURLHandle
- * objects, but NSURL provides a simplified API wrapping these objects.
- */
 @implementation NSURL
 
-static unsigned	urlAlign;
+static NSUInteger	urlAlign;
 
-/**
- * Create and return a file URL with the supplied path.<br />
- * The value of aPath must be a valid filesystem path.<br />
- * Calls -initFileURLWithPath: which escapes characters in the
- * path where necessary.
- */
 + (id) fileURLWithPath: (NSString*)aPath
 {
   return AUTORELEASE([[NSURL alloc] initFileURLWithPath: aPath]);
 }
 
++ (NSURL*) fileURLWithPathComponents: (NSArray*)components
+{
+  return [self fileURLWithPath: [NSString pathWithComponents: components]];
+}
+
 + (void) initialize
 {
   if (clientsLock == nil)
     {
-      urlAlign = objc_alignof_type(@encode(parsedURL));
+      NSGetSizeAndAlignment(@encode(parsedURL), &urlAlign, 0);
       clientsLock = [NSLock new];
     }
 }
 
-/**
- * Create and return a URL with the supplied string, which should
- * be a string (containing percent escape codes where necessary)
- * conforming to the description (in RFC2396) of an absolute URL.<br />
- * Calls -initWithString:
- */
 + (id) URLWithString: (NSString*)aUrlString
 {
   return AUTORELEASE([[NSURL alloc] initWithString: aUrlString]);
 }
 
-/**
- * Create and return a URL with the supplied string, which should
- * be a string (containing percent escape codes where necessary)
- * conforming to the description (in RFC2396) of a relative URL.<br />
- * Calls -initWithString:relativeToURL:
- */
 + (id) URLWithString: (NSString*)aUrlString
        relativeToURL: (NSURL*)aBaseUrl
 {
@@ -620,24 +635,88 @@ static unsigned	urlAlign;
 				     relativeToURL: aBaseUrl]);
 }
 
-/**
- * Initialise by building a URL string from the supplied parameters
- * and calling -initWithString:relativeToURL:<br />
- * This method adds percent escapes to aPath if it contains characters
- * which need escaping.<br />
- * Accepts RFC2732 style IPv6 host addresses either with or without the
- * enclosing square brackets (MacOS-X at least up to version 10.5 does
- * not handle these correctly, but GNUstep does).<br />
- * Permits the 'aHost' part to contain 'username:password@host:port' or
- * 'host:port' in addition to a simple host name or address.
- */
+- (id) initFileURLWithPath: (NSString*)aPath
+{
+  NSFileManager	*mgr = [NSFileManager defaultManager];
+  BOOL		flag = NO;
+
+  if (nil == aPath)
+    {
+      [NSException raise: NSInvalidArgumentException
+		  format: @"[%@ %@] nil string parameter",
+	NSStringFromClass([self class]), NSStringFromSelector(_cmd)];
+    }
+  if ([aPath isAbsolutePath] == NO)
+    {
+      aPath = [[mgr currentDirectoryPath]
+	stringByAppendingPathComponent: aPath];
+    }
+  if ([mgr fileExistsAtPath: aPath isDirectory: &flag] == YES)
+    {
+      if ([aPath isAbsolutePath] == NO)
+	{
+	  aPath = [aPath stringByStandardizingPath];
+	}
+      if (flag == YES && [aPath hasSuffix: @"/"] == NO)
+	{
+	  aPath = [aPath stringByAppendingString: @"/"];
+	}
+    }
+  self = [self initWithScheme: NSURLFileScheme
+			 host: @"localhost"
+			 path: aPath];
+  return self;
+}
+
+- (id) initFileURLWithPath: (NSString*)aPath isDirectory: (BOOL)isDir
+{
+  NSFileManager	*mgr = [NSFileManager defaultManager];
+  BOOL		flag = NO;
+
+  if (nil == aPath)
+    {
+      [NSException raise: NSInvalidArgumentException
+		  format: @"[%@ %@] nil string parameter",
+	NSStringFromClass([self class]), NSStringFromSelector(_cmd)];
+    }
+  if ([aPath isAbsolutePath] == NO)
+    {
+      aPath = [[mgr currentDirectoryPath]
+	stringByAppendingPathComponent: aPath];
+    }
+  if ([mgr fileExistsAtPath: aPath isDirectory: &flag] == YES)
+    {
+      if ([aPath isAbsolutePath] == NO)
+	{
+	  aPath = [aPath stringByStandardizingPath];
+	}
+      isDir = flag;
+    }
+  if (isDir == YES && [aPath hasSuffix: @"/"] == NO)
+    {
+      aPath = [aPath stringByAppendingString: @"/"];
+    }
+  self = [self initWithScheme: NSURLFileScheme
+			 host: @"localhost"
+			 path: aPath];
+  return self;
+}
+
 - (id) initWithScheme: (NSString*)aScheme
 		 host: (NSString*)aHost
 		 path: (NSString*)aPath
 {
   NSString	*aUrlString = [NSString alloc];
 
-  aPath = [aPath _stringByAddingPercentEscapes];
+  if ([aScheme isEqualToString: @"file"])
+    {
+      aPath = [aPath _stringByAddingPercentEscapes];
+    }
+  else
+    {
+      aPath = [aPath
+	stringByAddingPercentEscapesUsingEncoding: NSUTF8StringEncoding];
+    }
   if ([aHost length] > 0)
     {
       NSRange	r = [aHost rangeOfString: @"@"];
@@ -706,59 +785,12 @@ static unsigned	urlAlign;
   return self;
 }
 
-/**
- * Initialise as a file URL with the specified path (which must
- * be a valid path on the local filesystem).<br />
- * Converts relative paths to absolute ones.<br />
- * Appends a trailing slash to the path when necessary if it
- * specifies a directory.<br />
- * Calls -initWithScheme:host:path:
- */
-- (id) initFileURLWithPath: (NSString*)aPath
-{
-  NSFileManager	*mgr = [NSFileManager defaultManager];
-  BOOL		flag = NO;
-
-  if ([aPath isAbsolutePath] == NO)
-    {
-      aPath = [[mgr currentDirectoryPath]
-	stringByAppendingPathComponent: aPath];
-    }
-  if ([mgr fileExistsAtPath: aPath isDirectory: &flag] == YES)
-    {
-      if ([aPath isAbsolutePath] == NO)
-	{
-	  aPath = [aPath stringByStandardizingPath];
-	}
-      if (flag == YES && [aPath hasSuffix: @"/"] == NO)
-	{
-	  aPath = [aPath stringByAppendingString: @"/"];
-	}
-    }
-  self = [self initWithScheme: NSURLFileScheme
-			 host: @"localhost"
-			 path: aPath];
-  return self;
-}
-
-/**
- * Initialise as an absolute URL.<br />
- * Calls -initWithString:relativeToURL:
- */
 - (id) initWithString: (NSString*)aUrlString
 {
   self = [self initWithString: aUrlString relativeToURL: nil];
   return self;
 }
 
-/** <init />
- * Initialised using aUrlString and aBaseUrl.  The value of aBaseUrl
- * may be nil, but aUrlString must be non-nil.<br />
- * Accepts RFC2732 style IPv6 host addresses.<br />
- * Parses a string wihthout a scheme as a simple path.<br />
- * Parses an empty string as an empty path.<br />
- * If the string cannot be parsed the method returns nil.
- */
 - (id) initWithString: (NSString*)aUrlString
 	relativeToURL: (NSURL*)aBaseUrl
 {
@@ -773,7 +805,7 @@ static unsigned	urlAlign;
   if ([aUrlString isKindOfClass: [NSString class]] == NO)
     {
       [NSException raise: NSInvalidArgumentException
-		  format: @"[%@ %@] nil string parameter",
+		  format: @"[%@ %@] bad string parameter",
 	NSStringFromClass([self class]), NSStringFromSelector(_cmd)];
     }
   if (aBaseUrl != nil
@@ -1299,9 +1331,6 @@ static unsigned	urlAlign;
   return [[self absoluteString] isEqualToString: [other absoluteString]];
 }
 
-/**
- * Returns the full string describing the receiver resolved against its base.
- */
 - (NSString*) absoluteString
 {
   NSString	*absString = myData->absolute;
@@ -1319,10 +1348,6 @@ static unsigned	urlAlign;
   return absString;
 }
 
-/**
- * If the receiver is an absolute URL, returns self.  Otherwise returns an
- * absolute URL referring to the same resource as the receiver.
- */
 - (NSURL*) absoluteURL
 {
   if (_baseURL == nil)
@@ -1335,21 +1360,50 @@ static unsigned	urlAlign;
     }
 }
 
-/**
- * If the receiver is a relative URL, returns its base URL.<br />
- * Otherwise, returns nil.
- */
 - (NSURL*) baseURL
 {
   return _baseURL;
 }
 
-/**
- * Returns the fragment portion of the receiver or nil if there is no
- * fragment supplied in the URL.<br />
- * The fragment is everything in the original URL string after a '#'<br />
- * File URLs do not have fragments.
- */
+- (BOOL) checkResourceIsReachableAndReturnError: (NSError **)error
+{
+  NSString *errorStr = nil;
+
+  if ([self isFileURL])
+    {
+      NSFileManager *mgr = [NSFileManager defaultManager];
+      NSString *path = [self path];
+      
+      if ([mgr fileExistsAtPath: path])
+        {
+          if (![mgr isReadableFileAtPath: path])
+            {
+              errorStr = @"File not readable";
+            }
+        }
+      else
+        {
+          errorStr = @"File does not exist";
+        }
+    }
+  else
+    {
+      errorStr = @"No file URL";
+    }
+
+  if ((errorStr != nil) && (error != NULL))
+    {
+      NSDictionary	*info;
+
+      info = [NSDictionary dictionaryWithObjectsAndKeys:
+	errorStr, NSLocalizedDescriptionKey, nil];
+      *error = [NSError errorWithDomain: @"NSURLError"
+                                   code: 0 
+                               userInfo: info];
+    }
+  return nil == errorStr ? YES : NO;
+}
+
 - (NSString*) fragment
 {
   NSString	*fragment = nil;
@@ -1444,49 +1498,6 @@ static unsigned	urlAlign;
   return ptr;
 }
 
-- (NSString*) fullPath
-{
-  NSString	*path = nil;
-  unsigned int	len = 3;
-
-  if (_baseURL != nil)
-    {
-      if (baseData->path && *baseData->path)
-        {
-          len += strlen(baseData->path);
-	}
-      else if (baseData->hasNoPath == NO)
-	{
-	  len++;
-	}
-    }
-  if (myData->path && *myData->path)
-    {
-      len += strlen(myData->path);
-    }
-  else if (myData->hasNoPath == NO)
-    {
-      len++;
-    }
-  if (len > 3)
-    {
-      char		buf[len];
-      char		*ptr;
-
-      ptr = [self _path: buf];
-      path = [NSString stringWithUTF8String: ptr];
-    }
-  return path;
-}
-
-/**
- * Returns the host portion of the receiver or nil if there is no
- * host supplied in the URL.<br />
- * Percent escape sequences in the user string are translated and the string
- * treated as UTF8.<br />
- * Returns IPv6 addresses <em>without</em> the enclosing square brackets
- * required (by RFC2732) in URL strings.
- */
 - (NSString*) host
 {
   NSString	*host = nil;
@@ -1513,36 +1524,16 @@ static unsigned	urlAlign;
   return host;
 }
 
-/**
- * Returns YES if the receiver is a file URL, NO otherwise.
- */
 - (BOOL) isFileURL
 {
   return myData->isFile;
 }
 
-/**
- * Loads resource data for the specified client.
- * <p>
- *   If shouldUseCache is YES then an attempt
- *   will be made to locate a cached NSURLHandle to provide the
- *   resource data, otherwise a new handle will be created and
- *   cached.
- * </p>
- * <p>
- *   If the handle does not have the data available, it will be
- *   asked to load the data in the background by calling its
- *   loadInBackground  method.
- * </p>
- * <p>
- *   The specified client (if non-nil) will be set up to receive
- *   notifications of the progress of the background load process.
- * </p>
- * <p>
- *   The processes current run loop must be run in order for the
- *   background load operation to operate!
- * </p>
- */
+- (NSString*) lastPathComponent
+{
+  return [[self path] lastPathComponent];
+}
+
 - (void) loadResourceDataNotifyingClient: (id)client
 			      usingCache: (BOOL)shouldUseCache
 {
@@ -1586,13 +1577,6 @@ static unsigned	urlAlign;
     }
 }
 
-/**
- * Returns the parameter portion of the receiver or nil if there is no
- * parameter supplied in the URL.<br />
- * The parameters are everything in the original URL string after a ';'
- * but before the query.<br />
- * File URLs do not have parameters.
- */
 - (NSString*) parameterString
 {
   NSString	*parameters = nil;
@@ -1604,14 +1588,6 @@ static unsigned	urlAlign;
   return parameters;
 }
 
-/**
- * Returns the password portion of the receiver or nil if there is no
- * password supplied in the URL.<br />
- * Percent escape sequences in the user string are translated and the string
- * treated as UTF8 in GNUstep but this appears to be broken in MacOS-X.<br />
- * NB. because of its security implications it is recommended that you
- * do not use URLs with users and passwords unless necessary.
- */
 - (NSString*) password
 {
   NSString	*password = nil;
@@ -1626,17 +1602,6 @@ static unsigned	urlAlign;
   return password;
 }
 
-/**
- * Returns the path portion of the receiver.<br />
- * Replaces percent escapes with unescaped values, interpreting non-ascii
- * character sequences as UTF8.<br />
- * NB. This does not conform strictly to the RFCs, in that it includes a
- * leading slash ('/') character (whereas the path part of a URL strictly
- * should not) and the interpretation of non-ascii character is (strictly
- * speaking) undefined.<br />
- * Also, this breaks strict conformance in that a URL of file scheme is
- * treated as having a path (contrary to RFCs)
- */
 - (NSString*) path
 {
   NSString	*path = nil;
@@ -1682,12 +1647,16 @@ static unsigned	urlAlign;
   return path;
 }
 
-/**
- * Returns the port portion of the receiver or nil if there is no
- * port supplied in the URL.<br />
- * Percent escape sequences in the user string are translated in GNUstep
- * but this appears to be broken in MacOS-X.
- */
+- (NSArray*) pathComponents 
+{
+  return [[self path] pathComponents];
+}
+
+- (NSString*) pathExtension 
+{
+  return [[self path] pathExtension];
+}
+
 - (NSNumber*) port
 {
   NSNumber	*port = nil;
@@ -1702,10 +1671,6 @@ static unsigned	urlAlign;
   return port;
 }
 
-/**
- * Asks a URL handle to return the property for the specified key and
- * returns the result.
- */
 - (id) propertyForKey: (NSString*)propertyKey
 {
   NSURLHandle	*handle = [self URLHandleUsingCache: YES];
@@ -1713,13 +1678,6 @@ static unsigned	urlAlign;
   return [handle propertyForKey: propertyKey];
 }
 
-/**
- * Returns the query portion of the receiver or nil if there is no
- * query supplied in the URL.<br />
- * The query is everything in the original URL string after a '?'
- * but before the fragment.<br />
- * File URLs do not have queries.
- */
 - (NSString*) query
 {
   NSString	*query = nil;
@@ -1731,11 +1689,6 @@ static unsigned	urlAlign;
   return query;
 }
 
-/**
- * Returns the path of the receiver, without taking any base URL into account.
- * If the receiver is an absolute URL, -relativePath is the same as -path.<br />
- * Returns nil if there is no path specified for the URL.
- */
 - (NSString*) relativePath
 {
   if (nil == _baseURL)
@@ -1754,10 +1707,6 @@ static unsigned	urlAlign;
     }
 }
 
-/**
- * Returns the relative portion of the URL string.  If the receiver is not
- * a relative URL, this returns the same as absoluteString.
- */
 - (NSString*) relativeString
 {
   return _urlString;
@@ -1772,12 +1721,6 @@ static unsigned	urlAlign;
   return [super replacementObjectForPortCoder: aCoder];
 }
 
-/**
- * Loads the resource data for the represented URL and returns the result.
- * The shouldUseCache flag determines whether data previously retrieved by
- * an existing NSURLHandle can be used to provide the data, or if it should
- * be refetched.
- */
 - (NSData*) resourceDataUsingCache: (BOOL)shouldUseCache
 {
   NSURLHandle	*handle = [self URLHandleUsingCache: YES];
@@ -1791,10 +1734,6 @@ static unsigned	urlAlign;
   return data;
 }
 
-/**
- * Returns the resource specifier of the URL ... the part which lies
- * after the scheme.
- */
 - (NSString*) resourceSpecifier
 {
   NSRange	range = [_urlString rangeOfString: @"://"];
@@ -1834,9 +1773,6 @@ static unsigned	urlAlign;
     }
 }
 
-/**
- * Returns the scheme of the receiver.
- */
 - (NSString*) scheme
 {
   NSString	*scheme = nil;
@@ -1848,9 +1784,6 @@ static unsigned	urlAlign;
   return scheme;
 }
 
-/**
- * Calls [NSURLHandle-writeProperty:forKey:] to set the named property.
- */
 - (BOOL) setProperty: (id)property
 	      forKey: (NSString*)propertyKey
 {
@@ -1859,11 +1792,6 @@ static unsigned	urlAlign;
   return [handle writeProperty: property forKey: propertyKey];
 }
 
-/**
- * Calls [NSURLHandle-writeData:] to write the specified data object
- * to the resource identified by the receiver URL.<br />
- * Returns the result.
- */
 - (BOOL) setResourceData: (NSData*)data
 {
   NSURLHandle	*handle = [self URLHandleUsingCache: YES];
@@ -1883,9 +1811,6 @@ static unsigned	urlAlign;
   return YES;
 }
 
-/**
- * Returns a URL with '/./' and '/../' sequences resolved etc.
- */
 - (NSURL*) standardizedURL
 {
   char		*url = buildURL(baseData, myData, YES);
@@ -1901,12 +1826,6 @@ static unsigned	urlAlign;
   return tmp;
 }
 
-/**
- * Returns an NSURLHandle instance which may be used to write data to the
- * resource represented by the receiver URL, or read data from it.<br />
- * The shouldUseCache flag indicates whether a cached handle may be returned
- * or a new one should be created.
- */
 - (NSURLHandle*) URLHandleUsingCache: (BOOL)shouldUseCache
 {
   NSURLHandle	*handle = nil;
@@ -1928,14 +1847,6 @@ static unsigned	urlAlign;
   return handle;
 }
 
-/**
- * Returns the user portion of the receiver or nil if there is no
- * user supplied in the URL.<br />
- * Percent escape sequences in the user string are translated and
- * the whole is treated as UTF8 data.<br />
- * NB. because of its security implications it is recommended that you
- * do not use URLs with users and passwords unless necessary.
- */
 - (NSString*) user
 {
   NSString	*user = nil;
@@ -1950,6 +1861,49 @@ static unsigned	urlAlign;
   return user;
 }
 
+- (NSURL*) URLByAppendingPathComponent: (NSString*)pathComponent 
+{
+  return [self _URLBySettingPath:
+    [[self path] stringByAppendingPathComponent: pathComponent]];
+}
+
+- (NSURL*) URLByAppendingPathExtension: (NSString*)pathExtension
+{
+  return [self _URLBySettingPath:
+    [[self path] stringByAppendingPathExtension: pathExtension]];
+}
+
+- (NSURL*) URLByDeletingLastPathComponent 
+{
+  return [self _URLBySettingPath:
+    [[self path] stringByDeletingLastPathComponent]];
+}
+
+- (NSURL*) URLByDeletingPathExtension 
+{
+  return [self _URLBySettingPath:
+    [[self path] stringByDeletingPathExtension]];
+}
+
+- (NSURL*) URLByResolvingSymlinksInPath 
+{
+  if ([self isFileURL]) 
+    {
+      return [NSURL fileURLWithPath:
+	[[self path] stringByResolvingSymlinksInPath]];
+    }
+  return self;
+}
+
+- (NSURL*) URLByStandardizingPath 
+{
+  if ([self isFileURL]) 
+    {
+      return [NSURL fileURLWithPath: [[self path] stringByStandardizingPath]];
+    }
+  return self;
+}
+
 - (void) URLHandle: (NSURLHandle*)sender
   resourceDataDidBecomeAvailable: (NSData*)newData
 {
@@ -2051,3 +2005,41 @@ static unsigned	urlAlign;
 }
 
 @end
+
+@implementation NSURL (GNUstepBase)
+- (NSString*) fullPath
+{
+  NSString	*path = nil;
+  unsigned int	len = 3;
+
+  if (_baseURL != nil)
+    {
+      if (baseData->path && *baseData->path)
+        {
+          len += strlen(baseData->path);
+	}
+      else if (baseData->hasNoPath == NO)
+	{
+	  len++;
+	}
+    }
+  if (myData->path && *myData->path)
+    {
+      len += strlen(myData->path);
+    }
+  else if (myData->hasNoPath == NO)
+    {
+      len++;
+    }
+  if (len > 3)
+    {
+      char		buf[len];
+      char		*ptr;
+
+      ptr = [self _path: buf];
+      path = [NSString stringWithUTF8String: ptr];
+    }
+  return path;
+}
+@end
+
