@@ -77,6 +77,7 @@ zfree(void *opaque, void *mem)
 }
 + (void) purge: (NSNotification*)n;
 - (void) cache: (NSDate*)when;
+- (void) close;
 - (NSDate*) expires;
 - (id) initWithHost: (NSHost*)h port: (uint16_t)p forSSL: (BOOL)s;
 - (NSInputStream*) inputStream;
@@ -126,13 +127,28 @@ static NSLock		*pairLock = nil;
 
 - (void) cache: (NSDate*)when
 {
-  ASSIGN(expires, when);
+  NSTimeInterval	ti = [when timeIntervalSinceNow];
+
+  if (ti <= 0.0)
+    {
+      [self close];
+      return;
+    }
+  NSAssert(ip != nil, NSGenericException);
+  if (ti > 120.0)
+    {
+      ASSIGN(expires, [NSDate dateWithTimeIntervalSinceNow: 120.0]);
+    }
+  else
+    { 
+      ASSIGN(expires, when);
+    }
   [pairLock lock];
   [pairCache addObject: self];
   [pairLock unlock];
 }
 
-- (void) dealloc
+- (void) close
 {
   [ip setDelegate: nil];
   [op setDelegate: nil];
@@ -144,6 +160,11 @@ static NSLock		*pairLock = nil;
   [op close];
   DESTROY(ip);
   DESTROY(op);
+}
+
+- (void) dealloc
+{
+  [self close];
   DESTROY(host);
   DESTROY(expires);
   [super dealloc];
@@ -299,7 +320,7 @@ static NSURLProtocol	*placeholder = nil;
     }
   [super dealloc];
 }
-- (void) release
+- (oneway void) release
 {
   /* In a multi-threaded environment we could have two threads release the
    * class at the same time ... causing -dealloc to be called twice at the
@@ -679,10 +700,15 @@ static NSURLProtocol	*placeholder = nil;
 	     wasRedirectedToRequest: request
 		   redirectResponse: nil];
 	}
-      if (_isLoading == NO)
+      if (NO == _isLoading)
         {
-	  return;
+	  return;	// Loading cancelled
 	}
+      if (nil != this->input)
+	{
+	  return;	// Following redirection
+	}
+      // Fall through to continue original connect.
     }
 
   if (0 && this->cachedResponse)
@@ -843,6 +869,8 @@ static NSURLProtocol	*placeholder = nil;
 	  GSMimeHeader		*info;
 	  NSString		*enc;
 	  int			len = -1;
+	  NSString		*ct;
+	  NSString		*st;
 	  NSString		*s;
 
 	  info = [document headerNamed: @"http"];
@@ -878,11 +906,22 @@ static NSURLProtocol	*placeholder = nil;
 	      enc = [[document headerNamed: @"transfer-encoding"] value];
 	    }
 
+	  info = [document headerNamed: @"content-type"];
+	  ct = [document contentType];
+	  st = [document contentSubtype];
+	  if (ct && st)
+	    {
+	      ct = [ct stringByAppendingFormat: @"/%@", st];
+	    }
+	  else
+	    {
+	      ct = nil;
+	    }
 	  _response = [[NSHTTPURLResponse alloc]
 	    initWithURL: [this->request URL]
-	    MIMEType: nil
+	    MIMEType: ct
 	    expectedContentLength: len
-	    textEncodingName: nil];
+	    textEncodingName: [info parameterForKey: @"charset"]];
 	  [_response _setStatusCode: _statusCode text: s];
 	  [document deleteHeaderNamed: @"http"];
 	  [_response _setHeaders: [document allHeaders]];
@@ -1237,7 +1276,8 @@ static NSURLProtocol	*placeholder = nil;
   IF_NO_GC([[self retain] autorelease];)
 
 #if 0
-  NSLog(@"stream: %@ handleEvent: %x for: %@", stream, event, self);
+  NSLog(@"stream: %@ handleEvent: %x for: %@ (ip %p, op %p)",
+    stream, event, self, this->input, this->output);
 #endif
 
   if (stream == this->input) 
@@ -1479,6 +1519,7 @@ static NSURLProtocol	*placeholder = nil;
 		    }
 		  if (_shouldClose == YES)
 		    {
+		      [this->output setDelegate: nil];
 		      [this->output removeFromRunLoop:
 			[NSRunLoop currentRunLoop]
 			forMode: NSDefaultRunLoopMode];
@@ -1494,7 +1535,8 @@ static NSURLProtocol	*placeholder = nil;
     }
   else
     {
-      NSLog(@"Unexpected event %d occurred on stream %@ not being used by %@",
+      NSLog(@"Unexpected event %"PRIuPTR
+	" occurred on stream %@ not being used by %@",
 	event, stream, self);
     }
   if (event == NSStreamEventErrorOccurred)
@@ -1506,7 +1548,7 @@ static NSURLProtocol	*placeholder = nil;
     }
   else
     {
-      NSLog(@"Unexpected event %d ignored on stream %@ of %@",
+      NSLog(@"Unexpected event %"PRIuPTR" ignored on stream %@ of %@",
 	event, stream, self);
     }
 }
@@ -1640,7 +1682,8 @@ static NSURLProtocol	*placeholder = nil;
     }
   else
     {
-      NSLog(@"Unexpected event %d occurred on stream %@ not being used by %@",
+      NSLog(@"Unexpected event %"PRIuPTR
+	" occurred on stream %@ not being used by %@",
 	event, stream, self);
     }
   if (event == NSStreamEventErrorOccurred)
@@ -1652,7 +1695,7 @@ static NSURLProtocol	*placeholder = nil;
     }
   else
     {
-      NSLog(@"Unexpected event %d ignored on stream %@ of %@",
+      NSLog(@"Unexpected event %"PRIuPTR" ignored on stream %@ of %@",
 	event, stream, self);
     }
 }
