@@ -22,10 +22,15 @@
    Boston, MA 02111 USA.
 
    <title>NSUnarchiver class reference</title>
-   $Date: 2011-03-17 12:53:48 +0000 (Thu, 17 Mar 2011) $ $Revision: 32630 $
+   $Date: 2011-05-19 02:19:24 -0600 (Thu, 19 May 2011) $ $Revision: 33069 $
    */
 
 #import "common.h"
+
+#if !defined (__GNU_LIBOBJC__)
+#  include <objc/encoding.h>
+#endif
+
 #define	EXPOSE_NSUnarchiver_IVARS	1
 #include <string.h>
 #import "Foundation/NSDictionary.h"
