@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSValue class reference</title>
-   $Date: 2010-12-23 02:23:05 +0000 (Thu, 23 Dec 2010) $ $Revision: 31765 $
+   $Date: 2011-07-31 09:31:39 -0600 (Sun, 31 Jul 2011) $ $Revision: 33660 $
 */
 
 #import "common.h"
@@ -62,7 +62,7 @@
 @interface GSSizeValue : NSObject	// Help the compiler
 @end
 @class	NSDataStatic;		// Needed for decoding.
-@interface NSDataStatic : NSObject	// Help the compiler
+@interface NSDataStatic : NSData	// Help the compiler
 @end
 
 
@@ -103,9 +103,12 @@ static NSLock			*placeholderLock;
        */
       defaultPlaceholderValue = (GSPlaceholderValue*)
 	NSAllocateObject(GSPlaceholderValueClass, 0, NSDefaultMallocZone());
+      [[NSObject leakAt: (id*)&defaultPlaceholderValue] release];
       placeholderMap = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
 	NSNonRetainedObjectMapValueCallBacks, 0);
+      [[NSObject leakAt: (id*)&placeholderMap] release];
       placeholderLock = [NSLock new];
+      [[NSObject leakAt: (id*)&placeholderLock] release];
     }
 }
 
@@ -383,6 +386,7 @@ static NSLock			*placeholderLock;
 
 - (void) encodeWithCoder: (NSCoder *)coder
 {
+  NSUInteger	tsize;
   unsigned	size;
   const char	*data;
   const char	*objctype = [self objCType];
@@ -420,8 +424,8 @@ static NSLock			*placeholderLock;
       return;
     }
 
-  size = objc_sizeof_type(objctype);
-  data = (void *)NSZoneMalloc([self zone], size);
+  NSGetSizeAndAlignment(objctype, 0, &tsize);
+  data = (void *)NSZoneMalloc([self zone], tsize);
   [self getValue: (void*)data];
   d = [NSMutableData new];
   [d serializeDataAt: data ofObjCType: objctype context: nil];
@@ -439,6 +443,7 @@ static NSLock			*placeholderLock;
   const char	*objctype;
   Class		c;
   id		o;
+  NSUInteger	tsize;
   unsigned	size;
   int		ver;
 
@@ -562,10 +567,10 @@ static NSLock			*placeholderLock;
 	   * For performance, decode small values directly onto the stack,
 	   * For larger values we allocate and deallocate heap space.
 	   */
-	  size = objc_sizeof_type(objctype);
-	  if (size <= 64)
+	  NSGetSizeAndAlignment(objctype, 0, &tsize);
+	  if (tsize <= 64)
 	    {
-	      unsigned char data[size];
+	      unsigned char data[tsize];
 
 	      [coder decodeValueOfObjCType: @encode(id) at: &d];
 	      [d deserializeDataAt: data
@@ -579,7 +584,7 @@ static NSLock			*placeholderLock;
 	    {
 	      unsigned char *data;
 
-	      data = (void *)NSZoneMalloc(NSDefaultMallocZone(), size);
+	      data = (void *)NSZoneMalloc(NSDefaultMallocZone(), tsize);
 	      [coder decodeValueOfObjCType: @encode(id) at: &d];
 	      [d deserializeDataAt: data
 			ofObjCType: objctype
@@ -604,10 +609,10 @@ static NSLock			*placeholderLock;
        * For performance, decode small values directly onto the stack,
        * For larger values we allocate and deallocate heap space.
        */
-      size = objc_sizeof_type(objctype);
-      if (size <= 64)
+      NSGetSizeAndAlignment(objctype, 0, &tsize);
+      if (tsize <= 64)
 	{
-	  unsigned char	data[size];
+	  unsigned char	data[tsize];
 
 	  [coder decodeValueOfObjCType: @encode(unsigned) at: &size];
 	  {
@@ -628,7 +633,7 @@ static NSLock			*placeholderLock;
 	{
 	  void	*data;
 
-	  data = (void *)NSZoneMalloc(NSDefaultMallocZone(), size);
+	  data = (void *)NSZoneMalloc(NSDefaultMallocZone(), tsize);
 	  [coder decodeValueOfObjCType: @encode(unsigned) at: &size];
 	  {
 	    void	*serialized;
@@ -695,7 +700,7 @@ static NSLock			*placeholderLock;
   return 0;
 }
 
-- (void) release
+- (oneway void) release
 {
   return;		// placeholders never get released.
 }
