@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSZone class reference</title>
-   $Date: 2011-02-27 17:53:14 +0000 (Sun, 27 Feb 2011) $ $Revision: 32388 $
+   $Date: 2012-01-09 01:28:27 -0700 (Mon, 09 Jan 2012) $ $Revision: 34468 $
 */
 
 /*  Design goals:
@@ -102,8 +102,7 @@
 void *
 GSOutOfMemory(NSUInteger size, BOOL retry)
 {
-  fprintf(stderr, "GSOutOfMemory ... wanting %lu bytes.\n",
-    (unsigned long)size);
+  fprintf(stderr, "GSOutOfMemory ... wanting %"PRIuPTR" bytes.\n", size);
   return 0;
 }
 
@@ -227,8 +226,121 @@ NSZoneName (NSZone *zone)
     zone = NSDefaultMallocZone();
   return zone->name;
 }
+#if __OBJC_GC__
 
-#if	GS_WITH_GC
+#include <objc/objc-auto.h>
+
+__strong void *
+NSAllocateCollectable(NSUInteger size, NSUInteger options)
+{
+  if (!objc_collecting_enabled())
+    {
+      return calloc(1, size);
+    }
+  id obj = objc_gc_allocate_collectable(size, 
+             ((options & NSScannedOption) == NSScannedOption));
+  if ((options & NSCollectorDisabledOption) == NSCollectorDisabledOption)
+    {
+      obj = objc_gc_retain(obj);
+    }
+  return obj;
+}
+
+__strong void *
+NSReallocateCollectable(void *ptr, NSUInteger size, NSUInteger options)
+{
+  if (!objc_collecting_enabled())
+    {
+      return realloc(ptr, size);
+    }
+  return objc_gc_reallocate_collectable(ptr, size, 
+             ((options & NSScannedOption) == NSScannedOption));
+}
+
+id NSMakeCollectable(id obj)
+{
+  if (objc_collecting_enabled())
+    {
+      objc_gc_release(obj);
+    }
+  return obj;
+}
+
+NSZone*
+NSCreateZone (NSUInteger start, NSUInteger gran, BOOL canFree)
+{
+  return &default_zone;
+}
+
+NSZone*
+NSDefaultMallocZone (void)
+{
+  return &default_zone;
+}
+
+NSZone*
+NSZoneFromPointer (void *ptr)
+{
+  return &default_zone;
+}
+
+void
+NSRecycleZone (NSZone *zone) { }
+
+BOOL
+NSZoneCheck (NSZone *zone)
+{
+  return YES;
+}
+
+struct
+NSZoneStats NSZoneStats (NSZone *zone)
+{
+  struct NSZoneStats stats = { 0 };
+  return stats;
+}
+
+void
+GSMakeWeakPointer(Class theClass, const char *iVarName) { }
+
+BOOL
+GSAssignZeroingWeakPointer(void **destination, void *source)
+{
+  objc_assign_weak(source, (id*)destination);
+  return YES;
+}
+
+void*
+NSZoneMalloc (NSZone *zone, NSUInteger size)
+{
+  return NSZoneCalloc(zone, 1, size);
+}
+
+void*
+NSZoneCalloc (NSZone *zone, NSUInteger elems, NSUInteger bytes)
+{
+  if (objc_collecting_enabled())
+    {
+      // FIXME: Overflow checking
+      size_t size = elems * bytes;
+      return objc_gc_allocate_collectable(size, YES);
+    }
+  return calloc(elems, bytes);
+}
+
+void*
+NSZoneRealloc (NSZone *zone, void *ptr, NSUInteger size)
+{
+  if (objc_collecting_enabled())
+    {
+      return objc_gc_reallocate_collectable(ptr, size, YES);
+    }
+  return realloc(ptr, size);
+}
+
+void NSZoneFree (NSZone *zone, void *ptr) { }
+
+#elif	GS_WITH_GC
 
 #if	defined(DEBUG)
 #define	GC_DEBUG	1
