@@ -25,6 +25,10 @@
 
 #import "common.h"
 
+#if !defined (__GNU_LIBOBJC__)
+#  include <objc/encoding.h>
+#endif
+
 #ifdef HAVE_MALLOC_H
 #include <malloc.h>
 #endif
@@ -39,6 +43,8 @@
 
 #if defined(ALPHA) || (defined(MIPS) && (_MIPS_SIM == _ABIN32))
 typedef long long smallret_t;
+#elif defined(__sparc)
+typedef NSInteger smallret_t;
 #else
 typedef int smallret_t;
 #endif
