@@ -22,24 +22,19 @@
    Boston, MA 02111 USA.
    */
 
-/*
-    CAVEATS:
-	- unloading modules not implemented
-	- would like to raise exceptions without having to turn this into
-	  a .m file (right now NSBundle does this for us, ok?)
-
-*/
+/* PS: Unloading modules is not implemented.  */
 
 #import "common.h"
-
 #include <stdio.h>
-#include <objc/objc-api.h>
-#ifndef NeXT_RUNTIME
-# include <objc/objc-list.h>
-#else
+
+#if defined(NeXT_RUNTIME)
 # include <objc/objc-load.h>
 #endif
 
+#ifdef __GNUSTEP_RUNTIME__
+# include <objc/hooks.h>
+#endif
+
 #include "objc-load.h"
 #import "Foundation/NSException.h"
 
@@ -54,11 +49,6 @@ static BOOL	dynamic_loaded;
 /* Our current callback function */
 static void (*_objc_load_load_callback)(Class, struct objc_category *) = 0;
 
-/* List of modules we have loaded (by handle) */
-#ifndef NeXT_RUNTIME
-static struct objc_list *dynamic_handles = NULL;
-#endif
-
 /* Check to see if there are any undefined symbols. Print them out.
 */
 static int
@@ -186,7 +176,6 @@ GSPrivateLoadModule(NSString *filename, FILE *errorStream,
       _objc_load_callback = 0;
       return 1;
     }
-  dynamic_handles = list_cons(handle, dynamic_handles);
 
   /* If there are any undefined symbols, we can't load the bundle */
   if (objc_check_undefineds(errorStream))
@@ -222,7 +211,9 @@ GSPrivateLoadModule(NSString *filename, FILE *errorStream,
     }
 #endif /* not __ELF__ */
 
+#if !defined(__GNUSTEP_RUNTIME__) && !defined(__GNU_LIBOBJC__)
   __objc_resolve_class_links(); /* fill in subclass_list and sibling_class */
+#endif
   _objc_load_callback = 0;
   _objc_load_load_callback = 0;
   return 0;
@@ -247,6 +238,9 @@ GSPrivateUnloadModule(FILE *errorStream,
 
 
 #ifdef __MINGW__
+// FIXME: We can probably get rid of this now - MinGW should include a working
+// dladdr() wrapping this function, so we no longer need a Windows-only code
+// path
 NSString *
 GSPrivateSymbolPath(Class theClass, Category *theCategory)
 {
@@ -262,19 +256,33 @@ GSPrivateSymbolPath(Class theClass, Category *theCategory)
     }
   return s;
 }
+#elif LINKER_GETSYMBOL 
+NSString *GSPrivateSymbolPath(Class theClass, Category *theCategory)
+{
+	void *addr = (NULL == theCategory) ? (void*)theClass : (void*)theCategory;
+	Dl_info info;
+	// This is correct: dladdr() does the opposite thing to all other UNIX
+	// functions.
+	if (0 == dladdr(addr, &info))
+	{
+		return nil;
+	}
+	return [NSString stringWithUTF8String: info.dli_fname];
+}
 #else
 NSString *
 GSPrivateSymbolPath(Class theClass, Category *theCategory)
 {
   const char *ret;
   char        buf[125], *p = buf;
-  int         len = strlen(theClass->name);
+  const char *className = class_getName(theClass);
+  int         len = strlen(className);
 
   if (theCategory == NULL)
     {
       if (len + sizeof(char)*19 > sizeof(buf))
 	{
-	  p = objc_malloc(len + sizeof(char)*19);
+	  p = malloc(len + sizeof(char)*19);
 
 	  if (p == NULL)
 	    {
@@ -284,8 +292,7 @@ GSPrivateSymbolPath(Class theClass, Category *theCategory)
 	}
 
       memcpy(p, "__objc_class_name_", sizeof(char)*18);
-      memcpy(&p[18*sizeof(char)], theClass->name,
-	     strlen(theClass->name) + 1);
+	  memcpy(&p[18*sizeof(char)], className, strlen(className) + 1);
     }
   else
     {
@@ -293,7 +300,7 @@ GSPrivateSymbolPath(Class theClass, Category *theCategory)
 
       if (len + sizeof(char)*23 > sizeof(buf))
 	{
-	  p = objc_malloc(len + sizeof(char)*23);
+	  p = malloc(len + sizeof(char)*23);
 
 	  if (p == NULL)
 	    {
@@ -314,7 +321,7 @@ GSPrivateSymbolPath(Class theClass, Category *theCategory)
 
   if (p != buf)
     {
-      objc_free(p);
+      free(p);
     }
 
   if (ret)
