@@ -8,7 +8,6 @@
 
 #import "common.h"
 
-#import "GNUstepBase/preface.h"
 #import "Foundation/NSError.h"
 #import "Foundation/NSNotification.h"
 #import "Foundation/NSNotificationQueue.h"
@@ -27,9 +26,6 @@
 #ifdef HAVE_POLL_F
 #include <poll.h>
 #endif
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 
 #define	FDCOUNT	128
 
@@ -198,11 +194,11 @@ static const NSMapTableValueCallBacks WatcherMapValueCallBacks =
       GSIArrayInitWithZoneAndCapacity(watchers, z, 8);
       GSIArrayInitWithZoneAndCapacity(_trigger, z, 8);
 
-      _efdMap = NSCreateMapTable (NSIntMapKeyCallBacks,
+      _efdMap = NSCreateMapTable (NSIntegerMapKeyCallBacks,
 				      WatcherMapValueCallBacks, 0);
-      _rfdMap = NSCreateMapTable (NSIntMapKeyCallBacks,
+      _rfdMap = NSCreateMapTable (NSIntegerMapKeyCallBacks,
 				      WatcherMapValueCallBacks, 0);
-      _wfdMap = NSCreateMapTable (NSIntMapKeyCallBacks,
+      _wfdMap = NSCreateMapTable (NSIntegerMapKeyCallBacks,
 				      WatcherMapValueCallBacks, 0);
     }
   return self;
