@@ -24,9 +24,6 @@
 
 #import "common.h"
 
-#include <unistd.h>
-#include <errno.h>
-
 #import "Foundation/NSData.h"
 #import "Foundation/NSArray.h"
 #import "Foundation/NSDictionary.h"
@@ -195,7 +192,7 @@
     }
   else
     {
-      NSLog(@"_dispatch with unexpected status %d", [self streamStatus]);
+      NSLog(@"_dispatch with unexpected status %"PRIuPTR, [self streamStatus]);
     }
 }
 @end
@@ -324,7 +321,7 @@
     }
   else
     {
-      NSLog(@"_dispatch with unexpected status %d", [self streamStatus]);
+      NSLog(@"_dispatch with unexpected status %"PRIuPTR, [self streamStatus]);
     }
 }
 @end
@@ -354,8 +351,8 @@
              outputStream: (NSOutputStream **)outputStream
 {
   NSString *address = host ? (id)[host address] : (id)@"127.0.0.1";
-  GSSocketStream *ins = nil;
-  GSSocketStream *outs = nil;
+  id ins = nil;
+  id outs = nil;
 
   // try ipv4 first
   ins = AUTORELEASE([[GSInetInputStream alloc]
@@ -388,8 +385,8 @@
                    inputStream: (NSInputStream **)inputStream 
                   outputStream: (NSOutputStream **)outputStream
 {
-  GSSocketStream *ins = nil;
-  GSSocketStream *outs = nil;
+  id ins = nil;
+  id outs = nil;
 
   ins = AUTORELEASE([[GSLocalInputStream alloc] initToAddr: path]);
   outs = AUTORELEASE([[GSLocalOutputStream alloc] initToAddr: path]);
@@ -409,8 +406,8 @@
 + (void) pipeWithInputStream: (NSInputStream **)inputStream 
                 outputStream: (NSOutputStream **)outputStream
 {
-  GSSocketStream *ins = nil;
-  GSSocketStream *outs = nil;
+  id ins = nil;
+  id outs = nil;
   int fds[2];
   int pipeReturn;
 
