@@ -26,25 +26,24 @@
 
 #include "common.h"
 
-#include "GNUstepBase/preface.h"
-#include "Foundation/NSObject.h"
-#include "Foundation/NSData.h"
-#include "Foundation/NSArray.h"
-#include "Foundation/NSString.h"
-#include "Foundation/NSFileHandle.h"
-#include "GNUstepBase/GSFileHandle.h"
-#include "Foundation/NSException.h"
-#include "Foundation/NSRunLoop.h"
-#include "Foundation/NSNotification.h"
-#include "Foundation/NSHost.h"
-#include "Foundation/NSByteOrder.h"
-#include "Foundation/NSProcessInfo.h"
-#include "Foundation/NSUserDefaults.h"
-#include "Foundation/NSDebug.h"
+#import "Foundation/NSObject.h"
+#import "Foundation/NSData.h"
+#import "Foundation/NSArray.h"
+#import "Foundation/NSString.h"
+#import "Foundation/NSFileHandle.h"
+#import "../GSFileHandle.h"
+#import "Foundation/NSException.h"
+#import "Foundation/NSRunLoop.h"
+#import "Foundation/NSNotification.h"
+#import "Foundation/NSHost.h"
+#import "Foundation/NSByteOrder.h"
+#import "Foundation/NSProcessInfo.h"
+#import "Foundation/NSUserDefaults.h"
+#import "Foundation/NSDebug.h"
 
-#include "../Tools/gdomap.h"
-#include "../GSNetwork.h"
-#include "../GSPrivate.h"
+#import "../Tools/gdomap.h"
+#import "../GSPrivate.h"
+#import "../GSNetwork.h"
 
 #include <winsock2.h>
 #include <fcntl.h>
@@ -55,10 +54,6 @@
 #include <stdio.h>
 
 #include <string.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#include <errno.h>
 
 #ifndef	O_BINARY
 #ifdef	_O_BINARY
@@ -2206,7 +2201,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 		 extra: (void*)extra
 	       forMode: (NSString*)mode
 {
-  NSDebugMLLog(@"NSFileHandle", @"%@ event: %lu type: %lu extra: %lu mode: %@",
+  NSDebugMLLog(@"NSFileHandle", @"%@ event: %"PRIuPTR" type: %"PRIuPTR
+    " extra: %"PRIuPTR" mode: %@",
     self, (uintptr_t)data, (uintptr_t)type, (uintptr_t)extra, mode);
 
   if (isNonBlocking == NO)
@@ -2219,7 +2215,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
       if (type != ET_HANDLE)
 	{
-	  NSLog(@"Argh, TRIGGER on socket %lu %lu %@ with event %u desc %u",
+	  NSLog(@"Argh, TRIGGER on socket %"PRIuPTR" %"PRIuPTR
+	    " %@ with event %u desc %u",
 	    (uintptr_t)data, (uintptr_t)extra, mode, (uintptr_t)event,
 	    (uintptr_t)descriptor);
 /*
