@@ -8,7 +8,6 @@
 
 #import "common.h"
 
-#import "GNUstepBase/preface.h"
 #import "Foundation/NSError.h"
 #import "Foundation/NSNotification.h"
 #import "Foundation/NSNotificationQueue.h"
@@ -170,9 +169,9 @@ static const NSMapTableValueCallBacks WatcherMapValueCallBacks =
       GSIArrayInitWithZoneAndCapacity(watchers, z, 8);
       GSIArrayInitWithZoneAndCapacity(_trigger, z, 8);
 
-      handleMap = NSCreateMapTable(NSIntMapKeyCallBacks,
+      handleMap = NSCreateMapTable(NSIntegerMapKeyCallBacks,
               WatcherMapValueCallBacks, 0);
-      winMsgMap = NSCreateMapTable(NSIntMapKeyCallBacks,
+      winMsgMap = NSCreateMapTable(NSIntegerMapKeyCallBacks,
               WatcherMapValueCallBacks, 0);
     }
   return self;
