@@ -22,7 +22,6 @@
    */
 
 #include "common.h"
-#include "GNUstepBase/preface.h"
 #include "GNUstepBase/GSLock.h"
 #include "Foundation/NSArray.h"
 #include "Foundation/NSNotification.h"
@@ -48,7 +47,6 @@
 #include "../GSPortPrivate.h"
 
 #include <stdio.h>
-#include <errno.h>
 
 #define	UNISTR(X) \
 ((const unichar*)[(X) cStringUsingEncoding: NSUnicodeStringEncoding])
