@@ -20,7 +20,6 @@
 #import	"common.h"
 
 #include <stdio.h>
-#include <unistd.h>
 
 #import	"Foundation/NSArray.h"
 #import	"Foundation/NSAutoreleasePool.h"
@@ -45,12 +44,22 @@
 #include	"process.h"
 #endif
 
-#include <fcntl.h>
+
+#if	defined(HAVE_SYS_FCNTL_H)
+#  include	<sys/fcntl.h>
+#elif	defined(HAVE_FCNTL_H)
+#  include	<fcntl.h>
+#endif
+
 #ifdef	HAVE_SYSLOG_H
 #include <syslog.h>
 #endif
 
-#include        <signal.h>
+#if	defined(HAVE_SYS_SIGNAL_H)
+#  include	<sys/signal.h>
+#elif	defined(HAVE_SIGNAL_H)
+#  include	<signal.h>
+#endif
 
 #ifndef NSIG
 #define NSIG    32
@@ -536,7 +545,7 @@ ihandler(int sig)
 
   if (debugging)
     NSLog(@"Adding observer %llu for %@ %@",
-      anObserver, notificationName, anObject);
+      (unsigned long long)anObserver, notificationName, anObject);
 
   connection = [(NSDistantObject*)client connectionForProxy];
   clients = (NSMapTable*)NSMapGet(connections, connection);
@@ -818,7 +827,7 @@ ihandler(int sig)
 		{
 		  [obs->queue removeObjectAtIndex: 0];
   if (debugging)
-    NSLog(@"Posting to observer %llu with %@", obs->observer, n);
+    NSLog(@"Posting to observer %llu with %@", (unsigned long long)obs->observer, n);
 		  [obs->client->client postNotificationName: n->name
 						     object: n->object
 						   userInfo: n->info
@@ -842,7 +851,7 @@ ihandler(int sig)
 {
   if (debugging)
     NSLog(@"Removing observer %llu for %@ %@",
-      observer->observer, observer->notificationName,
+      (unsigned long long)observer->observer, observer->notificationName,
       observer->notificationObject);
 
   if (observer->notificationObject)
