@@ -1,7 +1,7 @@
 /*
    This tool produces a desktop link file for KDE and Gnome out of a GNUstep
    property list.
-   Copyright (C) 20010 Free Software Foundation, Inc.
+   Copyright (C) 2001, 2011 Free Software Foundation, Inc.
 
    Written by:  Fred Kiefer <FredKiefer@gmx.de>
    Created: December 2001
@@ -19,6 +19,8 @@
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
+#import <stdlib.h>
+
 #import	"common.h"
 
 #import	"Foundation/NSArray.h"
@@ -28,6 +30,7 @@
 #import	"Foundation/NSException.h"
 #import	"Foundation/NSFileManager.h"
 #import	"Foundation/NSProcessInfo.h"
+#import "Foundation/NSPathUtilities.h"
 
 int
 main(int argc, char** argv, char **env)
@@ -41,6 +44,8 @@ main(int argc, char** argv, char **env)
   NSDictionary	        *plist = nil;
   NSArray		*list;
   NSString		*entry;
+  NSString              *installPath = @"";
+  NSString              *appName = @"";
 
 #ifdef GS_PASS_ARGUMENTS
   GSInitializeProcess(argc, argv, env);
@@ -98,10 +103,11 @@ main(int argc, char** argv, char **env)
   [fileContents appendString:
     @"[Desktop Entry]\nEncoding=UTF-8\nType=Application\n"];
   [fileContents appendString:
-    @"Version=0.94\nCategories=GNUstep\n"];
+    @"Categories=X-GNUstep;\n"];
   entry = [plist objectForKey: @"ApplicationName"];
   if (entry != nil)
     {
+      appName = entry;
       [fileContents appendFormat: @"Name=%@\n", entry];
       if (destName == nil)
 	destName = [entry stringByAppendingString: @".desktop"];
@@ -111,45 +117,124 @@ main(int argc, char** argv, char **env)
     {
       [fileContents appendFormat: @"Comment=%@\n", entry];
     }
+
+  /* Try to guess where the application will be installed.  PS: At the
+     moment, this is only required for NSIcon, so I suppose we could
+     skip it if there is no NSIcon.  */
+  {
+    /* The default installation domain is the local domain.  Assume
+       that's the case unless something is specified.  */
+    NSSearchPathDomainMask domain = NSLocalDomainMask;
+    NSString *installDomain;
+    NSArray *installPaths;
+
+    installDomain = [[procinfo environment] objectForKey: @"GNUSTEP_INSTALLATION_DOMAIN"];
+    if(installDomain != nil)
+      {
+	if ([installDomain isEqualToString: @"SYSTEM"])
+	  {
+	    domain = NSSystemDomainMask;
+	  }
+	else if ([installDomain isEqualToString: @"NETWORK"])
+	  {
+	    domain = NSNetworkDomainMask;
+	  }
+	else if ([installDomain isEqualToString: @"USER"])
+	  {
+	    domain = NSUserDomainMask;
+	  }
+	
+	/* In all other cases, we leave domain == NSLocalDomainMask.  */
+      }
+
+    installPaths = NSSearchPathForDirectoriesInDomains (NSApplicationDirectory, domain, YES);
+    if ([installPaths count] > 0)
+      {
+	installPath = [installPaths objectAtIndex: 0];
+      }
+    else
+      {
+	/* Ahm - this should never happen.  */
+	GSPrintf(stderr, @"Error determining the application installation location\n");
+
+	/* Try to get any application installation path.  */
+	installPaths = NSSearchPathForDirectoriesInDomains (NSApplicationDirectory, NSAllDomainsMask, YES);
+	if ([installPaths count] > 0)
+	  {
+	    installPath = [installPaths objectAtIndex: 0];
+	    GSPrintf(stderr, @"Assuming it will be installed into '%@' (this may be wrong!)\n", installPath);
+	  }
+	else
+	  {
+	    [pool release];
+	    exit(EXIT_FAILURE);
+	  }
+      }
+  }
+
   entry = [plist objectForKey: @"NSIcon"];
   if (entry != nil)
-  {
-    if ([[entry pathExtension] isEqualToString: @""])
-      [fileContents appendFormat: @"Icon=%@.tiff\n", entry];
-    else
-      [fileContents appendFormat: @"Icon=%@\n", entry];
-  }
+    {
+      NSString *iconPath = [[[[installPath stringByAppendingPathComponent:appName] 
+			       stringByAppendingPathExtension:@"app"] 
+			      stringByAppendingPathComponent:@"Resources"]
+			     stringByAppendingPathComponent:entry];
+
+      if ([[iconPath pathExtension] isEqualToString: @""])
+	{
+	  [fileContents appendFormat: @"Icon=%@.tiff\n", iconPath];
+	}
+      else
+	{
+	  [fileContents appendFormat: @"Icon=%@\n", iconPath];
+	}
+    }
   entry = [plist objectForKey: @"NSExecutable"];
   if (entry != nil)
     {
-      [fileContents appendFormat: @"Exec=openapp %@.app\n", entry];
-      [fileContents appendFormat: @"#TryExec=%@.app\n", entry];
-      [fileContents appendFormat: @"FilePattern=%@.app;%@\n", entry, entry];
+      FILE *fp;
+      char line[130];
+      NSString *execPath = nil;
+      int l = 0;
+
+      fp = popen("which openapp","r");
+      fgets(line,sizeof line,fp);
+      l = strlen(line);
+      line[l-1] = '\0';
+
+      // Build the string to execute the application...
+      execPath = [NSString stringWithCString: line
+			   encoding: NSASCIIStringEncoding];
+      [fileContents appendFormat: @"Exec=%@ %@\n", execPath, entry];
+      [fileContents appendFormat: @"FilePattern=%@.app;%@;\n", entry, entry];
     }
 
   list = [plist objectForKey: @"NSTypes"];
   if (list != nil)
     {
-      unsigned int i;
-
-      [fileContents appendString: @"MimeType="];
-      for (i = 0; i < [list count]; i++)
-	{
-	  NSArray *types;
-	  unsigned int j;
-
-	  plist = [list objectAtIndex: i];
-	  types = [plist objectForKey: @"NSMIMETypes"];
-	  if (types != nil)
+      if([list count] > 0)
+	{  
+	  unsigned int i;
+      
+	  [fileContents appendString: @"MimeType="];
+	  for (i = 0; i < [list count]; i++)
 	    {
-	      for (j = 0; j < [types count]; j++)
+	      NSArray *types;
+	      unsigned int j;
+	      
+	      plist = [list objectAtIndex: i];
+	      types = [plist objectForKey: @"NSMIMETypes"];
+	      if (types != nil)
 		{
-		  entry = [types objectAtIndex: j];
-		  [fileContents appendFormat: @"%@;", entry];
+		  for (j = 0; j < [types count]; j++)
+		    {
+		      entry = [types objectAtIndex: j];
+		      [fileContents appendFormat: @"%@;", entry];
+		    }
 		}
 	    }
+	  [fileContents appendString: @"\n"];
 	}
-      [fileContents appendString: @"\n"];
     }
 
   if ([[fileContents dataUsingEncoding: NSUTF8StringEncoding]
