@@ -18,17 +18,15 @@
 
    */
 
-#include "config.h"
-#include	<Foundation/Foundation.h>
-#include	<Foundation/NSArray.h>
-#include	<Foundation/NSData.h>
-#include	<Foundation/NSException.h>
-#include	<Foundation/NSString.h>
-#include	<Foundation/NSProcessInfo.h>
-#include	<Foundation/NSUserDefaults.h>
-#include	<Foundation/NSDebug.h>
-#include	<Foundation/NSFileHandle.h>
-#include	<Foundation/NSAutoreleasePool.h>
+#import "common.h"
+
+#import	"Foundation/NSArray.h"
+#import	"Foundation/NSData.h"
+#import	"Foundation/NSException.h"
+#import	"Foundation/NSProcessInfo.h"
+#import	"Foundation/NSUserDefaults.h"
+#import	"Foundation/NSFileHandle.h"
+#import	"Foundation/NSAutoreleasePool.h"
 
 
 /** <p> This tool converts a text property list to a binary serialised
@@ -43,7 +41,7 @@ main(int argc, char** argv, char **env)
   unsigned		i;
 
 #ifdef GS_PASS_ARGUMENTS
-  [NSProcessInfo initializeWithArguments:argv count:argc environment:env];
+  GSInitializeProcess(argc, argv, env);
 #endif
   pool = [NSAutoreleasePool new];
   proc = [NSProcessInfo processInfo];
