@@ -1,11 +1,15 @@
 
 #include "objc-common.g"
 
-@interface Test : Object
+@interface Test 
 +(int) testResult;
 @end
 
 @implementation Test
++ (void) initialize
+{
+  return;
+}
 +(int) testResult
 {
   return -1;
